package com.bettersoft.api.config;

import org.apache.http.HttpHost;
import org.apache.http.client.HttpClient;
import org.apache.http.config.Registry;
import org.apache.http.config.RegistryBuilder;
import org.apache.http.config.SocketConfig;
import org.apache.http.conn.routing.HttpRoute;
import org.apache.http.conn.socket.ConnectionSocketFactory;
import org.apache.http.conn.socket.PlainConnectionSocketFactory;
import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.conn.ssl.TrustStrategy;
import org.apache.http.impl.DefaultConnectionReuseStrategy;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.DefaultConnectionKeepAliveStrategy;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.conn.BasicHttpClientConnectionManager;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.ssl.SSLContexts;
import org.springframework.boot.web.client.RestTemplateBuilder;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;
import javax.net.ssl.SSLContext;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.util.concurrent.TimeUnit;

@Component
@Configuration
public class RestTempConfig {

    @Bean(value = "restTemplate")
    public RestTemplate restTemplate() throws NoSuchAlgorithmException, KeyStoreException, KeyManagementException {
        TrustStrategy acceptingTrustStrategy = (cert, authType) -> true;
        SSLContext sslContext = SSLContexts.custom().loadTrustMaterial(null, acceptingTrustStrategy).build();
        SSLConnectionSocketFactory sslsf = new SSLConnectionSocketFactory(sslContext, NoopHostnameVerifier.INSTANCE);

        Registry<ConnectionSocketFactory> socketFactoryRegistry = RegistryBuilder.<ConnectionSocketFactory>create()
                .register("https", sslsf).register("http", new PlainConnectionSocketFactory()).build();

        BasicHttpClientConnectionManager connectionManager = new BasicHttpClientConnectionManager(
                socketFactoryRegistry);
        CloseableHttpClient httpClient = HttpClients.custom().setSSLSocketFactory(sslsf)
                .setConnectionManager(connectionManager).build();

        HttpComponentsClientHttpRequestFactory requestFactory = new HttpComponentsClientHttpRequestFactory(httpClient);

        requestFactory.setConnectionRequestTimeout(2 * 1000);
        requestFactory.setConnectTimeout(2 * 1000);
        requestFactory.setReadTimeout(2 * 1000);
        RestTemplate restTemplate = new RestTemplate(requestFactory);

        return restTemplate;
    }

//    public static HttpClient httpClient(){
//        SocketConfig socketConfig = SocketConfig.custom()
//                .setTcpNoDelay(false)
//                .setSoReuseAddress(true)
//                .setSoTimeout(500)
//                .setSoLinger(60)
//                .setSoKeepAlive(true)
//                .build();
//
//        PoolingHttpClientConnectionManager pcm = new PoolingHttpClientConnectionManager();
//        pcm.setDefaultSocketConfig(socketConfig);
//        pcm.setMaxTotal(600);
//        pcm.setDefaultMaxPerRoute(100);
//        HttpHost httpHost = new HttpHost("localhost", 54188);
//        pcm.setMaxPerRoute(new HttpRoute(httpHost), 200);
//
//        return HttpClientBuilder.create()
//                .setConnectionManager(pcm)
//                .setConnectionReuseStrategy(new DefaultConnectionReuseStrategy())
//                .setKeepAliveStrategy(new DefaultConnectionKeepAliveStrategy())
//                .setConnectionManagerShared(false)
//                .evictIdleConnections(20, TimeUnit.SECONDS)
//                .build();
//    }
}
