package com.bettersoft.api.controllers;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.text.MessageFormat;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;

import javax.annotation.Resource;
import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

import com.alibaba.fastjson.JSONObject;
import net.coobird.thumbnailator.Thumbnails;

import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.RestController;

import com.bettersoft.api.annotation.MenuCode;
import com.bettersoft.api.annotation.ValidatePermission;
import com.bettersoft.api.exceptions.TaskThreadOverloadException;
import com.bettersoft.api.exceptions.ValidateAuthorityFailedException;
import com.bettersoft.api.exceptions.ValidateDataFailedException;
import com.bettersoft.api.exceptions.ValidateSmsDynamicCodeException;
import com.bettersoft.api.exceptions.ValidateSmsDynamicCodeExpirationException;
import com.bettersoft.api.service.interfaces.IInitializingDataService;
import com.bettersoft.api.utils.BigDecimalAdapter;
import com.bettersoft.api.utils.ConfigParamHelper;
import com.bettersoft.api.utils.Constants;
import com.bettersoft.api.utils.EntityUtil;
import com.bettersoft.api.utils.HibernateProxyTypeAdapter;
import com.bettersoft.api.utils.JsonResult;
import com.bettersoft.api.utils.JwtUtils;
import com.bettersoft.api.utils.LoginUserInfo;
import com.bettersoft.api.utils.RedisCacheUtil;
import com.bettersoft.api.utils.StringDigestUtil;
import com.bettersoft.api.validmodel.BaseValidate;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

@RestController
public class BaseController {
    protected Log logger = LogFactory.getLog(getClass());
    protected final String STALE_DATA_ERROR = "STALEDATE_ERROR";

    @Resource
    protected ConfigParamHelper configParam;

    @Resource(name = "initializingDataService")
    protected IInitializingDataService initializingDataService;
    @Resource(name = "redisCacheUtil")
    protected RedisCacheUtil redisCacheUtil;

    protected final String SMS_CACHE_PREFIX = "SMS_";

    @InitBinder
    public void initBinder(WebDataBinder binder) {
        binder.setAutoGrowCollectionLimit(Integer.MAX_VALUE);
    }

    protected Gson gson = new GsonBuilder().serializeNulls()
            .registerTypeAdapterFactory(HibernateProxyTypeAdapter.FACTORY)
            // .registerTypeAdapter(Map.class, new MapTypeAdapter())
            .registerTypeAdapter(BigDecimal.class, new BigDecimalAdapter())
            .setDateFormat("yyyy-MM-dd HH:mm:ss").create();

    protected void renderResponse(Map<String, Object> resultMap,
                                  HttpServletResponse response) throws Exception, IOException {
        response.setContentType("text/json;charset=UTF-8");
        response.setCharacterEncoding(Constants.CHARACTERSET_CODING);
        response.getOutputStream().write(
                gson.toJson(resultMap, Map.class).toString()
                        .getBytes(Constants.CHARACTERSET_CODING));
    }

    protected void renderResponse(JsonResult jsonResult,
                                  HttpServletResponse response) throws Exception, IOException {
        response.setContentType("text/json;charset=UTF-8");
        response.setCharacterEncoding(Constants.CHARACTERSET_CODING);
        response.getOutputStream().write(
                gson.toJson(jsonResult, JsonResult.class).toString()
                        .getBytes(Constants.CHARACTERSET_CODING));
    }

    @SuppressWarnings("rawtypes")
    protected void renderResponse(List results, HttpServletResponse response)
            throws Exception, IOException {
        response.setContentType("text/json;charset=UTF-8");
        response.setCharacterEncoding(Constants.CHARACTERSET_CODING);
        response.getOutputStream().write(
                gson.toJson(results).toString()
                        .getBytes(Constants.CHARACTERSET_CODING));
    }

    protected LoginUserInfo getLoginUser(HttpServletRequest request)
            throws Exception {
		/*if(1==1)
		{
			LoginUserInfo loginUserInfo = new LoginUserInfo();
			loginUserInfo.setUserName("CGXU_TEST");
			return loginUserInfo;
		}*/
        String token = request.getHeader("Authorization");
        if (StringUtils.isEmpty(token))
            throw new ValidateAuthorityFailedException("未获得任何操作权限!");

        String operMenu = request.getParameter("operMenu");
        LoginUserInfo loginUser = EntityUtil.convertMapToEntity(
                LoginUserInfo.class,
                JwtUtils.verifyToken(token, redisCacheUtil));
        loginUser.setCurrentMenu(getMenuCode(operMenu));

        return loginUser;
    }

    protected String[] getAuthStores(HttpServletRequest request) throws Exception {
        LoginUserInfo loginUser = getLoginUser(request);
        if (!StringUtils.isEmpty(loginUser.getAuthStores()) && !loginUser.getAuthStores().equalsIgnoreCase("ALL")) {
            return loginUser.getAuthStores().split(",");
        }

        return null;
    }

    protected String generalThumbMediaData(InputStream inputStream,
                                           String destPath, String thumbName) throws IOException {
        destPath += "thumb_" + thumbName;

        BufferedImage srcImg = ImageIO.read(inputStream);
        int thumbWidth = new BigDecimal(Math.ceil(srcImg.getWidth() * 0.1))
                .intValue();
        int thumbHeight = new BigDecimal(Math.ceil(srcImg.getHeight() * 0.1))
                .intValue();

        Thumbnails.of(srcImg).size(thumbWidth, thumbHeight).toFile(destPath);
        return "thumb_" + thumbName;
    }

    protected String generalThumbMediaData(InputStream inputStream,
                                           String destPath, String thumbName, int thumbWidth, int thumbHeight)
            throws IOException {
        destPath += "thumb_" + thumbName;

        Thumbnails.of(inputStream).size(thumbWidth, thumbHeight)
                .toFile(destPath);
        return "thumb_" + thumbName;
    }

    protected void writeFile(String dataStr, String filePath) throws Exception {
        if (StringUtils.isEmpty(dataStr))
            return;

        FileOutputStream outputStream = null;
        try {
            String floder = StringUtils.substring(filePath, 0,
                    filePath.lastIndexOf("/") + 1);
            createFloder(floder);

            byte[] bytes = StringDigestUtil.decodeBase64Hex(dataStr.getBytes());
            outputStream = new FileOutputStream(filePath);

            outputStream.write(bytes);
            outputStream.flush();
        } finally {
            if (null != outputStream)
                outputStream.close();
        }
    }

    protected void writeFile(String srcPath, String destPath,
                             String fileFullName) throws Exception {
        File srcFile = new File(srcPath);
        if (!srcFile.exists())
            throw new FileNotFoundException();

        FileInputStream inputStream = new FileInputStream(new File(srcPath));

        writeFile(inputStream, destPath, fileFullName);
    }

    protected void writeFile(InputStream inputStream, String destPath,
                             String fileFullName) throws IOException {
        FileOutputStream outputStream = null;
        try {
            createFloder(destPath);
            outputStream = new FileOutputStream(destPath + "/" + fileFullName);

            byte[] buffer = new byte[1024 * 1024];
            int byteread = 0;
            while ((byteread = inputStream.read(buffer)) != -1) {
                outputStream.write(buffer, 0, byteread);
                outputStream.flush();
            }
        } finally {
            if (null != outputStream)
                outputStream.close();
        }
    }

    protected void checkAuthorization(HttpServletRequest request)
            throws Exception {
        String permCode = getValidatePermissionValue();
        String[] menuCodes = StringUtils.split(getMenuCode(), "|");

        LoginUserInfo loginUser = getLoginUser(request);

        if (null == loginUser)
            throw new ValidateAuthorityFailedException("未获得任何操作权限!");

        if (loginUser.isSupperUser())
            return;

        if (!loginUser.getPlatform().equals(Constants.PC_PLATFORM))
            return;

        if (StringUtils.isEmpty(permCode))
            return;

        if (null != loginUser.getAuths() && !loginUser.getAuths().isEmpty()) {
            boolean isAuthorized = false;
            for (String authCode : loginUser.getAuths()) {
                for (String menuCode : menuCodes) {
                    String chkAuthCode = MessageFormat.format("{0}:{1}",
                            menuCode, permCode);
                    if (authCode.equals(chkAuthCode)) {
                        isAuthorized = true;
                        break;
                    }
                }

                if (isAuthorized)
                    break;
            }

            if (!isAuthorized) {
                Map<String, Object> permMap = initializingDataService
                        .getPermissionByKey(permCode);

                throw new ValidateAuthorityFailedException(
                        MessageFormat.format("未获得{0}操作!", MapUtils.getString(
                                permMap, "permissionDesc", StringUtils.EMPTY)));
            } else
                return;
        }

        throw new ValidateAuthorityFailedException("未获得任何操作权限!");
    }

    protected void checkSmsCode(String mobile, String dynamicCode)
            throws Exception {
        if (StringUtils.isEmpty(dynamicCode))
            throw new IOException();

        if (!configParam.getEvn().equalsIgnoreCase("prod"))
            return;

        String validateCode = redisCacheUtil
                .getCache(SMS_CACHE_PREFIX + mobile);
        if (StringUtils.isEmpty(validateCode))
            throw new ValidateSmsDynamicCodeExpirationException();

        if (!validateCode.equals(dynamicCode))
            throw new ValidateSmsDynamicCodeException();

        redisCacheUtil.removeCacheByKey(SMS_CACHE_PREFIX + mobile);
    }

    protected String generalImgData2Base64(String fileFullName)
            throws IOException {
        FileInputStream inputStream = new FileInputStream(
                new File(fileFullName));
        return generalImgData2Base64(inputStream);
    }

    @SuppressWarnings("unused")
    protected String generalImgData2Base64(InputStream inputStream)
            throws IOException {
        try {
            StringBuffer sb = new StringBuffer();
            byte[] buffer = new byte[1024 * 1024];
            int byteread = 0;

            while ((byteread = inputStream.read(buffer)) != -1)
                sb.append(new String(StringDigestUtil.encodeBase64Hex(buffer)));

            return sb.toString();
        } finally {
            inputStream.close();
        }
    }

    protected void validateEnityBean(@Valid BaseValidate bean,
                                     BindingResult result) throws Exception {
        if (result.hasErrors())
            throw new ValidateDataFailedException(getErrorMaps(bean, result));
    }

    private String getErrorMaps(BaseValidate bean, BindingResult result)
            throws Exception {
        String errorTemp = "{0},{1}";
        String errorDesc = StringUtils.EMPTY;

        List<FieldError> errors = result.getFieldErrors();
        for (FieldError error : errors) {
            errorDesc = MessageFormat.format(
                    errorTemp,
                    EntityUtil.getFieldCommon(bean, error.getField()),
                    MessageFormat.format(error.getDefaultMessage(),
                            error.getArguments()));
            break;
        }

        return errorDesc;
    }

    protected void watchTaskThread() throws TaskThreadOverloadException {
        int taskThread = this.redisCacheUtil
                .getCache(Constants.TASK_THREAD_KEY);

        if (taskThread + 1 >= Constants.MAX_TASK_THREADS)
            throw new TaskThreadOverloadException();
    }

    private void createFloder(String floder) throws IOException {
        File file = new File(floder);
        if (!file.exists())
            file.mkdirs();
    }

    private String getMenuCode() throws ClassNotFoundException {
        MenuCode menuCode = Class.forName(this.getClass().getName())
                .getAnnotation(MenuCode.class);

        if (null != menuCode && !StringUtils.isEmpty(menuCode.value()))
            return menuCode.value();

        return StringUtils.EMPTY;
    }

    private String getMenuCode(String operMenu) throws ClassNotFoundException {
        MenuCode menuCode = Class.forName(this.getClass().getName())
                .getAnnotation(MenuCode.class);

        if (null != menuCode && !StringUtils.isEmpty(menuCode.value())) {
            if (!StringUtils.isEmpty(operMenu))
                return 0 <= menuCode.value().indexOf(operMenu) ? operMenu
                        : menuCode.value();
            else
                return menuCode.value();
        }

        return !StringUtils.isEmpty(operMenu) ? operMenu : StringUtils.EMPTY;
    }

    private String getValidatePermissionValue() throws Exception {
        String methodName = Thread.currentThread().getStackTrace()[3]
                .getMethodName();

        Method method = EntityUtil
                .getDeclareMethod(this.getClass(), methodName);

        if (null != method) {
            ValidatePermission validatePerm = method
                    .getAnnotation(ValidatePermission.class);

            if (null != validatePerm
                    && !StringUtils.isEmpty(validatePerm.value()))
                return validatePerm.value();
        }

        return StringUtils.EMPTY;
    }

    protected Boolean checkPwdValid(String pwd) throws Exception {
        String pattern = "^(?![0-9A-Za-z]+$)(?![0-9A-Z\\W]+$)(?![0-9a-z\\W]+$)(?![A-Za-z\\W]+$)[0-9A-Za-z~!@#$%^&*()_+={}<>?,]{8,}$";
        return Pattern.matches(pattern, pwd);
    }
}