package com.bettersoft.api.controllers;

import java.text.MessageFormat;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.bettersoft.api.encrytion.Base64Util;
import com.bettersoft.api.utils.Constants;
import com.bettersoft.api.utils.RedisCacheUtil;
import com.sun.xml.bind.v2.TODO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bettersoft.api.exceptions.ValidateAuthorityFailedException;
import com.bettersoft.api.service.interfaces.ICallJiakeApiService;
import com.bettersoft.api.service.interfaces.IOnlinePrepayService;
import com.bettersoft.api.service.interfaces.IOnlinestoreService;
import com.bettersoft.api.utils.JsonResult;

@RestController
@RequestMapping(value = "/online/callJiake")
public class CallJiakeApiController extends BaseController {
    @Autowired
    private ICallJiakeApiService callJiakeApiService;
    
    @Autowired
    private IOnlinestoreService onlinestoreService;
    
    @Autowired
    private IOnlinePrepayService onlinePrepayService;

    @Resource(name = "redisCacheUtil")
    private RedisCacheUtil redisCacheUtil;
    
    @RequestMapping(value = "/findRemark1/{strstorecode}")
    public void findOnlinestore(@PathVariable("strstorecode") String strstorecode, HttpServletRequest request,
                                HttpServletResponse response) throws Exception {
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult.getData().put("remark1", onlinestoreService.findOnlinestore(strstorecode).getRemark1());
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/callJiake/userDetail error.", ex);
        } finally {
        	logger.info("######获取remark1:"+jsonResult.getData());
            renderResponse(jsonResult, response);
        }
    }
	@RequestMapping(value = "/wx_user", method = RequestMethod.POST) 
    public void wx_user(HttpServletRequest request, HttpServletResponse response,
                        @RequestParam("packageJson") String packageJson) throws Exception {
        Map<String, Object> queryParam =  JSON.parseObject(packageJson);
        JsonResult jsonResult = new JsonResult();

        try {
            //checkAuthorization(request);
        	JSONObject result=callJiakeApiService.callJiakeApi_wxUser_Service(queryParam);
        	if(result==null || !"0".equals(result.get("jk_code"))) {
        		 jsonResult.setSuccess(false);
        		 jsonResult.setErrMsg(result.get("jk_msg")+"");
    		}else{
    			jsonResult.setSuccess(true);
    			jsonResult.getData().putAll(result.getInnerMap());
    		}
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("callJiake/wx_user error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }
    
	@RequestMapping(value = "/wxQuery", method = RequestMethod.POST) 
    public void wxQuery(HttpServletRequest request, HttpServletResponse response,
                        @RequestParam("packageJson") String packageJson) throws Exception {
		Map<String, Object> queryParam =  JSON.parseObject(packageJson);
        JsonResult jsonResult = new JsonResult();

        try {
            //checkAuthorization(request);
        	JSONObject result=callJiakeApiService.callJiakeApi_wxQuery_Service(queryParam);
        	if(result==null || !"0".equals(result.get("jk_code"))) {
        		 jsonResult.setSuccess(false);
        		 jsonResult.setErrMsg(result.get("jk_msg")+"");
    		}else{
    			jsonResult.setSuccess(true);
    			jsonResult.getData().putAll(result.getInnerMap());
    		}
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("callJiake/wx_query error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }
    
    
	@RequestMapping(value = "/wx_creat_foodorder", method = RequestMethod.POST) 
    public void wx_creat_foodorder(HttpServletRequest request, HttpServletResponse response,
                        @RequestParam("packageJson") String packageJson,String token) throws Exception {
		Map<String, Object> queryParam =  JSON.parseObject(packageJson);
        JsonResult jsonResult = new JsonResult();

        try {
            //checkAuthorization(request);
            //1. 前台修改out_trade_no为orderno
            //2. 后台通过orderno查找prepay，生成out，创建支付订单 TODO
            String outTradeNo = onlinePrepayService.searchOutTradeNo(queryParam.get("orderno").toString());
            queryParam.put("out_trade_no",outTradeNo);
            String str = new String(Base64Util.decryBASE64(token));
            JSONObject strAsJson = JSONObject.parseObject(str);
            String openId=strAsJson.get("openid").toString();
            String login_token=strAsJson.get("sessionKey").toString();
            queryParam.put("openid",openId);
            queryParam.put("login_token",login_token);
        	JSONObject result=callJiakeApiService.callJiakeApi_wxCFO_Service(queryParam);
        	if(result==null || !"0".equals(result.get("jk_code"))) {
        		 jsonResult.setSuccess(false);
        		 jsonResult.setErrMsg(result.get("jk_msg")+"");
    		}else{
    			onlinePrepayService.saveOnlinePrepay(result,queryParam,1);
    			jsonResult.setSuccess(true);
    			jsonResult.getData().putAll(result.getInnerMap());
    		}
        	
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("callJiake/wx_creat_foodorder error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }
    

	@RequestMapping(value = "/ali_user", method = RequestMethod.POST) 
    public void ali_user(HttpServletRequest request, HttpServletResponse response,
                        @RequestParam("packageJson") String packageJson) throws Exception {
		Map<String, Object> queryParam =  JSON.parseObject(packageJson);
        JsonResult jsonResult = new JsonResult();

        try {
            //checkAuthorization(request);
        	JSONObject result=callJiakeApiService.callJiakeApi_aliUser_Service(queryParam);
        	if(result==null || !"0".equals(result.get("jk_code"))) {
        		 jsonResult.setSuccess(false);
        		 jsonResult.setErrMsg(result.get("jk_msg")+"");
    		}else{
    			jsonResult.setSuccess(true);
    			jsonResult.getData().putAll(result.getInnerMap());
    		}
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("callJiake/ali_user error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }
   
	@RequestMapping(value = "/ali_query", method = RequestMethod.POST) 
    public void ali_query(HttpServletRequest request, HttpServletResponse response,
                        @RequestParam("packageJson") String packageJson) throws Exception {
		Map<String, Object> queryParam =  JSON.parseObject(packageJson);
        JsonResult jsonResult = new JsonResult();

        try {
            //checkAuthorization(request);
        	JSONObject result=callJiakeApiService.callJiakeApi_wxQuery_Service(queryParam);
        	if(result==null || !"0".equals(result.get("jk_code"))) {
        		 jsonResult.setSuccess(false);
        		 jsonResult.setErrMsg(result.get("jk_msg")+"");
    		}else{
    			jsonResult.setSuccess(true);
    			jsonResult.getData().putAll(result.getInnerMap());
    		}
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("callJiake/ali_query error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }
    
    
   
	@RequestMapping(value = "/ali_creat_foodorder", method = RequestMethod.POST) 
    public void ali_creat_foodorder(HttpServletRequest request, HttpServletResponse response,
                        @RequestParam("packageJson") String packageJson) throws Exception {
		Map<String, Object> queryParam =  JSON.parseObject(packageJson);
		JsonResult jsonResult = new JsonResult();

        try {
            //checkAuthorization(request);
        	JSONObject result=callJiakeApiService.callJiakeApi_aliCFO_Service(queryParam);
        	if(result==null || !"0".equals(result.get("jk_code"))) {
        		 jsonResult.setSuccess(false);
        		 jsonResult.setErrMsg(result.get("jk_msg")+"");
    		}else{
    			onlinePrepayService.saveOnlinePrepay(result,queryParam,2);
    			jsonResult.setSuccess(true);
    			jsonResult.getData().putAll(result.getInnerMap());
    		}
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("callJiake/ali_creat_foodorder error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

}

