package com.bettersoft.api.controllers;

import java.math.BigDecimal;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.alibaba.fastjson.JSONObject;
import com.bettersoft.api.encrytion.AESUtil;
import com.bettersoft.api.encrytion.Base64Util;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.alibaba.fastjson.JSON;
import com.bettersoft.api.exceptions.ValidateAuthorityFailedException;
import com.bettersoft.api.exceptions.ValidateDataFailedException;
import com.bettersoft.api.exceptions.ValidateFailedException;
import com.bettersoft.api.model.OnlineorderhBean;
import com.bettersoft.api.service.interfaces.ICallJiakeApiService;
import com.bettersoft.api.service.interfaces.IOnlineItemService;
import com.bettersoft.api.service.interfaces.IOnlineorderhService;
import com.bettersoft.api.service.interfaces.IOnlineorderpService;
import com.bettersoft.api.utils.Constants;
import com.bettersoft.api.utils.EntityUtil;
import com.bettersoft.api.utils.JsonResult;
import com.bettersoft.api.utils.RedisCacheUtil;
import com.bettersoft.api.utils.RequestMapUtils;
import com.bettersoft.api.validmodel.OnlineItemPackageValidate;
import com.bettersoft.api.validmodel.ShoppingCartValidate;

@RestController
@RequestMapping(value = "/online/buyCart")
public class OnlineBuyCartController extends BaseController {
    @Autowired
    private IOnlineorderhService onlineorderhService;
    @Autowired
    private IOnlineItemService onlineItemService;
    @Resource(name = "redisCacheUtil")
    private RedisCacheUtil redisCacheUtil;
    @Autowired
    private ICallJiakeApiService callJiakeApiService;
    @Autowired
    private IOnlineorderpService onlineorderpService;

    /**
     * 获取门店信息
     * @param storeCode
     * @param request
     * @param response
     * @throws Exception
     */
    @RequestMapping(value = "/storeStatus/{storeCode}", method = RequestMethod.POST)
    public void storeStatus(@PathVariable("storeCode") String storeCode, HttpServletRequest request, HttpServletResponse response,String token) throws Exception {
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult.setSuccess(true);
            jsonResult.getData().putAll(onlineorderhService.findStoreStatus(storeCode,token));
        } catch (ClassNotFoundException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("无法获取门店信息");
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("/online/buyCart/storeStatus error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }
    /**
     * 添加外卖购物车
     * @param storeCode
     * @param token
     * @param request
     * @param response
     * @param packageJson
     * @throws Exception
     */
    @RequestMapping(value = "/addTakeOutCart/{storeCode}", method = RequestMethod.POST)
    public void addTakeOutCart(@PathVariable("storeCode") String storeCode,
                              @RequestParam("token") String token,
                        HttpServletRequest request, HttpServletResponse response,
                        @RequestParam("packageJson") String packageJson) throws Exception {
        JsonResult jsonResult = new JsonResult();
        try {
            // 声明
            String str = new String(Base64Util.decryBASE64(token));
            JSONObject strAsJson = JSONObject.parseObject(str);
            String openId=strAsJson.get("openid").toString();
            OnlineItemPackageValidate itemCartValidate = gson.fromJson(packageJson, OnlineItemPackageValidate.class);
            String cartKey = MessageFormat.format(Constants.BUYCART_REDIS, storeCode, openId);
            ShoppingCartValidate shoppingCart = null;

            if (redisCacheUtil.containsKey(cartKey)&&redisCacheUtil.getCache(cartKey)!=null) {
                shoppingCart = redisCacheUtil.getCache(cartKey);
            } else {
                shoppingCart = new ShoppingCartValidate(storeCode, openId);
            }

            int qty = shoppingCart.addCartItemQrOrder(itemCartValidate);

            String orderNo = onlineorderhService.findUnSettlementOrderNo(storeCode, openId);
            if (!StringUtils.isEmpty(orderNo)) {
                searchOfflineItems(storeCode, orderNo, shoppingCart);
            }
            Map<String, Object> cartMap;
            cartMap = EntityUtil.convertEntityToMap(shoppingCart);
            cartMap.put("cartItems", shoppingCart.getCartItems());
            cartMap.put("orderItems", shoppingCart.getOrderItems());
            cartMap.put("offlineItems", shoppingCart.getOfflineItems());
            jsonResult.getData().put("shoppingCart", cartMap);
            jsonResult.setSuccess(true);
//            redisCacheUtil.setCache(cartKey, shoppingCart);

//            Map<String, Object> cartMap = EntityUtil.convertEntityToMap(shoppingCart);
//            cartMap.put("cartItems", shoppingCart.getCartItems());
//            jsonResult.setSuccess(true);
//            jsonResult.getData().put("shoppingCart", cartMap);
//            shoppingCart.delOrderOfflineItem();
            if(qty<=0){
                shoppingCart.removeCartItem(itemCartValidate);
            }
//86400
            redisCacheUtil.setCache(cartKey, shoppingCart,86400);

//            Map<String, Object> cartMap = EntityUtil.convertEntityToMap(shoppingCart);
//            cartMap.put("cartItems", shoppingCart.getCartItems());
//            jsonResult.setSuccess(true);
//            jsonResult.getData().put("shoppingCart", cartMap);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (ValidateDataFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("/online/buyCart/addTakeOutCart error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }


    /**
     * 添加购物车
     * @param storeCode
     * @param tableCode
     * @param request
     * @param response
     * @param packageJson
     * @throws Exception
     */
    @RequestMapping(value = "/addCart/{storeCode}/{tableCode}", method = RequestMethod.POST)
    public void addCart(@PathVariable("storeCode") String storeCode,
                        @PathVariable("tableCode") String tableCode,
                        HttpServletRequest request, HttpServletResponse response,
                        @RequestParam("packageJson") String packageJson,String token) throws Exception {
        JsonResult jsonResult = new JsonResult();
        try {
            // 声明
            OnlineItemPackageValidate itemCartValidate = gson.fromJson(packageJson, OnlineItemPackageValidate.class);
            String cartKey = MessageFormat.format(Constants.BUYCART_REDIS, storeCode, tableCode);
            ShoppingCartValidate shoppingCart = null;
            if (redisCacheUtil.containsKey(cartKey)) {
                shoppingCart = redisCacheUtil.getCache(cartKey);
            } else {
                shoppingCart = new ShoppingCartValidate(storeCode, tableCode);
            }

            int qty = shoppingCart.addCartItemQrOrder(itemCartValidate);
//            shoppingCart.addCartItem(itemCartValidate);

//            if(itemCartValidate.getQuantity()<=0){
//                shoppingCart.removeCartItem(itemCartValidate);
//            }
//            redisCacheUtil.setCache(cartKey, shoppingCart);

            String orderNo = onlineorderhService.findUnSettlementOrderNo(storeCode, tableCode);
            if (!StringUtils.isEmpty(orderNo)) {
                searchOfflineItems(storeCode, orderNo, shoppingCart);
            }
            Map<String, Object> cartMap;
            cartMap = EntityUtil.convertEntityToMap(shoppingCart);
            cartMap.put("cartItems", shoppingCart.getCartItems());
            cartMap.put("orderItems", shoppingCart.getOrderItems());
            cartMap.put("offlineItems", shoppingCart.getOfflineItems());
            jsonResult.getData().put("shoppingCart", cartMap);
            jsonResult.setSuccess(true);
//            redisCacheUtil.setCache(cartKey, shoppingCart);

//            Map<String, Object> cartMap = EntityUtil.convertEntityToMap(shoppingCart);
//            cartMap.put("cartItems", shoppingCart.getCartItems());
//            jsonResult.setSuccess(true);
//            jsonResult.getData().put("shoppingCart", cartMap);
//            shoppingCart.delOrderOfflineItem();
            if(qty<=0){
                shoppingCart.removeCartItem(itemCartValidate);
            }
            redisCacheUtil.setCache(cartKey, shoppingCart,86400);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (ValidateDataFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("/online/buyCart/addCart error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }
    /**
     * 清空外卖购物车
     * @param request
     * @param response
     * @param storeCode
     * @param token
     * @throws Exception
     */
    @RequestMapping(value = "/clearTakeOutCart/{storeCode}", method = RequestMethod.POST)
    public void clearTakeOutCart(HttpServletRequest request, HttpServletResponse response,
                          @PathVariable("storeCode") String storeCode,
                                 @RequestParam("token") String token) throws Exception {
        JsonResult jsonResult = new JsonResult();
        try {
            String str = new String(Base64Util.decryBASE64(token));
            JSONObject strAsJson = JSONObject.parseObject(str);
            String openId=strAsJson.get("openid").toString();
            String cartKey = MessageFormat.format(Constants.BUYCART_REDIS, storeCode, openId);
            ShoppingCartValidate shoppingCart = null;

            if (redisCacheUtil.containsKey(cartKey)&&redisCacheUtil.getCache(cartKey)!=null) {
                shoppingCart = redisCacheUtil.getCache(cartKey);
            } else {
                shoppingCart = new ShoppingCartValidate(storeCode, openId);
            }
            shoppingCart.resetCartItemsClear();

            redisCacheUtil.setCache(cartKey, shoppingCart,86400);
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (ValidateDataFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("/online/buyCart/clearTakeOutCart error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }
    /**
     * 清空购物车
     * @param request
     * @param response
     * @param storeCode
     * @param tableCode
     * @throws Exception
     */
    @RequestMapping(value = "/clearCart/{storeCode}/{tableCode}", method = RequestMethod.POST)
    public void clearCart(HttpServletRequest request, HttpServletResponse response,
                          @PathVariable("storeCode") String storeCode,
                          @PathVariable("tableCode") String tableCode,String token) throws Exception {
        JsonResult jsonResult = new JsonResult();
        try {
            String cartKey = MessageFormat.format(Constants.BUYCART_REDIS, storeCode, tableCode);
            ShoppingCartValidate shoppingCart = null;
            if (redisCacheUtil.containsKey(cartKey)) {
                shoppingCart = redisCacheUtil.getCache(cartKey);
            } else {
                shoppingCart = new ShoppingCartValidate(storeCode, tableCode);
            }
            shoppingCart.resetCartItemsClear();

            redisCacheUtil.setCache(cartKey, shoppingCart,86400);
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (ValidateDataFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("/online/buyCart/clearCart error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 查看购物车，没用到，findorder替代
     * @param request
     * @param response
     * @param storeCode
     * @param tableCode
     * @throws Exception
     */
    @RequestMapping(value = "/getCart/{storeCode}/{tableCode}", method = RequestMethod.POST)
    public void getCart(HttpServletRequest request, HttpServletResponse response,
                        @PathVariable("storeCode") String storeCode,
                        @PathVariable("tableCode") String tableCode) throws Exception {
        JsonResult jsonResult = new JsonResult();
        Map<String, Object> cartMap;
        try {
            String cartKey = MessageFormat.format(Constants.BUYCART_REDIS, storeCode, tableCode);
            ShoppingCartValidate shoppingCart = redisCacheUtil.getCache(cartKey);
            if (null != shoppingCart) {
                cartMap = EntityUtil.convertEntityToMap(shoppingCart);
                cartMap.put("cartItems", shoppingCart.getCartItems());

            } else {
                shoppingCart = new ShoppingCartValidate(storeCode, tableCode);
                cartMap = EntityUtil.convertEntityToMap(shoppingCart);
            }

            jsonResult.getData().put("shoppingCart", cartMap);
            jsonResult.setSuccess(true);
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("/online/buyCart/getCart error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }
    /**
     * 删除一个外卖购物项
     * @param request
     * @param response
     * @param storeCode
     * @param token
     * @param itemId
     * @throws Exception
     */
    @RequestMapping(value = "/deleteTakeOutItem/{storeCode}/{itemId}", method = RequestMethod.POST)
    public void deleteTakeOutItem(HttpServletRequest request, HttpServletResponse response,
                           @PathVariable("storeCode") String storeCode,
                                  @RequestParam("token") String token,
                           @PathVariable int itemId) throws Exception {
        JsonResult jsonResult = new JsonResult();
        try {
            // 声明
            String str = new String(Base64Util.decryBASE64(token));
            JSONObject strAsJson = JSONObject.parseObject(str);
            String openId=strAsJson.get("openid").toString();
            String cartKey = MessageFormat.format(Constants.BUYCART_REDIS, storeCode, openId);
            ShoppingCartValidate shoppingCart = redisCacheUtil.getCache(cartKey);
            if (null == shoppingCart)
                throw new ClassNotFoundException();

//            shoppingCart.removeCartItem(itemId);



            Map<String, Object> cartMap = EntityUtil.convertEntityToMap(shoppingCart);
            cartMap.put("cartItems", shoppingCart.getCartItems());
            jsonResult.setSuccess(true);
            jsonResult.getData().put("shoppingCart", cartMap);
            shoppingCart.removeCartItem(itemId);
            redisCacheUtil.setCache(cartKey, shoppingCart,86400);
        } catch (ClassNotFoundException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("此购物车不存在！");
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("/online/buyCart/deleteTakeOutItem error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 删除一个购物项,暂时没用到
     * @param request
     * @param response
     * @param storeCode
     * @param tableCode
     * @param itemId
     * @throws Exception
     */
    @RequestMapping(value = "/deleteItem/{storeCode}/{tableCode}/{itemId}", method = RequestMethod.POST)
    public void deleteItem(HttpServletRequest request, HttpServletResponse response,
                           @PathVariable("storeCode") String storeCode,
                           @PathVariable("tableCode") String tableCode,
                           @PathVariable int itemId,String token) throws Exception {
        JsonResult jsonResult = new JsonResult();
        try {
            // 声明
            String cartKey = MessageFormat.format(Constants.BUYCART_REDIS, storeCode, tableCode);
            ShoppingCartValidate shoppingCart = redisCacheUtil.getCache(cartKey);
            if (null == shoppingCart)
                throw new ClassNotFoundException();

            shoppingCart.removeCartItem(itemId);

            redisCacheUtil.setCache(cartKey, shoppingCart,86400);

            Map<String, Object> cartMap = EntityUtil.convertEntityToMap(shoppingCart);
            cartMap.put("cartItems", shoppingCart.getCartItems());
            jsonResult.setSuccess(true);
            jsonResult.getData().put("shoppingCart", cartMap);
        } catch (ClassNotFoundException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("此购物车不存在！");
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("/online/buyCart/deleteItem error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }
    /**
     * 查看外卖购物车
     * @param request
     * @param response
     * @param storeCode
     * @param token
     * @throws Exception
     */
    @RequestMapping(value = "/findTakeOutOrder/{storeCode}", method = RequestMethod.POST)
    public void findTakeOutOrder(HttpServletRequest request,
                          HttpServletResponse response,
                          @PathVariable("storeCode") String storeCode,
                                 @RequestParam("token") String token) throws Exception {
        JsonResult jsonResult = new JsonResult();
        Map<String, Object> cartMap;
        try {
            String str = new String(Base64Util.decryBASE64(token));
            JSONObject strAsJson = JSONObject.parseObject(str);
            String openId=strAsJson.get("openid").toString();
            String cartKey = MessageFormat.format(Constants.BUYCART_REDIS, storeCode, openId);
            ShoppingCartValidate shoppingCart = redisCacheUtil.getCache(cartKey);
            if (null == shoppingCart)
                shoppingCart = new ShoppingCartValidate(storeCode, openId);

            String orderNo = onlineorderhService.findUnSettlementOrderNo(storeCode, openId);
            if (!StringUtils.isEmpty(orderNo)) {
                searchOfflineItems(storeCode, orderNo, shoppingCart);
            }
            ArrayList<String> list=new ArrayList();
            if(shoppingCart.getCartItems()!=null){
                for(OnlineItemPackageValidate onlineItemPackageValidate:shoppingCart.getCartItems()){
                    if(onlineItemPackageValidate.getStrpagecode()!=null){
                        if(!list.contains(onlineItemPackageValidate.getStrpagecode())){
                            list.add(onlineItemPackageValidate.getStrpagecode());
                        }
                    }
                }
            }
            if(shoppingCart.getTotalOfflineAmount()==null){
                shoppingCart.setTotalOfflineAmount(new BigDecimal(0.0));
            }
            cartMap = EntityUtil.convertEntityToMap(shoppingCart);
            cartMap.put("cartItems", shoppingCart.getCartItems());
            cartMap.put("orderItems", shoppingCart.getOrderItems());
            cartMap.put("offlineItems", shoppingCart.getOfflineItems());
            cartMap.put("strpagecode",list);
            jsonResult.getData().put("shoppingCart", cartMap);

            jsonResult.setSuccess(true);
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/buyCart/findTakeOutOrder error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }




    /**
     * 查看购物车
     * @param request
     * @param response
     * @param storeCode
     * @param tableCode
     * @throws Exception
     */
    @RequestMapping(value = "/findOrder/{storeCode}/{tableCode}", method = RequestMethod.POST)
    public void findOrder(HttpServletRequest request,
                          HttpServletResponse response,
                          @PathVariable("storeCode") String storeCode,
                          @PathVariable("tableCode") String tableCode,String token) throws Exception {
        JsonResult jsonResult = new JsonResult();
        Map<String, Object> cartMap;
        try {
            String cartKey = MessageFormat.format(Constants.BUYCART_REDIS, storeCode, tableCode);
            ShoppingCartValidate shoppingCart = redisCacheUtil.getCache(cartKey);
            if (null == shoppingCart)
                shoppingCart = new ShoppingCartValidate(storeCode, tableCode);

            String orderNo = onlineorderhService.findUnSettlementOrderNo(storeCode, tableCode);
            if (!StringUtils.isEmpty(orderNo)) {
                searchOfflineItems(storeCode, orderNo, shoppingCart);
            }
            ArrayList<String> list=new ArrayList();
            if(shoppingCart.getCartItems()!=null){
                for(OnlineItemPackageValidate onlineItemPackageValidate:shoppingCart.getCartItems()){
                    if(onlineItemPackageValidate.getStrpagecode()!=null){
                        if(!list.contains(onlineItemPackageValidate.getStrpagecode())){
                            list.add(onlineItemPackageValidate.getStrpagecode());
                        }
                    }
                }
            }
            if(shoppingCart.getTotalOfflineAmount()==null){
                shoppingCart.setTotalOfflineAmount(new BigDecimal(0.00));
            }
            cartMap = EntityUtil.convertEntityToMap(shoppingCart);
            cartMap.put("cartItems", shoppingCart.getCartItems());
            cartMap.put("orderItems", shoppingCart.getOrderItems());
            cartMap.put("offlineItems", shoppingCart.getOfflineItems());
            cartMap.put("orderNo",orderNo);
            cartMap.put("strpagecode",list);
            jsonResult.getData().put("shoppingCart", cartMap);
            jsonResult.setSuccess(true);
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/buyCart/findOrder error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 下单前购物车详情
     * @param request
     * @param response
     * @param storeCode
     * @param tableCode
     * @throws Exception
     */
    @RequestMapping(value = "/getOrder/{storeCode}/{tableCode}", method = RequestMethod.POST)
    public void getOrder(HttpServletRequest request,
                          HttpServletResponse response,
                          @PathVariable("storeCode") String storeCode,
                          @PathVariable("tableCode") String tableCode,String token,String ms) throws Exception {
        JsonResult jsonResult = new JsonResult();
        Map<String, Object> cartMap;
        try {
            if(ms.equals("1")){
                String cartKey = MessageFormat.format(Constants.BUYCART_REDIS, storeCode, tableCode);
                ShoppingCartValidate shoppingCart = redisCacheUtil.getCache(cartKey);
                if (null == shoppingCart)
                    shoppingCart = new ShoppingCartValidate(storeCode, tableCode);
                String orderNo = onlineorderhService.findUnSettlementOrderNo(storeCode, tableCode);
                if (!StringUtils.isEmpty(orderNo)) {
                    searchOfflineItems(storeCode, orderNo, shoppingCart);
                }
                cartMap = EntityUtil.convertEntityToMap(shoppingCart);
                cartMap.put("cartItems", shoppingCart.getCartItems());
                cartMap.put("orderItems", shoppingCart.getOrderItems());
                cartMap.put("offlineItems", shoppingCart.getOfflineItems());
                cartMap.put("orderNo",orderNo);
            }else {
                String str = new String(Base64Util.decryBASE64(token));
                JSONObject strAsJson = JSONObject.parseObject(str);
                String openId=strAsJson.get("openid").toString();
                String cartKey = MessageFormat.format(Constants.BUYCART_REDIS, storeCode, openId);
                ShoppingCartValidate shoppingCart = redisCacheUtil.getCache(cartKey);
                if (null == shoppingCart)
                    shoppingCart = new ShoppingCartValidate(storeCode, openId);

                String orderNo = onlineorderhService.findUnSettlementOrderNo(storeCode, openId);
                if (!StringUtils.isEmpty(orderNo)) {
                    searchOfflineItems(storeCode, orderNo, shoppingCart);
                }
                cartMap = EntityUtil.convertEntityToMap(shoppingCart);
                cartMap.put("cartItems", shoppingCart.getCartItems());
                cartMap.put("orderItems", shoppingCart.getOrderItems());
                cartMap.put("offlineItems", shoppingCart.getOfflineItems());
                cartMap.put("orderNo",orderNo);
            }
            jsonResult.getData().put("shoppingCart", cartMap);
            jsonResult.setSuccess(true);
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/buyCart/findOrder error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }


    /**
     * 下单，新选择商品下单被保存
     * @param request
     * @param response
     * @param storeCode
     * @param tableCode
     * @throws Exception
     */
    @RequestMapping(value = "/saveOrder/{storeCode}/{tableCode}", method = RequestMethod.POST)
    public void saveOrder(HttpServletRequest request,
                          HttpServletResponse response,
                          @PathVariable("storeCode") String storeCode,
                          @PathVariable("tableCode") String tableCode,String token,Integer num) throws Exception {
        JsonResult jsonResult = new JsonResult();
        String remark = request.getParameter("remark");
        try {
            String cartKey = MessageFormat.format(Constants.BUYCART_REDIS, storeCode, tableCode);
            ShoppingCartValidate shoppingCart = redisCacheUtil.getCache(cartKey);
            shoppingCart.setRemark(remark);
            if (null != shoppingCart) {
                jsonResult.setWxData(onlineorderhService.saveOnlineOrder(shoppingCart, 0,token,null,num));
                logger.info(shoppingCart);
                shoppingCart.resetCartItems();
                logger.info(shoppingCart);
                redisCacheUtil.setCache(cartKey, shoppingCart,86400);
            }
            jsonResult.setSuccess(true);
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/buyCart/saveOrder error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 外卖下单，新选择商品下单被保存
     * @param request
     * @param response
     * @param storeCode
     * @param token
     * @throws Exception
     */
    @RequestMapping(value = "/saveOrderTakeOut/{storeCode}", method = RequestMethod.POST)
    public void saveOrderTakeOut(HttpServletRequest request,
                          HttpServletResponse response,
                          @PathVariable("storeCode") String storeCode, String token) throws Exception {
        JsonResult jsonResult = new JsonResult();
        String remark = request.getParameter("remark");
        String str = new String(Base64Util.decryBASE64(token));
        JSONObject strAsJson = JSONObject.parseObject(str);
        String openid = strAsJson.getString("openid");
        try {
            String cartKey = MessageFormat.format(Constants.BUYCART_REDIS, storeCode, openid);
            ShoppingCartValidate shoppingCart = redisCacheUtil.getCache(cartKey);
            shoppingCart.setRemark(remark);
            if (null != shoppingCart) {
                jsonResult.setWxData(onlineorderhService.saveOnlineOrder(shoppingCart, 0,token,"takeoff",0));
//                redisCacheUtil.removeCacheByKey(cartKey);
            }
            jsonResult.setSuccess(true);
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/buyCart/saveOrder error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 项目中未使用接口
     * @param storeCode
     * @param request
     * @param response
     * @param packageJson
     * @throws Exception
     */
    @RequestMapping(value = "/saveOrderScanPayment/{storeCode}", method = RequestMethod.POST)
    public void saveOrderScanPaymeny(@PathVariable("storeCode") String storeCode, 
                        HttpServletRequest request, HttpServletResponse response,
                        @RequestParam("packageJson") String packageJson) throws Exception {
        JsonResult jsonResult = new JsonResult();
        Map<String, Object> queryParam = null;
        String tableCode = "1" ;
        OnlineorderhBean order = null;
        Map<String, Object> orderInfoMap = null;
        try { 
            // 声明
        	queryParam =  JSON.parseObject(packageJson);
            String remark = queryParam.get("remark").toString();
            String paymentMoney = queryParam.get("paymentMoney").toString();
           
            ShoppingCartValidate shoppingCart = new ShoppingCartValidate(storeCode, tableCode);
            OnlineItemPackageValidate itemCartValidate =  new OnlineItemPackageValidate(); 
            
            itemCartValidate.setPrice(new BigDecimal(paymentMoney));
            itemCartValidate.setPackageType("I");
            //初始化值
            itemCartValidate.setItemId(999);
            shoppingCart.addCartItem(itemCartValidate);
            shoppingCart.setRemark(remark);
            
            if (null != shoppingCart) {
            	orderInfoMap  = onlineorderhService.saveOnlineOrderScanPaymeny(shoppingCart, 0);
            }
            jsonResult.setSuccess(true);
            jsonResult.getData().put("orderInfo", orderInfoMap);
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/buyCart/saveOrderScanPaymeny error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 结算前查看是否有这个订单，订单是否生效
     * @param request
     * @param response
     * @param storeCode
     * @param tableCode
     * @throws Exception
     */
    @RequestMapping(value = "/checkPayment/{storeCode}/{tableCode}", method = RequestMethod.POST)
    public void checkPayment(HttpServletRequest request,
                             HttpServletResponse response,
                             @PathVariable("storeCode") String storeCode,
                             @PathVariable("tableCode") String tableCode,String token) throws Exception {
        JsonResult jsonResult = new JsonResult();
        try {
            Map<String, Object> result = new HashMap<String, Object>();
            result.put("allowPay", onlineorderhService.checkPayment(storeCode, tableCode));
            jsonResult.setData(result);
            jsonResult.setSuccess(true);
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/buyCart/checkPayment error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 外卖结算前查看是否有这个订单，订单是否生效
     * @param request
     * @param response
     * @param storeCode
     * @param token
     * @throws Exception
     */
    @RequestMapping(value = "/checkPaymentTakeOut/{storeCode}", method = RequestMethod.POST)
    public void checkPaymentTakeOut(HttpServletRequest request,
                             HttpServletResponse response,
                             @PathVariable("storeCode") String storeCode, String token) throws Exception {
        JsonResult jsonResult = new JsonResult();
        String str = new String(Base64Util.decryBASE64(token));
        JSONObject strAsJson = JSONObject.parseObject(str);
        String openid = strAsJson.getString("openid");
        try {
            Map<String, Object> result = new HashMap<String, Object>();
            result.put("allowPay", onlineorderhService.checkPayment(storeCode, openid));
            jsonResult.setData(result);
            jsonResult.setSuccess(true);
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/buyCart/checkPayment error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 支付订单回调接口
     * @param request
     * @param response
     * @throws Exception
     */
    @RequestMapping(value = "/paymentOrder", method = RequestMethod.POST)
    public JSONObject paymentOrder(HttpServletRequest request, HttpServletResponse response,
                             @RequestBody JSONObject resultAsJson) throws Exception {
        JsonResult jsonResult = new JsonResult();
        if(!resultAsJson.get("event_type").equals("TRANSACTION.SUCCESS")){
            JSONObject result = new JSONObject();
            result.put("code","FAIL");
            result.put("message","失败");
            return result;
        }
        JSONObject resource = resultAsJson.getJSONObject("resource");
        String result = AESUtil.decryptToString(resource.getBytes("associated_data"),resource.getBytes("nonce"),resource.getBytes("ciphertext"));
        JSONObject resAsJson = JSONObject.parseObject(result);
        OnlineorderhBean onlineorderhBean = onlineorderhService.findOnlineorderh(resAsJson.getString("out_trade_no"));
        try {
            String cartKey = MessageFormat.format(Constants.BUYCART_REDIS, onlineorderhBean.getStrstorecode(), onlineorderhBean.getTablecode());
            ShoppingCartValidate shoppingCart = redisCacheUtil.getCache(cartKey);
            if (null != shoppingCart) {
                onlineorderhService.paymentOnlineOrder(onlineorderhBean.getStrstorecode(), onlineorderhBean.getTablecode());
                redisCacheUtil.removeCacheByKey(cartKey);
                onlineorderpService.insertIntoOnlineOrderp(onlineorderhBean,resAsJson);
            }
            jsonResult.setSuccess(true);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return null;
    }

    private void searchOfflineItems(String storeCode, String orderNo, ShoppingCartValidate shoppingCart) throws Exception {
        List<Map<String, Object>> offlineOrderDetails = onlineorderhService.searchOfflineOrderDetails(orderNo);

        if (null != offlineOrderDetails && !offlineOrderDetails.isEmpty()) {
            for (Map<String, Object> offlineOrder : offlineOrderDetails) {
                OnlineItemPackageValidate offlineItem = onlineItemService.findOfflineItem(storeCode, orderNo, offlineOrder);
                shoppingCart.setOrderItem(offlineItem);
            }
        }
    }

    private boolean setChildPackageItem(List<OnlineItemPackageValidate> childPackages, OnlineItemPackageValidate orderItem, boolean isChecked) {
        if (null != childPackages && !childPackages.isEmpty()) {
            for (OnlineItemPackageValidate childPackage : childPackages) {
                if (null != childPackage.getChildPackages() && !childPackage.getChildPackages().isEmpty()) {
                    isChecked = setChildPackageItem(childPackage.getChildPackages(), orderItem, isChecked);
                }

                isChecked = (childPackage.getItemId() == orderItem.getItemId() && childPackage.getPackageType().equals(childPackage.getPackageType())) || isChecked;
                childPackage.setChecked(isChecked);
            }
        }

        return isChecked;
    }

    /**
     * 项目中没有使用位置
     * @param request
     * @param response
     * @throws Exception
     */
    @RequestMapping(value = "/paymentOnlineSuccess", method = RequestMethod.POST)
    // @ValidatePermission("UPDATE")
     public void paymentOnlineSuccess(HttpServletRequest request, HttpServletResponse response) throws Exception {
         JsonResult jsonResult = new JsonResult();
         Map<String, Object> queryParam = RequestMapUtils.RequestMapToMap(request.getParameterMap());
         logger.info(queryParam.toString());
         try {
             //checkAuthorization(request);
             //String[] orderNoArr = StringUtils.split(orderNos, "|");
         	String orderNo=request.getParameter("orderNo");
         	int paymentCode=Integer.parseInt(request.getParameter("paymentCode"));
         	String if_orderNo=request.getParameter("if_orderNo");//(String)queryParam.get("if_orderNo");
         	String openid=request.getParameter("openid");
         	//String orderNo=(String)queryParam.get("orderNo");
             
             //String orderNo, int paymentCode,String if_orderNo,String openid
         	OnlineorderhBean order = onlineorderhService.paymentOnlineOrder(orderNo,paymentCode, if_orderNo,openid);
            
             String storeCode = order.getStrstorecode();//MapUtils.getString(order, "storeCode", StringUtils.EMPTY);
             String tableCode = order.getTablecode();//MapUtils.getString(order, "tableCode", StringUtils.EMPTY);

             String cartKey = MessageFormat.format(Constants.BUYCART_REDIS, storeCode, tableCode);
             if (redisCacheUtil.containsKey(cartKey))
                 redisCacheUtil.removeCacheByKey(cartKey);
     
             if(configParam.getAliPaymentCode().equals(paymentCode)){
 					callJiakeApiService.callJiakeApi_aliQuery_Service(queryParam);
 			 }else{
 					callJiakeApiService.callJiakeApi_wxQuery_Service(queryParam);
 			 }
             jsonResult.setSuccess(true);
         } catch (ValidateAuthorityFailedException ex) {
             jsonResult.setSuccess(false);
             jsonResult.setErrMsg(ex.getMessage());
         } catch (ClassNotFoundException ex) {
             jsonResult.setSuccess(false);
             jsonResult.setErrMsg(MessageFormat.format("订单号:{0}, 无法获取订单信息!", ex.getMessage()));
         } catch (ValidateFailedException ex) {
             jsonResult.setSuccess(false);
             jsonResult.setErrMsg(MessageFormat.format("订单号:{0}, 已被结账不允许重复操作!", ex.getMessage()));
         } catch (Exception ex) {
             jsonResult.setSuccess(false);
             logger.error("online/onlineorderh/paymentOnlineSuccess error.", ex);
         } finally {
             renderResponse(jsonResult, response);
         }
     }

    /**
     * 获取取餐号
     * @param request
     * @param response
     * @param orderNo
     *
     * @throws Exception
     */
    @RequestMapping(value = "/findDysn", method = RequestMethod.POST)
    public void findDysn(HttpServletRequest request,
                             HttpServletResponse response,
                              String orderNo,String token) throws Exception {
        JsonResult jsonResult = new JsonResult();
        try {
            Map<String, Object> result = new HashMap<String, Object>();
            OnlineorderhBean onlineorderhBean=onlineorderhService.findOnlineorderh(orderNo);
            if(onlineorderhBean!=null && onlineorderhBean.getDaysn()!=null)
            result.put("dysn",String.format("%04d",onlineorderhBean.getDaysn()));
            jsonResult.setData(result);
            jsonResult.setSuccess(true);
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/buyCart/findDysn error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }


    /**
     * 支付前查看购物车中商品是否下架
     * @param request
     * @param response
     * @param storeCode
     *
     * @throws Exception
     */
    @RequestMapping(value = "/checkBuyCartStatus", method = RequestMethod.POST)
    public void checkBuyCartStatus(HttpServletRequest request,
                         HttpServletResponse response,
                         String storeCode,String token,String tableCode) throws Exception {
        JsonResult jsonResult = new JsonResult();
        try {
            String str = new String(Base64Util.decryBASE64(token));
            JSONObject strAsJson = JSONObject.parseObject(str);
            String openId=strAsJson.get("openid").toString();
            String cartKey;
            if(StringUtils.isEmpty(tableCode)){
                cartKey = MessageFormat.format(Constants.BUYCART_REDIS, storeCode, openId);
            } else {
                cartKey = MessageFormat.format(Constants.BUYCART_REDIS, storeCode, tableCode);
            }
            ShoppingCartValidate shoppingCart = null;

            if (redisCacheUtil.containsKey(cartKey)&&redisCacheUtil.getCache(cartKey)!=null) {
                shoppingCart = redisCacheUtil.getCache(cartKey);
            }
            onlineItemService.checkBuyCartStatus(shoppingCart,jsonResult);
            redisCacheUtil.setCache(cartKey,shoppingCart,86400);
            jsonResult.setSuccess(true);
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/buyCart/checkBuyCartStatus error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }
}
