package com.bettersoft.api.controllers;

import com.alibaba.fastjson.JSONObject;
import com.bettersoft.api.annotation.MenuCode;
import com.bettersoft.api.annotation.ValidatePermission;
import com.bettersoft.api.encrytion.Base64Util;
import com.bettersoft.api.exceptions.ValidateAuthorityFailedException;
import com.bettersoft.api.exceptions.ValidateDataFailedException;
import com.bettersoft.api.exceptions.ValidateDataOccupiedException;
import com.bettersoft.api.service.interfaces.IOnlineItemService;
import com.bettersoft.api.utils.*;
import com.bettersoft.api.validmodel.OnlineItemPackageValidate;
import com.bettersoft.api.validmodel.OnlineItemValidate;
import com.bettersoft.api.validmodel.ShoppingCartValidate;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.text.MessageFormat;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping(value = "/online/onlineItem")
@MenuCode("SMG03")
public class OnlineItemController extends BaseController {
    @Autowired
    private IOnlineItemService onlineItemService;

    @Resource(name = "redisCacheUtil")
    private RedisCacheUtil redisCacheUtil;

    /**
     * 获取商品详细信息，包括客制化和套餐
     * @param storeCode
     * @param itemId
     * @param request
     * @param response
     * @throws Exception
     */
    @RequestMapping(value = "/findOnlineItem/{storeCode}/{itemId}", method = RequestMethod.POST)
    public void findOnlineItem(@PathVariable("storeCode") String storeCode,
                               @PathVariable("itemId") int itemId,
                               HttpServletRequest request,
                               HttpServletResponse response,String token) throws Exception {
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult.getData().put("onlineItem", onlineItemService.findOnlineItem(storeCode, itemId));
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/onlineItem/findOnlineItem error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 获取分栏下面的全部商品
     * @param request
     * @param response
     * @throws Exception
     */
    @RequestMapping(value = "/searchClientItems", method = RequestMethod.POST)
    public void searchClientItems(HttpServletRequest request, HttpServletResponse response,String token) throws Exception {
        Map<String, Object> queryParam = RequestMapUtils.RequestMapToMap(request.getParameterMap());
        JsonResult jsonResult = new JsonResult();
        System.out.println(queryParam+"====================================");
        try {
            // 查询上架商品
            queryParam.put("i.status", 1);
            //checkAuthorization(request);
            if(!queryParam.containsKey("tableCode")){
                String str = new String(Base64Util.decryBASE64(token));
                JSONObject strAsJson = JSONObject.parseObject(str);
                String openId=strAsJson.get("openid").toString();
                queryParam.put("tableCode",openId);
            }
            jsonResult.getData().putAll(onlineItemService.searchItemQr(queryParam));
//            queryParam.remove("tableCode");
//            jsonResult.getData().putAll(onlineItemService.searchItem(queryParam));
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/onlineItem/searchClientItems error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 后台管理系统使用，获取商品
     * @param request
     * @param response
     * @throws Exception
     */
    @SuppressWarnings("unchecked")
    @RequestMapping(value = "/searchItem")
    @ValidatePermission("SEARCH")
    public void searchItem(HttpServletRequest request, HttpServletResponse response) throws Exception {
        Map<String, Object> queryParam = RequestMapUtils.RequestMapToMap(request.getParameterMap());
        JsonResult jsonResult = new JsonResult();
        try {
            if (queryParam.containsKey("i.status")) {
                queryParam.put("i.status", Integer.parseInt(MapUtils.getString(queryParam, "i.status", "0")));
            }

            checkAuthorization(request);
            String[] authStores = getAuthStores(request);
            if (null != authStores && 0 < authStores.length) {
                queryParam.put(Constants.QUERY_SPEC_CONDITION + "strstorecode", "i.strstorecode in ('" + StringUtils.join(authStores, "','") + "')");
            }

            jsonResult.getData().putAll(onlineItemService.searchItem(queryParam));
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/onlineItem/searchOnlineItem error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 获取左侧分栏信息
     * @param strstorecode
     * @param request
     * @param response
     * @throws Exception
     */
    @SuppressWarnings("unchecked")
    @RequestMapping(value = "/getAllStrposfastkeyname")
    public void getAllStrposfastkeyname(@RequestParam String strstorecode, HttpServletRequest request, HttpServletResponse response,String token) throws Exception {
        JsonResult jsonResult = new JsonResult();
        try {
            //checkAuthorization(request);
            jsonResult.getData().putAll(onlineItemService.getAllStrposfastkeyname(strstorecode));
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/onlineItem/searchOnlineItem error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 项目暂时没有看到使用位置
     * @param request
     * @param response
     * @param onlineItemValidate
     * @param result
     * @throws Exception
     */
    @RequestMapping(value = "/saveOnlineItem", method = RequestMethod.POST)
    @ValidatePermission("INSERT")
    public void saveOnlineItem(HttpServletRequest request, HttpServletResponse response,
                               @Validated OnlineItemValidate onlineItemValidate, BindingResult result) throws Exception {
        JsonResult jsonResult = new JsonResult();
        LoginUserInfo loginUser = getLoginUser(request);

        try {
            checkAuthorization(request);
            validateEnityBean(onlineItemValidate, result);
            onlineItemService.saveOnlineItem(onlineItemValidate, loginUser);
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (ValidateDataFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (ValidateDataOccupiedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.getData().put("errMsg", "关键字已存在，不允许重复！");
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/onlineItem/saveRole error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 项目暂时没有看到使用位置
     * @param request
     * @param response
     * @param onlineItemCode
     * @param onlineItemModel
     * @param result
     * @throws Exception
     */
    @RequestMapping(value = "/updateOnlineItem/{onlineItemCode}", method = RequestMethod.POST)
    @ValidatePermission("UPDATE")
    public void updateOnlineItem(HttpServletRequest request, HttpServletResponse response,
                                 @PathVariable("onlineItemCode") String onlineItemCode,
                                 @Validated OnlineItemValidate onlineItemModel, BindingResult result) throws Exception {
        JsonResult jsonResult = new JsonResult();
        LoginUserInfo loginUser = getLoginUser(request);
        try {
            checkAuthorization(request);
            validateEnityBean(onlineItemModel, result);
            onlineItemService.updateOnlineItem(onlineItemModel, loginUser);
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (ValidateDataFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/onlineItem/updateDept error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 项目暂时没看到使用位置
     * @param request
     * @param response
     * @param onlineItemCode
     * @throws Exception
     */
    @RequestMapping(value = "/deleteOnlineItem/{onlineItemCode}", method = RequestMethod.POST)
    @ValidatePermission("DELETE")
    public void deleteOnlineItem(HttpServletRequest request, HttpServletResponse response,
                                 @PathVariable("onlineItemCode") String onlineItemCode) throws Exception {
        JsonResult jsonResult = new JsonResult();
        LoginUserInfo loginUser = getLoginUser(request);
        try {
            checkAuthorization(request);
            onlineItemService.deleteOnlineItem(onlineItemCode, loginUser);
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (ValidateDataFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/onlineItem/deleteRole error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 后台管理系统使用,商品上架
     * @param request
     * @param response
     * @param itemValidate
     * @throws Exception
     */
    @RequestMapping(value = "/itemOnline", method = RequestMethod.POST)
    @ValidatePermission("UPDATE")
    public void itemOnline(HttpServletRequest request, HttpServletResponse response, OnlineItemValidate itemValidate)
            throws Exception {
        JsonResult jsonResult = new JsonResult();
        LoginUserInfo loginUser = getLoginUser(request);
        try {
            checkAuthorization(request);
            onlineItemService.updateOnlineItemStatus(itemValidate, 1, loginUser);
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (ValidateDataFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/onlineItem/itemOnline error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 后台管理系统使用，商品下架
     * @param request
     * @param response
     * @param itemValidate
     * @throws Exception
     */
    @RequestMapping(value = "/itemOffline", method = RequestMethod.POST)
    @ValidatePermission("UPDATE")
    public void offlineItem(HttpServletRequest request, HttpServletResponse response, OnlineItemValidate itemValidate) throws Exception {
        JsonResult jsonResult = new JsonResult();
        LoginUserInfo loginUser = getLoginUser(request);
        try {
            checkAuthorization(request);
            onlineItemService.updateOnlineItemStatus(itemValidate, 0, loginUser);
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (ValidateDataFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/onlineItem/itemOffline error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 后台管理系统使用
     * @param imageFile
     * @param storeCode
     * @param itemId
     * @param request
     * @param response
     * @throws Exception
     */
    @RequestMapping(value = "/uploadImg/{storeCode}/{itemId}", method = RequestMethod.POST)
    public void uploadImg(@RequestParam("image") MultipartFile imageFile,
                          @PathVariable("storeCode") String storeCode,
                          @PathVariable("itemId") int itemId,
                          HttpServletRequest request, HttpServletResponse response)
            throws Exception {
        JsonResult jsonResult = new JsonResult();
        try {
            String imageType = StringUtils.substringAfterLast(
                    imageFile.getOriginalFilename(), ".");

            String imageName = MessageFormat.format("{0}.{1}",
                    itemId, imageType);
            String destPath = MessageFormat.format(configParam.getUploadImg(), storeCode);
            writeFile(imageFile.getInputStream(), destPath, imageName);
            onlineItemService.updateOnlineItemImage(storeCode, itemId, imageName, getLoginUser(request));
            jsonResult.setSuccess(true);
        } catch (ClassNotFoundException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("无法获取商品信息！");
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("系统错误，请与系统管理员联系！");
            logger.error("online/onlineItem/uploadImg error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }
}

