package com.bettersoft.api.controllers;

import com.bettersoft.api.exceptions.ValidateAuthorityFailedException;
import com.bettersoft.api.exceptions.ValidateDataFailedException;
import com.bettersoft.api.exceptions.ValidateFailedException;
import com.bettersoft.api.service.interfaces.IOnlineorderPosApiService;
import com.bettersoft.api.utils.Constants;
import com.bettersoft.api.utils.LoginUserInfo;
import com.bettersoft.api.validmodel.PosParameterValidate;
import com.google.gson.Gson;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.text.MessageFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping(value = "/online/pos")
public class OnlineorderPosInterfaceController extends BaseController {
    @Autowired
    private IOnlineorderPosApiService onlineorderPosApiService;

    /**
     * 提供给pos的接口，尽量不做更改
     * @param request
     * @param response
     * @param posParam
     * @param result
     * @throws Exception
     */
    @RequestMapping(value = "/executeApi", method = RequestMethod.POST)
    public void executeApi(HttpServletRequest request,
                           HttpServletResponse response,
                           @Validated PosParameterValidate posParam, BindingResult result) throws Exception {
        Map<String, Object> resultMap = new HashMap<String, Object>();

        try {
            validateEnityBean(posParam, result);
            if (!posParam.validateParameter(configParam.getSecret()))
                throw new ValidateAuthorityFailedException();

            Map<String, Object> reqParam = new Gson().fromJson(posParam.getPara(), Map.class);
            Map<String, Object> queryParam = new HashMap<String, Object>();
            String storeCode = MapUtils.getString(reqParam, "storecode", StringUtils.EMPTY);
            String orderNo = MapUtils.getString(reqParam, "orderno", StringUtils.EMPTY);
            String processFlag = MapUtils.getString(reqParam, "processflg", StringUtils.EMPTY);
            List<String> suborderno = (List<String>) MapUtils.getObject(reqParam, "suborderno", null);
            String posorderno = MapUtils.getString(reqParam, "posorderno", StringUtils.EMPTY);
            String tableCode = MapUtils.getString(reqParam, "tablecode", StringUtils.EMPTY);
            //wyj20200308 bug fix----S
            //String dataType = MapUtils.getString(reqParam, "dataType", StringUtils.EMPTY);
            String dataType = MapUtils.getString(reqParam, "datatype", StringUtils.EMPTY);
            //wyj20200308 bug fix----E
            List<Map<String, Object>> itemDetails = (List<Map<String, Object>>) MapUtils.getObject(reqParam, "detail", null);
            List<Map<String, Object>> payments = (List<Map<String, Object>>) MapUtils.getObject(reqParam, "payments", null);

            if (null != posParam) {
                switch (posParam.getMethod().toLowerCase()) {
                    case "getunprocessorders":
                        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
                        queryParam.put("strstorecode", storeCode);
                        queryParam.put("status", 1);
                        queryParam.put(Constants.QUERY_START_DATE + "dtorderstart",
                                sdf.format(new Date()));
                        queryParam.put(Constants.QUERY_END_DATE + "dtorderstart",
                                sdf.format(new Date()));
                        List<String> orderlist=onlineorderPosApiService.getUnsettleOrders(queryParam);
                        //queryParam.put("strstorecode", storeCode);
                        queryParam.put("status", 3);//已支付
                        List<String> orderlistPay=onlineorderPosApiService.getUnsettleOrders(queryParam);
                        if(null!=orderlistPay&&orderlistPay.size()>0) {
                        	 orderlist.addAll(orderlistPay);
                        }
                        logger.info("未处理订单:"+orderlist);
                        resultMap.put("orderlist",orderlist);
                        break;
                    case "getorderdetail":
                        resultMap = onlineorderPosApiService.getOrderDeatil(storeCode, orderNo);
                        break;
                    case "orderdone":
                        queryParam.clear();
                        queryParam.put("storeCode", storeCode);
                        queryParam.put("orderNo", orderNo);
                        queryParam.put("processFlag", processFlag);
                        queryParam.put("suborderNos", suborderno);
                        queryParam.put("posorderNo", posorderno);
                        if(!onlineorderPosApiService.confirmOrder(queryParam)){
                            resultMap.put("jk_msg", "该订单号:"+orderNo+"为外带订单，无法进行操作！");
                        }
                        break;
                    case "offlineorderupload":
                        queryParam.clear();
                        queryParam.put("storeCode", storeCode);
                        queryParam.put("tableCode", tableCode);
                        queryParam.put("posorderNo", posorderno);
                        queryParam.put("itemDetails", itemDetails);
                        queryParam.put("payments", payments);
                        if (StringUtils.isEmpty(tableCode))
                            throw new ValidateFailedException("无桌位参数信息");

                        if (dataType.equals("D")) {
                            onlineorderPosApiService.offlineOrderSync(queryParam, getLoginUser());
                        } else if (dataType.equals("P")) {
                            onlineorderPosApiService.onlineOrderSettlement(queryParam, getLoginUser());
                            String cartKey = MessageFormat.format(Constants.BUYCART_REDIS, storeCode, tableCode);
                            if (redisCacheUtil.containsKey(cartKey)) {
                                redisCacheUtil.removeCacheByKey(cartKey);
                            }
                        }
                        break;
                    case "ordercomplete":
                        queryParam.clear();
                        queryParam.put("storeCode", storeCode);
                        queryParam.put("orderNo", orderNo);
//                        queryParam.put("processFlag", processFlag);
//                        queryParam.put("suborderNos", suborderno);
                        queryParam.put("posorderNo", posorderno);
                        onlineorderPosApiService.sellOrder(queryParam);
                        break;
                }
            }
            resultMap.put("jk_code", "0");
        } catch (ValidateDataFailedException ex) {
            resultMap.put("jk_code", "2");
            resultMap.put("jk_msg", ex.getMessage());
        } catch (ValidateAuthorityFailedException ex) {
            resultMap.put("jk_code", "1");
            resultMap.put("jk_msg", "数字签名验证失败!");
        } catch (ValidateFailedException ex) {
            resultMap.put("jk_code", "2");
            resultMap.put("jk_msg", ex.getMessage());
        } catch (ClassNotFoundException ex) {
            resultMap.put("jk_code", "2");
            resultMap.put("jk_msg", MessageFormat.format("订单号:{0}, 无法获取订单信息!", ex.getMessage()));
        } catch (Exception ex) {
            resultMap.put("jk_code", "1");
            logger.error("online/pos/executeApi error.", ex);
        } finally {
            renderResponse(resultMap, response);
        }
    }

    private LoginUserInfo getLoginUser() {
        LoginUserInfo loginUser = new LoginUserInfo();
        loginUser.setUserCode("SYSTEM");
        loginUser.setUserName("SYSTEM");

        return loginUser;
    }
}
