package com.bettersoft.api.controllers;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.bettersoft.api.annotation.MenuCode;
import com.bettersoft.api.annotation.ValidatePermission;
import com.bettersoft.api.exceptions.ValidateAuthorityFailedException;
import com.bettersoft.api.exceptions.ValidateDataFailedException;
import com.bettersoft.api.exceptions.ValidateDataOccupiedException;
import com.bettersoft.api.service.interfaces.IOnlineorderdService;
import com.bettersoft.api.utils.LoginUserInfo;
import com.bettersoft.api.utils.RequestMapUtils;
import com.bettersoft.api.validmodel.OnlineorderdValidate;
import com.bettersoft.api.utils.JsonResult;

@RestController
@RequestMapping(value = "/online/onlineorderd")
//@MenuCode("PLM01")
public class OnlineorderdController extends BaseController
{
	@Autowired
	private IOnlineorderdService onlineorderdService;

	@RequestMapping(value = "/findOnlineorderd/{id}")
	@ValidatePermission("SEARCH")
	public void findOnlineorderd(@PathVariable("id") String id, HttpServletRequest request,
			HttpServletResponse response) throws Exception
	{
		JsonResult jsonResult = new JsonResult();

		try
		{
			checkAuthorization(request);

			jsonResult.getData().put("onlineorderd", onlineorderdService.findOnlineorderd(id));
			jsonResult.setSuccess(true);
		} 
		catch (ValidateAuthorityFailedException ex)
		{
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} 
		catch (Exception ex)
		{
			jsonResult.setSuccess(false);
			logger.error("online/user/userDetail error.", ex);
		} 
		finally
		{
			renderResponse(jsonResult, response);
		}
	}

	/**
	 * 项目中没有使用位置
	 * @param request
	 * @param response
	 * @throws Exception
	 */
	@SuppressWarnings("unchecked")
	@RequestMapping(value = "/searchByPage")
	public void searchByPage(HttpServletRequest request, HttpServletResponse response) throws Exception
	{
		Map<String, Object> queryParam = RequestMapUtils.RequestMapToMap(request.getParameterMap());
		JsonResult jsonResult = new JsonResult();

		try
		{
			//checkAuthorization(request);
			jsonResult.getData().putAll(onlineorderdService.searchByPage(queryParam));
			jsonResult.setSuccess(true);
		} 
		catch (ValidateAuthorityFailedException ex)
		{
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} 
		catch (Exception ex)
		{
			jsonResult.setSuccess(false);
			logger.error("online/onlineorderd/searchOnlineorderd error.", ex);
		} 
		finally
		{
			renderResponse(jsonResult, response);
		}
	}

	/**
	 * 项目中没有使用位置
	 * @param request
	 * @param response
	 * @param onlineorderdValidate
	 * @param result
	 * @throws Exception
	 */
	@RequestMapping(value = "/saveOnlineorderd", method = RequestMethod.POST)
	@ValidatePermission("INSERT")
	public void saveOnlineorderd(HttpServletRequest request, HttpServletResponse response,
			@Validated OnlineorderdValidate onlineorderdValidate, BindingResult result) throws Exception
	{
		JsonResult jsonResult = new JsonResult();
		LoginUserInfo loginUser = getLoginUser(request);

		try
		{
			checkAuthorization(request);
			validateEnityBean(onlineorderdValidate, result);
			onlineorderdService.saveOnlineorderd(onlineorderdValidate, loginUser);
			jsonResult.setSuccess(true);
		} 
		catch (ValidateAuthorityFailedException ex)
		{
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} 
		catch (ValidateDataFailedException ex)
		{
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} 
		catch (ValidateDataOccupiedException ex)
		{
			jsonResult.setSuccess(false);
			jsonResult.getData().put("errMsg", "关键字已存在，不允许重复！");
		} 
		catch (Exception ex)
		{
			jsonResult.setSuccess(false);
			logger.error("online/onlineorderd/saveRole error.", ex);
		} 
		finally
		{
			renderResponse(jsonResult, response);
		}
	}

	/**
	 * 项目中没有使用位置
	 * @param request
	 * @param response
	 * @param onlineorderdCode
	 * @param onlineorderdModel
	 * @param result
	 * @throws Exception
	 */
	@RequestMapping(value = "/updateOnlineorderd/{onlineorderdCode}", method = RequestMethod.POST)
	@ValidatePermission("UPDATE")
	public void updateOnlineorderd(HttpServletRequest request, HttpServletResponse response,
			@PathVariable("onlineorderdCode") String onlineorderdCode,
			@Validated OnlineorderdValidate onlineorderdModel, BindingResult result) throws Exception
	{
		JsonResult jsonResult = new JsonResult();
		LoginUserInfo loginUser = getLoginUser(request);
		try
		{
			checkAuthorization(request);
			validateEnityBean(onlineorderdModel, result);
			onlineorderdService.updateOnlineorderd(onlineorderdModel, loginUser);
			jsonResult.setSuccess(true);
		} 
		catch (ValidateAuthorityFailedException ex)
		{
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} 
		catch (ValidateDataFailedException ex)
		{
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} 
		catch (Exception ex)
		{
			jsonResult.setSuccess(false);
			logger.error("online/onlineorderd/updateDept error.", ex);
		} 
		finally
		{
			renderResponse(jsonResult, response);
		}
	}

	/**
	 * 项目中没有使用位置
	 * @param request
	 * @param response
	 * @param onlineorderdCode
	 * @throws Exception
	 */
	@RequestMapping(value = "/deleteOnlineorderd/{onlineorderdCode}", method = RequestMethod.POST)
	@ValidatePermission("DELETE")
	public void deleteOnlineorderd(HttpServletRequest request, HttpServletResponse response,
			@PathVariable("onlineorderdCode") String onlineorderdCode) throws Exception
	{
		JsonResult jsonResult = new JsonResult();
		LoginUserInfo loginUser = getLoginUser(request);
		try
		{
			checkAuthorization(request);
			onlineorderdService.deleteOnlineorderd(onlineorderdCode, loginUser);
			jsonResult.setSuccess(true);
		} 
		catch (ValidateAuthorityFailedException ex)
		{
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} 
		catch (ValidateDataFailedException ex)
		{
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} 
		catch (Exception ex)
		{
			jsonResult.setSuccess(false);
			logger.error("online/onlineorderd/deleteRole error.", ex);
		} 
		finally
		{
			renderResponse(jsonResult, response);
		}
	}
}

