package com.bettersoft.api.controllers;

import java.text.MessageFormat;
import java.text.SimpleDateFormat;
import java.util.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.alibaba.fastjson.JSONObject;
import com.bettersoft.api.dao.interfaces.IOnlinestoreDao;
import com.bettersoft.api.dao.interfaces.IYourenMemberDao;
import com.bettersoft.api.encrytion.Base64Util;
import com.bettersoft.api.exceptions.ValidateFailedException;
import com.bettersoft.api.model.Entity.OnlineorderpList;
import com.bettersoft.api.model.OnlineorderhBean;
import com.bettersoft.api.service.interfaces.ICallJiakeApiService;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.bettersoft.api.annotation.MenuCode;
import com.bettersoft.api.annotation.ValidatePermission;
import com.bettersoft.api.exceptions.ValidateAuthorityFailedException;
import com.bettersoft.api.service.interfaces.IOnlineorderhService;
import com.bettersoft.api.utils.RequestMapUtils;
import com.bettersoft.api.utils.Constants;
import com.bettersoft.api.utils.JsonResult;
import com.bettersoft.api.utils.LangUtils;

@RestController

@RequestMapping(value = "/online/onlineorderh")
@MenuCode("SMG02")
public class OnlineorderhController extends BaseController {
    @Autowired
    private IOnlineorderhService onlineorderhService;

    @Autowired
    private IYourenMemberDao yourenMemberDao;

    @Autowired
    private IOnlinestoreDao onlinestoreDao;

    @Autowired
    private ICallJiakeApiService callJiakeApiService;

    /**
     * 项目中没有用到
     *
     * @param id
     * @param request
     * @param response
     * @throws Exception
     */
    @RequestMapping(value = "/findOnlineorderh/{id}")
    @ValidatePermission("SEARCH")
    public void findOnlineorderh(@PathVariable("id") String id, HttpServletRequest request,
                                 HttpServletResponse response) throws Exception {
        JsonResult jsonResult = new JsonResult();

        try {
            checkAuthorization(request);

            jsonResult.getData().put("onlineorderh", onlineorderhService.findOnlineorderh(id));
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/user/userDetail error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 后台管理系统使用,查找订单
     *
     * @param request
     * @param response
     * @throws Exception
     */
    @SuppressWarnings("unchecked")
    @RequestMapping(value = "/searchByPage", method = RequestMethod.POST)
    @ValidatePermission("SEARCH")
    public void searchByPage(HttpServletRequest request, HttpServletResponse response) throws Exception {
        Map<String, Object> queryParam = RequestMapUtils.RequestMapToMap(request.getParameterMap());
        if (queryParam.containsKey("status"))
            queryParam.put("status", LangUtils.optInteger((queryParam.get("status"))));
        JsonResult jsonResult = new JsonResult();
//        queryParam.put("offlineupdateflg", 0);
//        if(queryParam.containsKey("tablecode")&&!queryParam.get("tablecode").equals("qr")){
//            queryParam.remove("tablecode");
//            queryParam.put(Constants.QUERY_NOTEQUAL + "tablecode", "qr");
//        }
        try {
            checkAuthorization(request);
            String[] authStores = getAuthStores(request);
            int auth = 0;
            if(authStores!=null&&queryParam.get("like.strstorecode")!=null){
                for (String authStore:authStores) {
                    if(queryParam.get("like.strstorecode").equals(authStore)){
                        auth =1;
                        break;
                    }
                }
            } else {
                auth =1;
            }
            if (null != authStores && 0 < authStores.length) {
                queryParam.put(Constants.QUERY_SPEC_CONDITION + "strstorecode", "strstorecode in ('" + StringUtils.join(authStores, "','") + "')");
            }
            if(auth==1){
                jsonResult.getData().putAll(onlineorderhService.searchByPage(queryParam));
                jsonResult.setSuccess(true);
            } else {
                jsonResult.setSuccess(false);
                jsonResult.setErrMsg("无操作权限!");
            }
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/onlineorderh/searchOnlineorderh error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 后台管理系统使用，查看订单详细信息
     *
     * @param orderNo
     * @param request
     * @param response
     * @throws Exception
     */
    @RequestMapping(value = "/findOrderDetails/{orderNo}", method = RequestMethod.POST)
    @ValidatePermission("SEARCH")
    public void findOrderDetails(@PathVariable("orderNo") String orderNo, HttpServletRequest request, HttpServletResponse response) throws Exception {
        JsonResult jsonResult = new JsonResult();
        try {
            checkAuthorization(request);
            jsonResult.getData().put("details", onlineorderhService.searchOrderDetails(orderNo));
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/onlineorderh/findOrderDetails error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 后台管理系统使用，接单
     *
     * @param orderNos
     * @param request
     * @param response
     * @throws Exception
     */
    @RequestMapping(value = "/acceptOrder/{orderNos}", method = RequestMethod.POST)
    @ValidatePermission("UPDATE")
    public void acceptOrder(@PathVariable("orderNos") String orderNos, HttpServletRequest request, HttpServletResponse response) throws Exception {
        JsonResult jsonResult = new JsonResult();
        try {
            checkAuthorization(request);
            String[] orderNoArr = StringUtils.split(orderNos, "|");
            onlineorderhService.acceptOnlineOrder(orderNoArr, getLoginUser(request));
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (ClassNotFoundException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(MessageFormat.format("订单号:{0}, 无法获取订单信息!", ex.getMessage()));
        } catch (ValidateFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(MessageFormat.format("订单号:{0}, 已被接单不允许重复操作!", ex.getMessage()));
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/onlineorderh/acceptOrder error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 后台管理系统使用，标记为线下结账
     *
     * @param orderNos
     * @param request
     * @param response
     * @throws Exception
     */
    @RequestMapping(value = "/paymentOrder/{orderNos}", method = RequestMethod.POST)
    @ValidatePermission("UPDATE")
    public void paymentOrder(@PathVariable("orderNos") String orderNos, HttpServletRequest request, HttpServletResponse response) throws Exception {
        JsonResult jsonResult = new JsonResult();
        try {
            checkAuthorization(request);
            String[] orderNoArr = StringUtils.split(orderNos, "|");
            List<Map<String, Object>> orders = onlineorderhService.paymentOffOnlineOrder(orderNoArr, getLoginUser(request));
            if (null != orders && !orders.isEmpty()) {
                for (Map<String, Object> order : orders) {
                    String storeCode = MapUtils.getString(order, "storeCode", StringUtils.EMPTY);
                    String tableCode = MapUtils.getString(order, "tableCode", StringUtils.EMPTY);

                    String cartKey = MessageFormat.format(Constants.BUYCART_REDIS, storeCode, tableCode);
                    if (redisCacheUtil.containsKey(cartKey))
                        redisCacheUtil.removeCacheByKey(cartKey);
                }
            }
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (ClassNotFoundException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(MessageFormat.format("订单号:{0}, 无法获取订单信息!", ex.getMessage()));
        } catch (ValidateFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(MessageFormat.format("订单号:{0}, 已被结账不允许重复操作!", ex.getMessage()));
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/onlineorderh/paymentOrder error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 暂时未使用
     *
     * @param request
     * @param response
     * @throws Exception
     */
    @RequestMapping(value = "/paymentOnlineSuccess", method = RequestMethod.POST)
    // @ValidatePermission("UPDATE")

    public void paymentOnlineSuccess(HttpServletRequest request, HttpServletResponse response) throws Exception {
        JsonResult jsonResult = new JsonResult();
        Map<String, Object> queryParam = RequestMapUtils.RequestMapToMap(request.getParameterMap());
        try {
            //checkAuthorization(request);
            //String[] orderNoArr = StringUtils.split(orderNos, "|");
            String orderNo = (String) queryParam.get("orderNo");
            int paymentCode = Integer.parseInt((String) queryParam.get("paymentCode"));
            String if_orderNo = (String) queryParam.get("if_orderNo");
            String openid = (String) queryParam.get("openid");
            //String orderNo=(String)queryParam.get("orderNo");


            //String orderNo, int paymentCode,String if_orderNo,String openid
            OnlineorderhBean order = onlineorderhService.paymentOnlineOrder(orderNo, paymentCode, if_orderNo, openid);

            String storeCode = order.getStrstorecode();//MapUtils.getString(order, "storeCode", StringUtils.EMPTY);
            String tableCode = order.getTablecode();//MapUtils.getString(order, "tableCode", StringUtils.EMPTY);

            String cartKey = MessageFormat.format(Constants.BUYCART_REDIS, storeCode, tableCode);
            if (redisCacheUtil.containsKey(cartKey))
                redisCacheUtil.removeCacheByKey(cartKey);

            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (ClassNotFoundException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(MessageFormat.format("订单号:{0}, 无法获取订单信息!", ex.getMessage()));
        } catch (ValidateFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(MessageFormat.format("订单号:{0}, 已被结账不允许重复操作!", ex.getMessage()));
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/onlineorderh/paymentOnlineSuccess error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    @RequestMapping(value = "/findOrderList", method = RequestMethod.POST)
    public void findOrderList(@RequestParam("token") String token, HttpServletResponse response, HttpServletRequest request) throws Exception {
        JsonResult jsonResult = new JsonResult();
        try {
            String str = new String(Base64Util.decryBASE64(token));
            JSONObject strAsJson = JSONObject.parseObject(str);

            String openId = strAsJson.get("openid").toString();
            List<OnlineorderpList> list = onlineorderhService.findOnlineorderhListByOpenId(openId);
            if (list != null && list.size() > 0) {
                for (OnlineorderpList onlineorderhBean : list) {
                    Date date = onlineorderhBean.getPaydate();
                    SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy年MM月dd日");
                    onlineorderhBean.setPaydates(simpleDateFormat.format(date));
                    SimpleDateFormat simpleDateFormat1 = new SimpleDateFormat("MM dd");
                    onlineorderhBean.setDate(simpleDateFormat1.format(date));
                    SimpleDateFormat simpleDateFormatdedate = new SimpleDateFormat("yyyy年MM月dd日 HH:mm:ss");
                    onlineorderhBean.setDedate(simpleDateFormatdedate.format(date));
                }
                jsonResult.getData().put("OnlineorderhList", list);
            }
            jsonResult.setSuccess(true);

        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/onlineorderh/findOrderList error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }


    /**
     * 前台点单系统使用，查看订单详细信息
     *
     * @param orderNo
     * @param request
     * @param response
     * @throws Exception
     */
    @RequestMapping(value = "/findOrderReceptionDetails/{orderNo}", method = RequestMethod.POST)
    public void findOrderReceptionDetails(@PathVariable("orderNo") String orderNo, @RequestParam("token") String token, HttpServletRequest request, HttpServletResponse response) throws Exception {
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult.getData().put("details", yourenMemberDao.general(orderNo, null));
            jsonResult.setWxData(onlineorderhService.findOnlineorderh(orderNo).getPosorderno());
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/onlineorderh/findOrderReceptionDetails error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    @PostMapping("/freshHistory")
    @ResponseBody
    public JsonResult freshHistory(HttpServletRequest request, HttpServletResponse response, String token) {
        JsonResult jsonResult = new JsonResult();
        try {
            String str = new String(Base64Util.decryBASE64(token));
            JSONObject strAsJson = JSONObject.parseObject(str);
            String openId = strAsJson.get("openid").toString();
            if (onlineorderhService.findOnlineorderhOrderNoByOpenId(openId)) {
                jsonResult.setSuccess(true);
                jsonResult.setErrMsg("无更新订单!");
            }
            jsonResult.setSuccess(true);
            jsonResult.setErrMsg("更新完成!");
        } catch (Exception e) {
            e.printStackTrace();
            logger.info(e);
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("更新失败!");
        }
        return jsonResult;
    }

    /**
     * 后台管理系统使用,导出订单数据
     *
     * @param request
     * @param response
     * @throws Exception
     */
    @SuppressWarnings("unchecked")
    @RequestMapping(value = "/exportExcel", method = RequestMethod.POST)
    @ValidatePermission("SEARCH")
    @ResponseBody
    public JsonResult exportExcel(HttpServletRequest request, HttpServletResponse response) throws Exception {
        Map<String, Object> queryParam = RequestMapUtils.RequestMapToMap(request.getParameterMap());
        if (queryParam.containsKey("status"))
            queryParam.put("status", LangUtils.optInteger((queryParam.get("status"))));
        JsonResult jsonResult = new JsonResult();
//        queryParam.put("offlineupdateflg", 0);
//        if(queryParam.containsKey("tablecode")&&!queryParam.get("tablecode").equals("qr")){
//            queryParam.remove("tablecode");
//            queryParam.put(Constants.QUERY_NOTEQUAL + "tablecode", "qr");
//        }
        try {
            checkAuthorization(request);
            String[] authStores = getAuthStores(request);
            if (null != authStores && 0 < authStores.length) {
                queryParam.put(Constants.QUERY_SPEC_CONDITION + "strstorecode", "strstorecode in ('" + StringUtils.join(authStores, "','") + "')");
            }
            if(onlineorderhService.exportExcel(response,queryParam)){
                jsonResult.setSuccess(true);
            }
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
            logger.error("online/onlineorderh/searchOnlineorderh error.", ex);
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/onlineorderh/searchOnlineorderh error.", ex);
        }
        return jsonResult;
    }

}
