package com.bettersoft.api.controllers;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.bettersoft.api.utils.Constants;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.bettersoft.api.annotation.MenuCode;
import com.bettersoft.api.annotation.ValidatePermission;
import com.bettersoft.api.exceptions.ValidateAuthorityFailedException;
import com.bettersoft.api.exceptions.ValidateDataFailedException;
import com.bettersoft.api.exceptions.ValidateDataOccupiedException;
import com.bettersoft.api.service.interfaces.IOnlineorderpService;
import com.bettersoft.api.utils.LoginUserInfo;
import com.bettersoft.api.utils.RequestMapUtils;
import com.bettersoft.api.validmodel.OnlineorderpValidate;
import com.bettersoft.api.utils.JsonResult;

@RestController
@RequestMapping(value = "/online/onlineorderp")
//@MenuCode("PLM01")
public class OnlineorderpController extends BaseController {
    @Autowired
    private IOnlineorderpService onlineorderpService;

    /**
     * 通过id获取订单支付详细信息
     * 项目中没有使用位置
     * @param id
     * @param request
     * @param response
     * @throws Exception
     */
    @RequestMapping(value = "/findOnlineorderp/{id}")
    @ValidatePermission("SEARCH")
    public void findOnlineorderp(@PathVariable("id") String id, HttpServletRequest request,
                                 HttpServletResponse response) throws Exception {
        JsonResult jsonResult = new JsonResult();

        try {
//            checkAuthorization(request);

            jsonResult.getData().put("onlineorderp", onlineorderpService.findOnlineorderp(id));
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/user/userDetail error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 项目中没有使用位置
     * @param request
     * @param response
     * @throws Exception
     */
    @SuppressWarnings("unchecked")
    @RequestMapping(value = "/searchByPage")
    public void searchByPage(HttpServletRequest request, HttpServletResponse response) throws Exception {
        Map<String, Object> queryParam = RequestMapUtils.RequestMapToMap(request.getParameterMap());
        JsonResult jsonResult = new JsonResult();

        try {
//            checkAuthorization(request);
            jsonResult.getData().putAll(onlineorderpService.searchByPage(queryParam));
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/onlineorderp/searchOnlineorderp error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 项目中没有使用位置
     * @param request
     * @param response
     * @param storeCode
     * @param tablecode
     * @param onlineorderpValidate
     * @param result
     * @throws Exception
     */
    @RequestMapping(value = "/saveOnlineorderp/{storeCoce}/{tableCode}", method = RequestMethod.POST)
    // @ValidatePermission("INSERT")
    public void saveOnlineorderp(HttpServletRequest request, HttpServletResponse response,
                                 @PathVariable("storeCode") String storeCode, @PathVariable("tableCode") String tablecode,
                                 @Validated OnlineorderpValidate onlineorderpValidate, BindingResult result) throws Exception {
        JsonResult jsonResult = new JsonResult();
        LoginUserInfo loginUser = getLoginUser(request);

        try {
            // checkAuthorization(request);
            validateEnityBean(onlineorderpValidate, result);
            onlineorderpService.saveOnlineorderp(onlineorderpValidate, loginUser);
            redisCacheUtil.removeCacheByKey(Constants.BUYCART_REDIS + storeCode + tablecode);
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (ValidateDataFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (ValidateDataOccupiedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.getData().put("errMsg", "关键字已存在，不允许重复！");
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/onlineorderp/saveRole error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 项目中没有使用位置
     * @param request
     * @param response
     * @param onlineorderpCode
     * @param onlineorderpModel
     * @param result
     * @throws Exception
     */
    @RequestMapping(value = "/updateOnlineorderp/{onlineorderpCode}", method = RequestMethod.POST)
    @ValidatePermission("UPDATE")
    public void updateOnlineorderp(HttpServletRequest request, HttpServletResponse response,
                                   @PathVariable("onlineorderpCode") String onlineorderpCode,
                                   @Validated OnlineorderpValidate onlineorderpModel, BindingResult result) throws Exception {
        JsonResult jsonResult = new JsonResult();
        LoginUserInfo loginUser = getLoginUser(request);
        try {
//            checkAuthorization(request);
            validateEnityBean(onlineorderpModel, result);
            onlineorderpService.updateOnlineorderp(onlineorderpModel, loginUser);
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (ValidateDataFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/onlineorderp/updateDept error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 项目中没有使用位置
     * @param request
     * @param response
     * @param onlineorderpCode
     * @throws Exception
     */
    @RequestMapping(value = "/deleteOnlineorderp/{onlineorderpCode}", method = RequestMethod.POST)
    @ValidatePermission("DELETE")
    public void deleteOnlineorderp(HttpServletRequest request, HttpServletResponse response,
                                   @PathVariable("onlineorderpCode") String onlineorderpCode) throws Exception {
        JsonResult jsonResult = new JsonResult();
        LoginUserInfo loginUser = getLoginUser(request);
        try {
//            checkAuthorization(request);
            onlineorderpService.deleteOnlineorderp(onlineorderpCode, loginUser);
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (ValidateDataFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/onlineorderp/deleteRole error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }
}

