package com.bettersoft.api.controllers;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.bettersoft.api.utils.*;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.bettersoft.api.annotation.MenuCode;
import com.bettersoft.api.annotation.ValidatePermission;
import com.bettersoft.api.exceptions.ValidateAuthorityFailedException;
import com.bettersoft.api.exceptions.ValidateDataFailedException;
import com.bettersoft.api.exceptions.ValidateDataOccupiedException;
import com.bettersoft.api.service.interfaces.IOnlinestoreService;
import com.bettersoft.api.validmodel.OnlinestoreValidate;

@RestController
@RequestMapping(value = "/online/onlinestore")
@MenuCode("SMG01")
public class OnlinestoreController extends BaseController {
    @Autowired
    private IOnlinestoreService onlinestoreService;

    /**
     * 后台管理系统使用，但也没用
     * @param strstorecode
     * @param request
     * @param response
     * @throws Exception
     */
    @RequestMapping(value = "/findOnlinestore/{strstorecode}")
    @ValidatePermission("SEARCH")
    public void findOnlinestore(@PathVariable("strstorecode") String strstorecode, HttpServletRequest request,
                                HttpServletResponse response) throws Exception {
        JsonResult jsonResult = new JsonResult();

        try {
            checkAuthorization(request);
            jsonResult.getData().put("onlinestore", onlinestoreService.findOnlinestore(strstorecode));
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/user/userDetail error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 后台管理系统使用，查找店铺
     * @param request
     * @param response
     * @throws Exception
     */
    @SuppressWarnings("unchecked")
    @RequestMapping(value = "/searchByPage")
    @ValidatePermission("SEARCH")
    public void searchByPage(HttpServletRequest request, HttpServletResponse response) throws Exception {
        Map<String, Object> queryParam = RequestMapUtils.RequestMapToMap(request.getParameterMap());
        JsonResult jsonResult = new JsonResult();
        if (queryParam.containsKey("o.status")) {
            queryParam.put("o.status", LangUtils.optInteger(queryParam.get("o.status")));
        }
        try {
            checkAuthorization(request);
            String[] authStores = getAuthStores(request);
            if (null != authStores && 0 < authStores.length) {
                queryParam.put(Constants.QUERY_SPEC_CONDITION + "strstorecode", "o.strstorecode in ('" + StringUtils.join(authStores, "','") + "')");
            }
            jsonResult.getData().putAll(onlinestoreService.searchByPage(queryParam));
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/onlinestore/searchOnlinestore error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 后台管理系统使用，但没用到
     * @param request
     * @param response
     * @param onlinestoreValidate
     * @param result
     * @throws Exception
     */
    @RequestMapping(value = "/saveOnlinestore", method = RequestMethod.POST)
    @ValidatePermission("INSERT")
    public void saveOnlinestore(HttpServletRequest request, HttpServletResponse response,
                                @Validated OnlinestoreValidate onlinestoreValidate, BindingResult result) throws Exception {
        JsonResult jsonResult = new JsonResult();
        LoginUserInfo loginUser = getLoginUser(request);

        try {
            checkAuthorization(request);
            validateEnityBean(onlinestoreValidate, result);
            onlinestoreService.saveOnlinestore(onlinestoreValidate, loginUser);
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (ValidateDataFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (ValidateDataOccupiedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.getData().put("errMsg", "关键字已存在，不允许重复！");
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/onlinestore/saveRole error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 后台管理系统使用，更新店铺信息，修改QR仅点单或者QR点单支付
     * @param strstorecodes
     * @param status
     * @param request
     * @param response
     * @throws Exception
     */
    @RequestMapping(value = "/updateOnlinestore", method = RequestMethod.POST)
    @ValidatePermission("UPDATE")
    public void updateOnlinestore(String strstorecodes, Integer status, HttpServletRequest request, HttpServletResponse response) throws Exception {
        JsonResult jsonResult = new JsonResult();
        LoginUserInfo loginUser = getLoginUser(request);
        try {
            checkAuthorization(request);
            onlinestoreService.updateOnlinestore(strstorecodes, status, loginUser);
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (ValidateDataFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/onlinestore/updateDept error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    /**
     * 删除店铺，后台管理系统使用，没用到
     * @param request
     * @param response
     * @param strstorecode
     * @throws Exception
     */
    @RequestMapping(value = "/deleteOnlinestore/{strstorecode}", method = RequestMethod.POST)
    @ValidatePermission("DELETE")
    public void deleteOnlinestore(HttpServletRequest request, HttpServletResponse response,
                                  @PathVariable("strstorecode") String strstorecode) throws Exception {
        JsonResult jsonResult = new JsonResult();
        LoginUserInfo loginUser = getLoginUser(request);
        try {
            checkAuthorization(request);
            onlinestoreService.deleteOnlinestore(strstorecode, loginUser);
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (ValidateDataFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("online/onlinestore/deleteRole error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }
}

