package com.bettersoft.api.controllers;

import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.bettersoft.api.annotation.MenuCode;
import com.bettersoft.api.annotation.ValidatePermission;
import com.bettersoft.api.exceptions.ValidateAuthorityFailedException;
import com.bettersoft.api.exceptions.ValidateDataFailedException;
import com.bettersoft.api.exceptions.ValidateDataOccupiedException;
import com.bettersoft.api.service.interfaces.IProvincialAreaService;
import com.bettersoft.api.utils.Constants;
import com.bettersoft.api.utils.JsonResult;
import com.bettersoft.api.utils.LoginUserInfo;
import com.bettersoft.api.utils.RequestMapUtils;
import com.bettersoft.api.validmodel.ProvincialAreaValidate;

@RestController
@RequestMapping(value = "/prov/area")
@MenuCode("OMC01")
public class ProvincialAreaController extends BaseController {
	@Resource(name = "provincialAreaService")
	private IProvincialAreaService provincialAreaService;

	//后台管理系统使用，增删改查地理位置信息，但是没用到


	@RequestMapping(value = "/{areaCode}", method = RequestMethod.POST)
	@ValidatePermission("SEARCH")
	public void provincialDetail(@PathVariable("areaCode") String areaCode,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		JsonResult jsonResult = new JsonResult();
		try {
			checkAuthorization(request);

			jsonResult.getData().put("provincial",
					provincialAreaService.findProvincialByCode(areaCode));
			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (ClassNotFoundException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("无法获取地理信息！");
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员联系！");
			logger.error("prov/area/provincialDetail error", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@SuppressWarnings("unchecked")
	@RequestMapping(value = "/searchProvincals", method = RequestMethod.POST)
	@ValidatePermission("SEARCH")
	public void searchProvincials(HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		JsonResult jsonResult = new JsonResult();
		Map<String, Object> queryParam = RequestMapUtils
				.RequestMapToMap(request.getParameterMap());

		try {
			checkAuthorization(request);

			jsonResult.getData().putAll(
					provincialAreaService.searchProvincials(queryParam));
			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员联系！");
			logger.error("prov/area/searchProvincials error", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@SuppressWarnings("unchecked")
	@RequestMapping(value = "/searchParentProvincials", method = RequestMethod.POST)
	public void searchParentProvincials(HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		JsonResult jsonResult = new JsonResult();
		Map<String, Object> queryParam = RequestMapUtils
				.RequestMapToMap(request.getParameterMap());

		queryParam.put(Constants.QUERY_ORDER_BY, "areaCode");

		try {
			jsonResult.getData().put("provincials",
					provincialAreaService.searchProvincialList(queryParam));
			jsonResult.setSuccess(true);
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员联系！");
			logger.error("prov/area/searchParentProvincals error", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@RequestMapping(value = "/saveProvincial", method = RequestMethod.POST)
	@ValidatePermission("INSERT")
	public void saveProvincial(HttpServletRequest request,
			HttpServletResponse response,
			@Validated ProvincialAreaValidate provincialValidate,
			BindingResult result) throws Exception {
		JsonResult jsonResult = new JsonResult();
		LoginUserInfo loginUser = getLoginUser(request);

		try {
			checkAuthorization(request);

			validateEnityBean(provincialValidate, result);

			provincialAreaService.saveProvincial(provincialValidate, loginUser);

			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (ValidateDataFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (ValidateDataOccupiedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("地区编号已存在，不允许重复！");
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员联系！");
			logger.error("prov/area/saveProvincial error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@RequestMapping(value = "/updateProvincial/{areaCode}", method = RequestMethod.POST)
	@ValidatePermission("UPDATE")
	public void updateProvincial(HttpServletRequest request,
			HttpServletResponse response,
			@PathVariable("areaCode") String areaCode,
			@Validated ProvincialAreaValidate provincialValidate,
			BindingResult result) throws Exception {
		JsonResult jsonResult = new JsonResult();
		LoginUserInfo loginUser = getLoginUser(request);
		try {
			checkAuthorization(request);

			validateEnityBean(provincialValidate, result);

			provincialAreaService.updateProvincial(provincialValidate,
					loginUser);

			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (ValidateDataFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员联系！");
			logger.error("prov/area/updateProvincial error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@RequestMapping(value = "/deleteProvincial/{areaCode}", method = RequestMethod.POST)
	@ValidatePermission("DELETE")
	public void deleteProvincial(@PathVariable("areaCode") String areaCode,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		JsonResult jsonResult = new JsonResult();
		LoginUserInfo loginUser = getLoginUser(request);

		try {
			checkAuthorization(request);

			provincialAreaService.deleteProvincial(areaCode, loginUser);

			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (ValidateDataFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员联系！");
			logger.error("prov/area/deleteProvincial error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}
}