package com.bettersoft.api.controllers;

import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.collections.MapUtils;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.bettersoft.api.annotation.MenuCode;
import com.bettersoft.api.annotation.ValidatePermission;
import com.bettersoft.api.exceptions.ValidateAuthorityFailedException;
import com.bettersoft.api.exceptions.ValidateDataFailedException;
import com.bettersoft.api.exceptions.ValidateDataOccupiedException;
import com.bettersoft.api.model.SystemDepartmentBean;
import com.bettersoft.api.service.interfaces.ISystemDepartmentService;
import com.bettersoft.api.utils.Constants;
import com.bettersoft.api.utils.JsonResult;
import com.bettersoft.api.utils.LoginUserInfo;
import com.bettersoft.api.utils.RequestMapUtils;
import com.bettersoft.api.validmodel.SystemDepartmentValidate;

@RestController
@RequestMapping(value = "/sys/dept")
@MenuCode("SYS01")
public class SystemDepartmentController extends BaseController {
	@Resource(name = "systemDepartmentService")
	private ISystemDepartmentService systemDepartmentService;

	@RequestMapping(value = "/{departmentCode}")
	@ValidatePermission("SEARCH")
	public void findDept(@PathVariable("departmentCode") String departmentCode,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		JsonResult jsonResult = new JsonResult();
		try {
			checkAuthorization(request);

			SystemDepartmentBean department = systemDepartmentService
					.searchDepatmentByCode(departmentCode);
			if (null == department)
				throw new ClassNotFoundException();

			jsonResult.getData().put("dept", department);
			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (ClassNotFoundException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("无法获取组织架构信息！");
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员系联！");
			logger.error("sys/dep/findDept error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@SuppressWarnings("unchecked")
	@RequestMapping(value = "/searchDeptList", method = RequestMethod.POST)
	@ValidatePermission("SEARCH")
	public void searchDeptList(HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		Map<String, Object> queryParam = RequestMapUtils
				.RequestMapToMap(request.getParameterMap());
		JsonResult jsonResult = new JsonResult();

		try {
			checkAuthorization(request);

			if (queryParam.containsKey("forcePage")
					&& !MapUtils
							.getBooleanValue(queryParam, "forcePage", false)) {
				queryParam.remove(Constants.QUERY_FIRST_RESULTS);
				queryParam.remove(Constants.QUERY_MAX_RESULTS);
			}

			queryParam.remove("forcePage");

			jsonResult.getData().putAll(
					systemDepartmentService.searchDepartments(queryParam));
			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员系联！");
			logger.error("sys/dep/searchDeptList error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@SuppressWarnings("unchecked")
	@RequestMapping(value = "/searchDepartments", method = RequestMethod.POST)
	@ValidatePermission("SEARCH")
	public void searchDepartments(HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		Map<String, Object> queryParam = RequestMapUtils
				.RequestMapToMap(request.getParameterMap());
		JsonResult jsonResult = new JsonResult();

		try {
			checkAuthorization(request);

			jsonResult.getData().putAll(
					systemDepartmentService.searchDepartment(queryParam));
			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员系联！");
			logger.error("sys/dep/searchDepartments error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@SuppressWarnings("unchecked")
	@RequestMapping(value = "/searchDeptTree", method = RequestMethod.POST)
	@ValidatePermission("SEARCH")
	public void searchDeptTree(HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		Map<String, Object> queryParam = RequestMapUtils
				.RequestMapToMap(request.getParameterMap());
		JsonResult jsonResult = new JsonResult();

		try {
			checkAuthorization(request);

			queryParam.remove(Constants.QUERY_FIRST_RESULTS);
			queryParam.remove(Constants.QUERY_MAX_RESULTS);

			jsonResult.getData().put("rows",
					systemDepartmentService.searchDepartmentTree(queryParam));
			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员系联！");
			logger.error("sys/dep/searchDeptList error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@RequestMapping(value = "/saveDept", method = RequestMethod.POST)
	@ValidatePermission("INSERT")
	public void saveDept(HttpServletRequest request,
			HttpServletResponse response,
			@Validated SystemDepartmentValidate departmentModel,
			BindingResult result) throws Exception {
		JsonResult jsonResult = new JsonResult();
		LoginUserInfo loginUser = getLoginUser(request);

		try {
			checkAuthorization(request);

			validateEnityBean(departmentModel, result);
			systemDepartmentService.saveDepartment(departmentModel, loginUser);

			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (ValidateDataFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (ValidateDataOccupiedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("组织编号已存在，不允许重复！");
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员系联！");
			logger.error("sys/dept/saveDept error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@RequestMapping(value = "/updateDept/{departmentCode}", method = RequestMethod.POST)
	@ValidatePermission("UPDATE")
	public void updateDept(HttpServletRequest request,
			HttpServletResponse response,
			@PathVariable("departmentCode") String departmentCode,
			@Validated SystemDepartmentValidate departmentModel,
			BindingResult result) throws Exception {
		JsonResult jsonResult = new JsonResult();
		LoginUserInfo loginUser = getLoginUser(request);
		try {
			checkAuthorization(request);

			validateEnityBean(departmentModel, result);
			systemDepartmentService
					.updateDepartment(departmentModel, loginUser);

			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (ValidateDataFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员系联！");
			logger.error("sys/dept/updateDept error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@RequestMapping(value = "/deleteDept/{departmentCode}", method = RequestMethod.POST)
	@ValidatePermission("DELETE")
	public void deleteDept(HttpServletRequest request,
			HttpServletResponse response,
			@PathVariable("departmentCode") String departmentCode)
			throws Exception {
		JsonResult jsonResult = new JsonResult();

		try {
			checkAuthorization(request);

			systemDepartmentService.deleteDepartment(departmentCode);

			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (ValidateDataFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员系联！");
			logger.error("sys/dept/updateDept error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}
}
