package com.bettersoft.api.controllers;

import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.bettersoft.api.annotation.ValidatePermission;
import com.bettersoft.api.exceptions.ValidateAuthorityFailedException;
import com.bettersoft.api.service.interfaces.ISystemLogService;
import com.bettersoft.api.utils.JsonResult;
import com.bettersoft.api.utils.RequestMapUtils;

@RestController
@RequestMapping(value = "/sys/log")
public class SystemLogController extends BaseController {
	@Resource(name = "systemLogService")
	private ISystemLogService systemLogService;

	@RequestMapping(value = "/{logId}", method = RequestMethod.POST)
	@ValidatePermission("SEARCH")
	public void logDetail(@PathVariable("logId") String logId,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		JsonResult jsonResult = new JsonResult();

		try {
			checkAuthorization(request);

			jsonResult.getData().put("syslog",
					systemLogService.findLogOperator(logId));
			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员系联！");
			logger.error("sys/log/logDetail error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@SuppressWarnings("unchecked")
	@RequestMapping(value = "/searchLogs", method = RequestMethod.POST)
	@ValidatePermission("SEARCH")
	public void searchLogs(HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		Map<String, Object> queryParam = RequestMapUtils
				.RequestMapToMap(request.getParameterMap());
		JsonResult jsonResult = new JsonResult();

		try {
			jsonResult.getData()
					.putAll(systemLogService.searchLogs(queryParam));
			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员系联！");
			logger.error("sys/log/searchLogs error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}
}
