package com.bettersoft.api.controllers;

import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.bettersoft.api.exceptions.ValidateAuthorityFailedException;
import com.bettersoft.api.service.interfaces.ISystemMessageService;
import com.bettersoft.api.utils.Constants;
import com.bettersoft.api.utils.JsonResult;
import com.bettersoft.api.utils.LoginUserInfo;
import com.bettersoft.api.utils.RequestMapUtils;

@RestController
@RequestMapping(value = "/sys/msg")
public class SystemMessageController extends BaseController {
    @Resource(name = "systemMessageService")
    private ISystemMessageService systemMessageService;

    @RequestMapping(value = "/{seqId}", method = RequestMethod.POST)
    public void findSysMsg(@PathVariable("seqId") int seqId,
                           HttpServletRequest request, HttpServletResponse response)
            throws Exception {
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult.getData().putAll(
                    systemMessageService.findSystemMsg(seqId));
            jsonResult.setSuccess(true);
        } catch (ClassNotFoundException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("无法获取系统消息！");
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("系统错误，请与系统管理员系联！");
            logger.error("sys/msg/searchSysMessages error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    @RequestMapping(value = "/searchSysMessages", method = RequestMethod.POST)
    public void searchSysMessages(HttpServletRequest request,
                                  HttpServletResponse response) throws Exception {
        JsonResult jsonResult = new JsonResult();
        LoginUserInfo loginUser = getLoginUser(request);
        try {
            jsonResult.getData().put("unRead", 0);
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("系统错误，请与系统管理员系联！");
            logger.error("sys/msg/searchSysMessages error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    @SuppressWarnings("unchecked")
    @RequestMapping(value = "/findTopMessage", method = RequestMethod.POST)
    public void findTopMessage(HttpServletRequest request,
                               HttpServletResponse response) throws Exception {
        Map<String, Object> queryParam = RequestMapUtils
                .RequestMapToMap(request.getParameterMap());
        JsonResult jsonResult = new JsonResult();
        try {
            queryParam.put(Constants.QUERY_ORDER_BY, "r.createDateTime desc");

//            jsonResult.getData().putAll(
//                    systemMessageService.findTopUnReadMessage(queryParam));
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("系统错误，请与系统管理员系联！");
            logger.error("sys/msg/findTopMessage error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    @RequestMapping(value = "/readMessage/{seqId}", method = RequestMethod.POST)
    public void readMessage(@PathVariable("seqId") int seqId,
                            HttpServletRequest request, HttpServletResponse response)
            throws Exception {
        JsonResult jsonResult = new JsonResult();
        LoginUserInfo loginUser = getLoginUser(request);
        try {
            systemMessageService.readMessage(seqId, loginUser);
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("系统错误，请与系统管理员系联！");
            logger.error("sys/msg/readMessage error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }
}
