package com.bettersoft.api.controllers;

import com.alibaba.fastjson.JSONObject;
import com.bettersoft.api.dao.interfaces.IOnlineorderhDao;
import com.bettersoft.api.encrytion.AESUtil;
import com.bettersoft.api.encrytion.Base64Util;
import com.bettersoft.api.encrytion.DateUtils;
import com.bettersoft.api.model.OnlineorderhBean;
import com.bettersoft.api.model.WxMaAuthResult;
import com.bettersoft.api.service.implement.CallJiakeApiService;
import com.bettersoft.api.service.interfaces.IOnlinePrepayService;
import com.bettersoft.api.service.interfaces.IOnlineorderhService;
import com.bettersoft.api.service.interfaces.IWxService;
import com.bettersoft.api.utils.JsonResult;
import org.apache.commons.lang.StringUtils;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.json.Json;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Controller
@RequestMapping("/wx")
public class WXController extends BaseController{

    @Autowired
    private IWxService wxService;
    @Autowired
    private IOnlinePrepayService onlinePrepayService;
    @Autowired
    private CallJiakeApiService callJiakeApiService;
    @Autowired
    private IOnlineorderhDao onlineorderhDao;
    @Autowired
    private IOnlineorderhService onlineorderhService;

    /**
     * 检查微信登录信息,暂时用作调试接口
     * @param orderNo
     * @param request
     * @param response
     */
    @PostMapping(value = "/checkWxLogin")
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
    public void checkWxLogin(String orderNo,HttpServletRequest request, HttpServletResponse response){
        JsonResult jsonResult = new JsonResult();
        try {
            if(onlineorderhDao.isOrderSettle("21001","A18")){
                jsonResult.setErrMsg("订单全部已完成");
            } else {
                jsonResult.setErrMsg("上一个订单未完成");
            }
            renderResponse(jsonResult,response);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 微信登录
     * @param code
     * @param request
     * @param response
     */
    @PostMapping(value = "/WxLogin")
    public void wxLogin(String code, HttpServletRequest request, HttpServletResponse response){
        JsonResult jsonResult = new JsonResult();
        if (StringUtils.isBlank(code)) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("code为空");
            jsonResult.setErrorCode("1");
            try {
                renderResponse(jsonResult,response);
            } catch (Exception e) {
                e.printStackTrace();
            }
            return;
        }
        //处理登录授权 获得openid,unionid和sessionkey,返回unionid,openid,phone,过期时间组成的加密数据,其中phone暂时为空
        Object result = wxService.dealLoginAuth(code);
        if(result!=null){
            jsonResult.setSuccess(true);
            jsonResult.setWxData(result);
            try {
                renderResponse(jsonResult,response);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        else {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("获取失败");
            jsonResult.setErrorCode("1");
            try {
                renderResponse(jsonResult,response);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 堂食支付
     * @param request
     * @param response
     * @param storeCode
     * @param tableCode
     */
    @PostMapping(value = "/WxPay/{storeCode}/{tableCode}")
    public void WxPay(HttpServletRequest request, HttpServletResponse response,
                      @PathVariable("storeCode") String storeCode,@PathVariable("tableCode") String tableCode,String token) throws Exception {
        JsonResult jsonResult = new JsonResult();
        String str = new String(Base64Util.decryBASE64(token));
        JSONObject strAsJson = JSONObject.parseObject(str);
        String openId = strAsJson.getString("openid");
        try {
            wxService.wxCallJiakePay(storeCode,tableCode,openId,strAsJson.getString("session_key"));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 外卖支付
     * @param request
     * @param response
     * @param storeCode
     */
    @PostMapping(value = "/WxPayTakeOff/{storeCode}")
    public void WxPayTakeOff(HttpServletRequest request, HttpServletResponse response,
                      @PathVariable("storeCode") String storeCode,String token) throws Exception {
        JsonResult jsonResult = new JsonResult();
        String str = new String(Base64Util.decryBASE64(token));
        JSONObject strAsJson = JSONObject.parseObject(str);
        String openId = strAsJson.getString("openid");
        try {
            wxService.wxCallJiakePay(storeCode,openId,openId,strAsJson.getString("session_key"));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    //无用
    @PostMapping(value = "/WxPay/{storeCode}")
    public void WxPay(HttpServletRequest request, HttpServletResponse response,
                      @PathVariable("storeCode") String storeCode,String token) throws Exception {
        JsonResult jsonResult = new JsonResult();
        String str = new String(Base64Util.decryBASE64(token));
        JSONObject strAsJson = JSONObject.parseObject(str);
        String openId = strAsJson.getString("openid");
        try {
            Object object = wxService.wxPay(openId,storeCode,openId);
            if(object==null){
                jsonResult.setSuccess(false);
                jsonResult.setErrorCode("1");
                jsonResult.setErrMsg("支付失败!");
                renderResponse(jsonResult,response);
                return;
            }
            jsonResult.setWxData(object);
            jsonResult.setSuccess(true);
            renderResponse(jsonResult,response);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 订阅消息
     * @param request
     * @param response
     * @param orderNo
     */
    @PostMapping(value = "/WxSubscribeMessage")
    public void WxSubscribeMessage(HttpServletRequest request,HttpServletResponse response,
                                   String orderNo,String token){
        JsonResult jsonResult = new JsonResult();
        //可能做的身份验证 TODO
        try {
            Object object = wxService.wxSubscribeMessage(orderNo);
            if(!object.equals("发送成功!")){
                jsonResult.setSuccess(false);
                jsonResult.setErrorCode("1");
                jsonResult.setErrMsg(object.toString());
                renderResponse(jsonResult,response);
                return;
            }
            jsonResult.setErrMsg(object.toString());
            jsonResult.setSuccess(true);
            renderResponse(jsonResult,response);
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    /**
     * 获取手机号
     * @param request
     * @param response
     * @param code
     */
    @PostMapping(value = "/WxGetPhone")
    public void WxGetPhone(HttpServletRequest request,HttpServletResponse response,
                                   String code,String token){
        JsonResult jsonResult = new JsonResult();
        //可能做的身份验证 TODO
        try {
            Object object = wxService.wxGetPhone(code,token);
            if(object==null){
                jsonResult.setSuccess(false);
                jsonResult.setErrorCode("1");
                jsonResult.setErrMsg("获取手机号失败!");
                renderResponse(jsonResult,response);
                return;
            }
            jsonResult.setWxData(object);
            jsonResult.setSuccess(true);
            renderResponse(jsonResult,response);
        } catch (Exception e) {
            e.printStackTrace();
        }

    }


//    /**
//     * 支付成功清除对应缓存
//     * @param request
//     * @param response
//     * @param orderno
//     */
//    @PostMapping(value = "/removeShoppingCart")
//    public void removeShoppingCart(HttpServletRequest request,HttpServletResponse response,
//                           String orderno,String token){
//        JsonResult jsonResult = new JsonResult();
//        //可能做的身份验证 TODO
//        try {
//            onlineorderhService.removeShoppingCart(orderno);
//            jsonResult.setErrorCode("200");
//            jsonResult.setSuccess(true);
//            renderResponse(jsonResult,response);
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//
//    }


    @GetMapping(value = "/WxGetOrderUnlimited")
    public void WxGetOrderUnlimited(HttpServletRequest request,HttpServletResponse response,
                                    String storeCode,String beginCode,String endCode,String flag){
        JsonResult jsonResult = new JsonResult();
        //可能做的身份验证 TODO
        try {
            wxService.wxSetOrderUnlimited(storeCode,beginCode,endCode,flag);
            List<String> path=wxService.wxGetOrderUnlimited(storeCode,beginCode,endCode,flag);
            if(path!=null){
                jsonResult.setErrorCode("200");
                jsonResult.setWxData(path);
                jsonResult.setSuccess(true);
            }else {
                jsonResult.setErrorCode("100");
                jsonResult.setErrMsg("图片获取失败");
                jsonResult.setSuccess(false);
            }
            renderResponse(jsonResult,response);
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    /**
     * 判断token是否过期
     * @param request
     * @param response
     * @param token
     */
    @PostMapping(value = "/WxToken")
    public void WxToken(HttpServletRequest request,HttpServletResponse response,
                          @RequestParam("token") String token){
        JsonResult jsonResult = new JsonResult();
        try {
            String temp = new String(Base64Util.decryBASE64(token));
            JSONObject jsonObject = JSONObject.parseObject(temp);
            String val = jsonObject.getString("day");
            System.out.println(val+"========================");
            if(!(DateUtils.getDifferDay(val, DateUtils.format(new Date())) > 0)) {
                jsonResult.setSuccess(true);
                jsonResult.setErrorCode("100");
                jsonResult.setErrMsg("登录过期,请重新登录!");
            }else{
                jsonResult.setErrorCode("200");
                jsonResult.setSuccess(true);
            }

            renderResponse(jsonResult,response);
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    /**
     * 判断token是否获取到手机号
     * @param request
     * @param response
     * @param token
     */
    @PostMapping(value = "/WxTokenPhone")
    public void WxTokenPhone(HttpServletRequest request,HttpServletResponse response,
                             @RequestParam("token") String token){
        JsonResult jsonResult = new JsonResult();
        try {
            String temp = new String(Base64Util.decryBASE64(token));
            JSONObject jsonObject = JSONObject.parseObject(temp);
            String val = jsonObject.getString("phone");
            System.out.println(val+"========================");
            if(val!=null && !val.equals("")) {
                jsonResult.setErrorCode("200");
                jsonResult.setSuccess(true);
            }else{
                jsonResult.setSuccess(true);
                jsonResult.setErrorCode("100");
                jsonResult.setErrMsg("获取手机失败，请重试!");
            }
            renderResponse(jsonResult,response);
        } catch (Exception e) {
            e.printStackTrace();
        }

    }


    /**
     * 获取外部访问链接
     * @param request
     * @param response
     */
    @GetMapping(value = "/WxGetUrl")
    public void WxGetUrl(HttpServletRequest request,HttpServletResponse response){
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult.setWxData(wxService.getUrl());
            renderResponse(jsonResult,response);
        } catch (Exception e) {
            e.printStackTrace();
        }

    }



}
