package com.bettersoft.api.controllers;

import com.alibaba.fastjson.JSONObject;
import com.bettersoft.api.dao.interfaces.IYourenMemberDao;
import com.bettersoft.api.encrytion.Base64Util;
import com.bettersoft.api.model.OnlineorderdBean;
import com.bettersoft.api.service.interfaces.IWxService;
import com.bettersoft.api.utils.Constants;
import com.bettersoft.api.utils.JsonResult;
import com.bettersoft.api.validmodel.ShoppingCartValidate;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.text.MessageFormat;
import java.util.List;

@RestController
@RequestMapping("/youren")
public class YourenController extends BaseController{

    @Autowired
    private IYourenMemberDao yourenMemberDao;
    @Resource
    private IWxService wxService;


    @PostMapping(value = "/checkMember")
    public JsonResult checkMember(HttpServletRequest request, HttpServletResponse response,String token) throws Exception {
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = yourenMemberDao.checkMember(token);
            return jsonResult;
        } catch (Exception e) {
            jsonResult.setSuccess(true);
            jsonResult.setErrMsg("获取会员信息失败!");
            jsonResult.setErrorCode("900");
            return jsonResult;
        }
    }

    @PostMapping(value = "/getPurchaseDiscount")
    public JsonResult getPurchaseDiscount(HttpServletRequest request, HttpServletResponse response,String token,String orderNo) throws Exception{
        JsonResult jsonResult = new JsonResult();

        if(orderNo==null||orderNo.equals("")){
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("订单号为空!");
            jsonResult.setErrMsg("101");
            return jsonResult;
        }
        try {
            jsonResult = yourenMemberDao.getPurchaseDiscount(token,orderNo);
            if(!jsonResult.getErrorCode().equals("200")){
                jsonResult.setWxData(yourenMemberDao.general(orderNo,jsonResult));
            } else {
                jsonResult.setWxData(yourenMemberDao.general(orderNo,null));
            }
            return jsonResult;
        } catch (Exception e) {
            logger.info(e.getMessage(),e);
            jsonResult.setSuccess(true);
            jsonResult.setWxData(yourenMemberDao.general(orderNo,jsonResult));
            jsonResult.setErrMsg("会员接口接口调用失败，请重试!");
            jsonResult.setErrorCode("900");
            return jsonResult;
        }
    }

    @PostMapping(value = "/getPurchaseDiscountTakeOff")
    public JsonResult getPurchaseDiscountTakeOff(HttpServletRequest request, HttpServletResponse response,String token,String storeCode,String remark) throws Exception{
        JsonResult jsonResult = new JsonResult();
        String str = new String(Base64Util.decryBASE64(token));
        JSONObject strAsJson = JSONObject.parseObject(str);
        String openid = strAsJson.getString("openid");
        String cartKey = MessageFormat.format(Constants.BUYCART_REDIS, storeCode, openid);
        ShoppingCartValidate shoppingCart = redisCacheUtil.getCache(cartKey);
        if(shoppingCart==null){
            logger.error("用户:"+openid+"购物车为空!");
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("请添加商品!");
            jsonResult.setErrorCode("901");
            return jsonResult;
        }
        List<OnlineorderdBean> orderDetails = yourenMemberDao.getOrderDetail(shoppingCart,token);
        try {
            jsonResult = yourenMemberDao.getPurchaseDiscountTakeOff(orderDetails,openid,remark,storeCode);
            if(!jsonResult.getErrorCode().equals("200")){
                jsonResult.setWxData(yourenMemberDao.generalTakeOff(orderDetails,jsonResult,remark));
            } else {
                jsonResult.setWxData(yourenMemberDao.generalTakeOff(orderDetails,null,remark));
            }
            return jsonResult;
        } catch (Exception e) {
            logger.error(e.getMessage(),e);
            jsonResult.setWxData(yourenMemberDao.generalTakeOff(orderDetails,jsonResult,remark));
            jsonResult.setSuccess(true);
            jsonResult.setErrMsg("会员接口接口调用失败，请重试!");
            jsonResult.setErrorCode("900");
            return jsonResult;
        }
    }

    @PostMapping(value = "/getStoreCity")
    public JsonResult getStoreCity(HttpServletRequest request,HttpServletResponse response){
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult.setWxData(wxService.getStoreCity());
            jsonResult.setSuccess(true);
            jsonResult.setErrorCode("200");
            jsonResult.setErrMsg("获取成功");
            return jsonResult;
        } catch (Exception e) {
            e.printStackTrace();
            jsonResult.setSuccess(false);
            jsonResult.setErrorCode("900");
            jsonResult.setErrMsg("获取城市失败,请重试!");
            return jsonResult;
        }
    }


    @PostMapping(value = "/getStore")
    public JsonResult getStore(HttpServletRequest request,HttpServletResponse response,String token,String city){
        JsonResult jsonResult = new JsonResult();
        try {
            String openId;
            if(StringUtils.isNotBlank(token)){
                String str = new String(Base64Util.decryBASE64(token));
                JSONObject strAsJson = JSONObject.parseObject(str);
                openId = strAsJson.getString("openid");
            } else {
                openId = null;
            }
            jsonResult.setWxData(wxService.getStore(openId,city));
            jsonResult.setSuccess(true);
            jsonResult.setErrorCode("200");
            jsonResult.setErrMsg("获取成功");
            return jsonResult;
        } catch (Exception e) {
            e.printStackTrace();
            jsonResult.setSuccess(false);
            jsonResult.setErrorCode("900");
            jsonResult.setErrMsg("获取门店失败,请重试!");
            return jsonResult;
        }
    }

    @PostMapping(value = "/checkStoreStatus")
    public JsonResult checkStoreStatus(HttpServletRequest request,HttpServletResponse response,String token,String storeId){
        JsonResult jsonResult = new JsonResult();
        try {
            if(wxService.checkStoreStatus(storeId)){
                jsonResult.setSuccess(true);
                jsonResult.setErrorCode("200");
                jsonResult.setErrMsg("获取成功");
            } else {
                jsonResult.setSuccess(true);
                jsonResult.setErrorCode("100");
                jsonResult.setErrMsg("门店已闭店!");
            }
            return jsonResult;
        } catch (Exception e) {
            logger.error(e.getMessage(),e);
            jsonResult.setSuccess(false);
            jsonResult.setErrorCode("900");
            jsonResult.setErrMsg("检查门店状态失败,请重试!");
            return jsonResult;
        }
    }


//    @PostMapping(value = "/getPurchaseDiscountTakeOut/{storeCode}")
//    public JsonResult getPurchaseDiscountTakeOut(HttpServletRequest request, HttpServletResponse response,String token,
//                                          @PathVariable("storeCode")String storeCode,String orderNo) throws Exception{
//        JsonResult jsonResult = new JsonResult();
//        String str = new String(Base64Util.decryBASE64(token));
//        JSONObject strAsJson = JSONObject.parseObject(str);
//        String openId=strAsJson.get("openid").toString();
//        String cartKey = MessageFormat.format(Constants.BUYCART_REDIS, storeCode, openId);
//        ShoppingCartValidate shoppingCart = null;
//
//        if (redisCacheUtil.containsKey(cartKey)) {
//            shoppingCart = redisCacheUtil.getCache(cartKey);
//        } else {
//            shoppingCart = new ShoppingCartValidate(storeCode, openId);
//        }
//        try {
//            jsonResult = yourenMemberDao.getPurchaseDiscount(token,shoppingCart);
//            return jsonResult;
//        } catch (Exception e) {
//            e.printStackTrace();
//            jsonResult.setSuccess(false);
//            jsonResult.setErrMsg("未知异常!");
//            jsonResult.setErrorCode("900");
//            return jsonResult;
//        }
//    }

    @PostMapping(value = "/checkPurchaseDiscount")
    public JsonResult checkPurchaseDiscount(HttpServletRequest request, HttpServletResponse response,String token,String orderNo) throws Exception{
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult.setWxData(yourenMemberDao.general(orderNo,jsonResult));
            jsonResult.setSuccess(true);
            return jsonResult;
        } catch (Exception e) {
            e.printStackTrace();
            jsonResult.setSuccess(true);
            jsonResult.setWxData(yourenMemberDao.general(orderNo,jsonResult));
            jsonResult.setErrMsg("获取折扣信息失败,请重试!");
            jsonResult.setErrorCode("900");
            return jsonResult;
        }
    }

    @PostMapping(value = "/checkPurchaseDiscountTakeOff")
    public JsonResult checkPurchaseDiscountTakeOff(HttpServletRequest request, HttpServletResponse response,String token,String storeCode,String remark) throws Exception{
        JsonResult jsonResult = new JsonResult();
        String str = new String(Base64Util.decryBASE64(token));
        JSONObject strAsJson = JSONObject.parseObject(str);
        String openid = strAsJson.getString("openid");
        String cartKey = MessageFormat.format(Constants.BUYCART_REDIS, storeCode, openid);
        ShoppingCartValidate shoppingCart = redisCacheUtil.getCache(cartKey);
        List<OnlineorderdBean> orderDetails = yourenMemberDao.getOrderDetail(shoppingCart,token);
        try {
            jsonResult.setWxData(yourenMemberDao.generalTakeOff(orderDetails,jsonResult,remark));
            jsonResult.setSuccess(true);
            return jsonResult;
        } catch (Exception e) {
            e.printStackTrace();
            jsonResult.setWxData(yourenMemberDao.generalTakeOff(orderDetails,null,remark));
            jsonResult.setSuccess(true);
            jsonResult.setErrMsg("获取折扣信息失败,请重试!");
            jsonResult.setErrorCode("900");
            return jsonResult;
        }
    }

    @PostMapping(value = "/test")
    public JsonResult test(HttpServletRequest request, HttpServletResponse response,String token,String storeCode,String remark) throws Exception{
        JsonResult jsonResult = new JsonResult();
        String str = new String(Base64Util.decryBASE64(token));
        JSONObject strAsJson = JSONObject.parseObject(str);
        String openid = strAsJson.getString("openid");
        String cartKey = MessageFormat.format(Constants.BUYCART_REDIS, storeCode, openid);
        ShoppingCartValidate shoppingCart = redisCacheUtil.getCache(cartKey);
        if(shoppingCart==null){
            logger.error("用户:"+openid+"购物车为空!");
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("请添加商品!");
            jsonResult.setErrorCode("901");
            return jsonResult;
        }
        List<OnlineorderdBean> orderDetails = yourenMemberDao.getOrderDetail(shoppingCart,token);
        try {
            jsonResult = yourenMemberDao.test(orderDetails,openid,remark,storeCode);
            if(!jsonResult.getErrorCode().equals("200")){
                jsonResult.setWxData(yourenMemberDao.generalTakeOff(orderDetails,jsonResult,remark));
            } else {
                jsonResult.setWxData(yourenMemberDao.generalTakeOff(orderDetails,null,remark));
            }
            return jsonResult;
        } catch (Exception e) {
            logger.error(e.getMessage(),e);
            jsonResult.setWxData(yourenMemberDao.generalTakeOff(orderDetails,jsonResult,remark));
            jsonResult.setSuccess(true);
            jsonResult.setErrMsg("会员接口接口调用失败，请重试!");
            jsonResult.setErrorCode("900");
            return jsonResult;
        }
    }
}
