package com.bettersoft.api.dao.implement;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Repository;

import com.bettersoft.api.dao.interfaces.IBaseDao;
import com.bettersoft.api.model.ProvincialAreaBean;
import com.bettersoft.api.model.SystemConfigurationBean;
import com.bettersoft.api.utils.Constants;
import com.bettersoft.api.utils.RedisCacheUtil;

import comm.hibernate.BaseHibernateDaoImpl;

@Repository("baseDao")
public class BaseDao extends BaseHibernateDaoImpl implements IBaseDao {
	@Resource(name = "redisCacheUtil")
	private RedisCacheUtil redisCacheUtil;

	public SystemConfigurationBean findConfigurationByCode(String configCode)
			throws Exception {
		SystemConfigurationBean config = redisCacheUtil.getCache(
				Constants.SYSCONFIG_KEY, configCode);
		if (null == config) {
			config = this.findEntity(SystemConfigurationBean.class, configCode);
			if (null != config)
				redisCacheUtil.setCache(Constants.SYSCONFIG_KEY,
						config.getConfigCode(), config);
		}

		return config;
	}

	public List<Map<String, Object>> searchPermissions() throws Exception {
		List<Map<String, Object>> permissions = redisCacheUtil
				.getCasheHashList(Constants.PERMISSION_KEY);

		if (null == permissions || permissions.isEmpty()) {
			String fromHql = "select new map(permissionCode as permissionCode, permissionDesc as permissionDesc) from SystemPermissionBean";
			permissions = this
					.generalSearch(fromHql, null, "order by priority");

			if (null != permissions && !permissions.isEmpty()) {
				for (Map<String, Object> perm : permissions) {
					String permissionCode = MapUtils.getString(perm,
							"permissionCode", StringUtils.EMPTY);
					redisCacheUtil.setCache(Constants.PERMISSION_KEY,
							permissionCode, perm);
				}
			}
		}

		return permissions;
	}

	public List<SystemConfigurationBean> searchConfigurations()
			throws Exception {
		List<SystemConfigurationBean> configurations = redisCacheUtil
				.getCasheHashList(Constants.SYSCONFIG_KEY);

		if (null == configurations || configurations.isEmpty()) {
			String fromHql = "from SystemConfigurationBean";

			configurations = this.generalSearch(fromHql, null,
					"order by createDateTime");
			if (null != configurations && !configurations.isEmpty()) {
				for (SystemConfigurationBean config : configurations) {
					redisCacheUtil.setCache(Constants.SYSCONFIG_KEY,
							config.getConfigCode(), config);
				}
			}
		}

		return configurations;
	}

	public List<ProvincialAreaBean> searchProvincials() throws Exception {
		List<ProvincialAreaBean> provincials = redisCacheUtil
				.getCasheHashList(Constants.PROVINCIAL_KEY);

		if (null == provincials || provincials.isEmpty()) {
			String fromHql = "from ProvincialAreaBean";
			Map<String, Object> queryParam = new HashMap<String, Object>();
			queryParam.put(Constants.QUERY_SPEC_CONDITION + "parentProvincial",
					"parentProvincial is null");

			provincials = this.generalSearch(fromHql, queryParam,
					"order by areaCode");

			if (null != provincials && !provincials.isEmpty()) {
				for (ProvincialAreaBean provincial : provincials) {
					redisCacheUtil.setCache(Constants.PROVINCIAL_KEY,
							provincial.getAreaCode(), provincial);
				}
			}
		}

		return provincials;
	}

	public Number nextSequance(String sequanceName) throws Exception {
		List<Map<String, Number>> res = generalSearchBySql("select NEXT_VAL('"
				+ sequanceName + "') AS seq from dual", null, null);

		if (null != res && !res.isEmpty())
			return MapUtils.getNumber(res.get(0), "seq", 0);

		return null;
	}
}
