package com.bettersoft.api.dao.implement;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.*;

import com.bettersoft.api.utils.DateUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.checkerframework.checker.units.qual.C;
import org.springframework.stereotype.Repository;

import com.bettersoft.api.dao.interfaces.IDashboardDao;
import com.bettersoft.api.utils.Constants;

@Repository("dashboardDao")
public class DashboardDao extends BaseDao implements IDashboardDao {

    public List<Map<String, Object>> searchStores(Map<String, Object> queryParam) throws Exception {
        String fromHql = "select new map(s.strStoreCode as storeCode, s.strStoreName as storeName) from OnlinestoreBean o, StoreBean s where o.strstorecode = s.strStoreCode";

        return this.generalSearch(fromHql, queryParam, MapUtils.getString(queryParam, Constants.QUERY_ORDER_BY, StringUtils.EMPTY));
    }

    public Map<String, Integer> dailyOnlineOrderQuantity(String storeCode, Date date) throws Exception {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Map<String, Integer> result = new HashMap<>();
        Map<String, Object> queryParam = new HashMap<String, Object>();
        queryParam.put(Constants.QUERY_START_DATE + "dtorderstart",
                sdf.format(date));
        queryParam.put(Constants.QUERY_END_DATE + "dtorderstart",
                sdf.format(date));
        if (storeCode != null) {
            queryParam.put("strstorecode", storeCode);
        }
        queryParam.put("offlineupdateflg", 0);
        String fromHql = "from OnlineorderhBean";
        queryParam.put(Constants.QUERY_SPEC_CONDITION + "status", "status in (1, 3, 9)");
        queryParam.put("tablecode", "qr");
        result.put("qrOrderTakeOff", this.generalCount(fromHql, queryParam).intValue());

        //增加退单订单的获取 20240301 郭
        queryParam.put(Constants.QUERY_SPEC_CONDITION + "status", "status in (5)");
        result.put("qrOrderTakeOffReturn", this.generalCount(fromHql, queryParam).intValue());
        result.put("qrOrderTakeOffTotal", result.get("qrOrderTakeOff")+result.get("qrOrderTakeOffReturn"));

        queryParam.remove("tablecode");
        queryParam.put(Constants.QUERY_NOTEQUAL + "tablecode", "qr");
        queryParam.put(Constants.QUERY_SPEC_CONDITION + "status", "status in (3, 9)");
        result.put("qrOrderStore", this.generalCount(fromHql, queryParam).intValue());
        result.put("qrOrder", result.get("qrOrderStore") + result.get("qrOrderTakeOff"));
        return result;
    }

    public Map<String, List<Object>> weeklyOnlineOrders(String storeCode, Date date) throws Exception {
        Map<String, List<Object>> results = new HashMap<>();
        Calendar cal = Calendar.getInstance();
        cal.setTime(new Date());
        Calendar cal1 = Calendar.getInstance();
        cal1.setTime(date);
        if (cal1.get(Calendar.WEEK_OF_YEAR) != cal.get(Calendar.WEEK_OF_YEAR)) {
            //选中天和当前天不为同一周，且前端传来日期不可能在当前天之后，取选中天那一周的数据
            results = getCurrentWeeklyOrders(storeCode, date);
        } else {
            if (cal.get(Calendar.DAY_OF_WEEK) == Calendar.SUNDAY) {
                //选中天和当前天为同一周，且当前天为周日，取本周数据
                results = getCurrentWeeklyOrders(storeCode, new Date());
            } else {
                //选中天和当前天为同一周，且当前天不为周日，取前七天数据
                results = getPreSevenOrders(storeCode, date);
            }
        }

        return results;
    }

    /**
     * 获取某店铺日期所在周七天的订单数
     *
     * @param storeCode
     * @param date
     * @return
     */
    private Map<String, List<Object>> getCurrentWeeklyOrders(String storeCode, Date date) {
        Map<String, List<Object>> results = new HashMap<>();
        List<Integer> orders = new ArrayList<>();
        List<BigDecimal> amounts = new ArrayList<>();
        List<String> dates = new ArrayList<>();
        Calendar cal = Calendar.getInstance();
        cal.setFirstDayOfWeek(Calendar.MONDAY);
        cal.setTime(date);
        Map<String, Object> queryParam = new HashMap<String, Object>();
        if (storeCode != null) {
            queryParam.put("strstorecode", storeCode);
        }
        queryParam.put("offlineupdateflg", 0);
        queryParam.put(Constants.QUERY_SPEC_CONDITION + "year",
                "year(dtorderstart) = " + cal.get(Calendar.YEAR));
        queryParam.put(Constants.QUERY_LESS_EQUAL_DATE + "dtorderstart",
                new Date());
        String fromHql = "select sum(amountsf) as amount from OnlineorderhBean";
        List<Double> summaryAmount = new ArrayList<>();
        for (int i = 2; i <= 8; i++) {
            cal.set(Calendar.DAY_OF_WEEK, i % 7);
            queryParam.put(Constants.QUERY_SPEC_CONDITION + "month",
                    "month(dtorderstart) = " + (cal.get(Calendar.MONTH) + 1));
            queryParam.put(Constants.QUERY_SPEC_CONDITION + "day",
                    "DATE_PART('day', dtorderstart) = " + cal.get(Calendar.DAY_OF_MONTH));
            queryParam.remove(Constants.QUERY_NOTEQUAL + "tablecode");
            queryParam.put(Constants.QUERY_SPEC_CONDITION + "status", "status in (1, 3, 9)");
            queryParam.put("tablecode", "qr");
            Integer tempOrderQr = this.generalCount("from OnlineorderhBean", queryParam).intValue();
            BigDecimal tempOrderQrAmounts;
            summaryAmount = this.generalSearch(fromHql, queryParam, null);
            if (null != summaryAmount && !summaryAmount.isEmpty() && summaryAmount.get(0) != null) {
                tempOrderQrAmounts = BigDecimal.valueOf(summaryAmount.get(0));
            } else {
                tempOrderQrAmounts = new BigDecimal(0L);
            }
            queryParam.remove("tablecode");
            queryParam.put(Constants.QUERY_NOTEQUAL + "tablecode", "qr");
            queryParam.put(Constants.QUERY_SPEC_CONDITION + "status", "status in (3, 9)");
            Integer tempOrderStore = this.generalCount("from OnlineorderhBean", queryParam).intValue();
            BigDecimal tempOrderStoreAmounts;
            summaryAmount = this.generalSearch(fromHql, queryParam, null);
            if (null != summaryAmount && !summaryAmount.isEmpty() && summaryAmount.get(0) != null) {
                tempOrderStoreAmounts = BigDecimal.valueOf(summaryAmount.get(0));
            } else {
                tempOrderStoreAmounts = new BigDecimal(0L);
            }
            orders.add(tempOrderQr + tempOrderStore);
            dates.add((cal.get(Calendar.MONTH) + 1) + "-" + cal.get(Calendar.DAY_OF_MONTH));
            amounts.add(tempOrderQrAmounts.add(tempOrderStoreAmounts));
        }
        results.put("dates", Collections.singletonList(dates));
        results.put("orders", Collections.singletonList(orders));
        results.put("amounts", Collections.singletonList(amounts));
        return results;
    }

    /**
     * 获取某店铺前七天的订单数
     *
     * @param storeCode
     * @param date
     * @return
     */
    private Map<String, List<Object>> getPreSevenOrders(String storeCode, Date date) {
        Map<String, List<Object>> results = new HashMap<>();
        List<Integer> orders = new ArrayList<>();
        List<BigDecimal> amounts = new ArrayList<>();
        List<String> dates = new ArrayList<>();
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        Map<String, Object> queryParam = new HashMap<String, Object>();
        if (storeCode != null) {
            queryParam.put("strstorecode", storeCode);
        }
        queryParam.put(Constants.QUERY_SPEC_CONDITION + "year",
                "year(dtorderstart) = " + cal.get(Calendar.YEAR));

        queryParam.put("offlineupdateflg", 0);
        queryParam.put(Constants.QUERY_LESS_EQUAL_DATE + "dtorderstart",
                new Date());
        cal.add(Calendar.DATE, -6);
        String fromHql = "select sum(amountsf) as amount from OnlineorderhBean";
        List<Double> summaryAmount = new ArrayList<>();
        for (int i = 0; i < 7; i++) {
            queryParam.put(Constants.QUERY_SPEC_CONDITION + "month",
                    "month(dtorderstart) = " + (cal.get(Calendar.MONTH) + 1));
            queryParam.put(Constants.QUERY_SPEC_CONDITION + "day",
                    "DATE_PART('day', dtorderstart) = " + cal.get(Calendar.DAY_OF_MONTH));
            queryParam.remove(Constants.QUERY_NOTEQUAL + "tablecode");
            queryParam.put(Constants.QUERY_SPEC_CONDITION + "status", "status in (1, 3, 9)");
            queryParam.put("tablecode", "qr");
            Integer tempOrderQr = this.generalCount("from OnlineorderhBean", queryParam).intValue();
            BigDecimal tempOrderQrAmounts;
            summaryAmount = this.generalSearch(fromHql, queryParam, null);
            if (null != summaryAmount && !summaryAmount.isEmpty() && summaryAmount.get(0) != null) {
                tempOrderQrAmounts = BigDecimal.valueOf(summaryAmount.get(0));
            } else {
                tempOrderQrAmounts = new BigDecimal(0L);
            }
            queryParam.remove("tablecode");
            queryParam.put(Constants.QUERY_NOTEQUAL + "tablecode", "qr");
            queryParam.put(Constants.QUERY_SPEC_CONDITION + "status", "status in (3, 9)");
            Integer tempOrderStore = this.generalCount("from OnlineorderhBean", queryParam).intValue();
            BigDecimal tempOrderStoreAmounts;
            summaryAmount = this.generalSearch(fromHql, queryParam, null);
            if (null != summaryAmount && !summaryAmount.isEmpty() && summaryAmount.get(0) != null) {
                tempOrderStoreAmounts = BigDecimal.valueOf(summaryAmount.get(0));
            } else {
                tempOrderStoreAmounts = new BigDecimal(0L);
            }
            orders.add(tempOrderQr + tempOrderStore);
            dates.add((cal.get(Calendar.MONTH) + 1) + "-" + cal.get(Calendar.DAY_OF_MONTH));
            amounts.add(tempOrderQrAmounts.add(tempOrderStoreAmounts));
            cal.add(Calendar.DATE, 1);
        }
        results.put("dates", Collections.singletonList(dates));
        results.put("orders", Collections.singletonList(orders));
        results.put("amounts", Collections.singletonList(amounts));
        return results;
    }

    public int dailyUnProcessOrderQuantity(String storeCode, Date date) throws Exception {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Map<String, Object> queryParam = new HashMap<String, Object>();
        queryParam.put(Constants.QUERY_START_DATE + "dtorderstart",
                sdf.format(date));
        queryParam.put(Constants.QUERY_END_DATE + "dtorderstart",
                sdf.format(date));
        queryParam.put("strstorecode", storeCode);
        queryParam.put("offlineupdateflg", 0);
        queryParam.put("status", 1);
        String fromHql = "from OnlineorderhBean";
        return this.generalCount(fromHql, queryParam).intValue();
    }

    public int dailyNotPaymentOrderQuantity(String storeCode, Date date) throws Exception {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Map<String, Object> queryParam = new HashMap<String, Object>();
        queryParam.put(Constants.QUERY_START_DATE + "dtorderstart",
                sdf.format(date));
        queryParam.put(Constants.QUERY_END_DATE + "dtorderstart",
                sdf.format(date));
        queryParam.put("strstorecode", storeCode);
        queryParam.put("offlineupdateflg", 0);
        queryParam.put(Constants.QUERY_SPEC_CONDITION + "status", "status not in (3, 4, 9)");
        String fromHql = "from OnlineorderhBean";
        return this.generalCount(fromHql, queryParam).intValue();
    }

    public Map<String, String> dailyOnlinePaymentAmount(String storeCode, Date date) throws Exception {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Map<String, Object> queryParam = new HashMap<String, Object>();
        Map<String, BigDecimal> result = new HashMap<>();
        Map<String, String> finalResult = new HashMap<>();
        queryParam.put(Constants.QUERY_START_DATE + "dtorderstart",
                sdf.format(date));
        queryParam.put(Constants.QUERY_END_DATE + "dtorderstart",
                sdf.format(date));
        if (storeCode != null) {
            queryParam.put("strstorecode", storeCode);
        }
        queryParam.put("offlineupdateflg", 0);
        queryParam.put(Constants.QUERY_SPEC_CONDITION + "status", "status in (1, 3, 9)");
        queryParam.put("tablecode", "qr");
        String fromHql = "select sum(amountsf) as amount from OnlineorderhBean";
        List<Double> summaryAmount = this.generalSearch(fromHql, queryParam, null);
        if (null != summaryAmount && !summaryAmount.isEmpty() && summaryAmount.get(0) != null) {
            result.put("qrAmountTakeOff", BigDecimal.valueOf(summaryAmount.get(0)).setScale(2, RoundingMode.HALF_UP));
        } else {
            result.put("qrAmountTakeOff", new BigDecimal("0.00"));
        }

        //增加退单金额获取的逻辑 20240301 郭
        queryParam.put(Constants.QUERY_SPEC_CONDITION + "status", "status in (5)");
        summaryAmount = this.generalSearch(fromHql, queryParam, null);
        if (null != summaryAmount && !summaryAmount.isEmpty() && summaryAmount.get(0) != null) {
            result.put("qrAmountTakeOffReturn", BigDecimal.valueOf(summaryAmount.get(0)).setScale(2, RoundingMode.HALF_UP));
        } else {
            result.put("qrAmountTakeOffReturn", new BigDecimal("0.00"));
        }
        result.put("qrAmountTakeOffTotal", result.get("qrAmountTakeOff").add(result.get("qrAmountTakeOffReturn")).setScale(2, RoundingMode.HALF_UP));

        queryParam.remove("tablecode");
        queryParam.put(Constants.QUERY_NOTEQUAL + "tablecode", "qr");
        queryParam.put(Constants.QUERY_SPEC_CONDITION + "status", "status in (3, 9)");
        summaryAmount = this.generalSearch(fromHql, queryParam, null);
        if (null != summaryAmount && !summaryAmount.isEmpty() && summaryAmount.get(0) != null) {
            result.put("qrAmountStore", BigDecimal.valueOf(summaryAmount.get(0)).setScale(2, RoundingMode.HALF_UP));
        } else {
            result.put("qrAmountStore", new BigDecimal("0.00"));
        }
        result.put("qrAmount", result.get("qrAmountTakeOff").add(result.get("qrAmountStore")).setScale(2, RoundingMode.HALF_UP));

        finalResult.put("qrAmountTakeOff",result.get("qrAmountTakeOff").toString());
        finalResult.put("qrAmountTakeOffReturn",result.get("qrAmountTakeOffReturn").toString());
        finalResult.put("qrAmountTakeOffTotal",result.get("qrAmountTakeOffTotal").toString());
        finalResult.put("qrAmountStore",result.get("qrAmountStore").toString());
        finalResult.put("qrAmount",result.get("qrAmount").toString());
        return finalResult;
    }

    public List<BigDecimal> weeklyOnlinePayments(String storeCode, Date date) throws Exception {
        List<BigDecimal> results = new ArrayList<BigDecimal>();
        Calendar cal = Calendar.getInstance();
        cal.setTime(new Date());
        int weekNum = cal.get(Calendar.DAY_OF_WEEK);
        Map<String, Object> queryParam = new HashMap<String, Object>();
        queryParam.put(Constants.QUERY_SPEC_CONDITION + "year",
                "year(dtorderstart) = " + cal.get(Calendar.YEAR));
        queryParam.put(Constants.QUERY_SPEC_CONDITION + "month",
                "month(dtorderstart) = " + (cal.get(Calendar.MONTH) + 1));
        queryParam.put(Constants.QUERY_LESS_EQUAL_DATE + "dtorderstart",
                new Date());
        queryParam.put("strstorecode", storeCode);
        queryParam.put("offlineupdateflg", 0);
        queryParam.put(Constants.QUERY_SPEC_CONDITION + "status", "status not in (3, 4)");
        String fromHql = "select sum(amountsf) as amount from OnlineorderhBean";
        for (int i = 1; i <= weekNum; i++) {
            queryParam.put(Constants.QUERY_SPEC_CONDITION + "week",
                    "DATE_PART('dow', dtorderstart) = " + (i == weekNum ? 0 : i));
            List<Double> summaryAmount = this.generalSearch(fromHql, queryParam, null);
            if (null != summaryAmount && !summaryAmount.isEmpty() && summaryAmount.get(0) != null) {
                results.add(BigDecimal.valueOf(summaryAmount.get(0)));
            }
        }

        return results;
    }

    public List<Map<String, Object>> searchPaymentPersent(String storeCode) throws Exception {
        String fromHql = "select new map(case when p.paymentcode = 1 then '微信' when p.paymentcode = 2 then '支付宝' else '线下' end as name, sum(p.amount) as value) " +
                "from OnlineorderhBean h, OnlineorderpBean p where h.orderno = p.orderno group by p.paymentcode";
        Map<String, Object> queryParam = new HashMap<String, Object>();
        queryParam.put("h.strstorecode", storeCode);

        return this.generalSearch(fromHql, queryParam, null);
    }
}
