package com.bettersoft.api.dao.implement;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.bettersoft.api.model.ItemBean;
import com.bettersoft.api.model.OnlineorderdBean;
import com.bettersoft.api.utils.PageInfoUtils;
import com.bettersoft.api.validmodel.OnlineItemPackageValidate;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Repository;

import com.bettersoft.api.dao.interfaces.IOnlineItemDao;
import com.bettersoft.api.model.OnlineItemBean;
import com.bettersoft.api.utils.Constants;

@Repository("onlineItemDao")
public class OnlineItemDao extends BaseDao implements IOnlineItemDao {
    public OnlineItemBean findOnlinItem(String storeCode, int itemId) throws Exception {
        Map<String, Object> queryParam = new HashMap<String, Object>();
        queryParam.put("strstorecode", storeCode);
        queryParam.put("intitemno", itemId);
        List<OnlineItemBean> items = this.generalSearch("from OnlineItemBean", queryParam, null);
        if (null != items && !items.isEmpty())
            return items.get(0);

        return null;
    }

    public List<Map<String, Object>> searchList(Map<String, Object> queryParam) throws Exception {
        StringBuilder sqlBuilder = new StringBuilder();
        sqlBuilder.append("select * from (select a.strposfastkeyname,d.strstorecode,s.strstorename, b.strpagecode,b.strpagename,c.intkeyorder,c.intitemno,c.strtext,d.iconpath,");
        sqlBuilder.append("d.status,d.imgpath,d.price from posfastkey a,posfastkey_page b,posfastkey_key c,onlineitem d, Store s where a.intposfastkeyno=b.intposfastkeyno and ");
        sqlBuilder.append("s.strstorecode = d.strstorecode and b.intposfastkeyno=c.intposfastkeyno and b.strpagecode=c.strpagecode and a.intposfastkeyno=c.intposfastkeyno ");
        sqlBuilder.append("and c.intitemno=d.intitemno and d.strstorecode = a.strstorecode ");
        sqlBuilder.append("and a.strposfastkeyname like '%QR' and c.intitemno is not null) i");

        return this.generalSearchBySql(sqlBuilder.toString(), queryParam, MapUtils.getString(queryParam, Constants.QUERY_ORDER_BY, StringUtils.EMPTY));
    }

    public Number searchCount(Map<String, Object> queryParam) throws Exception {
        StringBuilder sqlBuilder = new StringBuilder();
        sqlBuilder.append("select * from (select a.strposfastkeyname,d.strstorecode,s.strstorename, b.strpagecode,b.strpagename,c.intkeyorder,c.intitemno,c.strtext,d.iconpath,");
        sqlBuilder.append("d.status,d.imgpath,d.price from posfastkey a,posfastkey_page b,posfastkey_key c,onlineitem d, Store s where a.intposfastkeyno=b.intposfastkeyno and ");
        sqlBuilder.append("s.strstorecode = d.strstorecode and b.intposfastkeyno=c.intposfastkeyno and b.strpagecode=c.strpagecode and a.intposfastkeyno=c.intposfastkeyno ");
        sqlBuilder.append("and c.intitemno=d.intitemno and d.strstorecode = a.strstorecode ");
        sqlBuilder.append("and a.strposfastkeyname like '%QR' and c.intitemno is not null) i");

        return this.generalCountBySql(sqlBuilder.toString(), queryParam);
    }

    public List<Map<String, String>> getAllStrposfastkeyname(String strstorecode) throws Exception {
        String fromSql = "select * from (select distinct b.strpagecode as pageCode, b.strpagename as pageName from posfastkey a,posfastkey_page b,posfastkey_key c " +
                "where a.intposfastkeyno=b.intposfastkeyno and (b.intposfastkeyno=c.intposfastkeyno and b.strpagecode=c.strpagecode and a.intposfastkeyno=c.intposfastkeyno) " +
                "and a.strstorecode='" + strstorecode + "' and a.strposfastkeyname like '%QR' and c.intitemno is not null) t";
        return this.generalSearchBySql(fromSql, null, "order by pagecode");
    }

    public List<OnlineItemPackageValidate> searchPackageGroup(int itemId) throws Exception {
        List<OnlineItemPackageValidate> packages = new ArrayList<OnlineItemPackageValidate>();
        String fromSql = "select strgroupcode as packagecode, strgroupname as packagename, dblqty as quantity, intitemno as itemid from smig";
        Map<String, Object> queryParam = new HashMap<String, Object>();
        queryParam.put("intitemno", itemId);

        List<Map<String, Object>> groups = this.generalSearchBySql(fromSql, queryParam, "order by strgroupcode");
        if (null != groups && !groups.isEmpty()) {
            for (Map<String, Object> group : groups) {
                OnlineItemPackageValidate packageGroup = new OnlineItemPackageValidate();
                packageGroup.setPackageCode(MapUtils.getString(group, "packagecode", StringUtils.EMPTY));
                packageGroup.setPackageDesc(MapUtils.getString(group, "packagename", StringUtils.EMPTY));
                packageGroup.setQuantity(MapUtils.getIntValue(group, "quantity", 0));
                packageGroup.setPackageType("P");
                packageGroup.setItemId(itemId);

                packages.add(packageGroup);
            }
        }

        return packages;
    }

    public List<OnlineItemPackageValidate> searchPackageDetail(String groupCode, int itemId) throws Exception {
        List<OnlineItemPackageValidate> packages = new ArrayList<OnlineItemPackageValidate>();
        StringBuilder sqlBuilder = new StringBuilder();
        sqlBuilder.append("SELECT * ");
        sqlBuilder.append("FROM (SELECT g.strgroupcode           AS packagecode,");
        sqlBuilder.append("             g.strgroupname           AS packagename,");
        sqlBuilder.append("             gi.intingredientitemno   AS itemid,");
        sqlBuilder.append("             i.stritemnameshort       AS itemdesc,");
        sqlBuilder.append("             gi.intitemno             AS pitemid,");
        sqlBuilder.append("             0                        AS price");
        sqlBuilder.append("      FROM smig g, smigi gi, item i");
        sqlBuilder.append("      WHERE     g.strgroupcode = gi.strgroupcode");
        sqlBuilder.append("            AND gi.intingredientitemno = i.intitemno");
        sqlBuilder.append("      UNION");
        sqlBuilder.append("      SELECT g.strgroupcode        AS packagecode,");
        sqlBuilder.append("             g.strgroupname        AS packagename,");
        sqlBuilder.append("             gc.intreplaceitemno   AS itemid,");
        sqlBuilder.append("             i.stritemnameshort    AS itemdesc,");
        sqlBuilder.append("             gc.intitemno          AS pitemid,");
        sqlBuilder.append("             gc.curupdateprice     AS price");
        sqlBuilder.append("      FROM smig g, smigc gc, item i");
        sqlBuilder.append("      WHERE     g.strgroupcode = gc.strgroupcode");
        sqlBuilder.append("            AND gc.intreplaceitemno = i.intitemno) t");
        Map<String, Object> queryParam = new HashMap<String, Object>();
        queryParam.put("packagecode", groupCode);
        queryParam.put("pitemid", itemId);

        List<Map<String, Object>> groupItems = this.generalSearchBySql(sqlBuilder.toString(), queryParam, "order by packagecode, itemid");
        if (null != groupItems && !groupItems.isEmpty()) {
            for (Map<String, Object> groupItem : groupItems) {
                OnlineItemPackageValidate packageGroup = new OnlineItemPackageValidate();
                packageGroup.setPackageDesc(MapUtils.getString(groupItem, "itemdesc", StringUtils.EMPTY));
                packageGroup.setPackageCode(MapUtils.getString(groupItem, "packagecode", StringUtils.EMPTY));
                packageGroup.setItemId(MapUtils.getIntValue(groupItem, "itemid", 0));
                packageGroup.setParentPackage(itemId + "");
                packageGroup.setPackageType("I");
                packageGroup.setPrice(new BigDecimal(MapUtils.getFloatValue(groupItem, "price", 0.0f)));

                packages.add(packageGroup);
            }
        }
        return packages;
    }

    public List<OnlineItemPackageValidate> searchItemModifiers(String storeCode, int itemId) throws Exception {
        Map<String, OnlineItemPackageValidate> modifiPackageMap = new HashMap<String, OnlineItemPackageValidate>();
        StringBuilder sqlBuilder = new StringBuilder();
        sqlBuilder.append("SELECT * ");
        sqlBuilder.append("FROM (SELECT m.modifierno,");
        sqlBuilder.append("             m.modifiername,");
        sqlBuilder.append("             m.sortno,");
        sqlBuilder.append("             m.itemdesc,");
        sqlBuilder.append("             m.pitemid,");
        sqlBuilder.append("             m.itemid,");
        sqlBuilder.append("             p.strstorecode   AS storecode,");
        sqlBuilder.append("             m.defaultflag,");
        sqlBuilder.append("             m.defaultselected,");
        sqlBuilder.append("             p.curprice       AS price");
        sqlBuilder.append("      FROM (SELECT b.intmodifierno       AS modifierno,");
        sqlBuilder.append("                   a.strmodifiername     AS modifiername,");
        sqlBuilder.append("                   b.intsortno           AS sortno,");
        sqlBuilder.append("                   b.strtext             AS itemdesc,");
        sqlBuilder.append("                   c.intitemno           AS pitemid,");
        sqlBuilder.append("                   b.intitemno           AS itemid,");
        sqlBuilder.append("                   b.ysndefaultselected  AS defaultselected,");
        sqlBuilder.append("                   a.ysnrequired         AS defaultflag");
        sqlBuilder.append("            FROM modifierhdr        a");
        sqlBuilder.append("                 LEFT JOIN modifierdtl b");
        sqlBuilder.append("                    ON a.intmodifierno = b.intmodifierno");
        sqlBuilder.append("                 LEFT JOIN itemmodifier c");
        sqlBuilder.append("                    ON a.intmodifierno = c.intmodifierno) m");
        sqlBuilder.append("           LEFT JOIN item n ON m.itemid = n.intitemno");
        sqlBuilder.append("           LEFT JOIN price p");
        sqlBuilder.append("              ON     m.itemid = p.intitemno");
        sqlBuilder.append("                 AND p.ysnactive = 'T'");
        sqlBuilder.append("                 AND p.strstorecode IS NOT NULL) t");

        Map<String, Object> queryParam = new HashMap<String, Object>();
        queryParam.put(Constants.QUERY_SPEC_CONDITION + "storecode", "(storecode ='" + storeCode + "' OR storecode = 'ALL' or itemid is null)");
        queryParam.put("pitemid", itemId);
        List<Map<String, Object>> modifies = this.generalSearchBySql(sqlBuilder.toString(), queryParam, "order by modifierno, sortno");
        if (null != modifies && !modifies.isEmpty()) {
            for (Map<String, Object> modifi : modifies) {
                OnlineItemPackageValidate modifiPackage = new OnlineItemPackageValidate();
                String modifiNo = MapUtils.getString(modifi, "modifierno", StringUtils.EMPTY);

                modifiPackage.setPackageCode(MapUtils.getIntValue(modifi, "itemid", 0) + "");
                modifiPackage.setPackageDesc(MapUtils.getString(modifi, "itemdesc", StringUtils.EMPTY));
//                modifiPackage.setPrice(new BigDecimal(MapUtils.getFloatValue(modifi, "price", 0.0f)));
                modifiPackage.setParentPackage(modifiNo);
                if (modifiPackageMap.containsKey(modifiNo)) {
                    OnlineItemPackageValidate itemPackage = (OnlineItemPackageValidate) MapUtils.getObject(modifiPackageMap, modifiNo, null);
                    modifiPackage.setPackageType("A");
                    modifiPackage.setPrice(new BigDecimal(MapUtils.getFloatValue(modifi, "price", 0.0f)));
                    modifiPackage.setItemId(MapUtils.getIntValue(modifi, "itemid", 0));
                    if(modifi.get("defaultselected")!=null&&modifi.get("defaultselected").toString().equals("T")){
                        modifiPackage.setChecked(true);
                    }
                    itemPackage.getChildPackages().add(modifiPackage);
                } else {
                    modifiPackage.setPackageCode(modifiNo);
                    modifiPackage.setPackageDesc(MapUtils.getString(modifi, "modifiername", StringUtils.EMPTY));
                    modifiPackage.setParentPackage(itemId + "");
                    modifiPackage.setItemId(MapUtils.getIntValue(modifi, "pitemid", 0));
                    modifiPackage.setPackageType("M");
                    modifiPackage.setDefaultflag(MapUtils.getString(modifi, "defaultflag", ""));
                    OnlineItemPackageValidate itemPackage = new OnlineItemPackageValidate();
                    itemPackage.setPackageCode(MapUtils.getIntValue(modifi, "itemid", 0) + "");
                    itemPackage.setPackageDesc(MapUtils.getString(modifi, "itemdesc", StringUtils.EMPTY));
                    itemPackage.setPrice(new BigDecimal(MapUtils.getFloatValue(modifi, "price", 0.0f)));
                    itemPackage.setParentPackage(modifiNo);
                    itemPackage.setItemId(MapUtils.getIntValue(modifi, "itemid", 0));
                    itemPackage.setPackageType("A");
                    if(modifi.get("defaultselected")!=null&&modifi.get("defaultselected").toString().equals("T")){
                        itemPackage.setChecked(true);
                    }
                    modifiPackage.getChildPackages().add(itemPackage);

                    modifiPackageMap.put(modifiNo, modifiPackage);
                }
            }
        }

        return new ArrayList<OnlineItemPackageValidate>(modifiPackageMap.values());
    }

    public List<OnlineItemPackageValidate> searchItemModifiers(String storeCode, String orderNo, int itemId, List<OnlineorderdBean> offlineItems) throws Exception {
        Map<String, OnlineItemPackageValidate> modifiPackageMap = new HashMap<String, OnlineItemPackageValidate>();
        StringBuilder sqlBuilder = new StringBuilder();
        sqlBuilder.append("SELECT * ");
        sqlBuilder.append("FROM (SELECT m.modifierno,");
        sqlBuilder.append("             m.modifiername,");
        sqlBuilder.append("             m.sortno,");
        sqlBuilder.append("             m.itemdesc,");
        sqlBuilder.append("             m.pitemid,");
        sqlBuilder.append("             m.itemid,");
        sqlBuilder.append("             p.strstorecode   AS storecode,");
        sqlBuilder.append("             p.curprice       AS price");
        sqlBuilder.append("      FROM (SELECT b.intmodifierno       AS modifierno,");
        sqlBuilder.append("                   a.strmodifiername     AS modifiername,");
        sqlBuilder.append("                   b.intsortno           AS sortno,");
        sqlBuilder.append("                   b.strtext             AS itemdesc,");
        sqlBuilder.append("                   c.intitemno           AS pitemid,");
        sqlBuilder.append("                   b.intitemno           AS itemid");
        sqlBuilder.append("            FROM modifierhdr        a");
        sqlBuilder.append("                 LEFT JOIN modifierdtl b");
        sqlBuilder.append("                    ON a.intmodifierno = b.intmodifierno");
        sqlBuilder.append("                 LEFT JOIN itemmodifier c");
        sqlBuilder.append("                    ON a.intmodifierno = c.intmodifierno) m");
        sqlBuilder.append("           LEFT JOIN item n ON m.itemid = n.intitemno");
        sqlBuilder.append("           LEFT JOIN price p");
        sqlBuilder.append("              ON     m.itemid = p.intitemno");
        sqlBuilder.append("                 AND p.ysnactive = 'T'");
        sqlBuilder.append("                 AND p.strstorecode IS NOT NULL) t");

        Map<String, Object> queryParam = new HashMap<String, Object>();
        queryParam.put(Constants.QUERY_SPEC_CONDITION + "storecode", "(storecode ='" + storeCode + "' OR storecode = 'ALL' or itemid is null)");
        queryParam.put("pitemid", itemId);
        List<Map<String, Object>> modifies = this.generalSearchBySql(sqlBuilder.toString(), queryParam, "order by modifierno, sortno");
        if (null != modifies && !modifies.isEmpty()) {
            for (Map<String, Object> modifi : modifies) {
                if (isModifyOfflineItem(offlineItems, modifi)) {
                    OnlineItemPackageValidate modifiPackage = new OnlineItemPackageValidate();
                    String modifiNo = MapUtils.getString(modifi, "modifierno", StringUtils.EMPTY);

                    modifiPackage.setOrderNo(orderNo);
                    modifiPackage.setOffline(true);
                    modifiPackage.setPackageCode(MapUtils.getIntValue(modifi, "itemid", 0) + "");
                    modifiPackage.setPackageDesc(MapUtils.getString(modifi, "itemdesc", StringUtils.EMPTY));
                    modifiPackage.setParentPackage(modifiNo);
                    if (modifiPackageMap.containsKey(modifiNo)) {
                        OnlineItemPackageValidate itemPackage = (OnlineItemPackageValidate) MapUtils.getObject(modifiPackageMap, modifiNo, null);
                        modifiPackage.setPackageType("A");
                        itemPackage.getChildPackages().add(modifiPackage);
                    } else {
                        modifiPackage.setPackageCode(modifiNo);
                        modifiPackage.setOrderNo(orderNo);
                        modifiPackage.setOffline(true);
                        modifiPackage.setPackageDesc(MapUtils.getString(modifi, "modifiername", StringUtils.EMPTY));
                        modifiPackage.setParentPackage(itemId + "");
                        modifiPackage.setItemId(MapUtils.getIntValue(modifi, "pitemid", 0));
                        modifiPackage.setPackageType("M");
                        OnlineItemPackageValidate itemPackage = new OnlineItemPackageValidate();
                        itemPackage.setOrderNo(orderNo);
                        itemPackage.setOffline(true);
                        itemPackage.setPackageCode(MapUtils.getIntValue(modifi, "itemid", 0) + "");
                        itemPackage.setPackageDesc(MapUtils.getString(modifi, "itemdesc", StringUtils.EMPTY));
                        itemPackage.setPrice(new BigDecimal(MapUtils.getFloatValue(modifi, "price", 0.0f)));
                        itemPackage.setParentPackage(modifiNo);
                        itemPackage.setItemId(MapUtils.getIntValue(modifi, "itemid", 0));
                        itemPackage.setPackageType("A");
                        modifiPackage.getChildPackages().add(itemPackage);

                        modifiPackageMap.put(modifiNo, modifiPackage);
                    }
                }
            }
        }

        return new ArrayList<OnlineItemPackageValidate>(modifiPackageMap.values());
    }

    public boolean isPackageItem(int itemId) throws Exception {
        ItemBean item = findEntity(ItemBean.class, itemId);

        return null != item && !StringUtils.isEmpty(item.getYsnSetMeal()) && item.getYsnSetMeal().equals("T");
    }

    private boolean isModifyOfflineItem(List<OnlineorderdBean> offlineItems, Map<String, Object> modify) {
        boolean isOffline = false;
        if (null != offlineItems && !offlineItems.isEmpty()) {
            for (OnlineorderdBean offlineItem : offlineItems) {
                if (!StringUtils.isEmpty(offlineItem.getPmodify())) {
                    int modifyId = MapUtils.getIntValue(modify, "itemid", 0);
                    String modifyName = MapUtils.getString(modify, "itemdesc", StringUtils.EMPTY);
                    String[] modifies = offlineItem.getPmodify().split("\\|");

                    for (String pmodify : modifies) {
                        isOffline = pmodify.equals(modifyName) || modifyId == offlineItem.getIntitemno();
                        if (isOffline)
                            break;
                    }

                    if (isOffline)
                        break;
                }
            }
        }

        return isOffline;
    }
}
