package com.bettersoft.api.dao.implement;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bettersoft.api.config.WxConfig;
import com.bettersoft.api.dao.interfaces.*;
import com.bettersoft.api.encrytion.Base64Util;
import com.bettersoft.api.encrytion.DateUtils;
import com.bettersoft.api.encrytion.MD5Util;
import com.bettersoft.api.model.*;
import com.bettersoft.api.service.interfaces.IOnlineorderhService;
import com.bettersoft.api.utils.EntityUtil;
import com.bettersoft.api.utils.HttpUtils;
import com.bettersoft.api.utils.JsonResult;
import com.bettersoft.api.utils.RedisCacheUtil;
import com.bettersoft.api.validmodel.OnlineItemPackageValidate;
import com.bettersoft.api.validmodel.ShoppingCartValidate;
import org.apache.commons.lang.ObjectUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.http.client.ClientHttpRequest;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.HttpMessageConverterExtractor;
import org.springframework.web.client.RequestCallback;
import org.springframework.web.client.RestTemplate;
import sun.swing.StringUIClientPropertyKey;

import javax.annotation.Resource;
import javax.validation.constraints.DecimalMax;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.*;

@Repository("YourenMemberDao")
public class YourenMemberDao extends BaseDao implements IYourenMemberDao {

    @Resource(name = "onlineuserDao")
    private IOnlineuserDao onlineuserDao;
    @Resource(name = "restTemplate")
    private RestTemplate restTemplate;
    @Resource
    private IOnlineorderhDao onlineorderhDao;
    @Resource
    private IOnlineorderdDao onlineorderdDao;
    @Resource
    private IBarcodeDao barcodeDao;
    @Resource
    private IOnlineorderhService onlineorderhService;
    @Resource(name = "redisCacheUtil")
    private RedisCacheUtil redisCacheUtil;
    @Resource
    private OnlineItemDao onlineItemDao;


    @Override
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
    public JsonResult checkMember(String token) throws Exception {
        JsonResult jsonResult = new JsonResult();
        String str = new String(Base64Util.decryBASE64(token));
        JSONObject strAsJson = JSONObject.parseObject(str);
        try {
            HashMap<String, Object> paramMap = new HashMap<>();
            paramMap.put("openId", strAsJson.get("openid"));
            if (strAsJson.getString("unionid") != null) {
                paramMap.put("unionId", strAsJson.get("unionid"));
            } else {
//                paramMap.put("unionId","oID9MxNm_naZVRyRvAFGOKZViIw8");     //非会员
//                paramMap.put("unionId", "oID9MxD2tBHivjBuYmCYNN5Hw8-g");     //会员
                paramMap.put("unionId", "");     //会员
            }
            paramMap.put("appId", WxConfig.appId);
            paramMap.put("mobile", strAsJson.get("phone"));
            JSONObject bodyAsJson = restTemplate.getForObject(WxConfig.CHECK_MEMBER, JSONObject.class, paramMap);
            logger.info(bodyAsJson);
            if (!bodyAsJson.containsKey("data")) {
//                jsonResult.setSuccess(false);
                jsonResult.setSuccess(true);
                jsonResult.setErrorCode("404");
                jsonResult.setErrMsg("404 not found");
                return jsonResult;
            }
            JSONObject data = bodyAsJson.getJSONObject("data");
            OnlineUserBean onlineUserBean = onlineuserDao.findEntity(OnlineUserBean.class, strAsJson.getString("openid"));
            if (data.get("barcode") != null) {
                onlineUserBean.setMemberflg(1);
                onlineUserBean.setMemberCard(data.getString("barcode"));
                onlineUserBean.setUpdateDate(new Date());
                onlineuserDao.saveOrUpdate(onlineUserBean);
                jsonResult.setSuccess(true);
                jsonResult.setErrorCode("200");
                jsonResult.setErrMsg("该用户为会员!");
                return jsonResult;
            } else {
                onlineUserBean.setMemberflg(0);
                onlineUserBean.setUpdateDate(new Date());
                onlineuserDao.saveOrUpdate(onlineUserBean);
                jsonResult.setSuccess(true);
                jsonResult.setErrorCode("100");
                jsonResult.setErrMsg("该用户非会员!");
                return jsonResult;
            }
        } catch (Exception e) {
            OnlineUserBean onlineUserBean = onlineuserDao.findEntity(OnlineUserBean.class, strAsJson.getString("openid"));
            if (onlineUserBean != null && onlineUserBean.getMemberflg() == null) {
                onlineUserBean.setMemberflg(0);
                onlineUserBean.setUpdateDate(new Date());
                onlineuserDao.saveOrUpdate(onlineUserBean);
            }
            logger.info(e.getMessage(), e);
            throw e;
        }
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
    public JsonResult getPurchaseDiscount(String token, String orderNo) throws Exception {
        JsonResult jsonResult = new JsonResult();
        Map<String, Object> resMap = new HashMap<>();
        List<OnlineorderdBean> orderDetails = onlineorderhService.searchOrderDetails(orderNo);
        List<OnlineorderdBean> tempDetail = new ArrayList<>();
        for (OnlineorderdBean detail : orderDetails) {
            if (!detail.getLinetype().equals("SI")) {
                tempDetail.add(detail);
            }
        }
        try {
            String str = new String(Base64Util.decryBASE64(token));
            JSONObject strAsJson = JSONObject.parseObject(str);
            OnlineUserBean onlineUserBean = onlineuserDao.findEntity(OnlineUserBean.class, strAsJson.getString("openid"));
            OnlineorderhBean onlineorderhBean = onlineorderhDao.findEntity(OnlineorderhBean.class, orderNo);
            StringBuilder serialNumber = new StringBuilder();
            serialNumber.append(DateUtils.format(onlineorderhBean.getDtorderstart(), DateUtils.DATE_TIME));
            String daysn = String.format("%04d", onlineorderhBean.getDaysn());
            serialNumber.append(daysn);
            StringBuilder commodityInfo = new StringBuilder();
            int index = 1;
            StringBuilder remark = new StringBuilder();
            BigDecimal betotalAmount = new BigDecimal(0);
            for (OnlineorderdBean detail : tempDetail) {
                ItemBean item = onlineItemDao.findEntity(ItemBean.class, detail.getIntitemno());
                if (item == null) {
                    logger.info("获取折扣时,商品号" + detail.getIntitemno() + "对应商品在item表中不存在.");
                } else if (StringUtils.isEmpty(item.getStrExtSRef1())) {
                    logger.info("获取折扣时,商品号" + detail.getIntitemno() + "没有barcode");
                }
                if (item != null && !StringUtils.isEmpty(item.getStrExtSRef1())) {
                    commodityInfo.append(item.getStrExtSRef1());
                }
                commodityInfo.append(":");
                commodityInfo.append(detail.getQty());
                commodityInfo.append(":");
                commodityInfo.append(String.format("%.2f", detail.getPrice()));
                commodityInfo.append(":");
                commodityInfo.append(String.format("%.2f", detail.getAmount()));
                commodityInfo.append(":");
                commodityInfo.append(detail.getLine());
                commodityInfo.append(",");
                if (detail.getSuborderno() == index) {
                    if (detail.getRemark() != null && !detail.getRemark().isEmpty()) {
                        remark.append(detail.getRemark());
                        remark.append(";");
                    }
                    index++;
                }
                betotalAmount = betotalAmount.add(BigDecimal.valueOf(detail.getAmount()));
            }
            String commdity = commodityInfo.substring(0, commodityInfo.length() - 1);
            if (StringUtils.isEmpty(onlineorderhBean.getCrm_serialnumber())) {
//                Long cache = Long.parseLong(redisCacheUtil.getCache(onlineorderhBean.getStrstorecode()).toString());
//                redisCacheUtil.setCache(onlineorderhBean.getStrstorecode(), cache + 1);
//                redisCacheUtil.setCache(onlineorderhBean.getStrstorecode() + strAsJson.getString("openid"), cache.toString(), 28800);
                Long cache = onlineorderhService.getSerialNumber(onlineorderhBean.getStrstorecode(), strAsJson.getString("openid"));
                if (onlineUserBean != null && onlineUserBean.getMemberflg() != null && onlineUserBean.getMemberflg() == 1) {
                    logger.info("用户openId:" + strAsJson.getString("openid") + ",会员卡号:" + onlineUserBean.getMemberCard() + ",首次获得序列号并放入缓存:" + cache);
                } else {
                    logger.info("用户openId:" + strAsJson.getString("openid") + ",会员卡号:  ,首次获得序列号并放入缓存:" + cache);
                }
                onlineorderhBean.setCrm_serialnumber(cache.toString());
                onlineorderhDao.update(onlineorderhBean);
//                Long temp = cache + 1;
//                logger.info("店铺号:" + onlineorderhBean.getStrstorecode() + ",当前流水号:" + temp);
            } else {
                logger.info("openId:" + strAsJson.getString("openid") + "已经获得的序列号为:" + onlineorderhBean.getCrm_serialnumber());
                JSONObject canParam = new JSONObject();
                if (onlineUserBean != null && onlineUserBean.getMemberflg() != null && onlineUserBean.getMemberflg() == 1) {
                    canParam.put("cardCode", onlineUserBean.getMemberCard());
                } else {
                    canParam.put("cardCode", "");
                }
                canParam.put("shopCode", onlineorderhBean.getStrstorecode());
                canParam.put("posNo", onlineorderhBean.getStrstorecode() + "Q");
//                canParam.put("serialNumber", redisCacheUtil.getCache(strAsJson.getString("openid")));
                canParam.put("serialNumber", onlineorderhBean.getCrm_serialnumber());
                logger.info("用户openId: " + strAsJson.getString("openid") + ",会员卡号: " + canParam.get("cardCode") + ",解绑优惠前序列号: " + canParam.get("serialNumber"));
                canParam.put("accessToken", MD5Util.encode("sendPurchaseCancelToYRccc" + canParam.getString("cardCode")));
                HttpHeaders canheaders = new HttpHeaders();
                MediaType type = MediaType.parseMediaType("application/json; charset=UTF-8");
                canheaders.setContentType(type);
                HttpEntity<JSONObject> canrequest = new HttpEntity<JSONObject>(canParam, canheaders);
                logger.info("解绑折扣参数：" + canParam);
//                JSONObject bodyAsJson = restTemplate.postForObject(WxConfig.CANCEL_YR, canrequest, JSONObject.class, canParam);
                String bodyAsString = HttpUtils.doPost(WxConfig.CANCEL_YR, JSONObject.toJSONString(canParam));
                JSONObject bodyAsJson = JSONObject.parseObject(bodyAsString);
                logger.info("解绑折扣返回值：" + bodyAsJson);
            }
            JSONObject param = new JSONObject();
            if (onlineUserBean != null && onlineUserBean.getMemberflg() != null && onlineUserBean.getMemberflg() == 1) {
                param.put("cardCode", onlineUserBean.getMemberCard());
            } else {
//                param.put("cardCode", "100000001958276964");
                param.put("cardCode", "");
            }
            //有注释参数暂时写死，需要询问
//            param.put("shopCode", "12002");
            param.put("shopCode", onlineorderhBean.getStrstorecode());
//            param.put("posNo", "03");
            param.put("posNo", onlineorderhBean.getStrstorecode() + "Q");
//            param.put("serialNumber","2102240001");
            param.put("serialNumber", onlineorderhBean.getCrm_serialnumber());
            logger.info("用户openId: " + strAsJson.getString("openid") + ",会员卡号: " + param.get("cardCode") + ",获得优惠前序列号: " + param.get("serialNumber"));
            param.put("saledDatetime", DateUtils.formatDateTime(new Date()));
//            param.put("commodityInfo","4993418033677:1:38.00:38.00:1,100981710000:1:79.60:79.60:2");
            param.put("commodityInfo", commdity);
            param.put("accessToken", MD5Util.encode("sendPurchaseInfoToYRccc" + param.getString("cardCode")));
            HttpHeaders headers = new HttpHeaders();
            MediaType type = MediaType.parseMediaType("application/json; charset=UTF-8");
            headers.setContentType(type);
            HttpEntity<JSONObject> request = new HttpEntity<JSONObject>(param, headers);
            logger.info("获取折扣参数：" + param);
//            JSONObject bodyAsJson = restTemplate.postForObject(WxConfig.GET_DISCOUNT, request, JSONObject.class, param);
            String bodyAsString = HttpUtils.doPost(WxConfig.GET_DISCOUNT, JSONObject.toJSONString(param));
            JSONObject bodyAsJson = JSONObject.parseObject(bodyAsString);
//            onlineorderhBean.setCrm_serialnumber(redisCacheUtil.getCache(strAsJson.getString("openid")));
//            onlineorderdDao.update(onlineorderhBean);
            logger.info("获取折扣返回值：" + bodyAsJson);
            if (bodyAsJson.getString("resultCode").equals("100")) {
                jsonResult.setSuccess(true);
                jsonResult.setErrMsg("获取成功!");
                jsonResult.setErrorCode("200");
                JSONArray bodyAsList = bodyAsJson.getJSONArray("commodityInfo");
                BigDecimal totalAmount = new BigDecimal(0);
                for (OnlineorderdBean orderDetail : tempDetail) {
                    JSONObject temp = bodyAsList.getJSONObject(tempDetail.indexOf(orderDetail));
                    orderDetail.setAmountafterdiscount(Double.parseDouble(temp.getString("subTotalPrice")));
                    orderDetail.setDiscountamount(orderDetail.getAmount() - orderDetail.getAmountafterdiscount());
                    orderDetail.setDiscountcode(temp.getString("discountId"));
                    orderDetail.setDiscountname(temp.getString("discountMsg"));
                    totalAmount = totalAmount.add(BigDecimal.valueOf(orderDetail.getAmountafterdiscount()));
//                    if (orderDetail.getPmodify().contains("|")) {
//                        String[] modify = orderDetail.getPmodify().split("\\|");
//                        StringBuilder modifyBulider = new StringBuilder();
//                        for (String s : modify) {
//                            modifyBulider.append("[");
//                            modifyBulider.append(s);
//                            modifyBulider.append("] ");
//                        }
//                        orderDetail.setPmodify(modifyBulider.substring(0, modifyBulider.length()));
//                    } else if (!orderDetail.getPmodify().contains("[")) {
//                        orderDetail.setPmodify("[" + orderDetail.getPmodify() + "]");
//                    }
                }
//                Map<String, Object> resMap = new HashMap<>();
                resMap.put("totalAmount", String.format("%.2f", totalAmount));
                resMap.put("remark", remark);
                onlineorderdDao.saveOrUpdateAll(tempDetail);
//                List<OnlineorderdBean> curOrderDetails = onlineorderhService.searchOrderDetails(orderNo);
//                jsonResult.setWxData(general(tempDetail));
                jsonResult.setData(resMap);
                return jsonResult;
            } else {
                logger.info(bodyAsJson);
                resetD(tempDetail);
                resMap.put("totalAmount", String.format("%.2f", betotalAmount));
                resMap.put("remark", remark);
                jsonResult.setErrMsg("该订单无折扣信息!");
                jsonResult.setData(resMap);
                jsonResult.setSuccess(true);
                jsonResult.setErrorCode("100");
                return jsonResult;
            }
        } catch (Exception e) {
//            resetD(tempDetail);
            logger.error(e.getMessage(), e);
            throw e;
        }
    }

    private void resetD(List<OnlineorderdBean> tempDetail) {
        for (OnlineorderdBean detail : tempDetail) {
            detail.setDiscountamount(0.00);
            detail.setDiscountcode("");
            detail.setDiscountname("");
            detail.setAmountafterdiscount(detail.getAmount());
            onlineorderdDao.update(detail);
        }
    }

    @Override
    public List<OnlineorderdBean> general(String orderNo, JsonResult jsonResult) {
        try {
            Double totalAmount = 0.0;
            List<OnlineorderdBean> tempDetail = onlineorderhService.searchOrderDetails(orderNo);
            StringBuilder remark = new StringBuilder();
            int index = 1;
            for (int i = 0; i < tempDetail.size(); ) {
                OnlineorderdBean orderDetail = tempDetail.get(i);
                if (orderDetail.getSuborderno() == index) {
                    if (orderDetail.getRemark() != null && !orderDetail.getRemark().isEmpty()) {
                        remark.append(orderDetail.getRemark());
                        remark.append(";");
                    }
                    index++;
                }
                if (!orderDetail.getLinetype().equals("A")) {
                    Double subTotalPrice = orderDetail.getAmountafterdiscount();
                    Double amount = orderDetail.getAmount();
                    Double price = orderDetail.getPrice();
                    Double discountAmount = orderDetail.getDiscountamount();
                    if (subTotalPrice == null) {
                        subTotalPrice = 0.0;
                    }
                    if (amount == null) {
                        amount = 0.0;
                    }
                    if (price == null) {
                        price = 0.0;
                    }
                    if (discountAmount == null) {
                        discountAmount = 0.0;
                    }
                    StringBuilder discountId = new StringBuilder();
                    StringBuilder discountMsg = new StringBuilder();
                    if (orderDetail.getDiscountcode() != null && !orderDetail.getDiscountcode().isEmpty()) {
                        discountId.append(orderDetail.getDiscountcode());
                    }
                    if (orderDetail.getDiscountname() != null && !orderDetail.getDiscountname().isEmpty()) {
                        discountMsg.append(orderDetail.getDiscountname());
                    }
                    for (int j = i + 1; j <= tempDetail.size(); j++) {
                        if (j == tempDetail.size()) {
                            i = j;
                            break;
                        }
                        OnlineorderdBean curDetail = tempDetail.get(j);
                        if (curDetail.getLinetype().equals("A")) {
                            if (curDetail.getAmountafterdiscount() == null) {
                                subTotalPrice += 0.0;
                            } else {
                                subTotalPrice += curDetail.getAmountafterdiscount();
                            }
                            if (curDetail.getAmount() == null) {
                                amount += 0.0;
                            } else {
                                amount += curDetail.getAmount();
                            }
                            if (curDetail.getPrice() == null) {
                                price += 0.0;
                            } else {
                                price += curDetail.getPrice();
                            }
                            if (curDetail.getDiscountamount() == null) {
                                discountAmount += 0.0;
                            } else {
                                discountAmount += curDetail.getDiscountamount();
                            }
//                            if(curDetail.getDiscountcode()!=null&&!curDetail.getDiscountcode().isEmpty()){
//                                discountId.append(";");
//                                discountId.append(curDetail.getDiscountcode());
//                            }
//                            if(curDetail.getDiscountname()!=null&&!curDetail.getDiscountname().isEmpty()){
//                                discountMsg.append(";");
//                                discountMsg.append(curDetail.getDiscountname());
//                            }
                            if (curDetail.getDiscountcode() != null && !curDetail.getDiscountcode().isEmpty()) {
                                if (discountId == null || discountId.toString().equals("")) {
                                    discountId.append(curDetail.getDiscountcode());
                                }
                            }
                            if (curDetail.getDiscountname() != null && !curDetail.getDiscountname().isEmpty()) {
                                if (discountMsg == null || discountId.toString().equals("")) {
                                    discountMsg.append(curDetail.getDiscountname());
                                }
                            }
                        } else if (!curDetail.getLinetype().equals("SI")) {
                            i = j;
                            break;
                        } else {
                            curDetail.setDiscountamount(0.0);
                        }
                    }
                    orderDetail.setPrice(price);
                    totalAmount += subTotalPrice;
                    orderDetail.setAmountafterdiscount(subTotalPrice);
                    orderDetail.setDiscountamount(discountAmount);
                    orderDetail.setAmount(amount);
                    orderDetail.setDiscountcode(discountId.toString());
                    orderDetail.setDiscountname(discountMsg.toString());
                }
            }
            if (jsonResult != null) {
                Map<String, Object> resMap = new HashMap<>();
                resMap.put("totalAmount", String.format("%.2f", totalAmount));
                resMap.put("remark", remark);
                jsonResult.setData(resMap);
            }
            return tempDetail;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }


    @Override
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = true, rollbackFor = Exception.class)
    public List<OnlineorderdBean> getOrderDetail(ShoppingCartValidate shoppingCart, String token) {
        List<OnlineorderdBean> resultDetail = new ArrayList<>();
        int lineNo = 1;
        try {
            for (OnlineItemBean bi : shoppingCart.getOnlineItemsTakeOff()) {
                OnlineorderdBean tempDetial = new OnlineorderdBean();
                OnlineItemBean i = onlineorderhDao.findOnlineItem(shoppingCart.getStoreCode(), bi.getIntitemno());
                String itemDesc = "";
                itemDesc = null != i.getItem() ? i.getItem().getStrItemNameShort() : StringUtils.EMPTY;
                tempDetial.setBrandcode(i.getBrandcode());
                tempDetial.setLine(lineNo);
                lineNo++;
                tempDetial.setLinetype(bi.getItemType());
                tempDetial.setPname(itemDesc);
                tempDetial.setAmountafterdiscount(bi.getQuantity() * bi.getPrice());
                tempDetial.setPmodify(StringUtils.removeEnd(bi.getPmodify(), " "));
                tempDetial.setQty(bi.getQuantity());
                tempDetial.setPrice(bi.getPrice());
                tempDetial.setOrderfrom(0);
                tempDetial.setConfirmuser("SYSTEM");
                tempDetial.setDtorder(new Date());
                tempDetial.setAmount(bi.getQuantity() * bi.getPrice());
                tempDetial.setIntitemno(i.getIntitemno());
                resultDetail.add(tempDetial);
            }
        } catch (Exception e) {
            logger.error(e.getMessage(),e);
        }
        return resultDetail;
    }

    /**
     *
     * @param orderDetails
     * @param openid
     * @param remark
     * @param storeCode
     * @author gm
     * @apiNote 整理orderDetails的数据，从游仁请求折扣的数据；首次生成serialNumber
     */
    @Override
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = true, rollbackFor = Exception.class)
    public JsonResult getPurchaseDiscountTakeOff(List<OnlineorderdBean> orderDetails, String openid, String remark, String storeCode) throws Exception {
        JsonResult jsonResult = new JsonResult();
        try {
            OnlineUserBean onlineUserBean = onlineuserDao.findEntity(OnlineUserBean.class, openid);
            List<OnlineorderdBean> tempDetail = new ArrayList<>();
            for (OnlineorderdBean detail : orderDetails) {
                if (!detail.getLinetype().equals("SI")) {
                    tempDetail.add(detail);
                }
            }
            StringBuilder commodityInfo = new StringBuilder();
            for (OnlineorderdBean detail : tempDetail) {
                ItemBean item = onlineItemDao.findEntity(ItemBean.class, detail.getIntitemno());
                if (item == null) {
                    logger.info("获取折扣时,商品号" + detail.getIntitemno() + "对应商品在item表中不存在.");
                } else if (StringUtils.isEmpty(item.getStrExtSRef1())) {
                    logger.info("获取折扣时,商品号" + detail.getIntitemno() + "没有barcode");
                }
                if (item != null && !StringUtils.isEmpty(item.getStrExtSRef1())) {
                    commodityInfo.append(item.getStrExtSRef1());
                }
                commodityInfo.append(":");
                commodityInfo.append(detail.getQty());
                commodityInfo.append(":");
                commodityInfo.append(String.format("%.2f", detail.getPrice()));
                commodityInfo.append(":");
                commodityInfo.append(String.format("%.2f", detail.getAmount()));
                commodityInfo.append(":");
                commodityInfo.append(detail.getLine());
                commodityInfo.append(",");
            }
            String commdity = commodityInfo.substring(0, commodityInfo.length() - 1);
            if (!redisCacheUtil.containsKey(storeCode + openid) || redisCacheUtil.getCache(storeCode + openid) == null) {
//                Long cache = Long.parseLong(redisCacheUtil.getCache(storeCode).toString());
//                redisCacheUtil.setCache(storeCode, cache + 1);
//                redisCacheUtil.setCache(storeCode + openid, cache.toString(), 28800);
                Long cache = onlineorderhService.getSerialNumber(storeCode, openid);
                if (onlineUserBean != null && onlineUserBean.getMemberflg() != null && onlineUserBean.getMemberflg() == 1) {
                    logger.info("用户openId:" + openid + ",会员卡号:" + onlineUserBean.getMemberCard() + ",首次获得序列号并放入缓存:" + cache);
                } else {
                    logger.info("用户openId:" + openid + ",会员卡号:  ,首次获得序列号并放入缓存:" + cache);
                }
//                Long temp = cache + 1;
//                logger.info("店铺号:" + storeCode + ",当前流水号:" + temp);
            } else {
                logger.info("用户openId:" + openid + ",已经获得的序列号为:" + redisCacheUtil.getCache(storeCode + openid));
                JSONObject canParam = new JSONObject();
                if (onlineUserBean != null && onlineUserBean.getMemberflg() != null && onlineUserBean.getMemberflg() == 1) {
                    canParam.put("cardCode", onlineUserBean.getMemberCard());
                } else {
                    canParam.put("cardCode", "");
                }
                canParam.put("shopCode", storeCode);
                canParam.put("posNo", storeCode + "Q");
                canParam.put("serialNumber", redisCacheUtil.getCache(storeCode + openid));
                logger.info("用户openId: " + openid + ",会员卡号: " + canParam.get("cardCode") + ",解绑优惠前序列号: " + canParam.get("serialNumber"));
                canParam.put("accessToken", MD5Util.encode("sendPurchaseCancelToYRccc" + canParam.getString("cardCode")));
                HttpHeaders canheaders = new HttpHeaders();
                MediaType type = MediaType.parseMediaType("application/json; charset=UTF-8");
                canheaders.setContentType(type);
                HttpEntity<JSONObject> canrequest = new HttpEntity<JSONObject>(canParam, canheaders);
                logger.info("解绑折扣参数：" + canParam);
//                JSONObject bodyAsJson = restTemplate.postForObject(WxConfig.CANCEL_YR, canrequest, JSONObject.class, canParam);
                String bodyAsString = HttpUtils.doPost(WxConfig.CANCEL_YR, JSONObject.toJSONString(canParam));
                JSONObject bodyAsJson = JSONObject.parseObject(bodyAsString);
                logger.info("解绑折扣返回值：" + bodyAsJson);
            }
            JSONObject param = new JSONObject();
            if (onlineUserBean != null && onlineUserBean.getMemberflg() != null && onlineUserBean.getMemberflg() == 1) {
                param.put("cardCode", onlineUserBean.getMemberCard());
            } else {
                param.put("cardCode", "");
            }
            //有注释参数暂时写死，需要询问
            param.put("shopCode", storeCode);
            param.put("posNo", storeCode + "Q");
            param.put("serialNumber", redisCacheUtil.getCache(storeCode + openid));
            param.put("saledDatetime", DateUtils.formatDateTime(new Date()));
            logger.info("用户openId: " + openid + ",会员卡号: " + param.get("cardCode") + ",获取优惠前序列号: " + param.get("serialNumber"));
            param.put("commodityInfo", commdity);
            param.put("accessToken", MD5Util.encode("sendPurchaseInfoToYRccc" + param.getString("cardCode")));
//            logger.info(param.toString());
            HttpHeaders headers = new HttpHeaders();
            MediaType type = MediaType.parseMediaType("application/json; charset=UTF-8");
            headers.setContentType(type);
            HttpEntity<JSONObject> request = new HttpEntity<JSONObject>(param, headers);
            logger.info("获取折扣参数：" + param);
            String bodyAsString = HttpUtils.doPost(WxConfig.GET_DISCOUNT, JSONObject.toJSONString(param));
            JSONObject bodyAsJson = JSONObject.parseObject(bodyAsString);
            logger.info("获取折扣返回值：" + bodyAsJson);
            if (bodyAsJson.getString("resultCode").equals("100")) {
                jsonResult.setSuccess(true);
                jsonResult.setErrMsg("获取成功!");
                jsonResult.setErrorCode("200");
                JSONArray bodyAsList = bodyAsJson.getJSONArray("commodityInfo");
                BigDecimal totalAmount = new BigDecimal(0);
                for (OnlineorderdBean orderDetail : tempDetail) {
                    JSONObject temp = bodyAsList.getJSONObject(tempDetail.indexOf(orderDetail));
                    orderDetail.setAmountafterdiscount(Double.parseDouble(temp.getString("subTotalPrice")));
                    orderDetail.setDiscountamount(orderDetail.getAmount() - orderDetail.getAmountafterdiscount());
                    orderDetail.setDiscountcode(temp.getString("discountId"));
                    orderDetail.setDiscountname(temp.getString("discountMsg"));
                    totalAmount = totalAmount.add(BigDecimal.valueOf(orderDetail.getAmountafterdiscount()));
                    if (orderDetail.getPmodify().contains("|")) {
                        String[] modify = orderDetail.getPmodify().split("\\|");
                        StringBuilder modifyBulider = new StringBuilder();
                        for (String s : modify) {
                            modifyBulider.append("[");
                            modifyBulider.append(s);
                            modifyBulider.append("] ");
                        }
                        orderDetail.setPmodify(modifyBulider.substring(0, modifyBulider.length()));
                    } else if (!orderDetail.getPmodify().contains("[")) {
                        orderDetail.setPmodify("[" + orderDetail.getPmodify() + "]");
                    }
                }
                Map<String, Object> resMap = new HashMap<>();
                resMap.put("totalAmount", String.format("%.2f", totalAmount));
                resMap.put("remark", remark);
                jsonResult.setData(resMap);
                jsonResult.setWxData(tempDetail);
                return jsonResult;
            } else {
                logger.info(bodyAsJson);
                jsonResult.setErrMsg("该订单无折扣信息!");
                jsonResult.setSuccess(true);
                jsonResult.setErrorCode("100");
                return jsonResult;
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw e;
        }
    }

    @Override
    public Object generalTakeOff(List<OnlineorderdBean> tempDetail, JsonResult jsonResult, String remark) {
        try {
            Double totalAmount = 0.0;
            for (int i = 0; i < tempDetail.size(); ) {
                OnlineorderdBean orderDetail = tempDetail.get(i);
                if (!orderDetail.getLinetype().equals("A")) {
                    Double subTotalPrice = orderDetail.getAmountafterdiscount();
                    Double amount = orderDetail.getAmount();
                    Double price = orderDetail.getPrice();
                    Double discountAmount = orderDetail.getDiscountamount();
                    if (subTotalPrice == null) {
                        subTotalPrice = 0.0;
                    }
                    if (amount == null) {
                        amount = 0.0;
                    }
                    if (price == null) {
                        price = 0.0;
                    }
                    if (discountAmount == null) {
                        discountAmount = 0.0;
                    }
                    StringBuilder discountId = new StringBuilder();
                    StringBuilder discountMsg = new StringBuilder();
                    if (orderDetail.getDiscountcode() != null && !orderDetail.getDiscountcode().isEmpty()) {
                        discountId.append(orderDetail.getDiscountcode());
                    }
                    if (orderDetail.getDiscountname() != null && !orderDetail.getDiscountname().isEmpty()) {
                        discountMsg.append(orderDetail.getDiscountname());
                    }
                    for (int j = i + 1; j <= tempDetail.size(); j++) {
                        if (j == tempDetail.size()) {
                            i = j;
                            break;
                        }
                        OnlineorderdBean curDetail = tempDetail.get(j);
                        if (curDetail.getLinetype().equals("A")) {
                            if (curDetail.getAmountafterdiscount() == null) {
                                subTotalPrice += 0.0;
                            } else {
                                subTotalPrice += curDetail.getAmountafterdiscount();
                            }
                            if (curDetail.getAmount() == null) {
                                amount += 0.0;
                            } else {
                                amount += curDetail.getAmount();
                            }
                            if (curDetail.getPrice() == null) {
                                price += 0.0;
                            } else {
                                price += curDetail.getPrice();
                            }
                            if (curDetail.getDiscountamount() == null) {
                                discountAmount += 0.0;
                            } else {
                                discountAmount += curDetail.getDiscountamount();
                            }
                            if (curDetail.getDiscountcode() != null && !curDetail.getDiscountcode().isEmpty()) {
                                if (discountId == null || discountId.toString().equals("")) {
                                    discountId.append(curDetail.getDiscountcode());
                                }
                            }
                            if (curDetail.getDiscountname() != null && !curDetail.getDiscountname().isEmpty()) {
                                if (discountMsg == null || discountMsg.toString().equals("")) {
                                    discountMsg.append(curDetail.getDiscountname());
                                }
                            }
                        } else if (!curDetail.getLinetype().equals("SI")) {
                            i = j;
                            break;
                        } else {
                            curDetail.setDiscountamount(0.0);
                            curDetail.setAmountafterdiscount(0.0);
                        }
                    }
                    orderDetail.setPrice(price);
                    orderDetail.setAmountafterdiscount(subTotalPrice);
                    totalAmount += subTotalPrice;
                    orderDetail.setDiscountamount(discountAmount);
                    orderDetail.setAmount(amount);
                    orderDetail.setDiscountcode(discountId.toString());
                    orderDetail.setDiscountname(discountMsg.toString());
                }
            }
            if (jsonResult != null) {
                Map<String, Object> resMap = new HashMap<>();
                resMap.put("totalAmount", String.format("%.2f", totalAmount));
                resMap.put("remark", remark);
                jsonResult.setData(resMap);
            }
            return tempDetail;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = true, rollbackFor = Exception.class)
    public JsonResult test(List<OnlineorderdBean> orderDetails, String openid, String remark, String storeCode) throws Exception{
        JsonResult jsonResult = new JsonResult();
        try {
            OnlineUserBean onlineUserBean = onlineuserDao.findEntity(OnlineUserBean.class, openid);
            int daySN = onlineorderhDao.getDaysn(storeCode);
            List<OnlineorderdBean> tempDetail = new ArrayList<>();
            for (OnlineorderdBean detail : orderDetails) {
                if (!detail.getLinetype().equals("SI")) {
                    tempDetail.add(detail);
                }
            }
            StringBuilder serialNumber = new StringBuilder();
            serialNumber.append(DateUtils.format(new Date(), DateUtils.DATE_TIME));
            String daysn = String.format("%04d", daySN);
            serialNumber.append(daysn);
            StringBuilder commodityInfo = new StringBuilder();
            for (OnlineorderdBean detail : tempDetail) {
//                BarcodeBean barcodeBean = barcodeDao.findEntity(BarcodeBean.class, detail.getIntitemno());
//                commodityInfo.append(barcodeBean.getStrbarcode());
                ItemBean item = onlineItemDao.findEntity(ItemBean.class, detail.getIntitemno());
                if (item == null) {
                    logger.info("获取折扣时,商品号" + detail.getIntitemno() + "对应商品在item表中不存在.");
                } else if (StringUtils.isEmpty(item.getStrExtSRef1())) {
                    logger.info("获取折扣时,商品号" + detail.getIntitemno() + "没有barcode");
                }
                if (item != null && !StringUtils.isEmpty(item.getStrExtSRef1())) {
                    commodityInfo.append(item.getStrExtSRef1());
                }
//                commodityInfo.append("4993418033677");
                commodityInfo.append(":");
                commodityInfo.append(detail.getQty());
                commodityInfo.append(":");
//                commodityInfo.append(String.format("%.2f", item.getTotalPrice()));
                commodityInfo.append(String.format("%.2f", detail.getPrice()));
                commodityInfo.append(":");
//                commodityInfo.append(String.format("%.2f",item.getTotalPrice().multiply(new BigDecimal(item.getQuantity()))));
                commodityInfo.append(String.format("%.2f", detail.getAmount()));
                commodityInfo.append(":");
                commodityInfo.append(detail.getLine());
                commodityInfo.append(",");
            }
            String commdity = commodityInfo.substring(0, commodityInfo.length() - 1);
            Long cache = onlineorderhService.getSerialNumber(storeCode, openid);
            if (!redisCacheUtil.containsKey(storeCode + openid) || redisCacheUtil.getCache(storeCode + openid) == null) {
//                Long cache = Long.parseLong(redisCacheUtil.getCache(storeCode).toString());
//                redisCacheUtil.setCache(storeCode, cache + 1);
//                redisCacheUtil.setCache(storeCode + openid, cache.toString(), 28800);
                if (onlineUserBean != null && onlineUserBean.getMemberflg() != null && onlineUserBean.getMemberflg() == 1) {
//                    logger.info("用户openId:" + openid + ",会员卡号:" + onlineUserBean.getMemberCard() + ",首次获得序列号并放入缓存:" + cache);
                } else {
//                    logger.info("用户openId:" + openid + ",会员卡号:  ,首次获得序列号并放入缓存:" + cache);
                }
                Long temp = cache + 1;
//                logger.info("店铺号:" + storeCode + ",当前流水号:" + temp);
            } else {
//                logger.info("用户openId:" + openid + ",已经获得的序列号为:" + redisCacheUtil.getCache(storeCode + openid));
                JSONObject canParam = new JSONObject();
                if (onlineUserBean != null && onlineUserBean.getMemberflg() != null && onlineUserBean.getMemberflg() == 1) {
                    canParam.put("cardCode", onlineUserBean.getMemberCard());
                } else {
                    canParam.put("cardCode", "");
                }
                canParam.put("shopCode", storeCode);
                canParam.put("posNo", storeCode + "Q");
                canParam.put("serialNumber", redisCacheUtil.getCache(storeCode + openid));
//                logger.info("用户openId: " + openid + ",会员卡号: " + canParam.get("cardCode") + ",解绑优惠前序列号: " + canParam.get("serialNumber"));
                canParam.put("accessToken", MD5Util.encode("sendPurchaseCancelToYRccc" + canParam.getString("cardCode")));
                HttpHeaders canheaders = new HttpHeaders();
                MediaType type = MediaType.parseMediaType("application/json; charset=UTF-8");
                canheaders.setContentType(type);
                HttpEntity<JSONObject> canrequest = new HttpEntity<JSONObject>(canParam, canheaders);
//                logger.info("解绑折扣参数：" + canParam);
//                JSONObject bodyAsJson = restTemplate.postForObject(WxConfig.CANCEL_YR, canrequest, JSONObject.class, canParam);
            }
            JSONObject param = new JSONObject();
            if (onlineUserBean != null && onlineUserBean.getMemberflg() != null && onlineUserBean.getMemberflg() == 1) {
                param.put("cardCode", onlineUserBean.getMemberCard());
            } else {
//                param.put("cardCode", "100000001958276964");
                param.put("cardCode", "");
            }
            //有注释参数暂时写死，需要询问
            param.put("shopCode", storeCode);
//            param.put("posNo", "03");
            param.put("posNo", storeCode + "Q");
//            param.put("posNo", "");
//            param.put("serialNumber","2102240001");
            param.put("serialNumber", redisCacheUtil.getCache(storeCode + openid));
            param.put("saledDatetime", DateUtils.formatDateTime(new Date()));
//            logger.info("用户openId: " + openid + ",会员卡号: " + param.get("cardCode") + ",获取优惠前序列号: " + param.get("serialNumber"));
//            param.put("commodityInfo","4993418033677:1:38.00:38.00:1,100981710000:1:79.60:79.60:2");
            param.put("commodityInfo", commdity);
            param.put("accessToken", MD5Util.encode("sendPurchaseInfoToYRccc" + param.getString("cardCode")));
//            logger.info(param.toString());
            HttpHeaders headers = new HttpHeaders();
            MediaType type = MediaType.parseMediaType("application/json; charset=UTF-8");
            headers.setContentType(type);
            HttpEntity<JSONObject> request = new HttpEntity<JSONObject>(param, headers);
//            logger.info("获取折扣参数：" + param);
//            JSONObject bodyAsJson = restTemplate.postForObject(WxConfig.GET_DISCOUNT, request, JSONObject.class, param);
            return jsonResult;
        } catch (Exception e) {
            e.printStackTrace();
            throw e;
        }
    }

}


