package com.bettersoft.api.filter;

import javax.annotation.Resource;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.shiro.web.filter.AccessControlFilter;
import org.springframework.http.HttpStatus;

import com.bettersoft.api.exceptions.ValidateAuthorityFailedException;
import com.bettersoft.api.exceptions.ValidateExpirationFailedException;
import com.bettersoft.api.utils.JwtUtils;
import com.bettersoft.api.utils.RedisCacheUtil;
import com.google.gson.Gson;

public class BettersoftApiFilter extends AccessControlFilter {
	private Log logger = LogFactory.getLog(getClass());

	@Resource(name = "redisCacheUtil")
	private RedisCacheUtil redisCacheUtil;

	protected boolean isAccessAllowed(ServletRequest servletRequest,
			ServletResponse servletResponse, Object mappedValue)
			throws Exception {
		logger.info("is access allowed");

		HttpServletRequest request = (HttpServletRequest) servletRequest;
		HttpServletResponse response = (HttpServletResponse) servletResponse;

		response.setHeader("Access-control-Allow-Origin",
				request.getHeader("Origin"));
		response.setHeader("Access-Control-Allow-Methods", "*");
		response.setHeader("Access-Control-Max-Age", "0");
		response.setHeader("Access-Control-Allow-Credentials", "true");
		response.setHeader("Access-Control-Allow-Headers",
				"x-requested-with,Authorization");
		// 防止乱码，用于传输JSON数据
		response.setHeader("Content-Type", "application/json;charset=UTF-8");

		logger.info("request url: " + request.getRequestURI());
		logger.info("requestParams:"
				+ new Gson().toJson(request.getParameterMap()));
		String ticket = request.getHeader("Authorization");
		if (StringUtils.isEmpty(ticket))
			return false;

		try {
			JwtUtils.verifyToken(ticket, redisCacheUtil);
		} catch (Exception ex) {
			return false;
		}

		return true;
	}

	protected boolean onAccessDenied(ServletRequest servletRequest,
			ServletResponse servletResponse) throws Exception {
		boolean result = true;
		HttpServletRequest request = (HttpServletRequest) servletRequest;
		HttpServletResponse response = (HttpServletResponse) servletResponse;
		response.setHeader("Access-control-Allow-Origin",
				request.getHeader("Origin"));
		response.setHeader("Access-Control-Allow-Methods", "*");
		response.setHeader("Access-Control-Max-Age", "0");
		response.setHeader("Access-Control-Allow-Credentials", "true");
		response.setHeader("Access-Control-Allow-Headers",
				"x-requested-with,Authorization");
		// 防止乱码，用于传输JSON数据
		response.setHeader("Content-Type", "application/json;charset=UTF-8");

		try {
			String ticket = request.getHeader("Authorization");
			if (StringUtils.isEmpty(ticket))
				throw new ValidateAuthorityFailedException();

			JwtUtils.verifyToken(ticket, redisCacheUtil);
		} catch (ValidateAuthorityFailedException ex) {
			result = false;
		} catch (ValidateExpirationFailedException ex) {
			result = false;
			response.getOutputStream().write(
					HttpStatus.UNAUTHORIZED.toString().getBytes());
		} catch (Exception ex) {
			result = false;
		}

		return result;
	}
}
