package com.bettersoft.api.model;

import java.io.Serializable;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.annotations.Cascade;
import org.hibernate.annotations.CascadeType;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;
import org.hibernate.annotations.Filter;
import org.hibernate.annotations.FilterDef;
import org.hibernate.annotations.FilterDefs;
import org.hibernate.annotations.Filters;
import org.hibernate.annotations.NotFound;
import org.hibernate.annotations.NotFoundAction;
import org.hibernate.annotations.ParamDef;

import com.bettersoft.api.annotation.NotCopy;
import com.bettersoft.api.annotation.NotMapped;

@Entity
@DynamicInsert
@DynamicUpdate
@Table(name = "bs_menutree")
@FilterDefs({ @FilterDef(name = "enableMenuFilter", parameters = @ParamDef(name = "enable", type = "boolean")) })
public class MenuTreeBean extends BaseBean implements Serializable {
	private static final long serialVersionUID = 1555113821412422886L;

	@NotCopy
	private String menuCode;
	private String menuName;
	private int priority;
	private boolean enable;
	private String menuUrl;
	private String menuRoute;
	private String menuIcon;
	private String menuType;
	@NotCopy
	private int menuLevel;

	transient private MenuTreeBean parentMenu;
	@NotCopy
	@NotMapped
	private List<MenuTreeBean> childMenus = new ArrayList<MenuTreeBean>();
	@NotCopy
	@NotMapped
	private List<SystemMenuPermissionBean> permissions = new ArrayList<SystemMenuPermissionBean>();

	@com.bettersoft.api.annotation.Cascade(mappedBy = "parentMenu", property = "menuCode")
	private String parentCode;
	@com.bettersoft.api.annotation.Cascade(mappedBy = "parentMenu", property = "menuName")
	private String parentName;

	private boolean retention;

	@Id
	@Column(name = "bs_menucode")
	public String getMenuCode() {
		return menuCode;
	}

	public void setMenuCode(String menuCode) {
		this.menuCode = menuCode;
	}

	@Column(name = "bs_menuname")
	public String getMenuName() {
		return menuName;
	}

	public void setMenuName(String menuName) {
		this.menuName = menuName;
	}

	@Column(name = "bs_priority")
	public int getPriority() {
		return priority;
	}

	public void setPriority(int priority) {
		this.priority = priority;
	}

	@Column(name = "bs_isenable")
	public boolean isEnable() {
		return enable;
	}

	public void setEnable(boolean enable) {
		this.enable = enable;
	}

	@Column(name = "bs_menuurl")
	public String getMenuUrl() {
		return menuUrl;
	}

	public void setMenuUrl(String menuUrl) {
		this.menuUrl = menuUrl;
	}

	@Column(name = "bs_menurouter")
	public String getMenuRoute() {
		return menuRoute;
	}

	public void setMenuRoute(String menuRoute) {
		this.menuRoute = menuRoute;
	}

	@Column(name = "bs_menuicon")
	public String getMenuIcon() {
		return menuIcon;
	}

	public void setMenuIcon(String menuIcon) {
		this.menuIcon = menuIcon;
	}

	@Column(name = "bs_menutype")
	public String getMenuType() {
		return menuType;
	}

	public void setMenuType(String menuType) {
		this.menuType = menuType;
	}

	@Column(name = "bs_menulevel")
	public int getMenuLevel() {
		return menuLevel;
	}

	public void setMenuLevel(int menuLevel) {
		this.menuLevel = menuLevel;
	}

	@Column(name = "bs_createdatetime")
	public Timestamp getCreateDateTime() {
		return createDateTime;
	}

	public void setCreateDateTime(Timestamp createDateTime) {
		this.createDateTime = createDateTime;
	}

	@ManyToOne
	@NotFound(action = NotFoundAction.IGNORE)
	@JoinColumn(name = "bs_parentcode")
	public MenuTreeBean getParentMenu() {
		return parentMenu;
	}

	public void setParentMenu(MenuTreeBean parentMenu) {
		this.parentMenu = parentMenu;
	}

	@OneToMany(mappedBy = "parentMenu", orphanRemoval = true)
	@Cascade(value = { CascadeType.ALL })
	@OrderBy("priority")
	@Filters({ @Filter(name = "enableMenuFilter", condition = ":enable=bs_isenable") })
	public List<MenuTreeBean> getChildMenus() {
		return childMenus;
	}

	public void setChildMenus(List<MenuTreeBean> childMenus) {
		this.childMenus = childMenus;
	}

	@OneToMany(mappedBy = "menu", orphanRemoval = true)
	@Cascade(value = { CascadeType.ALL })
	public List<SystemMenuPermissionBean> getPermissions() {
		return permissions;
	}

	public void setPermissions(List<SystemMenuPermissionBean> permissions) {
		this.permissions = permissions;
	}

	@Transient
	public String getParentCode() {
		return parentCode;
	}

	public void setParentCode(String parentCode) {
		this.parentCode = parentCode;
	}

	@Transient
	public String getParentName() {
		return parentName;
	}

	public void setParentName(String parentName) {
		this.parentName = parentName;
	}

	@Transient
	public boolean isRetention() {
		return retention;
	}

	public void setRetention(boolean retention) {
		this.retention = retention;
	}
}
