package com.bettersoft.api.model;

import java.io.Serializable;
import java.sql.Timestamp;
import java.time.DateTimeException;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import javafx.util.converter.TimeStringConverter;
import org.hibernate.annotations.Cascade;
import org.hibernate.annotations.CascadeType;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;
import org.hibernate.annotations.NotFound;
import org.hibernate.annotations.NotFoundAction;

import com.bettersoft.api.annotation.NotCopy;

@Entity
@DynamicInsert
@DynamicUpdate
@Table(name = "bs_sysuser")
public class SystemUserBean extends BaseBean implements Serializable {
	private static final long serialVersionUID = -923638266249159286L;

	@NotCopy
	private String userCode;
	private String userName;
	private String password;
	private String mobile;
	private String headPhoto;
	private boolean enable;
	private String email;
	private boolean supper;
	private String departmentCode;
	private Integer pwdErrorCount;
	private Timestamp freeLockTime;

	@NotCopy
	transient private SystemDepartmentBean department;
	@NotCopy
	transient private List<SystemUserRoleBean> authRoles = new ArrayList<SystemUserRoleBean>();
	@NotCopy
	transient  private List<SystemUserStoreBean> stores = new ArrayList<SystemUserStoreBean>();
	@com.bettersoft.api.annotation.Cascade(mappedBy = "department", property = "departmentDesc")
	private String departmentName;

	@Id
	@Column(name = "bs_usercode")
	public String getUserCode() {
		return userCode;
	}

	public void setUserCode(String userCode) {
		this.userCode = userCode;
	}

	@Column(name = "bs_username")
	public String getUserName() {
		return userName;
	}

	public void setUserName(String userName) {
		this.userName = userName;
	}

	@Column(name = "bs_password")
	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	@Column(name = "bs_mobile")
	public String getMobile() {
		return mobile;
	}

	public void setMobile(String mobile) {
		this.mobile = mobile;
	}

	@Column(name = "bs_headphoto")
	public String getHeadPhoto() {
		return headPhoto;
	}

	public void setHeadPhoto(String headPhoto) {
		this.headPhoto = headPhoto;
	}

	@Column(name = "bs_isenable")
	public boolean isEnable() {
		return enable;
	}

	public void setEnable(boolean enable) {
		this.enable = enable;
	}

	@Column(name = "bs_email")
	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	@Column(name = "bs_issupper")
	public boolean isSupper() {
		return supper;
	}

	public void setSupper(boolean supper) {
		this.supper = supper;
	}

	@Column(name = "bs_depcode")
	public String getDepartmentCode() {
		return departmentCode;
	}

	public void setDepartmentCode(String departmentCode) {
		this.departmentCode = departmentCode;
	}

	@Column(name = "bs_createby")
	public String getCreateBy() {
		return createBy;
	}

	public void setCreateBy(String createBy) {
		this.createBy = createBy;
	}

	@Column(name = "bs_createdatetime")
	public Timestamp getCreateDateTime() {
		return createDateTime;
	}

	public void setCreateDateTime(Timestamp createDateTime) {
		this.createDateTime = createDateTime;
	}

	@Column(name = "bs_createusername")
	public String getCreateUserName() {
		return createUserName;
	}

	public void setCreateUserName(String createUserName) {
		this.createUserName = createUserName;
	}

	@Column(name = "bs_lastmodiby")
	public String getLastModifyBy() {
		return lastModifyBy;
	}

	public void setLastModifyBy(String lastModifyBy) {
		this.lastModifyBy = lastModifyBy;
	}

	@Column(name = "bs_lastmodidatetime")
	public Timestamp getLastModifyDateTime() {
		return lastModifyDateTime;
	}

	public void setLastModifyDateTime(Timestamp lastModifyDateTime) {
		this.lastModifyDateTime = lastModifyDateTime;
	}

	@Column(name = "bs_lastmodiusername")
	public String getLastModifyUserName() {
		return lastModifyUserName;
	}

	public void setLastModifyUserName(String lastModifyUserName) {
		this.lastModifyUserName = lastModifyUserName;
	}

	@ManyToOne
	@NotFound(action = NotFoundAction.IGNORE)
	@JoinColumn(name = "bs_depcode", insertable = false, updatable = false)
	public SystemDepartmentBean getDepartment() {
		return department;
	}

	public void setDepartment(SystemDepartmentBean department) {
		this.department = department;
	}

	@OneToMany(mappedBy = "systemUser", orphanRemoval = true)
	@Cascade(value = { CascadeType.ALL })
	public List<SystemUserRoleBean> getAuthRoles() {
		return authRoles;
	}

	public void setAuthRoles(List<SystemUserRoleBean> authRoles) {
		this.authRoles = authRoles;
	}

	@OneToMany(mappedBy = "systemUser", orphanRemoval = true)
	@Cascade(value = { CascadeType.ALL })
	public List<SystemUserStoreBean> getStores() { return stores; }

	public void setStores(List<SystemUserStoreBean> stores) { this.stores = stores; }

	@Transient
	public String getDepartmentName() {
		return departmentName;
	}

	public void setDepartmentName(String departmentName) {
		this.departmentName = departmentName;
	}

	@Column(name = "pwd_error_count")
	public Integer getPwdErrorCount() {
		return pwdErrorCount;
	}

	public void setPwdErrorCount(Integer pwdErrorCount) {
		this.pwdErrorCount = pwdErrorCount;
	}

	@Column(name = "free_lock_time")
	public Timestamp getFreeLockTime() {
		return freeLockTime;
	}

	public void setFreeLockTime(Timestamp freeLockTime) {
		this.freeLockTime = freeLockTime;
	}


}
