package com.bettersoft.api.realm;

import javax.annotation.Resource;

import io.jsonwebtoken.Claims;

import org.apache.commons.lang.StringUtils;
import org.apache.shiro.authc.AuthenticationException;
import org.apache.shiro.authc.AuthenticationInfo;
import org.apache.shiro.authc.AuthenticationToken;
import org.apache.shiro.authc.IncorrectCredentialsException;
import org.apache.shiro.authc.SimpleAuthenticationInfo;
import org.apache.shiro.authc.UnknownAccountException;
import org.apache.shiro.authz.AuthorizationInfo;
import org.apache.shiro.authz.SimpleAuthorizationInfo;
import org.apache.shiro.realm.AuthorizingRealm;
import org.apache.shiro.subject.PrincipalCollection;

import com.bettersoft.api.exceptions.ValidateAuthorityFailedException;
import com.bettersoft.api.service.interfaces.ILoginService;
import com.bettersoft.api.utils.EntityUtil;
import com.bettersoft.api.utils.JwtUtils;
import com.bettersoft.api.utils.LoginUserInfo;
import com.bettersoft.api.utils.RedisCacheUtil;
import com.bettersoft.api.validmodel.LoginAccountValidate;

public class BettersoftApiAuthorizingRealm extends AuthorizingRealm {
	@Resource(name = "loginService")
	private ILoginService loginService;

	@Resource(name = "redisCacheUtil")
	private RedisCacheUtil redisCacheUtil;

	/**
	 * 认证(登录时调用)
	 */
	protected AuthenticationInfo doGetAuthenticationInfo(
			AuthenticationToken token) throws AuthenticationException {
		BettersoftUsernamePasswordToken bettersoftToken = ((BettersoftUsernamePasswordToken) token);
		String account = (String) bettersoftToken.getPrincipal();
		String password = StringUtils.EMPTY;

		if (null != token.getCredentials())
			password = new String((char[]) token.getCredentials());

		LoginUserInfo loginUser = null;
		try {
			LoginAccountValidate loginValidate = new LoginAccountValidate();
			loginValidate.setAccount(account);
			loginValidate.setPassword(password);
			loginValidate.setSessionId(bettersoftToken.getSessionId());
			loginValidate.setLoginType(bettersoftToken.getLoginType());
			loginValidate.setPlatform(bettersoftToken.getLoginPlatform());

			loginUser = loginService.loginUser(loginValidate);
		} catch (ClassNotFoundException ex) {
			throw new UnknownAccountException("此账号不存在！");
		} catch (ValidateAuthorityFailedException ex) {
			throw new IncorrectCredentialsException(ex.getMessage());
		} catch (Exception e) {
			e.printStackTrace();
		}

		return new SimpleAuthenticationInfo(loginUser, password, getName());
	}

	/**
	 * 授权(验证权限时调用)
	 */
	protected AuthorizationInfo doGetAuthorizationInfo(
			PrincipalCollection principals) {
		String jwt = (String) principals.getPrimaryPrincipal();
		try {
			Claims claims = JwtUtils.verifyToken(jwt, redisCacheUtil);

			LoginUserInfo loginUser = EntityUtil.convertMapToEntity(
					LoginUserInfo.class, claims);

			SimpleAuthorizationInfo info = new SimpleAuthorizationInfo();
			info.setStringPermissions(loginUser.getAuths());
			// info.setStringPermissions();
			// simpleAuthorizationInfo.addRole("DOCTOR");

			return info;
		} catch (Exception e) {
		}

		return null;
	}

}
