package com.bettersoft.api.service.implement;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.Serializable;
import java.sql.Timestamp;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Pattern;

import javax.annotation.Resource;

import com.bettersoft.api.encrytion.DateUtils;
import com.bettersoft.api.utils.*;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.math.NumberUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.transaction.annotation.Transactional;

import com.bettersoft.api.dao.interfaces.IBaseDao;
import com.bettersoft.api.model.BaseBean;
import com.bettersoft.api.model.SystemConfigurationBean;
import com.bettersoft.api.model.SystemLogOperatorBean;
import com.bettersoft.api.model.SystemLogOperatorDetailBean;
import com.bettersoft.api.service.interfaces.IBaseService;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

@Transactional(readOnly = true)
public abstract class BaseService implements IBaseService {
	protected Log logger = LogFactory.getLog(getClass());

	@Resource(name = "baseDao")
	private IBaseDao baseDao;
	@Resource
	private RedisCacheUtil redisCacheUtil;

	protected Gson gson = new GsonBuilder()
			.setDateFormat("yyyy-MM-dd HH:mm:ss").serializeNulls().create();

	public abstract boolean checkEntityOccupied(Serializable id)
			throws Exception;

	public Map<String, String> findImageUploadPath() throws Exception {
		Map<String, String> pathMap = new HashMap<String, String>();
		SystemConfigurationBean config = baseDao
				.findConfigurationByCode(Constants.CONF_IMG_UPLOADPATH);

		if (null == config)
			throw new ClassNotFoundException();

		String[] paths = StringUtils.split(config.getConfigValue(), ";");

		pathMap.put(Constants.ACCESS_URL_KEY, paths[0]);
		pathMap.put(Constants.TEMP_PATH_KEY, paths[1]);
		pathMap.put(Constants.DEST_PATH_KEY, paths[2]);

		return pathMap;
	}

	public Map<String, String> findResourceUploadPath(String resourceType)
			throws Exception {
		Map<String, String> pathMap = new HashMap<String, String>();
		SystemConfigurationBean config = null;

		if (resourceType.equals(Constants.VIDEO_RES_TYPE)
				|| resourceType.equals(Constants.EDUCATION_RES_TYPE)) {
			config = baseDao
					.findConfigurationByCode(Constants.CONF_VOD_UPLOADPATH);
		} else {
			config = baseDao
					.findConfigurationByCode(Constants.CONF_AOD_UPLOADPATH);
		}

		if (null == config)
			throw new ClassNotFoundException();

		String[] paths = StringUtils.split(config.getConfigValue(), ";");

		pathMap.put(Constants.TEMP_PATH_KEY, paths[0]);
		pathMap.put(Constants.DEST_PATH_KEY, paths[1]);

		return pathMap;
	}

	protected SystemLogOperatorBean generalLogOperator(String operatorType,
			LoginUserInfo loginUser, BaseBean... args) throws Exception {
		BaseBean srcObj = null != args && 0 < args.length ? args[0] : null;
		BaseBean tarObj = null != args && 1 < args.length ? args[1] : null;

		SystemLogOperatorBean syslog = new SystemLogOperatorBean();
		syslog.setOperatorType(operatorType);
		syslog.setOperatorObject(loginUser.getCurrentMenu());
		syslog.setLoginUserCode(loginUser.getUserCode());
		syslog.setLoginUserName(!StringUtils.isEmpty(loginUser.getUserName()) ? loginUser
				.getUserName() : loginUser.getUserCode());
		syslog.setCreateDateTime(new Timestamp(System.currentTimeMillis()));

		if (!operatorType.equals(Constants.LOGIN_OPER_TYPE)
				&& !operatorType.equals(Constants.LOGOUT_OPER_TYPE)) {
			SystemLogOperatorDetailBean logdetail = new SystemLogOperatorDetailBean();
			logdetail.setLogId(syslog.getLogId());

			if (null != srcObj)
				logdetail.setSourceValue(gson.toJson(EntityUtil
						.convertEntityToMap(srcObj)));

			if (null != tarObj)
				logdetail.setTargetValue(gson.toJson(EntityUtil
						.convertEntityToMap(tarObj)));

			logdetail.setLogOper(syslog);

			syslog.setOperatorDetail(logdetail);
		}

		return syslog;
	}

	protected void moveFile2DestPath(String srcPath, String destPath,
			boolean... autoCreate) {
		FileOutputStream outputStream = null;
		FileInputStream inputStream = null;
		File srcFile = null;

		try {
			if (null != autoCreate && 0 < autoCreate.length && autoCreate[0]) {
				String floder = StringUtils.substringBeforeLast(destPath, "/");

				createFloder(floder);
			}

			srcFile = new File(srcPath);

			if (srcFile.exists()) {
				inputStream = new FileInputStream(srcFile);
				outputStream = new FileOutputStream(destPath);
				byte[] buffer = new byte[1024 * 1024];
				int byteread = 0;

				while ((byteread = inputStream.read(buffer)) != -1) {
					outputStream.write(buffer, 0, byteread);
					outputStream.flush();
				}
			}
		} catch (Exception ex) {
			logger.error("moveFile2DestPath error.", ex);
		} finally {
			try {
				if (null != outputStream)
					outputStream.close();

				if (null != inputStream)
					inputStream.close();

				if (null != srcFile && srcFile.exists())
					srcFile.delete();
			} catch (Exception ex) {
			}
		}
	}

	protected void removeFile(String fileFullPath) throws Exception {
		File file = new File(fileFullPath);
		if (null != file && file.exists()) {
			file.delete();
		}
	}

	private void createFloder(String floder) throws IOException {
		File file = new File(floder);
		if (null != file && !file.exists())
			file.mkdirs();
	}

	/**
	 * @param storeCode
	 * @param openId
	 * @return SerialNumber
	 * @author gm
	 * 从redis拿店铺的序列号然后给顾客，序列号加一后放回店铺的缓存中
	 */
	@Override
	public synchronized Long getSerialNumber(String storeCode, String openId) throws Exception {
		logger.info("当前线程进入同步方法时间");
		Object storeSerialNumber = redisCacheUtil.getCache(storeCode);
		if(storeSerialNumber==null||"".equals(storeSerialNumber.toString())|| !NumberUtils.isNumber(storeSerialNumber.toString())) {
			String serialNumber = (DateUtils.format(new Date(), DateUtils.DATE_TIME) + "8001").substring(2);
			logger.error("日期:" + DateUtils.format(new Date(), DateUtils.DATE_PATTERN) + ",店铺:" + storeCode + ",序列号为空或无法转换成数字,重新生成序列号:" + serialNumber);
			redisCacheUtil.setCache(storeCode, serialNumber);
		}
		long cache = Long.parseLong(redisCacheUtil.getCache(storeCode).toString());
		logger.info("店铺:"+storeCode+"更新前当前序列号为:"+ cache);

		String strCache = String.valueOf(cache);
		if(!StringUtils.equals(strCache.substring(0,6),DateUtils.format(new Date(), DateUtils.DATE_TIME).substring(2))){
			logger.info("店铺:"+storeCode+"当前序列号为:"+ cache + ",和当天日期不符合");
			String newCache = (DateUtils.format(new Date(), DateUtils.DATE_TIME) + strCache.substring(6)).substring(2);
			logger.info("店铺:"+storeCode+"重置当前序列号为:"+ newCache);
			cache = Long.parseLong(newCache);
		}

		long temp = cache+1;
		redisCacheUtil.setCache(storeCode, Long.toString(temp));
		redisCacheUtil.setCache(storeCode + openId, Long.toString(cache), 28800);

		logger.info("用户:"+openId+",更新店铺序列号,店铺:"+storeCode+"当前序列号为:"+temp);
		logger.info("用户:"+openId+",当前序列号："+cache);
		logger.info("当前线程退出同步方法时间");
		return cache;
	}

	@Override
	public Boolean checkPwdValid(String pwd) throws Exception {
		String pattern = "^(?![0-9A-Za-z]+$)(?![0-9A-Z\\W]+$)(?![0-9a-z\\W]+$)(?![A-Za-z\\W]+$)[0-9A-Za-z~!@#$%^&*()_+={}<>?,]{8,}$";
		return Pattern.matches(pattern, pwd);
	}
}
