package com.bettersoft.api.service.implement;

import java.io.Serializable;
import java.util.*;

import javax.annotation.Resource;

import com.bettersoft.api.encrytion.DateUtils;
import com.bettersoft.api.utils.Constants;
import com.bettersoft.api.utils.LoginUserInfo;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;

import com.bettersoft.api.dao.interfaces.IDashboardDao;
import com.bettersoft.api.service.interfaces.IDashboardService;

@Service("dashboardService")
public class DashboardService extends BaseService implements IDashboardService {
    @Resource(name = "dashboardDao")
    private IDashboardDao dashboardDao;

    public Map<String, Object> initParam(LoginUserInfo loginUser) throws Exception {
        String[] authStores;
        Map<String, Object> resultParam = new HashMap<String, Object>();
        Map<String, Object> queryParam = new HashMap<String, Object>();
        if (!StringUtils.isEmpty(loginUser.getAuthStores()) && !"ALL".equalsIgnoreCase(loginUser.getAuthStores())) {
            authStores = loginUser.getAuthStores().split(",");
            queryParam.put(Constants.QUERY_SPEC_CONDITION + "storeCode", "s.strStoreCode in ('" + StringUtils.join(authStores, "','") + "')");
        }
        queryParam.put(Constants.QUERY_NOTEQUAL + "s.strStoreCode", "ALL");
        queryParam.put(Constants.QUERY_ORDER_BY, "order by s.strStoreCode");
        List<Map<String,Object>> stores = new ArrayList<>();
        if(loginUser.isSupperUser()){
            Map<String,Object> temp = new HashMap<>();
            temp.put("storeName","全部门店");
            temp.put("storeCode","");
            stores.add(temp);
        }
        stores.addAll(dashboardDao.searchStores(queryParam));
        resultParam.put("stores", stores);

        return resultParam;
    }

    public Map<String, Object> reportData(String storeCode, String date) throws Exception {
        Map<String, Object> resultMap = new HashMap<String, Object>();
        Date transferDate=new Date();
        if(date!=null){
            transferDate = DateUtils.getDate(date);
            if(transferDate==null){
                logger.error("===========日期转换错误---DashboardDao.java:41---展示当天数据===========");
                transferDate = new Date();
            }
        }
        resultMap.putAll(dashboardDao.dailyOnlineOrderQuantity(storeCode,transferDate));
        resultMap.putAll(dashboardDao.dailyOnlinePaymentAmount(storeCode,transferDate));
        Map<String, List<Object>> weeklyData = dashboardDao.weeklyOnlineOrders(storeCode, transferDate);
        resultMap.put("weeklyOrders", weeklyData.get("orders"));
        resultMap.put("weeklyOrdersDate",weeklyData.get("dates"));
        resultMap.put("weeklyOrderAmount", weeklyData.get("amounts"));
        //未结账，未接单，支付方式，三个未改动
        resultMap.put("dailyOrderNoProcQty", dashboardDao.dailyUnProcessOrderQuantity(storeCode, transferDate));
        resultMap.put("dailyOrderNotPayQty", dashboardDao.dailyNotPaymentOrderQuantity(storeCode, transferDate));
        resultMap.put("paymentPersent", dashboardDao.searchPaymentPersent(storeCode));

        return resultMap;
    }

    public boolean checkEntityOccupied(Serializable id) throws Exception {
        return false;
    }
}
