package com.bettersoft.api.service.implement;

import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.context.ApplicationListener;
import org.springframework.context.event.ContextRefreshedEvent;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.bettersoft.api.dao.interfaces.IBaseDao;
import com.bettersoft.api.service.interfaces.IInitializingDataService;
import com.bettersoft.api.utils.CacheUtil;
import com.bettersoft.api.utils.Constants;
import com.bettersoft.api.utils.RedisCacheUtil;

@Service("initializingDataService")
@Transactional(readOnly = true)
public class InitializingDataService implements IInitializingDataService,
		ApplicationListener<ContextRefreshedEvent> {
	private Log logger = LogFactory.getLog(getClass());

	@Resource(name = "baseDao")
	private IBaseDao baseDao;
	@Resource(name = "redisCacheUtil")
	private RedisCacheUtil redisCacheUtil;

	@Override
	@Async(value = "executor")
	public void onApplicationEvent(ContextRefreshedEvent event) {
		try {
			if (null == event.getApplicationContext().getParent()) {
				if (!redisCacheUtil.containsKey(Constants.PERMISSION_KEY))
					baseDao.searchPermissions();

				if (!redisCacheUtil.containsKey(Constants.SYSCONFIG_KEY))
					baseDao.searchConfigurations();

				if (!redisCacheUtil.containsKey(Constants.PROVINCIAL_KEY))
					baseDao.searchProvincials();

				redisCacheUtil.setCache(Constants.TASK_THREAD_KEY, 0);
			}
		} catch (Exception ex) {
			logger.error("InitializingDataService error.", ex);
		}
	}

	@SuppressWarnings("unchecked")
	public List<Map<String, Object>> getPermissions() throws Exception {
		List<Map<String, Object>> perms = CacheUtil
				.get(Constants.PERMISSION_KEY);
		if (null == perms || perms.isEmpty())
			CacheUtil
					.put(Constants.PERMISSION_KEY, baseDao.searchPermissions());

		return (List<Map<String, Object>>) CacheUtil
				.get(Constants.PERMISSION_KEY);
	}

	public Map<String, Object> getPermissionByKey(String key) throws Exception {
		List<Map<String, Object>> perms = getPermissions();
		if (null != perms && !perms.isEmpty()) {
			for (Map<String, Object> perm : perms) {
				String permissionCode = MapUtils.getString(perm,
						"permissionCode", StringUtils.EMPTY);
				if (permissionCode.equals(key))
					return perm;
			}
		}

		return null;
	}

	@SuppressWarnings("unchecked")
	public List<Map<String, Object>> getConfigurations() throws Exception {
		List<Map<String, Object>> configs = CacheUtil
				.get(Constants.SYSCONFIG_KEY);
		if (null == configs || configs.isEmpty())
			CacheUtil.put(Constants.SYSCONFIG_KEY, baseDao.searchPermissions());

		return (List<Map<String, Object>>) CacheUtil
				.get(Constants.SYSCONFIG_KEY);
	}

	public Map<String, Object> getConfigurationByKey(String key)
			throws Exception {
		List<Map<String, Object>> configs = getConfigurations();
		if (null != configs && !configs.isEmpty()) {
			for (Map<String, Object> config : configs) {
				String configCode = MapUtils.getString(config, "configCode",
						StringUtils.EMPTY);
				if (configCode.equals(key))
					return config;
			}
		}

		return null;
	}

	@Async(value = "executor")
	public void refreshProvincials() throws Exception {
		redisCacheUtil.removeCacheByKey(Constants.PROVINCIAL_KEY);
		redisCacheUtil.setCacheList(Constants.PROVINCIAL_KEY,
				baseDao.searchProvincials());
	}
}
