package com.bettersoft.api.service.implement;

import java.io.Serializable;
import java.sql.Timestamp;
import java.util.*;

import javax.annotation.Resource;

import com.bettersoft.api.model.*;
import com.bettersoft.api.utils.StringDigestUtil;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;

import com.bettersoft.api.dao.interfaces.IAuthorizationDao;
import com.bettersoft.api.exceptions.ValidateAuthorityFailedException;
import com.bettersoft.api.service.interfaces.ILoginService;
import com.bettersoft.api.utils.Constants;
import com.bettersoft.api.utils.JwtUtils;
import com.bettersoft.api.utils.LoginUserInfo;
import com.bettersoft.api.validmodel.LoginAccountValidate;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

@Service("loginService")
public class LoginService extends BaseService implements ILoginService {
    @Resource(name = "authorizationDao")
    private IAuthorizationDao authorizationDao;

    public LoginUserInfo loginUser(LoginAccountValidate loginAccount)
            throws Exception {
        LoginUserInfo loginUser = new LoginUserInfo();

        if (loginAccount.getLoginType().equals(Constants.PCPWD_LOGIN_TYPE)) {
            SystemUserBean chkUser = authorizationDao.findEntity(
                    SystemUserBean.class, loginAccount.getAccount());

            if (null == chkUser)
                throw new ValidateAuthorityFailedException("不存在此账号！");

            if (!chkUser.isEnable())
                throw new ValidateAuthorityFailedException("不存在此账号！");

            if (!chkUser.getPassword().equals(loginAccount.getPassword()))
                throw new ValidateAuthorityFailedException("用户账号或密码错误！");

            loginUser.setUserCode(chkUser.getUserCode());
            loginUser.setUserName(chkUser.getUserName());
            loginUser.setPhotoUrl(chkUser.getHeadPhoto());
            loginUser.setAuths(parseAuthPermission(chkUser));
            loginUser.setAuthMenus(parseAuthMenus(chkUser));
            if (!chkUser.isSupper()) {
                String authStores = authStores(chkUser.getUserCode());
                if (StringUtils.isEmpty(authStores))
                    throw new ValidateAuthorityFailedException("无任何操作权限！");
                loginUser.setAuthStores(authStores);
            } else {
                loginUser.setAuthStores("ALL");
            }

            loginUser.setSupperUser(chkUser.isSupper());
        }

        loginUser.setSessionId(loginAccount.getSessionId());
        loginUser.setPlatform(loginAccount.getPlatform());
        loginUser.setToken(StringDigestUtil.generalUUID());
        loginUser.setExpireTime(new Timestamp(System.currentTimeMillis() + Constants.LOGIN_EXPIRE_TIME));
        loginUser.setRefreshToken(StringDigestUtil.generalUUID());
        loginUser.setTicket(JwtUtils.createToken(loginUser, Constants.LOGIN_EXPIRE_TIME));

        return loginUser;
    }

    public boolean checkEntityOccupied(Serializable id) throws Exception {
        return false;
    }

    private Set<String> parseAuthPermission(SystemUserBean chkUser)
            throws Exception {
        Set<String> auths = new HashSet<String>();
        if (chkUser.isSupper()) {
            Map<String, Object> queryParam = new HashMap<String, Object>();
            queryParam.put("menuType", Constants.MENU_TYPE);
            queryParam.put("enable", true);

            List<MenuTreeBean> menus = authorizationDao
                    .searchMenuTrees(queryParam);

            if (null != menus && !menus.isEmpty()) {
                for (MenuTreeBean menu : menus) {
                    String menuCode = menu.getMenuCode();
                    if (null != menu.getPermissions()
                            && !menu.getPermissions().isEmpty()) {
                        for (SystemMenuPermissionBean menuPerm : menu
                                .getPermissions())
                            auths.add(menuCode + ":"
                                    + menuPerm.getPermissionCode());
                    }
                }
            }
        } else {
            if (null != chkUser.getAuthRoles()
                    && !chkUser.getAuthRoles().isEmpty()) {
                for (SystemUserRoleBean authRole : chkUser.getAuthRoles()) {
                    if (null != authRole.getSystemRole().getAuthMenus()
                            && !authRole.getSystemRole().getAuthMenus()
                            .isEmpty()) {
                        for (SystemRoleMenuBean roleMenu : authRole
                                .getSystemRole().getAuthMenus()) {
                            auths.add(roleMenu.getAuthCode());
                        }
                    }
                }
            }
        }

        return auths;
    }

    private String parseAuthMenus(SystemUserBean chkUser) throws Exception {
        Set<String> authMenus = new HashSet<String>();
        if (chkUser.isSupper()) {
            Map<String, Object> queryParam = new HashMap<String, Object>();
            queryParam.put("menuType", Constants.MENU_TYPE);
            queryParam.put("enable", true);

            List<MenuTreeBean> menus = authorizationDao
                    .searchMenuTrees(queryParam);

            if (null != menus && !menus.isEmpty()) {
                for (MenuTreeBean menu : menus) {
                    authMenus.add(menu.getMenuCode());
                }
            }
        } else {
            if (null != chkUser.getAuthRoles()
                    && !chkUser.getAuthRoles().isEmpty()) {
                for (SystemUserRoleBean authRole : chkUser.getAuthRoles()) {
                    if (null != authRole.getSystemRole()
                            && null != authRole.getSystemRole().getAuthMenus()
                            && !authRole.getSystemRole().getAuthMenus()
                            .isEmpty()) {
                        for (SystemRoleMenuBean roleMenu : authRole
                                .getSystemRole().getAuthMenus()) {
                            authMenus.add(roleMenu.getMenuCode());
                        }
                    }
                }
            }
        }

        return StringUtils.join(authMenus, ",");
    }

//    private String parseAuthData(List<SystemUserRoleBean> authRoles)
//            throws Exception {
//        String authData = StringUtils.EMPTY;
//        if (null != authRoles && !authRoles.isEmpty()) {
//            List<String> authDatas = new ArrayList<String>();
//            for (SystemUserRoleBean authRole : authRoles) {
//                if (null != authRole.getSystemRole().getAuthDatas()
//                        && !authRole.getSystemRole().getAuthDatas().isEmpty()) {
//                    for (SystemRoleDepartmentBean permData : authRole
//                            .getSystemRole().getAuthDatas()) {
//                        authDatas.add(permData.getDepartmentCode());
//                    }
//                }
//            }
//
//            authData = StringUtils.join(authDatas, ",");
//        }
//
//        return authData;
//    }

    private String authStores(String userCode) throws Exception {
        List<String> authStores = new ArrayList<String>();
        List<SystemUserStoreBean> userStores = authorizationDao.searchAuthStores(userCode);
        if (null != userStores && !userStores.isEmpty()) {
            for (SystemUserStoreBean userStore : userStores) {
                authStores.add(userStore.getStoreCode());
            }
        }

        return StringUtils.join(authStores.toArray(), ",");
    }
}
