package com.bettersoft.api.service.implement;

import java.io.Serializable;
import java.math.BigDecimal;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.mail.FetchProfile;
import javax.mail.Message;

import com.bettersoft.api.model.ItemBean;
import com.bettersoft.api.model.OnlineorderdBean;
import com.bettersoft.api.utils.*;
import com.bettersoft.api.validmodel.OnlineItemPackageValidate;
import com.bettersoft.api.validmodel.ShoppingCartValidate;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.bettersoft.api.dao.interfaces.IOnlineItemDao;
import com.bettersoft.api.model.OnlineItemBean;
import com.bettersoft.api.service.interfaces.IOnlineItemService;
import com.bettersoft.api.validmodel.OnlineItemValidate;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

@Repository
@SuppressWarnings({"unchecked", "rawtypes"})
public class OnlineItemService extends BaseService implements IOnlineItemService {
    @Resource(name = "onlineItemDao")
    private IOnlineItemDao onlineItemDao;
    @Resource
    private ConfigParamHelper configParam;

    @Resource(name = "redisCacheUtil")
    private RedisCacheUtil redisCacheUtil;

    protected Gson gson = new GsonBuilder().setDateFormat("yyyy-MM-dd HH:mm:ss").serializeNulls().create();

    public Map<String, Object> searchItem(Map<String, Object> queryParam) throws Exception {
        if (queryParam.containsKey("i.intitemno")) {
            queryParam.put("i.intitemno", Integer.parseInt(MapUtils.getString(queryParam, "i.intitemno", "0")));
        }
        List<Map<String, Object>> items = onlineItemDao.searchList(queryParam);
        if (null != items && !items.isEmpty()) {
            for (Map<String, Object> item : items) {
                String imgPath = MapUtils.getString(item, "imgpath", StringUtils.EMPTY);
                if (StringUtils.isEmpty(imgPath)) {
                    item.put("imgpath", configParam.getDefaultImg());
                } else {
                    String storeCode = MapUtils.getString(item, "strstorecode", StringUtils.EMPTY);
                    item.put("imgpath", MessageFormat.format(configParam.getAccessImgUrl(), storeCode, imgPath));
                }
                item.put("imageData", "");
                item.put("showImgPreview", false);
            }
        }
        Number count = onlineItemDao.searchCount(queryParam);

        return new PageInfoUtils(queryParam, items, count).getPageInfoMap();
    }

    @Override
    public Map<String, Object> searchItemQr(Map<String, Object> queryParam) throws Exception {
        String cartKey = MessageFormat.format(Constants.BUYCART_REDIS, queryParam.get("strstorecode"), queryParam.get("tableCode"));
        ShoppingCartValidate shoppingCart = null;
        if (redisCacheUtil.containsKey(cartKey)&&redisCacheUtil.getCache(cartKey)!=null) {
            shoppingCart = redisCacheUtil.getCache(cartKey);
        } else {
            shoppingCart = new ShoppingCartValidate(queryParam.get("strstorecode").toString(), queryParam.get("tableCode").toString());
        }
        List<OnlineItemPackageValidate> cartItems = shoppingCart.getCartItems();

        if (queryParam.containsKey("i.intitemno")) {
            queryParam.put("i.intitemno", Integer.parseInt(MapUtils.getString(queryParam, "i.intitemno", "0")));
        }
        queryParam.remove("tableCode");
        List<Map<String, Object>> items = onlineItemDao.searchList(queryParam);
        if (null != items && !items.isEmpty()) {
            for (Map<String, Object> item : items) {
                String imgPath = MapUtils.getString(item, "imgpath", StringUtils.EMPTY);
                if (StringUtils.isEmpty(imgPath)) {
                    item.put("imgpath", configParam.getDefaultImg());
                } else {
                    String storeCode = MapUtils.getString(item, "strstorecode", StringUtils.EMPTY);
                    item.put("imgpath", MessageFormat.format(configParam.getAccessImgUrl(), storeCode, imgPath));
                }
                item.put("imageData", "");
                item.put("showImgPreview", false);
                item.put("quantity","");
                int quantity = 0;
                for(OnlineItemPackageValidate cartItem:cartItems){
                    if(cartItem.getItemId()==Integer.parseInt(item.get("intitemno").toString())){
                        quantity+=cartItem.getQuantity();
                    }
                }
                if(quantity!=0){
                    item.put("quantity",quantity);
                }
            }
        }
        Number count = onlineItemDao.searchCount(queryParam);

        return new PageInfoUtils(queryParam, items, count).getPageInfoMap();
    }

    public Map<String, Object> getAllStrposfastkeyname(String strstorecode) throws Exception {
        Map<String, Object> result = new HashMap<String, Object>();
        result.put("allStrposfastkeyname", onlineItemDao.getAllStrposfastkeyname(strstorecode));
        return result;
    }

    public OnlineItemPackageValidate findOnlineItem(String storeCode, int itemId) throws Exception {
        OnlineItemBean onlineItem = onlineItemDao.findOnlinItem(storeCode, itemId);
        if (null == onlineItem)
            throw new ClassNotFoundException();
        ItemBean item = onlineItemDao.findEntity(ItemBean.class, itemId);
        if (null == item)
            throw new ClassNotFoundException();

        OnlineItemPackageValidate packageValidate = new OnlineItemPackageValidate();
        packageValidate.setPackageCode(itemId + "");
        packageValidate.setParentPackage(itemId + "");
        packageValidate.setPackageDesc(item.getStrItemNameShort());
        packageValidate.setPrice(new BigDecimal(onlineItem.getPrice()));
        packageValidate.setItemId(itemId);
        packageValidate.setImagePath(MessageFormat.format(configParam.getDefaultImg(), storeCode, itemId));
        if (!StringUtils.isEmpty(onlineItem.getImgpath())) {
            packageValidate.setImagePath(MessageFormat.format(configParam.getAccessImgUrl(), storeCode, onlineItem.getImgpath()));
        }

        // 判定是否为套餐
        if (onlineItemDao.isPackageItem(onlineItem.getIntitemno())) {
            packageValidate.setPackageType("P");
            List<OnlineItemPackageValidate> packageGroups = onlineItemDao.searchPackageGroup(onlineItem.getIntitemno());
            if (null != packageGroups && !packageGroups.isEmpty()) {
                for (OnlineItemPackageValidate packageGroup : packageGroups) {
                    List<OnlineItemPackageValidate> packageDetails = onlineItemDao.searchPackageDetail(packageGroup.getPackageCode(), packageGroup.getItemId());
                    if (null != packageDetails && !packageDetails.isEmpty()) {
                        for (OnlineItemPackageValidate detail : packageDetails) {
                            detail.setChildPackages(onlineItemDao.searchItemModifiers(storeCode, detail.getItemId()));
                        }
                    }
                    packageGroup.setChildPackages(packageDetails);
                }
            }

            packageValidate.setChildPackages(packageGroups);
        } else {
            packageValidate.setPackageType("I");
            packageValidate.setChildPackages(onlineItemDao.searchItemModifiers(storeCode, itemId));
        }

        return packageValidate;
    }

    public OnlineItemPackageValidate findOfflineItem(String storeCode, String orderNo, Map<String, Object> offlineOrder) throws Exception {
        OnlineItemPackageValidate packageValidate = new OnlineItemPackageValidate();
        OnlineorderdBean orderDetail = (OnlineorderdBean) MapUtils.getObject(offlineOrder, "packageHeader", null);
        List<OnlineorderdBean> orderChildren = (List<OnlineorderdBean>) MapUtils.getObject(offlineOrder, "packageChildren", null);

        if (null != orderDetail) {
            OnlineItemBean onlineItem = onlineItemDao.findOnlinItem(storeCode, orderDetail.getIntitemno());
            ItemBean item = onlineItemDao.findEntity(ItemBean.class, onlineItem.getIntitemno());
            if (null == onlineItem || null == item)
                return null;

            packageValidate.setOrderNo(orderNo);
            packageValidate.setOffline(true);
            packageValidate.setPackageCode(onlineItem.getIntitemno() + "");
            packageValidate.setParentPackage(onlineItem.getIntitemno() + "");
            packageValidate.setPackageDesc(item.getStrItemNameShort());
            packageValidate.setPrice(new BigDecimal(onlineItem.getPrice()));
            packageValidate.setItemId(onlineItem.getIntitemno());
            packageValidate.setImagePath(MessageFormat.format(configParam.getDefaultImg(), storeCode, onlineItem.getIntitemno()));
            if (!StringUtils.isEmpty(onlineItem.getImgpath())) {
                packageValidate.setImagePath(MessageFormat.format(configParam.getAccessImgUrl(), storeCode, onlineItem.getImgpath()));
            }

            if (onlineItemDao.isPackageItem(onlineItem.getIntitemno())) {
                packageValidate.setPackageType("P");
                List<OnlineItemPackageValidate> packageGroups = onlineItemDao.searchPackageGroup(onlineItem.getIntitemno());
                if (null != packageGroups && !packageGroups.isEmpty()) {
                    for (OnlineItemPackageValidate packageGroup : packageGroups) {
                        packageGroup.setOrderNo(orderNo);
                        packageGroup.setOffline(true);
                        List<OnlineItemPackageValidate> packageDetails = onlineItemDao.searchPackageDetail(packageGroup.getPackageCode(), packageGroup.getItemId());
                        if (null != packageDetails && !packageDetails.isEmpty()) {
                            for (OnlineItemPackageValidate detail : packageDetails) {
                                for (OnlineorderdBean orderChild : orderChildren) {
                                    if (orderChild.getIntitemno() == detail.getItemId()) {
                                        List<OnlineItemPackageValidate> modifies = onlineItemDao.searchItemModifiers(storeCode, orderNo, detail.getItemId(), orderChildren);
                                        detail.setChildPackages(modifies);
                                        detail.setOrderNo(orderNo);
                                        detail.setOffline(true);
                                        packageGroup.getChildPackages().add(detail);
                                        break;
                                    }
                                }
                            }
                        }
                    }

                    packageValidate.setChildPackages(packageGroups);
                }
            } else {
                packageValidate.setPackageType("I");
                List<OnlineItemPackageValidate> modifies = onlineItemDao.searchItemModifiers(storeCode, orderNo, orderDetail.getIntitemno(), orderChildren);
                packageValidate.setChildPackages(modifies);
                // packageValidate.setChildPackages(onlineItemDao.searchItemModifiers(storeCode, onlineItem.getIntitemno()));
            }
        }

        return packageValidate;
    }

    public OnlineItemBean getSingleOnlineItem(String storeCode, int itemId) {
        String hql = "from OnlineItemBean";
        Map<String, Object> queryParam = new HashMap<String, Object>();
        queryParam.put("strstroecode", storeCode);
        queryParam.put("intitemno", itemId);
        List<OnlineItemBean> list = onlineItemDao.generalSearch(hql, queryParam, null);
        if (null != list && list.size() > 0) {
            OnlineItemBean onlineItemBean = list.get(0);
            return onlineItemBean;
        } else
            return null;
    }

    public OnlineItemBean findOnlineItem(Long l) throws Exception {
        OnlineItemBean onlineItemBean = onlineItemDao.findEntity(OnlineItemBean.class, l);
        return onlineItemBean;
    }

    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
    public void saveOnlineItem(OnlineItemValidate onlineItemValidate, LoginUserInfo loginUser) throws Exception {
        OnlineItemBean onlineItemBean = new OnlineItemBean();
        EntityUtil.copyEntityProperty(onlineItemValidate, onlineItemBean);
        onlineItemBean.setUpdateuser(loginUser.getUserCode());
        onlineItemBean.setUpdateDate(new Date());
        onlineItemBean.setCreatuser(loginUser.getUserName());
        onlineItemBean.setCreatDate(new Date());
        onlineItemDao.save(generalLogOperator(Constants.INSERT_OPER_TYPE, loginUser, onlineItemBean));
        onlineItemDao.save(onlineItemBean);
    }

    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
    public void updateOnlineItem(OnlineItemValidate onlineItemValidate, LoginUserInfo loginUser) throws Exception {
		/*
		String priKey = onlineItemValidate.getPriKey();
		OnlineItemBean edit = onlineItemDao.findEntity(OnlineItemBean.class, priKey);
		
		EntityUtil.compareCopyEntity(onlineItemValidate, edit);
		if (null == edit)
			throw new ClassNotFoundException();

		onlineItemBean.setUpdateuser(loginUser.getUserCode());
		onlineItemBean.setUpdateDate(new Date());
		onlineItemDao.save(generalLogOperator(Constants.UPDATE_OPER_TYPE, loginUser, edit));
		onlineItemDao.update(edit);
		*/
    }

    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
    public void deleteOnlineItem(String str, LoginUserInfo loginUser) throws Exception {
        OnlineItemBean del = onlineItemDao.findEntity(OnlineItemBean.class, str);
        if (null == del)
            throw new ClassNotFoundException();
        onlineItemDao.save(generalLogOperator(Constants.DELETE_OPER_TYPE, loginUser, del));
        onlineItemDao.delete(del);
    }

    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
    public void deleteOnlineItem(Long l, LoginUserInfo loginUser) throws Exception {
        OnlineItemBean del = onlineItemDao.findEntity(OnlineItemBean.class, l);
        if (null == del)
            throw new ClassNotFoundException();
        onlineItemDao.save(generalLogOperator(Constants.DELETE_OPER_TYPE, loginUser, del));
        onlineItemDao.delete(del);
    }

    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
    public void updateOnlineItemImage(String storeCode, int itemId, String imageUrl, LoginUserInfo loginUser) throws Exception {
        OnlineItemBean item = onlineItemDao.findOnlinItem(storeCode, itemId);
        if (null == item)
            throw new ClassNotFoundException();

        item.setImgpath(imageUrl);
        onlineItemDao.save(generalLogOperator(Constants.UPDATE_OPER_TYPE, loginUser, item));
        onlineItemDao.update(item);
    }

    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
    public void updateOnlineItemStatus(OnlineItemValidate onlineItemValidate, int status, LoginUserInfo loginUser) throws Exception {
        if (null != onlineItemValidate.getItemStatuses() && !onlineItemValidate.getItemStatuses().isEmpty()) {
            for (Map<String, Object> statusMap : onlineItemValidate.getItemStatuses()) {
                String storeCode = MapUtils.getString(statusMap, "storeCode", StringUtils.EMPTY);
                int itemId = MapUtils.getIntValue(statusMap, "itemId", 0);
                updateOnlineItemStatus(storeCode, itemId, status, loginUser);
            }
        }
    }

    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
    public void updateOnlineItemStatus(String storeCode, int itemId, int status, LoginUserInfo loginUser) throws Exception {
        OnlineItemBean item = onlineItemDao.findOnlinItem(storeCode, itemId);
        if (null == item)
            throw new ClassNotFoundException();

        item.setStatus(status);
        onlineItemDao.save(generalLogOperator(Constants.UPDATE_OPER_TYPE, loginUser, item));
        onlineItemDao.update(item);
    }

    public boolean checkEntityOccupied(Serializable id) throws Exception {
        OnlineItemBean existsDept = onlineItemDao.findEntity(OnlineItemBean.class, id);

        return null != existsDept;
    }

    @Override
    public void checkBuyCartStatus(ShoppingCartValidate shoppingCart, JsonResult jsonResult) throws Exception{
        String storeCode = shoppingCart.getStoreCode();
        List<OnlineItemPackageValidate> removeItem = new ArrayList<>();
        for(OnlineItemPackageValidate item:shoppingCart.getCartItems()){
            OnlineItemBean onlineItem = onlineItemDao.findOnlinItem(storeCode, item.getItemId());
            if(onlineItem.getStatus()==0){
//                shoppingCart.removeCartItem(onlineItem.getIntitemno());
                removeItem.add(item);
                jsonResult.setErrorCode("100");
            }
        }
        removeItem.forEach(shoppingCart::removeCartItem);
        jsonResult.setWxData(removeItem);
    }
}
