package com.bettersoft.api.service.implement;

import java.io.Serializable;
import java.util.Date;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import com.alibaba.fastjson.JSONObject;
import com.bettersoft.api.dao.interfaces.IOnlineorderhDao;
import com.bettersoft.api.utils.*;
import org.apache.commons.lang.StringUtils;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.bettersoft.api.dao.interfaces.IOnlineorderpDao;
import com.bettersoft.api.model.OnlineorderhBean;
import com.bettersoft.api.model.OnlineorderpBean;
import com.bettersoft.api.service.interfaces.IOnlineorderpService;
import com.bettersoft.api.validmodel.OnlineorderpValidate;

@Repository
@SuppressWarnings({ "unchecked", "rawtypes" })
public class OnlineorderpService extends BaseService implements IOnlineorderpService
{
	@Resource(name = "onlineorderpDao")
	private IOnlineorderpDao onlineorderpDao;
	@Autowired
	private IOnlineorderhDao onlineorderhDao;

	protected Gson gson = new GsonBuilder().setDateFormat("yyyy-MM-dd HH:mm:ss").serializeNulls().create();

	public Map<String, Object> searchByPage(Map<String, Object> queryParam) throws Exception
	{
		List<OnlineorderpBean> list = onlineorderpDao.searchList(queryParam);
		Number count = onlineorderpDao.searchCount(queryParam);
		return new PageInfoUtils(queryParam, list, count).getPageInfoMap();
	}

	public OnlineorderpBean findOnlineorderp(String str) throws Exception
	{
		OnlineorderpBean onlineorderpBean = onlineorderpDao.findEntity(OnlineorderpBean.class, str);
		return onlineorderpBean;
	}
	
	public OnlineorderpBean findOnlineorderp(Long l) throws Exception
	{
		OnlineorderpBean onlineorderpBean = onlineorderpDao.findEntity(OnlineorderpBean.class, l);
		return onlineorderpBean;
	}

	@Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
	public void saveOnlineorderp(OnlineorderpValidate onlineorderpValidate, LoginUserInfo loginUser) throws Exception
	{
		OnlineorderpBean onlineorderpBean = new OnlineorderpBean();
		EntityUtil.copyEntityProperty(onlineorderpValidate, onlineorderpBean);
		/*onlineorderpBean.setUpdateuser(loginUser.getUserCode());
		onlineorderpBean.setUpdateDate(new Date());*/
		onlineorderpBean.setCreatuser(loginUser.getUserName());
		onlineorderpBean.setCreatDate(new Date());
		onlineorderpDao.save(generalLogOperator(Constants.INSERT_OPER_TYPE, loginUser, onlineorderpBean));
		onlineorderpDao.save(onlineorderpBean);
	}
	
	@Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
	public void updateOnlineorderp(OnlineorderpValidate onlineorderpValidate, LoginUserInfo loginUser) throws Exception
	{
		/*
		String priKey = onlineorderpValidate.getPriKey();
		OnlineorderpBean edit = onlineorderpDao.findEntity(OnlineorderpBean.class, priKey);
		
		EntityUtil.compareCopyEntity(onlineorderpValidate, edit);
		if (null == edit)
			throw new ClassNotFoundException();

		onlineorderpBean.setUpdateuser(loginUser.getUserCode());
		onlineorderpBean.setUpdateDate(new Date());
		onlineorderpDao.save(generalLogOperator(Constants.UPDATE_OPER_TYPE, loginUser, edit));
		onlineorderpDao.update(edit);
		*/
	}

	@Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
	public void deleteOnlineorderp(String str, LoginUserInfo loginUser) throws Exception
	{
		OnlineorderpBean del = onlineorderpDao.findEntity(OnlineorderpBean.class, str);
		if (null == del)
			throw new ClassNotFoundException();
		onlineorderpDao.save(generalLogOperator(Constants.DELETE_OPER_TYPE, loginUser, del));
		onlineorderpDao.delete(del);
	}
	
	@Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
	public void deleteOnlineorderp(Long l, LoginUserInfo loginUser) throws Exception
	{
		OnlineorderpBean del = onlineorderpDao.findEntity(OnlineorderpBean.class, l);
		if (null == del)
			throw new ClassNotFoundException();
		onlineorderpDao.save(generalLogOperator(Constants.DELETE_OPER_TYPE, loginUser, del));
		onlineorderpDao.delete(del);
	}
	@Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
	public void updateOnlineorderpRemark(String orderNo, int line, String remark) throws Exception {
		OnlineorderpBean order = onlineorderpDao.findEntity(OnlineorderpBean.class, orderNo);
		order.setRemark(remark);
		order.setPayDate(new Date());
		OnlineorderhBean orderh = onlineorderhDao.findEntity(OnlineorderhBean.class,orderNo);
		//TODO
		if(orderh.getTablecode()!=null &&orderh.getTablecode().equals("qr") && orderh.getStatus()==0){
			orderh.setStatus(1);
		} else if(orderh.getTablecode()!=null&&!orderh.getTablecode().equals("qr") && (orderh.getStatus()==1||orderh.getStatus()==2)){
			orderh.setStatus(3);
		}
		orderh.setAmountsf(order.getAmount());
		orderh.setDiscountamount(orderh.getAmountyf()-orderh.getAmountsf());

		onlineorderhDao.update(orderh);
		if (null == order)
	            throw new ClassNotFoundException(orderNo);
//		onlineorderpDao.updateOnlineorderpRemark(orderNo, line, remark);
		onlineorderpDao.update(order);
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
	public void insertIntoOnlineOrderp(OnlineorderhBean onlineorderhBean, JSONObject resAsJson) throws Exception{
		try {
			OnlineorderpBean onlineorderpBean = new OnlineorderpBean();
			onlineorderpBean.setCreatDate(new Date());
			onlineorderpBean.setCreatuser("SYSTEM");
			onlineorderpBean.setAmount(Double.parseDouble(resAsJson.getJSONObject("amount").getString("total"))/100);
			onlineorderpBean.setOrderno(onlineorderhBean.getOrderno());
			onlineorderpBean.setBrandcode(onlineorderhBean.getBrandcode());
			onlineorderpBean.setLine(1);
			onlineorderpBean.setPaymentcode(1);
			onlineorderpBean.setPaymentname("微信");
			onlineorderpBean.setIf_orderno(resAsJson.getString("transaction_id"));
			onlineorderpBean.setOpenid(resAsJson.getJSONObject("payer").getString("openid"));
			onlineorderpBean.setPayDate(DateUtils.stringToDate(resAsJson.getString("success_time"),DateUtils.DATETIME_FORMAT));
			onlineorderpDao.saveOrUpdate(onlineorderpBean);
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	public boolean checkEntityOccupied(Serializable id) throws Exception
	{
		OnlineorderpBean existsDept = onlineorderpDao.findEntity(OnlineorderpBean.class, id);

		return null != existsDept;
	}
}
