package com.bettersoft.api.service.implement;

import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import com.bettersoft.api.dao.interfaces.IOnlinestoreDao;
import com.bettersoft.api.model.OnlinestoreBean;
import com.bettersoft.api.service.interfaces.IOnlinestoreService;
import com.bettersoft.api.utils.Constants;
import com.bettersoft.api.utils.EntityUtil;
import com.bettersoft.api.utils.LoginUserInfo;
import com.bettersoft.api.utils.PageInfoUtils;
import com.bettersoft.api.validmodel.OnlinestoreValidate;
import org.springframework.util.ObjectUtils;

@Repository
@SuppressWarnings({"unchecked", "rawtypes"})
public class OnlinestoreService extends BaseService implements IOnlinestoreService {
    @Resource(name = "onlinestoreDao")
    private IOnlinestoreDao onlinestoreDao;

    protected Gson gson = new GsonBuilder().setDateFormat("yyyy-MM-dd HH:mm:ss").serializeNulls().create();

    public Map<String, Object> searchByPage(Map<String, Object> queryParam) throws Exception {
        List<Map<String, Object>> list = onlinestoreDao.searchList(queryParam);
        SimpleDateFormat simpleDateFormatdedate = new SimpleDateFormat ("HH:mm");
        list.forEach(store->{
            if(!ObjectUtils.isEmpty(store.get("beginTime"))){
                store.put("beginTime",simpleDateFormatdedate.format(store.get("beginTime")));
            }
            if(!ObjectUtils.isEmpty(store.get("endTime"))){
                store.put("endTime",simpleDateFormatdedate.format(store.get("endTime")));
            }
        });
        Number count = onlinestoreDao.searchCount(queryParam);
        return new PageInfoUtils(queryParam, list, count).getPageInfoMap();
    }

    public OnlinestoreBean findOnlinestore(String str) throws Exception {
        OnlinestoreBean onlinestoreBean = onlinestoreDao.findEntity(OnlinestoreBean.class, str);
        return onlinestoreBean;
    }

    public OnlinestoreBean findOnlinestore(Long l) throws Exception {
        OnlinestoreBean onlinestoreBean = onlinestoreDao.findEntity(OnlinestoreBean.class, l);
        return onlinestoreBean;
    }

    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
    public void saveOnlinestore(OnlinestoreValidate onlinestoreValidate, LoginUserInfo loginUser) throws Exception {
        OnlinestoreBean onlinestoreBean = new OnlinestoreBean();
        EntityUtil.copyEntityProperty(onlinestoreValidate, onlinestoreBean);
        onlinestoreBean.setUpdateuser(loginUser.getUserName());
        onlinestoreBean.setUpdateDate(new Date());
        onlinestoreBean.setCreatuser(loginUser.getUserName());
        onlinestoreBean.setCreatDate(new Date());
        onlinestoreDao.save(generalLogOperator(Constants.INSERT_OPER_TYPE, loginUser, onlinestoreBean));
        onlinestoreDao.save(onlinestoreBean);
    }

    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
    public void updateOnlinestore(String strstorecodes, Integer status, LoginUserInfo loginUser) throws Exception {
        String fromHql = "from OnlinestoreBean where strstorecode in ('" + strstorecodes.replace(",", "','") + "')";
        List<OnlinestoreBean> list = onlinestoreDao.generalSearch(fromHql, null, null);
        for (OnlinestoreBean edit : list) {
            edit.setUpdateuser(loginUser.getUserName());
            edit.setUpdateDate(new Date());
            edit.setStatus(status);
            onlinestoreDao.save(generalLogOperator(Constants.UPDATE_OPER_TYPE, loginUser, edit));
            onlinestoreDao.update(edit);
        }
    }

    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
    public void deleteOnlinestore(String str, LoginUserInfo loginUser) throws Exception {
        OnlinestoreBean del = onlinestoreDao.findEntity(OnlinestoreBean.class, str);
        if (null == del)
            throw new ClassNotFoundException();
        onlinestoreDao.save(generalLogOperator(Constants.DELETE_OPER_TYPE, loginUser, del));
        onlinestoreDao.delete(del);
    }

    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
    public void deleteOnlinestore(Long l, LoginUserInfo loginUser) throws Exception {
        OnlinestoreBean del = onlinestoreDao.findEntity(OnlinestoreBean.class, l);
        if (null == del)
            throw new ClassNotFoundException();
        onlinestoreDao.save(generalLogOperator(Constants.DELETE_OPER_TYPE, loginUser, del));
        onlinestoreDao.delete(del);
    }

    public boolean checkEntityOccupied(Serializable id) throws Exception {
        OnlinestoreBean existsDept = onlinestoreDao.findEntity(OnlinestoreBean.class, id);

        return null != existsDept;
    }
}
