package com.bettersoft.api.service.implement;

import java.io.Serializable;
import java.sql.Timestamp;
import java.text.MessageFormat;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.bettersoft.api.dao.interfaces.IProvincialAreaDao;
import com.bettersoft.api.exceptions.ValidateDataOccupiedException;
import com.bettersoft.api.model.ProvincialAreaBean;
import com.bettersoft.api.service.interfaces.IProvincialAreaService;
import com.bettersoft.api.utils.Constants;
import com.bettersoft.api.utils.EntityUtil;
import com.bettersoft.api.utils.LoginUserInfo;
import com.bettersoft.api.utils.PageInfoUtils;
import com.bettersoft.api.utils.RedisCacheUtil;
import com.bettersoft.api.validmodel.ProvincialAreaValidate;

@Service("provincialAreaService")
public class ProvincialAreaService extends BaseService implements
		IProvincialAreaService {
	@Resource(name = "provincialAreaDao")
	private IProvincialAreaDao provincialAreaDao;
	@Resource(name = "redisCacheUtil")
	private RedisCacheUtil redisCacheUtil;

	public ProvincialAreaValidate findProvincialByCode(String areaCode)
			throws Exception {
		ProvincialAreaBean provincial = provincialAreaDao.findEntity(
				ProvincialAreaBean.class, areaCode);
		if (null == provincial)
			throw new ClassNotFoundException();

		if (null != provincial.getParentProvincial()) {
			provincial.setParentAreaCode(provincial.getParentProvincial()
					.getAreaCode());
			provincial.setParentAreaName(provincial.getParentProvincial()
					.getAreaName());
		}

		return (ProvincialAreaValidate) EntityUtil.cloneToEntity(
				ProvincialAreaValidate.class, provincial);
	}

	public Map<String, Object> searchProvincials(Map<String, Object> queryParam)
			throws Exception {
		if (queryParam.containsKey("areaDesc")) {
			String areaDesc = MapUtils.getString(queryParam, "areaDesc",
					StringUtils.EMPTY);

			queryParam.put(Constants.QUERY_SPEC_CONDITION + "areaDesc",
					MessageFormat.format(
							"a.areaCode like '%{0}%' or a.areaName like '%{0}%'",
							areaDesc));
			queryParam.remove("areaDesc");
		}
		
		List<Map<String, Object>> provincials = provincialAreaDao
				.searchProvincials(queryParam);

		Number count = provincialAreaDao.searchProvincialCount(queryParam);

		return new PageInfoUtils(queryParam, provincials, count)
				.getPageInfoMap();
	}

	public List<Map<String, Object>> searchProvincialList(
			Map<String, Object> queryParam) throws Exception {
//		List<ProvincialAreaBean> provincials = null;
//		List<Map<String, Object>> provincialMaps = new ArrayList<Map<String, Object>>();

//		if (queryParam.containsKey("areaDesc")) {
//			String areaDesc = MapUtils.getString(queryParam, "areaDesc",
//					StringUtils.EMPTY);
//
//			queryParam.put(Constants.QUERY_SPEC_CONDITION + "areaDesc",
//					MessageFormat.format(
//							"areaCode like '%{0}%' or areaName like '%{0}%'",
//							areaDesc));
//			queryParam.remove("areaDesc");
//		}
//
//		if (!queryParam.isEmpty()) {
//			provincialMaps = provincialAreaDao.searchProvincials(queryParam);
//		} else {
//			provincials = provincialAreaDao.searchProvincials();
//		}
//
//		if (null != provincials && !provincials.isEmpty()) {
//			for (ProvincialAreaBean provincial : provincials) {
//				Map<String, Object> provincialMap = EntityUtil
//						.convertEntityToMap(provincial);
//				provincialMap.put("parentArea", StringUtils.EMPTY);
//
//				if (null != provincial.getParentProvincial())
//					provincialMap.put("parentArea", provincial
//							.getParentProvincial().getAreaName());
//
//				provincialMap.put("isParent",
//						null != provincial.getChildAreas()
//								&& !provincial.getChildAreas().isEmpty());
//
//				provincialMaps.add(provincialMap);
//			}
//		}

		return provincialAreaDao.searchProvincials(queryParam);
	}

	@Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
	public void saveProvincial(ProvincialAreaValidate provincialValidate,
			LoginUserInfo loginUser) throws Exception {
		ProvincialAreaBean provincial = new ProvincialAreaBean();
		EntityUtil.copyEntityProperty(provincialValidate, provincial);
		provincial.setAreaCode(provincialValidate.getAreaCode());

		if (checkEntityOccupied(provincialValidate.getAreaCode()))
			throw new ValidateDataOccupiedException();

		if (!StringUtils.isEmpty(provincial.getParentAreaCode())) {
			ProvincialAreaBean parentProvincial = provincialAreaDao.findEntity(
					ProvincialAreaBean.class, provincial.getParentAreaCode());

			if (null == parentProvincial)
				throw new ClassNotFoundException();

			provincial.setParentProvincial(parentProvincial);
		} else {
			provincial.setParentProvincial(null);
		}

		provincial.setCreateBy(loginUser.getUserCode());
		provincial.setCreateDateTime(new Timestamp(System.currentTimeMillis()));
		provincial.setCreateUserName(loginUser.getUserName());

		ProvincialAreaBean topProvincial = findTopProvincial(provincial);
		redisCacheUtil.setCache(Constants.PROVINCIAL_KEY,
				topProvincial.getAreaCode(), topProvincial);

		provincialAreaDao.save(provincial);
		provincialAreaDao.save(generalLogOperator(Constants.INSERT_OPER_TYPE,
				loginUser, null, provincial));
	}

	@Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
	public void updateProvincial(ProvincialAreaValidate provincialValidate,
			LoginUserInfo loginUser) throws Exception {
		String areaCode = provincialValidate.getAreaCode();

		ProvincialAreaBean editProvincial = provincialAreaDao.findEntity(
				ProvincialAreaBean.class, areaCode);

		ProvincialAreaBean srcProvincial = EntityUtil
				.cloneEntity(editProvincial);

		EntityUtil.compareCopyEntity(provincialValidate, editProvincial);

		if (!StringUtils.isEmpty(editProvincial.getParentAreaCode())) {
			ProvincialAreaBean parentProvincial = provincialAreaDao.findEntity(
					ProvincialAreaBean.class,
					editProvincial.getParentAreaCode());

			if (null == parentProvincial)
				throw new ClassNotFoundException();

			editProvincial.setParentProvincial(parentProvincial);
		} else {
			editProvincial.setParentProvincial(null);
		}

		editProvincial.setLastModifyBy(loginUser.getUserCode());
		editProvincial.setLastModifyDateTime(new Timestamp(System
				.currentTimeMillis()));
		editProvincial.setLastModifyUserName(loginUser.getUserName());

		provincialAreaDao.update(editProvincial);
		provincialAreaDao.save(generalLogOperator(Constants.UPDATE_OPER_TYPE,
				loginUser, srcProvincial, editProvincial));
		
		ProvincialAreaBean topProvincial = findTopProvincial(editProvincial);
		redisCacheUtil.setCache(Constants.PROVINCIAL_KEY,
				topProvincial.getAreaCode(), topProvincial);
	}

	@Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
	public void deleteProvincial(String areaCode, LoginUserInfo loginUser)
			throws Exception {
		ProvincialAreaBean delProvincial = provincialAreaDao.findEntity(
				ProvincialAreaBean.class, areaCode);

		if (null == delProvincial)
			throw new ClassNotFoundException();

		ProvincialAreaBean topProvincial = findTopProvincial(delProvincial);
		if (topProvincial.getAreaCode().equals(areaCode))
			redisCacheUtil.removeCacheByKey(Constants.PROVINCIAL_KEY, areaCode);
		else
			redisCacheUtil.setCache(Constants.PROVINCIAL_KEY,
					topProvincial.getAreaCode(), topProvincial);

		provincialAreaDao.save(generalLogOperator(Constants.DELETE_OPER_TYPE,
				loginUser, delProvincial));
		provincialAreaDao.delete(delProvincial);
	}

	public boolean checkEntityOccupied(Serializable id) throws Exception {
		ProvincialAreaBean existsProvincial = provincialAreaDao.findEntity(
				ProvincialAreaBean.class, id);

		return null != existsProvincial;
	}

	public ProvincialAreaBean findTopProvincial(ProvincialAreaBean provincial)
			throws Exception {
		if (null != provincial.getParentProvincial())
			return findTopProvincial(provincial.getParentProvincial());

		return provincial;
	}
}
