package com.bettersoft.api.service.implement;

import java.io.Serializable;
import java.sql.Timestamp;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.bettersoft.api.dao.interfaces.ISystemConfigurationDao;
import com.bettersoft.api.exceptions.ValidateDataOccupiedException;
import com.bettersoft.api.model.SystemConfigurationBean;
import com.bettersoft.api.service.interfaces.ISystemConfigurationService;
import com.bettersoft.api.utils.Constants;
import com.bettersoft.api.utils.EntityUtil;
import com.bettersoft.api.utils.LoginUserInfo;
import com.bettersoft.api.utils.PageInfoUtils;
import com.bettersoft.api.utils.RedisCacheUtil;
import com.bettersoft.api.validmodel.SystemConfigurationValidate;

@Service("systemConfigurationService")
@Transactional(readOnly = true)
public class SystemConfigurationService extends BaseService implements
		ISystemConfigurationService {
	@Resource(name = "systemConfigurationDao")
	private ISystemConfigurationDao systemConfigurationDao;
	@Resource(name = "redisCacheUtil")
	private RedisCacheUtil redisCacheUtil;

	public SystemConfigurationBean findConfigurationByCode(String configCode)
			throws Exception {
		SystemConfigurationBean config = systemConfigurationDao.findEntity(
				SystemConfigurationBean.class, configCode);

		if (null == config)
			throw new ClassNotFoundException();

		return config;
	}

	public Map<String, Object> searchConfigurations(
			Map<String, Object> queryParam) throws Exception {
		List<SystemConfigurationBean> configurations = systemConfigurationDao
				.searchConfigurations(queryParam);

		Number count = systemConfigurationDao
				.searchConfigurationCount(queryParam);

		return new PageInfoUtils(queryParam, configurations, count)
				.getPageInfoMap();
	}

	@Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
	public void saveConfiguration(SystemConfigurationValidate configModel,
			LoginUserInfo loginUser) throws Exception {
		SystemConfigurationBean config = new SystemConfigurationBean();
		EntityUtil.copyEntityProperty(configModel, config);
		config.setConfigCode(configModel.getConfigCode());

		if (checkEntityOccupied(config.getConfigCode()))
			throw new ValidateDataOccupiedException();

		config.setCreateBy(loginUser.getUserCode());
		config.setCreateUserName(loginUser.getUserName());
		config.setCreateDateTime(new Timestamp(System.currentTimeMillis()));

		systemConfigurationDao.save(config);
		systemConfigurationDao.save(generalLogOperator(
				Constants.INSERT_OPER_TYPE, loginUser, config));
		redisCacheUtil.setCache(Constants.SYSCONFIG_KEY,
				config.getConfigCode(), config);
	}

	@Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
	public void updateConfiguration(String configCode,
			SystemConfigurationValidate configModel, LoginUserInfo loginUser)
			throws Exception {
		SystemConfigurationBean editConfig = systemConfigurationDao.findEntity(
				SystemConfigurationBean.class, configCode);

		SystemConfigurationBean srcConfig = EntityUtil.cloneEntity(editConfig);

		EntityUtil.compareCopyEntity(configModel, editConfig);

		editConfig.setLastModifyBy(loginUser.getUserCode());
		editConfig.setLastModifyDateTime(new Timestamp(System
				.currentTimeMillis()));
		editConfig.setLastModifyUserName(loginUser.getUserName());

		systemConfigurationDao.update(editConfig);
		systemConfigurationDao.save(generalLogOperator(
				Constants.UPDATE_OPER_TYPE, loginUser, srcConfig, editConfig));
		redisCacheUtil.setCache(Constants.SYSCONFIG_KEY,
				editConfig.getConfigCode(), editConfig);
	}

	@Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
	public void deleteConfiguration(String configCode, LoginUserInfo loginUser)
			throws Exception {
		SystemConfigurationBean delConfig = systemConfigurationDao.findEntity(
				SystemConfigurationBean.class, configCode);

		if (null == delConfig)
			throw new ClassNotFoundException();

		redisCacheUtil.removeCacheByKey(Constants.SYSCONFIG_KEY,
				delConfig.getConfigCode());
		systemConfigurationDao.save(generalLogOperator(
				Constants.DELETE_OPER_TYPE, loginUser, delConfig));
		systemConfigurationDao.delete(delConfig);
	}

	@Override
	public SystemConfigurationBean findConfigurationByCodeNoException(String configCode) {
		return systemConfigurationDao.findEntity(
				SystemConfigurationBean.class, configCode);
	}

	public boolean checkEntityOccupied(Serializable id) throws Exception {
		SystemConfigurationBean existsConfig = systemConfigurationDao
				.findEntity(SystemConfigurationBean.class, id);

		return null != existsConfig;
	}

}
