package com.bettersoft.api.service.implement;

import java.io.Serializable;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.bettersoft.api.dao.interfaces.ISystemDepartmentDao;
import com.bettersoft.api.exceptions.ValidateDataOccupiedException;
import com.bettersoft.api.model.SystemDepartmentBean;
import com.bettersoft.api.service.interfaces.ISystemDepartmentService;
import com.bettersoft.api.utils.Constants;
import com.bettersoft.api.utils.EntityUtil;
import com.bettersoft.api.utils.LoginUserInfo;
import com.bettersoft.api.utils.PageInfoUtils;
import com.bettersoft.api.validmodel.SystemDepartmentValidate;

@Service("systemDepartmentService")
public class SystemDepartmentService extends BaseService implements
		ISystemDepartmentService {
	@Resource(name = "systemDepartmentDao")
	private ISystemDepartmentDao systemDepartmentDao;

	public SystemDepartmentBean searchDepatmentByCode(String departmentCode)
			throws Exception {
		SystemDepartmentBean department = systemDepartmentDao.findEntity(
				SystemDepartmentBean.class, departmentCode);

		if (null != department.getParentDepartment()) {
			department.setParentCode(department.getParentDepartment()
					.getDepartmentCode());
			department.setParentDesc(department.getParentDepartment()
					.getDepartmentDesc());
		}

		return department;
	}

	public Map<String, Object> searchDepartments(Map<String, Object> queryParam)
			throws Exception {
		List<Map<String, Object>> departments = systemDepartmentDao
				.searchDepartments(queryParam);

		Number count = systemDepartmentDao.searchDepartmentCount(queryParam);

		return new PageInfoUtils(queryParam, departments, count)
				.getPageInfoMap();
	}

	public Map<String, Object> searchDepartment(Map<String, Object> queryParam)
			throws Exception {
		List<SystemDepartmentBean> departments = systemDepartmentDao
				.inqueryDepartments(queryParam);
		List<Map<String, Object>> depMaps = new ArrayList<Map<String, Object>>();

		if (null != departments && !departments.isEmpty()) {
			for (SystemDepartmentBean dep : departments) {
				Map<String, Object> depMap = EntityUtil.convertEntityToMap(dep);

				depMap.put("hasChild", null != dep.getChildDepartments()
						&& !dep.getChildDepartments().isEmpty());

				depMaps.add(depMap);
			}
		}

		Number count = systemDepartmentDao.inqueryDepartmentCount(queryParam);

		return new PageInfoUtils(queryParam, depMaps, count).getPageInfoMap();
	}

	public List<SystemDepartmentBean> searchDepartmentTree(
			Map<String, Object> queryParam) throws Exception {
		return systemDepartmentDao.searchDepartmentTree(queryParam);
	}

	@Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
	public void saveDepartment(SystemDepartmentValidate departmentValidate,
			LoginUserInfo loginUser) throws Exception {
		SystemDepartmentBean department = new SystemDepartmentBean();
		EntityUtil.copyEntityProperty(departmentValidate, department);

		department.setDepartmentCode(departmentValidate.getDepartmentCode());

		if (checkEntityOccupied(department.getDepartmentCode()))
			throw new ValidateDataOccupiedException();

		if (!StringUtils.isEmpty(department.getParentCode())) {
			SystemDepartmentBean parentDep = systemDepartmentDao.findEntity(
					SystemDepartmentBean.class, department.getParentCode());
			if (null == parentDep)
				throw new ClassNotFoundException();

			department.setDepartmentLevel(parentDep.getDepartmentLevel() + 1);
			department.setParentDepartment(parentDep);
		} else {
			department.setDepartmentLevel(0);
			department.setParentDepartment(null);
		}

		department.setCreateBy(loginUser.getUserCode());
		department.setCreateDateTime(new Timestamp(System.currentTimeMillis()));
		department.setCreateUserName(loginUser.getUserName());

		systemDepartmentDao.save(generalLogOperator(Constants.INSERT_OPER_TYPE,
				loginUser, null, department));
		systemDepartmentDao.save(department);
	}

	@Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
	public void updateDepartment(SystemDepartmentValidate departmentValidate,
			LoginUserInfo loginUser) throws Exception {
		String departmentCode = departmentValidate.getDepartmentCode();

		SystemDepartmentBean editDep = systemDepartmentDao.findEntity(
				SystemDepartmentBean.class, departmentCode);

		EntityUtil.compareCopyEntity(departmentValidate, editDep);

		if (!StringUtils.isEmpty(editDep.getParentCode())) {
			SystemDepartmentBean parentDep = systemDepartmentDao.findEntity(
					SystemDepartmentBean.class, editDep.getParentCode());
			if (null == parentDep)
				throw new ClassNotFoundException();

			editDep.setDepartmentLevel(parentDep.getDepartmentLevel() + 1);
			editDep.setParentDepartment(parentDep);
		} else {
			editDep.setDepartmentLevel(0);
			editDep.setParentCode(null);
			editDep.setParentDepartment(null);
		}

		editDep.setLastModifyBy(loginUser.getUserCode());
		editDep.setLastModifyDateTime(new Timestamp(System.currentTimeMillis()));
		editDep.setLastModifyUserName(loginUser.getUserName());

		systemDepartmentDao.update(editDep);
	}

	@Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
	public void deleteDepartment(String departmentCode) throws Exception {
		SystemDepartmentBean delDep = systemDepartmentDao.findEntity(
				SystemDepartmentBean.class, departmentCode);
		if (null == delDep)
			throw new ClassNotFoundException();

		if (null != delDep.getChildDepartments()
				&& !delDep.getChildDepartments().isEmpty())
			throw new ValidateDataOccupiedException();

		systemDepartmentDao.delete(delDep);
	}

	public boolean checkEntityOccupied(Serializable id) throws Exception {
		SystemDepartmentBean existsDept = systemDepartmentDao.findEntity(
				SystemDepartmentBean.class, id);

		return null != existsDept;
	}
}