package com.bettersoft.api.service.implement;

import java.io.Serializable;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.bettersoft.api.dao.interfaces.ISystemMenuTreeDao;
import com.bettersoft.api.exceptions.ValidateDataOccupiedException;
import com.bettersoft.api.model.MenuTreeBean;
import com.bettersoft.api.model.SystemMenuPermissionBean;
import com.bettersoft.api.model.SystemPermissionBean;
import com.bettersoft.api.service.interfaces.ISystemMenuTreeService;
import com.bettersoft.api.utils.Constants;
import com.bettersoft.api.utils.EntityUtil;
import com.bettersoft.api.utils.LoginUserInfo;
import com.bettersoft.api.utils.PageInfoUtils;
import com.bettersoft.api.validmodel.SystemMenuTreeValidate;

@Service("systemMenuTreeService")
public class SystemMenuTreeService extends BaseService implements
		ISystemMenuTreeService {
	@Resource(name = "systemMenuTreeDao")
	private ISystemMenuTreeDao systemMenuTreeDao;

	public SystemMenuTreeValidate findMenuByCode(String menuCode)
			throws Exception {
		MenuTreeBean menu = systemMenuTreeDao.findEntity(MenuTreeBean.class,
				menuCode);
		if (null == menu)
			throw new ClassNotFoundException();

		if (null != menu.getParentMenu()) {
			menu.setParentCode(menu.getParentMenu().getMenuCode());
			menu.setParentName(menu.getParentMenu().getMenuName());
		}

		SystemMenuTreeValidate menuValid = (SystemMenuTreeValidate) EntityUtil
				.cloneToEntity(SystemMenuTreeValidate.class, menu);

		if (null != menu.getPermissions() && !menu.getPermissions().isEmpty()) {
			List<String> perms = new ArrayList<String>();
			for (SystemMenuPermissionBean menuPerm : menu.getPermissions()) {
				if (null != menuPerm.getPermission()) {
					perms.add(menuPerm.getPermission().getPermissionCode());
				}
			}

			menuValid.setPerms(StringUtils.join(perms, ","));
		}

		return menuValid;
	}

	public Map<String, Object> searchMenuTrees(Map<String, Object> queryParam)
			throws Exception {
		List<MenuTreeBean> menus = systemMenuTreeDao
				.searchMenuTrees(queryParam);
		List<Map<String, Object>> menuMaps = new ArrayList<Map<String, Object>>();

		convertMenuTrees2MapList(menus, menuMaps);

		return new PageInfoUtils(queryParam, menuMaps, menuMaps.size())
				.getPageInfoMap();
	}

	public List<MenuTreeBean> searchMenuTrees(Map<String, Object> queryParam,
			LoginUserInfo loginUser) throws Exception {
		List<MenuTreeBean> menus = systemMenuTreeDao
				.searchMenuTrees(queryParam);
		if (null != menus && !menus.isEmpty()) {
			for (MenuTreeBean menu : menus) {
				setParentMenu(menu);
			}
		}

		if (null != loginUser.getAuthMenus()
				&& !loginUser.getAuthMenus().isEmpty()) {
			String[] authMenus = loginUser.getAuthMenus().split(",");
			if (null != authMenus && 0 < authMenus.length) {
				for (String menuCode : authMenus) {
					setRetentionMenuByMenuCode(menus, menuCode);
				}
			}

			removeMenusNoRetention(menus);
		}

		return menus;
	}

	public Map<String, Object> searchParentMenus(Map<String, Object> queryParam)
			throws Exception {
		List<Map<String, Object>> menuMaps = new ArrayList<Map<String, Object>>();

		List<MenuTreeBean> menus = systemMenuTreeDao
				.searchMenuTrees(queryParam);

		Number count = systemMenuTreeDao.searchMenuTreeCount(queryParam);
		if (null != menus && !menus.isEmpty()) {
			for (MenuTreeBean menu : menus) {
				Map<String, Object> menuMap = EntityUtil
						.convertEntityToMap(menu);
				menuMap.put("isParent", null != menu.getChildMenus()
						&& !menu.getChildMenus().isEmpty());

				menuMaps.add(menuMap);
			}
		}

		return new PageInfoUtils(queryParam, menuMaps, count).getPageInfoMap();
	}

	public List<SystemPermissionBean> searchPermissions() throws Exception {
		Map<String, Object> queryParam = new HashMap<String, Object>();
		queryParam.put(Constants.QUERY_ORDER_BY, "priority");

		return systemMenuTreeDao.searchPermissions(queryParam);
	}

	@Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
	public void saveMenuTree(SystemMenuTreeValidate menuValidate,
			LoginUserInfo loginUser) throws Exception {
		MenuTreeBean menuTree = new MenuTreeBean();
		EntityUtil.copyEntityProperty(menuValidate, menuTree);
		menuTree.setMenuCode(menuValidate.getMenuCode());

		if (checkEntityOccupied(menuTree.getMenuCode()))
			throw new ValidateDataOccupiedException();

		if (!StringUtils.isEmpty(menuTree.getParentCode())) {
			MenuTreeBean parentMenu = systemMenuTreeDao.findEntity(
					MenuTreeBean.class, menuTree.getParentCode());
			if (null == parentMenu)
				throw new ClassNotFoundException();

			menuTree.setMenuLevel(parentMenu.getMenuLevel() + 1);
			menuTree.setParentMenu(parentMenu);
		} else {
			menuTree.setMenuLevel(0);
			menuTree.setParentMenu(null);
		}

		if (!StringUtils.isEmpty(menuValidate.getPerms())) {
			String[] perms = StringUtils.split(menuValidate.getPerms(), ",");
			menuTree.getPermissions().clear();
			for (String permCode : perms) {

				SystemPermissionBean perm = systemMenuTreeDao.findEntity(
						SystemPermissionBean.class, permCode);
				if (null == perm)
					throw new ClassNotFoundException();

				SystemMenuPermissionBean menuPerm = new SystemMenuPermissionBean();
				menuPerm.setMenuCode(menuTree.getMenuCode());
				menuPerm.setPermissionCode(perm.getPermissionCode());
				menuPerm.setMenu(menuTree);
				menuPerm.setPermission(perm);
				menuPerm.setCreateDateTime(new Timestamp(System
						.currentTimeMillis()));

				menuTree.getPermissions().add(menuPerm);
			}
		}

		menuTree.setCreateDateTime(new Timestamp(System.currentTimeMillis()));

		systemMenuTreeDao.save(menuTree);
	}

	@Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
	public void updateMenuTree(SystemMenuTreeValidate menuValidate,
			LoginUserInfo loginUser) throws Exception {
		String menuCode = menuValidate.getMenuCode();
		MenuTreeBean editMenu = systemMenuTreeDao.findEntity(
				MenuTreeBean.class, menuCode);

		EntityUtil.compareCopyEntity(menuValidate, editMenu);

		if (!StringUtils.isEmpty(editMenu.getParentCode())) {
			MenuTreeBean parentMenu = systemMenuTreeDao.findEntity(
					MenuTreeBean.class, editMenu.getParentCode());
			if (null == parentMenu)
				throw new ClassNotFoundException();

			editMenu.setMenuLevel(parentMenu.getMenuLevel() + 1);
			editMenu.setParentMenu(parentMenu);
		} else {
			editMenu.setMenuLevel(0);
			editMenu.setParentCode(null);
			editMenu.setParentMenu(null);
		}

		editMenu.getPermissions().clear();
		if (null != menuValidate.getPerms()
				&& !menuValidate.getPerms().isEmpty()) {
			String[] perms = StringUtils.split(menuValidate.getPerms(), ",");
			for (String permCode : perms) {
				SystemMenuPermissionBean menuPerm = systemMenuTreeDao
						.searchMenuPermission(menuCode, permCode);

				if (null == menuPerm) {
					menuPerm = new SystemMenuPermissionBean();

					SystemPermissionBean perm = systemMenuTreeDao.findEntity(
							SystemPermissionBean.class, permCode);
					if (null == perm)
						throw new ClassNotFoundException();

					menuPerm.setMenuCode(editMenu.getMenuCode());
					menuPerm.setPermissionCode(perm.getPermissionCode());
					menuPerm.setMenu(editMenu);
					menuPerm.setPermission(perm);
					menuPerm.setCreateDateTime(new Timestamp(System
							.currentTimeMillis()));
				}

				editMenu.getPermissions().add(menuPerm);
			}
		}

		systemMenuTreeDao.update(editMenu);
	}

	@Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
	public void deleteMenuTree(String menuCode) throws Exception {
		MenuTreeBean delMenu = systemMenuTreeDao.findEntity(MenuTreeBean.class,
				menuCode);
		if (null == delMenu)
			throw new ClassNotFoundException();

		systemMenuTreeDao.delete(delMenu);
	}

	public boolean checkEntityOccupied(Serializable id) throws Exception {
		MenuTreeBean existsMenu = systemMenuTreeDao.findEntity(
				MenuTreeBean.class, id);

		return null != existsMenu;
	}

	private void setParentMenu(MenuTreeBean menu) {
		if (null != menu.getParentMenu()) {
			menu.setParentCode(menu.getParentMenu().getMenuCode());
			menu.setParentName(menu.getParentMenu().getMenuName());
		}

		if (null != menu.getChildMenus() && !menu.getChildMenus().isEmpty()) {
			for (MenuTreeBean childMenu : menu.getChildMenus()) {
				childMenu.setParentCode(menu.getMenuCode());
				childMenu.setParentName(menu.getMenuName());

				if (null != childMenu.getChildMenus()
						&& !menu.getChildMenus().isEmpty())
					setParentMenu(childMenu);
			}
		}

		if (null != menu.getPermissions() && !menu.getPermissions().isEmpty()) {
			for (SystemMenuPermissionBean menuPerm : menu.getPermissions())
				menuPerm.setPermissionDesc(menuPerm.getPermission()
						.getPermissionDesc());
		}
	}

	private void setRetentionMenuByMenuCode(List<MenuTreeBean> menus,
			String menuCode) {
		for (MenuTreeBean menu : menus) {
			if (null != menu.getChildMenus() && !menu.getChildMenus().isEmpty())
				setRetentionMenuByMenuCode(menu.getChildMenus(), menuCode);

			if (!menu.isRetention())
				menu.setRetention(menu.getMenuCode().equals(menuCode));
		}
	}

	private void removeMenusNoRetention(List<MenuTreeBean> menus) {
		Iterator<MenuTreeBean> menuIter = menus.iterator();
		while (menuIter.hasNext()) {
			MenuTreeBean menu = menuIter.next();
			if (null != menu.getChildMenus() && !menu.getChildMenus().isEmpty())
				removeMenusNoRetention(menu.getChildMenus());

			if ((menu.getMenuType().equals(Constants.MENU_TYPE) && !menu
					.isRetention())
					|| (menu.getMenuType().equals(Constants.DICTIONARY_TYPE) && menu
							.getChildMenus().isEmpty()))
				menuIter.remove();
		}
	}

	private void convertMenuTrees2MapList(List<MenuTreeBean> menus,
			List<Map<String, Object>> menuMaps) {
		if (null != menus && !menus.isEmpty()) {
			for (MenuTreeBean menu : menus) {
				Map<String, Object> menuMap = EntityUtil.convertEntityToMap(
						menu, "menuCode", "menuName", "priority", "enable",
						"menuUrl", "menuRoute", "menuIcon", "menuType");

				menuMap.put("id", menu.getMenuCode());
				menuMap.put("level", menu.getMenuLevel());
				menuMap.put("isLeaf",
						menu.getMenuType().equals(Constants.MENU_TYPE));

				menuMap.put("parent", null);
				menuMap.put("parentName", null);

				if (null != menu.getParentMenu()) {
					menuMap.put("parent", menu.getParentMenu().getMenuCode());
					menuMap.put("parentName", menu.getParentMenu()
							.getMenuName());
				}

				menuMaps.add(menuMap);

				if (null != menu.getChildMenus()
						&& !menu.getChildMenus().isEmpty()) {
					convertMenuTrees2MapList(menu.getChildMenus(), menuMaps);
				}
			}
		}
	}
}
