package com.bettersoft.api.service.implement;

import java.io.Serializable;
import java.sql.Timestamp;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.bettersoft.api.dao.interfaces.ISystemMessageDao;
import com.bettersoft.api.model.SystemMessageBean;
import com.bettersoft.api.model.SystemMessageReadBean;
import com.bettersoft.api.service.interfaces.ISystemMessageService;
import com.bettersoft.api.utils.Constants;
import com.bettersoft.api.utils.LoginUserInfo;
import com.bettersoft.api.utils.PageInfoUtils;

@Service("systemMessageService")
public class SystemMessageService extends BaseService implements
		ISystemMessageService {
	@Resource(name = "systemMessageDao")
	private ISystemMessageDao systemMessageDao;

	public Map<String, Object> findSystemMsg(int seqId) throws Exception {
		Map<String, Object> resultMap = new HashMap<String, Object>();

		SystemMessageBean msg = systemMessageDao.findEntity(
				SystemMessageBean.class, seqId);
		if (null == msg)
			throw new ClassNotFoundException();

		resultMap.put("msgTitle", msg.getMsgTitle());
		resultMap.put("msgContent", msg.getMsgContent());

		return resultMap;
	}

	public Map<String, Object> searchSystemMessages(LoginUserInfo loginUser)
			throws Exception {
		Map<String, Object> queryParam = new HashMap<String, Object>();
		queryParam.put(Constants.QUERY_SPEC_CONDITION + "userCode",
				"r.userCode = '" + loginUser.getUserCode()
						+ "' or r.userCode is null");
		queryParam.put(Constants.QUERY_ORDER_BY, "m.createDateTime desc");

		List<Map<String, Object>> sysMsgs = systemMessageDao
				.searchSystemMessages(queryParam);

		Number count = systemMessageDao.searchSystemMessageCount(queryParam);

		return new PageInfoUtils(queryParam, sysMsgs, count).getPageInfoMap();
	}

	public int searchUnReadMessagCount(LoginUserInfo loginUser)
			throws Exception {
		Map<String, Object> queryParam = new HashMap<String, Object>();
		queryParam
				.put(Constants.QUERY_SPEC_CONDITION + "userCode",
						"m.seqId not in (select new map(r.messageId as messageId) from SystemMessageReadBean r where r.userCode = '"
								+ loginUser.getUserCode() + "')");

		return systemMessageDao.searchUnReadMessageCount(queryParam).intValue();
	}

	public Map<String, Object> findTopUnReadMessage(
			Map<String, Object> queryParam) throws Exception {
		queryParam.put(Constants.QUERY_SPEC_CONDITION + "r.userCode",
				"r.userCode is null");
		queryParam.put(Constants.QUERY_FIRST_RESULTS, 0);
		queryParam.put(Constants.QUERY_MAX_RESULTS, 1);

		List<Map<String, Object>> msgs = systemMessageDao
				.searchSystemMessages(queryParam);
		if (null != msgs && !msgs.isEmpty())
			return msgs.get(0);

		return null;
	}

	@Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
	public void readMessage(int seqId, LoginUserInfo loginUser)
			throws Exception {
		Map<String, Object> queryParam = new HashMap<String, Object>();
		queryParam.put("messageId", seqId);
		queryParam.put("userCode", loginUser.getUserCode());
		SystemMessageReadBean msgRead = systemMessageDao
				.findMessageRead(queryParam);

		if (null == msgRead) {
			msgRead = new SystemMessageReadBean();
			msgRead.setMessageId(seqId);
			msgRead.setUserCode(loginUser.getUserCode());
			msgRead.setCreateDateTime(new Timestamp(System.currentTimeMillis()));

			systemMessageDao.save(msgRead);
		}
	}

	@Override
	public boolean checkEntityOccupied(Serializable id) throws Exception {
		// TODO Auto-generated method stub
		return false;
	}

}
