package com.bettersoft.api.service.implement;

import java.io.Serializable;
import java.sql.Timestamp;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.bettersoft.api.dao.interfaces.ISystemPermissionDao;
import com.bettersoft.api.exceptions.ValidateDataOccupiedException;
import com.bettersoft.api.model.SystemPermissionBean;
import com.bettersoft.api.service.interfaces.ISystemPermissionService;
import com.bettersoft.api.utils.EntityUtil;
import com.bettersoft.api.utils.LoginUserInfo;
import com.bettersoft.api.utils.PageInfoUtils;
import com.bettersoft.api.validmodel.SystemPermissionValidate;

@Service("systemPermissionService")
public class SystemPermissionService extends BaseService implements
		ISystemPermissionService {
	@Resource(name = "systemPermissionDao")
	private ISystemPermissionDao systemPermissionDao;

	public SystemPermissionBean findPermissionByCode(String permissionCode)
			throws Exception {
		SystemPermissionBean perm = systemPermissionDao.findEntity(
				SystemPermissionBean.class, permissionCode);
		if (null == perm)
			throw new ClassNotFoundException();

		return perm;
	}

	public Map<String, Object> searchPermissions(Map<String, Object> queryParam)
			throws Exception {
		List<SystemPermissionBean> permissions = systemPermissionDao
				.searchPermissions(queryParam);

		Number count = systemPermissionDao.searchPermissionCount(queryParam);

		return new PageInfoUtils(queryParam, permissions, count)
				.getPageInfoMap();
	}

	@Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
	public void savePermission(SystemPermissionValidate peremissionValidate,
			LoginUserInfo loginUser) throws Exception {
		SystemPermissionBean permission = new SystemPermissionBean();
		EntityUtil.copyEntityProperty(peremissionValidate, permission);

		permission.setPermissionCode(peremissionValidate.getPermissionCode());

		if (checkEntityOccupied(permission.getPermissionCode()))
			throw new ValidateDataOccupiedException();

		permission.setCreateDateTime(new Timestamp(System.currentTimeMillis()));

		systemPermissionDao.save(permission);
	}

	@Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
	public void updatePermission(SystemPermissionValidate peremissionValidate,
			LoginUserInfo loginUser) throws Exception {
		String permissionCode = peremissionValidate.getPermissionCode();
		SystemPermissionBean editPerm = systemPermissionDao.findEntity(
				SystemPermissionBean.class, permissionCode);
		EntityUtil.compareCopyEntity(peremissionValidate, editPerm);

		systemPermissionDao.update(editPerm);
	}

	@Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
	public void deletePermission(String permissionCode) throws Exception {
		SystemPermissionBean delPerm = systemPermissionDao.findEntity(
				SystemPermissionBean.class, permissionCode);

		if (null == delPerm)
			throw new ClassNotFoundException();

		systemPermissionDao.delete(delPerm);
	}

	public boolean checkEntityOccupied(Serializable id) throws Exception {
		SystemPermissionBean existsPerm = systemPermissionDao.findEntity(
				SystemPermissionBean.class, id);

		return null != existsPerm;
	}
}
