package com.bettersoft.api.service.implement;

import java.io.Serializable;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.bettersoft.api.dao.interfaces.ISystemRoleDao;
import com.bettersoft.api.exceptions.ValidateDataOccupiedException;
import com.bettersoft.api.model.MenuTreeBean;
import com.bettersoft.api.model.SystemDepartmentBean;
import com.bettersoft.api.model.SystemMenuPermissionBean;
import com.bettersoft.api.model.SystemRoleBean;
import com.bettersoft.api.model.SystemRoleDepartmentBean;
import com.bettersoft.api.model.SystemRoleMenuBean;
import com.bettersoft.api.service.interfaces.ISystemRoleService;
import com.bettersoft.api.utils.Constants;
import com.bettersoft.api.utils.EntityUtil;
import com.bettersoft.api.utils.LoginUserInfo;
import com.bettersoft.api.utils.PageInfoUtils;
import com.bettersoft.api.validmodel.SystemRoleValidate;

@Service("systemRoleService")
public class SystemRoleService extends BaseService implements
		ISystemRoleService {
	@Resource(name = "systemRoleDao")
	private ISystemRoleDao systemRoleDao;

	public SystemRoleValidate findRoleByCode(String roleCode) throws Exception {
		SystemRoleBean role = systemRoleDao.findEntity(SystemRoleBean.class,
				roleCode);
		if (null == role)
			throw new ClassNotFoundException();

		SystemRoleValidate roleValidate = (SystemRoleValidate) EntityUtil
				.cloneToEntity(SystemRoleValidate.class, role);

		if (null != role.getDepartment()) {
			roleValidate.setDepartmentCode(role.getDepartment()
					.getDepartmentCode());
			roleValidate.setDepartmentName(role.getDepartment()
					.getDepartmentDesc());
		}

		if (null != role.getAuthMenus() && !role.getAuthMenus().isEmpty()) {
			for (SystemRoleMenuBean menuPerm : role.getAuthMenus()) {
				Map<String, Object> menuPermMap = EntityUtil
						.convertEntityToMap(menuPerm);

				roleValidate.getMenuPerms().add(menuPermMap);
			}
		}

		if (null != role.getAuthDatas() && !role.getAuthDatas().isEmpty()) {
			List<String> depts = new ArrayList<String>();
			for (SystemRoleDepartmentBean roleDept : role.getAuthDatas())
				depts.add(roleDept.getDepartmentCode());

			roleValidate.setDeptDatas(StringUtils.join(depts, ","));
		}

		return roleValidate;
	}

	public Map<String, Object> searchRoles(Map<String, Object> queryParam)
			throws Exception {
		List<SystemRoleBean> roles = systemRoleDao.searchRoles(queryParam);

		if (null != roles && !roles.isEmpty()) {
			for (SystemRoleBean role : roles) {
				if (null != role.getDepartment()) {
					role.setDepartmentCode(role.getDepartment()
							.getDepartmentCode());
					role.setDepartmentDesc(role.getDepartment()
							.getDepartmentDesc());
				}
			}
		}
		Number count = systemRoleDao.searchRoleCount(queryParam);

		return new PageInfoUtils(queryParam, roles, count).getPageInfoMap();
	}

	public List<MenuTreeBean> searchMenus() throws Exception {
		Map<String, Object> queryParam = new HashMap<String, Object>();
		queryParam.put(Constants.QUERY_SPEC_CONDITION, "parentMenu is null");
		queryParam.put(Constants.QUERY_ORDER_BY, "priority");

		List<MenuTreeBean> menus = systemRoleDao.searchMenus(queryParam);
		if (null != menus && !menus.isEmpty()) {
			for (MenuTreeBean menu : menus) {
				setParentMenu(menu);
			}
		}

		return menus;
	}

	public List<SystemDepartmentBean> searchDepartments() throws Exception {
		Map<String, Object> queryParam = new HashMap<String, Object>();
		queryParam.put(Constants.QUERY_SPEC_CONDITION,
				"parentDepartment is null");
		queryParam.put(Constants.QUERY_ORDER_BY, "departmentCode");

		List<SystemDepartmentBean> departments = systemRoleDao
				.searchDepartments(queryParam);
		if (null != departments && !departments.isEmpty()) {
			for (SystemDepartmentBean dept : departments) {
				setParentDepartment(dept);
			}
		}

		return departments;
	}

	@Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
	public void saveRole(SystemRoleValidate roleValidate,
			LoginUserInfo loginUser) throws Exception {
		SystemRoleBean role = (SystemRoleBean) EntityUtil.cloneToEntity(
				SystemRoleBean.class, roleValidate);

		role.setRoleCode(roleValidate.getRoleCode());

		if (checkEntityOccupied(role.getRoleCode()))
			throw new ValidateDataOccupiedException();

		if (!StringUtils.isEmpty(role.getDepartmentCode())) {
			SystemDepartmentBean dept = systemRoleDao.findEntity(
					SystemDepartmentBean.class, role.getDepartmentCode());
			if (null == dept)
				throw new ClassNotFoundException();

			role.setDepartment(dept);
		}

		if (null != roleValidate.getMenuPerms()
				&& !roleValidate.getMenuPerms().isEmpty()) {
			for (Map<String, Object> menuPermMap : roleValidate.getMenuPerms()) {
				SystemRoleMenuBean roleMenu = EntityUtil.convertMapToEntity(
						SystemRoleMenuBean.class, menuPermMap);

				roleMenu.setRoleCode(role.getRoleCode());
				roleMenu.setCreateDateTime(new Timestamp(System
						.currentTimeMillis()));

				role.getAuthMenus().add(roleMenu);
			}
		}

		if (!StringUtils.isEmpty(roleValidate.getDeptDatas())) {
			String deptDatas[] = StringUtils.split(roleValidate.getDeptDatas(),
					",");

			for (String deptCode : deptDatas) {
				SystemDepartmentBean dept = systemRoleDao.findEntity(
						SystemDepartmentBean.class, deptCode);
				if (null == dept)
					throw new ClassNotFoundException();

				SystemRoleDepartmentBean roleDept = new SystemRoleDepartmentBean();
				roleDept.setRoleCode(role.getRoleCode());
				roleDept.setDepartmentCode(deptCode);

				roleDept.setCreateDateTime(new Timestamp(System
						.currentTimeMillis()));

				role.getAuthDatas().add(roleDept);
			}
		}

		role.setCreateBy(loginUser.getUserCode());
		role.setCreateDateTime(new Timestamp(System.currentTimeMillis()));
		role.setCreateUserName(loginUser.getUserName());

		systemRoleDao.save(role);
	}

	@Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
	public void updateRole(SystemRoleValidate roleValidate,
			LoginUserInfo loginUser) throws Exception {
		String roleCode = roleValidate.getRoleCode();

		SystemRoleBean editRole = systemRoleDao.findEntity(
				SystemRoleBean.class, roleCode);

		EntityUtil.compareCopyEntity(roleValidate, editRole);

		if (!StringUtils.isEmpty(editRole.getDepartmentCode())) {
			SystemDepartmentBean dept = systemRoleDao.findEntity(
					SystemDepartmentBean.class, editRole.getDepartmentCode());
			if (null == dept)
				throw new ClassNotFoundException();

			editRole.setDepartment(dept);
		}

		if (null != roleValidate.getMenuPerms()
				&& !roleValidate.getMenuPerms().isEmpty()) {
			editRole.getAuthMenus().clear();
			Map<String, Object> queryParam = new HashMap<String, Object>();
			queryParam.put("roleCode", roleCode);

			for (Map<String, Object> menuPermMap : roleValidate.getMenuPerms()) {
				String menuCode = MapUtils.getString(menuPermMap, "menuCode",
						StringUtils.EMPTY);
				String permissionCode = MapUtils.getString(menuPermMap,
						"permissionCode", StringUtils.EMPTY);

				queryParam.put("menuCode", menuCode);
				queryParam.put("permissionCode", permissionCode);

				SystemRoleMenuBean roleMenu = systemRoleDao
						.searchRoleMenu(queryParam);

				if (null == roleMenu) {
					roleMenu = EntityUtil.convertMapToEntity(
							SystemRoleMenuBean.class, menuPermMap);

					roleMenu.setRoleCode(roleCode);
					roleMenu.setCreateDateTime(new Timestamp(System
							.currentTimeMillis()));
				}

				editRole.getAuthMenus().add(roleMenu);
			}
		}

		if (!StringUtils.isEmpty(roleValidate.getDeptDatas())) {
			String deptDatas[] = StringUtils.split(roleValidate.getDeptDatas(),
					",");
			editRole.getAuthDatas().clear();
			Map<String, Object> queryParam = new HashMap<String, Object>();
			queryParam.put("roleCode", roleCode);

			for (String deptCode : deptDatas) {
				queryParam.put("departmentCode", deptCode);

				SystemRoleDepartmentBean roleDept = systemRoleDao
						.searchRoleDept(queryParam);
				if (null == roleDept) {
					roleDept = new SystemRoleDepartmentBean();
					SystemDepartmentBean dept = systemRoleDao.findEntity(
							SystemDepartmentBean.class, deptCode);
					if (null == dept)
						throw new ClassNotFoundException();

					roleDept.setRoleCode(roleCode);
					roleDept.setDepartmentCode(deptCode);

					roleDept.setCreateDateTime(new Timestamp(System
							.currentTimeMillis()));
				}

				editRole.getAuthDatas().add(roleDept);
			}
		}

		editRole.setLastModifyBy(loginUser.getUserCode());
		editRole.setLastModifyDateTime(new Timestamp(System.currentTimeMillis()));
		editRole.setLastModifyUserName(loginUser.getUserName());

		systemRoleDao.update(editRole);
	}

	@Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
	public void deleteRole(String roleCode) throws Exception {
		SystemRoleBean delRole = systemRoleDao.findEntity(SystemRoleBean.class,
				roleCode);

		if (null == delRole)
			throw new ClassNotFoundException();

		if (0 < systemRoleDao.searchRoleInUseCount(roleCode).longValue())
			throw new ValidateDataOccupiedException();

		systemRoleDao.delete(delRole);
	}

	public boolean checkEntityOccupied(Serializable id) throws Exception {
		SystemRoleBean existsRole = systemRoleDao.findEntity(
				SystemRoleBean.class, id);

		return null != existsRole;
	}

	private void setParentMenu(MenuTreeBean menu) {
		if (null != menu.getParentMenu()) {
			menu.setParentCode(menu.getParentMenu().getMenuCode());
			menu.setParentName(menu.getParentMenu().getMenuName());
		}

		if (null != menu.getChildMenus() && !menu.getChildMenus().isEmpty()) {
			for (MenuTreeBean childMenu : menu.getChildMenus()) {
				childMenu.setParentCode(menu.getMenuCode());
				childMenu.setParentName(menu.getMenuName());

				if (null != childMenu.getChildMenus()
						&& !menu.getChildMenus().isEmpty())
					setParentMenu(childMenu);
			}
		}

		if (null != menu.getPermissions() && !menu.getPermissions().isEmpty()) {
			for (SystemMenuPermissionBean menuPerm : menu.getPermissions())
				menuPerm.setPermissionDesc(menuPerm.getPermission()
						.getPermissionDesc());
		}
	}

	private void setParentDepartment(SystemDepartmentBean dept) {
		if (null != dept.getParentDepartment()) {
			dept.setParentCode(dept.getParentDepartment().getDepartmentCode());
			dept.setParentDesc(dept.getParentDepartment().getDepartmentDesc());
		}

		if (null != dept.getChildDepartments()
				&& !dept.getChildDepartments().isEmpty()) {
			for (SystemDepartmentBean childDept : dept.getChildDepartments()) {
				childDept.setParentCode(childDept.getParentDepartment()
						.getDepartmentCode());
				childDept.setParentDesc(childDept.getParentDepartment()
						.getDepartmentDesc());

				setParentDepartment(childDept);
			}
		}
	}
}
