package com.bettersoft.api.utils;

import java.util.List;

import org.apache.commons.httpclient.Header;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.NameValuePair;
import org.apache.commons.httpclient.SimpleHttpConnectionManager;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.params.HttpClientParams;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class HttpClientHelper {
	private static Log httpClientLog = LogFactory
			.getLog(HttpClientHelper.class);
	private final static int TIMEOUT = 20000;

	public String getHtmlContext(String url, List<NameValuePair> parameters,
			String type, Header... headers) throws Exception {
		String result = null;
		Exception ex = null;
		HttpClient httpclient = new HttpClient();
		HttpClientParams httpClientParams = httpclient.getParams();
		httpClientParams.setContentCharset("UTF-8");
		HttpMethod httpMethod = null;
		if ("get".equalsIgnoreCase(type)) {
			httpMethod = createGetMethod(url, parameters, headers);
		} else {
			httpMethod = createPostMethod(url, parameters, headers);
		}
		try {
			int r = httpclient.executeMethod(httpMethod);
			result = httpMethod.getResponseBodyAsString();

			httpClientLog.info("respsone status :" + r);
		} catch (Exception e) {
			e.printStackTrace();
			ex = e;
		} finally {
			if (httpMethod != null) {
				httpMethod.releaseConnection();
			}
			SimpleHttpConnectionManager connectManager = (SimpleHttpConnectionManager) httpclient
					.getHttpConnectionManager();
			connectManager.shutdown();
		}
		if (ex != null) {
			throw new Exception("访问" + url + "失败");
		}
		return result;
	}

	public String getHtmlContext(String url, List<NameValuePair> parameters,
			String Type, String timeout) throws Exception {
		String result = null;
		Exception ex = null;
		HttpClient httpclient = new HttpClient();
		HttpClientParams httpClientParams = httpclient.getParams();
		httpClientParams.setContentCharset("UTF-8");
		int s = 5000;
		try {
			s = Integer.parseInt(timeout);
		} catch (Exception e) {
			s = TIMEOUT;
		}
		httpclient.getHttpConnectionManager().getParams()
				.setConnectionTimeout(s);
		httpclient.getHttpConnectionManager().getParams().setSoTimeout(s);
		HttpMethod httpMethod = null;
		if ("get".equalsIgnoreCase(Type)) {
			httpMethod = createGetMethod(url, parameters);
		} else {
			httpMethod = createPostMethod(url, parameters);
		}
		try {
			int r = httpclient.executeMethod(httpMethod);
			result = httpMethod.getResponseBodyAsString();

			httpClientLog.info("respsone status :" + r);
		} catch (Exception e) {
			e.printStackTrace();
			ex = e;
		} finally {
			if (httpMethod != null) {
				httpMethod.releaseConnection();
			}
			SimpleHttpConnectionManager connectManager = (SimpleHttpConnectionManager) httpclient
					.getHttpConnectionManager();
			connectManager.shutdown();
		}
		if (ex != null) {
			throw new Exception("访问" + url + "失败");
		}
		return result;
	}

	private HttpMethod createGetMethod(String url,
			List<NameValuePair> parameters, Header... headers) {
		GetMethod getMethod = new GetMethod(url);
		if (parameters != null && parameters.size() > 0) {
			NameValuePair[] pairs = new NameValuePair[parameters.size()];
			for (int i = 0; i < parameters.size(); i++) {
				pairs[i] = parameters.get(i);
			}
			getMethod.setQueryString(pairs);
		}

		if (null != headers && 0 < headers.length) {
			for (Header header : headers)
				getMethod.addRequestHeader(header);
		}

		return getMethod;
	}

	private HttpMethod createPostMethod(String url,
			List<NameValuePair> parameters, Header... headers) {
		PostMethod postMethod = new PostMethod(url);
		if (parameters != null && parameters.size() > 0) {
			NameValuePair[] pairs = new NameValuePair[parameters.size()];
			for (int i = 0; i < parameters.size(); i++) {
				pairs[i] = parameters.get(i);
			}
			postMethod.addParameters(pairs);
		}

		if (null != headers && 0 < headers.length) {
			for (Header header : headers)
				postMethod.addRequestHeader(header);
		}

		return postMethod;
	}
}
