package com.bettersoft.api.utils;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.InputStream;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;

import org.apache.commons.codec.binary.Base64;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.NameValuePair;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicHeader;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;
import org.apache.log4j.Logger;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSONObject;
import com.bettersoft.api.utils.Constants.CommonConsts;


@Component
public class HttpUtils {

	private static Logger logger = Logger.getLogger(HttpUtils.class);
	private static final String HMAC_SHA1_ALGORITHM = "HmacSHA1";  

	private HttpUtils() {
	}

	/**
	 * 调用JiakeAPI接口
	 * 
	 * @param target
	 *            请求接口信息
	 * @param param
	 *            请求参数
	 * 
	 * @return 请求返回结果
	 */
	public static JSONObject callJiakeApi(JSONObject param,String  method) {
		JSONObject result = null;
		
		SimpleDateFormat format = new SimpleDateFormat(CommonConsts.COMPACT_DATETIME_FORMAT);
		String ts = format.format(new Date()).toString();
		Properties prop = new HttpUtils().getProperties();
		
		JSONObject target =new JSONObject();
		target.put("ts", ts);
		target.put("appid", prop.get("jiakeAppid").toString());
		target.put("sign", getSign(ts, prop));
		target.put("method", method);
		target.put("para", param.toString());

		String responseBody = null;

		try {
			responseBody = HttpUtils.post(prop.get("jiakeUrl").toString(), null, null, target);
		} catch (Exception e) {
			logger.error("JiakeAPI请求失败", e);
			return null;
		}

		try {
			result = JSONObject.parseObject(responseBody);
		} catch (Exception e) {
			logger.error("JiakeAPI请求结果解析失败", e);
		}

		return result;
	}
	

	

	/**
	 * 生成签名并返回
	 * 
	 * @return 加密后的签名
	 */
	private static String getSign(String ts, Properties prop) {

		String sign=MD5Utils.MD5Encode(
				prop.get("jiakeAppid").toString() + prop.get("jiakeSecret").toString() + ts,
				CommonConsts.DEFAULT_ENCODE);
		return  sign;
	}
	
	/**
	 * 生成叫号接口签名并返回
	 * 
	 * @return 加密后的签名
	 */
	private static String getQMSign(String ts, Properties prop,int intNonce) {
		//int intNonce =  rand.nextInt(9989)+10;
		String strSorce = "GrantCode=" + 
				prop.get("GrantCode").toString() +
				"&Nonce="+
				intNonce +
				"&OpenId="+
				prop.get("OpenId").toString() +
				"&Timestamp="+ts;
		
		//String strSorce ="GrantCode=ba67d4fa46&Nonce=11886&OpenId=d14c1559e87b747d577c834b275a4310&Timestamp=1465185768";
		
		String OpenKey =prop.get("OpenKey").toString();// "LyvrkvkxRkG2R6aM55bXpPwjYAbkEXTbVnKwfDYvVHjNwNFAmx";
		
		String signStr = genHMAC(strSorce,OpenKey);
		//signStr = base64encoder.encode(signStr.getBytes());
		System.out.println("signStr:"+signStr);
		//signStr = base64encoder.encode(signStr.getBytes());
		System.out.println("signStr_encode:"+signStr);

		return signStr;
	}
	

	  
	    /** 
	     * 使用 HMAC-SHA1 签名方法对data进行签名 
	     *  
	     * @param data 
	     *            被签名的字符串 
	     * @param key 
	     *            密钥      
	     * @return  
	                      加密后的字符串 
	     */  
	    public static String genHMAC(String data, String key) {  
	        byte[] result = null;  
	        try {  
	            //根据给定的字节数组构造一个密钥,第二参数指定一个密钥算法的名称    
	            SecretKeySpec signinKey = new SecretKeySpec(key.getBytes(), HMAC_SHA1_ALGORITHM);  
	            //生成一个指定 Mac 算法 的 Mac 对象    
	            Mac mac = Mac.getInstance(HMAC_SHA1_ALGORITHM);  
	            //用给定密钥初始化 Mac 对象    
	            mac.init(signinKey);  
	            //完成 Mac 操作     
	            byte[] rawHmac = mac.doFinal(data.getBytes());  
	            result = Base64.encodeBase64(rawHmac);  
	  
	        } catch (NoSuchAlgorithmException e) {  
	            System.err.println(e.getMessage());  
	        } catch (InvalidKeyException e) {  
	            System.err.println(e.getMessage());  
	        }  
	        if (null != result) {  
	            return new String(result);  
	        } else {  
	            return null;  
	        }  
	    }  

	/**
	 * 通用HTTP协议GET请求方式
	 * 
	 * @param url
	 *            请求地址
	 * @param param
	 *            请求参数
	 * 
	 * @return 请求返回信息
	 * @throws Exception
	 */
	public static String get(String url, Map<String, Object> param) throws Exception {

		CloseableHttpClient httpClient = HttpClients.createDefault();

		StringBuffer buffer = new StringBuffer();
		for (String name : param.keySet()) {
			if (!((String) param.get(name)).isEmpty()) {
				buffer.append(name).append("=")
						.append(java.net.URLEncoder.encode((String) param.get(name), CommonConsts.DEFAULT_ENCODE))
						.append("&");
			}
		}

		HttpGet httpGet = new HttpGet(url + "?" + buffer.substring(0, buffer.length() - 1));
		CloseableHttpResponse httpresponse = null;
		String responseBody = null;

		try {

			httpresponse = httpClient.execute(httpGet);
			int statusCode = httpresponse.getStatusLine().getStatusCode();
			if (statusCode == 200) {
				HttpEntity entity = httpresponse.getEntity();
				responseBody = EntityUtils.toString(entity);
				logger.info("发送GET请求结束, 请求结果 = " + responseBody);
			} else {
				logger.info("发送GET请求错误, 服务器为正常返回, 错误Code = " + statusCode);
				return null;
			}
		} catch (Exception e) {
			logger.info("发送GET请求错误", e);
			return null;
		} finally {
			if (httpClient != null) {
				httpClient.close();
			}
			if (httpresponse != null) {
				httpresponse.close();
			}
		}

		return responseBody;
	}

	/**
	 * 通用HTTP协议POST请求方式
	 * 
	 * @param url
	 *            请求地址
	 * @param encode
	 *            编码
	 * @param header
	 *            请求头信息
	 * @param param
	 *            请求参数
	 * @return 请求返回信息
	 * @throws Exception
	 */
	public static String post(String url, String encode, Map<String, String> header, Map<String, Object> param)
			throws Exception {
		CloseableHttpClient httpClient = HttpClients.createDefault();
		String responseBody = null;

		HttpPost httpPost = new HttpPost(url);
		CloseableHttpResponse httpresponse = null;

		try {

			// 设置请求头
			if (header != null) {
				for (String key : header.keySet()) {
					httpPost.setHeader(key, header.get(key));
				}
			}

			// 设置请求参数
			List<NameValuePair> nameValuePairs = new ArrayList<NameValuePair>();
			if (param != null) {
				for (String name : param.keySet()) {
					nameValuePairs.add(new BasicNameValuePair(name, String.valueOf(param.get(name))));
				}
			}

			// 设置字符编码
			if (encode == null) {
				encode = CommonConsts.DEFAULT_ENCODE;
			}

			httpPost.setEntity(new UrlEncodedFormEntity(nameValuePairs, encode));
			

			httpresponse = httpClient.execute(httpPost);
			int statusCode = httpresponse.getStatusLine().getStatusCode();
			if (statusCode == 200) {
				HttpEntity entity = httpresponse.getEntity();
				responseBody = EntityUtils.toString(entity);
				logger.info("发送POST请求结束, 请求结果 = " + responseBody);
			} else {
				logger.info("发送POST请求错误, 服务器为正常返回, 错误Code = " + statusCode);
				return null;
			}
		} catch (Exception e) {
			logger.error("发送POST请求错误", e);
			return null;
		} finally {
			if (httpClient != null) {
				httpClient.close();
			}
			if (httpresponse != null) {
				httpresponse.close();
			}
		}

		return responseBody;
	}

	
	private Properties getProperties() {
		Properties properties = new Properties();
		String configFile = "D:\\jiake\\jiakeApi.properties";

		try (InputStream is = new BufferedInputStream(new FileInputStream(configFile));) {
			properties.load(is);
		} catch (Exception e) {
			logger.error("jiake配置文件读取失败", e);
			return null;
		}

		return properties;
	}

	@SuppressWarnings("resource")
	public static JSONObject callHttpsJiakeApiPost(JSONObject target ){
		JSONObject jsonObjectResult=null;
		String errorMsg="";
		
        HttpClient httpClient = null;
        HttpPost httpPost = null;
        String result = null;
        HttpResponse response=null;
 
		SimpleDateFormat format = new SimpleDateFormat(CommonConsts.COMPACT_DATETIME_FORMAT);
		String ts = format.format(new Date()).toString();
		Properties prop = new HttpUtils().getProperties();
		
		target.put("ts", ts);
		target.put("appid", prop.get("jiakeAppid"));		
		target.put("sign", getSign(ts, prop));

		String url = prop.get("jiakeUrl").toString();
        logger.info("######调用参数明细：target"+target);

        try{
           /* httpClient = new SSLClient();
            httpPost = new HttpPost(url);
            //httpPost.addHeader("Content-Type", "application/json");
            StringEntity se = new StringEntity(target.toString(),"utf-8");
           // se.setContentType("application/x-www-form-urlencoded;charset=UTF-8");
            //se.setContentEncoding("UTF-8");  
           // se.setContentType("application/x-www-form-urlencoded;charset=UTF-8");
            //se.setContentEncoding(new BasicHeader("Content-Type", "application/json;charset=utf-8"));
            // se.setContentEncoding(new BasicHeader("Content-Type", "application/x-www-form-urlencoded;charset=utf-8"));
            //httpPost.setHeader("Accept","aplication/json");  
            
            //httpPost.addHeader("Content-Type","application/json;charset=UTF-8");  
            
            httpPost.addHeader("Content-Type","application/x-www-form-urlencoded;charset=UTF-8");
            httpPost.setEntity(se);
            response = httpClient.execute(httpPost);
        	 se.setContentType("text/json");
            se.setContentEncoding(new BasicHeader("Content-Type", "application/json"));
            httpPost.setEntity(se);*/
        	httpClient = new SSLClient();
            httpPost = new HttpPost(url);
        	
        	// 设置请求参数
			List<NameValuePair> nameValuePairs = new ArrayList<NameValuePair>();
			if (target != null) {
				for (String name : target.keySet()) {
					nameValuePairs.add(new BasicNameValuePair(name, String.valueOf(target.get(name))));
				}
			}
			httpPost.setEntity(new UrlEncodedFormEntity(nameValuePairs, "utf-8"));

            response = httpClient.execute(httpPost);
            if(response != null && response.getStatusLine().getStatusCode()==200){
                HttpEntity resEntity = response.getEntity();
                if(resEntity != null){ 
                    result = EntityUtils.toString(resEntity,"utf-8");
                    System.out.println(result);
                    try {
                    	jsonObjectResult=JSONObject.parseObject(result);
            		} catch (Exception e) {
            			errorMsg="jiakeAPI请求失败！";
            			logger.error("JiakeAPI请求结果解析失败", e);
            		}
                }    
            }else{
            	/*HttpEntity resEntity = response.getEntity();
            	result = EntityUtils.toString(resEntity,"utf-8");
            	System.out.println(result);*/
            	errorMsg="jiakeAPI请求失败！请检查参数是否异常！";
            	if (response != null) {
            		logger.error("jiakeAPI请求失败！getStatusCode= "+response.getStatusLine().getStatusCode());
            	}
            }
        }catch(Exception ex){
            errorMsg="jiakeAPI请求失败！请联系系统管理员！";
            logger.error("jiakeAPI请求失败！", ex);
        }
        if(jsonObjectResult==null){
        	jsonObjectResult= new JSONObject();
        	jsonObjectResult.put("jk_code", 2);
        	jsonObjectResult.put("jk_msg", errorMsg);
        }
        return  jsonObjectResult;
    }
	public static void main(String[] args) {
		JSONObject target = new JSONObject();
		JSONObject param = new JSONObject();
		
	
		param.put("code", "031gTNFa1AbGpz0ZvrIa18L6e32gTNFg");
		target.put("para", param.toString());
		target.put("method", "wx_user");
		target.put("sub_mch_id","1491775212");
		
		callHttpsJiakeApiPost(target);
	}

	public static String doPost(String url, String paramMaps) {
		String result = null;

		HttpPost httpPost = new HttpPost(url);
		HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
		CloseableHttpClient closeableHttpClient = httpClientBuilder.build();
		try {
			httpPost.setHeader("Content-Type","application/json");
			httpPost.setEntity(new StringEntity(paramMaps
					, ContentType.create("application/json", "utf-8")));

			HttpResponse response = closeableHttpClient.execute( httpPost );
			if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
				result = EntityUtils.toString(response.getEntity(), "UTF-8").trim();
			}
		} catch (Exception e) {
			throw new RuntimeException("HTTP post failed.", e);
		} finally {
			httpPost.abort();

			try {
				closeableHttpClient.close();
			} catch (Exception e2) {
				throw new RuntimeException("Close CloseableHttpClient failed.", e2);
			}
		}

		return result;
	}
}
