package com.bettersoft.api.utils;

import java.util.Random;
import java.util.UUID;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.DESKeySpec;
import javax.crypto.spec.IvParameterSpec;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang.StringUtils;

public class StringDigestUtil {
	private static final String DEFAULT_CHARSET = "UTF-8";

	public static String generalUUID() {
		return UUID.randomUUID().toString().replaceAll("-", "").toLowerCase();
	}

	public static String encodeMD5Hex(String targetStr) {
		return DigestUtils.md5Hex(targetStr);
	}

	public static String encodeBase64Hex(String targetStr) throws Exception {
		return new String(encodeBase64Hex(targetStr.getBytes(DEFAULT_CHARSET)));
	}

	public static byte[] encodeBase64Hex(byte[] bytes) {
		return Base64.encodeBase64(bytes);
	}

	public static String decodeBase64Hex(String encodeStr) throws Exception {
		return new String(decodeBase64Hex(encodeStr.getBytes()),
				DEFAULT_CHARSET);
	}

	public static byte[] decodeBase64Hex(byte[] bytes) {
		return Base64.decodeBase64(bytes);
	}

	public static String generalKey() throws Exception {
		KeyGenerator keyGenerator = KeyGenerator.getInstance("DES");
		SecretKey secretKey = keyGenerator.generateKey();

		return new String(Base64.encodeBase64(secretKey.getEncoded()));
	}

	public static String encrypt(String message, String secretKey)
			throws Exception {
		Cipher cipher = Cipher.getInstance("DES/CBC/PKCS5Padding");

		byte[] keyBytes = decodeBase64Hex(secretKey.getBytes(DEFAULT_CHARSET));
		DESKeySpec desKeySpec = new DESKeySpec(keyBytes);
		SecretKeyFactory keyFactory = SecretKeyFactory.getInstance("DES");
		SecretKey key = keyFactory.generateSecret(desKeySpec);

		IvParameterSpec iv = new IvParameterSpec(keyBytes);
		cipher.init(Cipher.ENCRYPT_MODE, key, iv);

		return new String(encodeBase64Hex(cipher.doFinal(message
				.getBytes("UTF-8"))));
	}

	public static String encryptHex(String message, String secretKey)
			throws Exception {
		Cipher cipher = Cipher.getInstance("DES/CBC/PKCS5Padding");

		byte[] keyBytes = decodeBase64Hex(secretKey.getBytes(DEFAULT_CHARSET));
		DESKeySpec desKeySpec = new DESKeySpec(keyBytes);
		SecretKeyFactory keyFactory = SecretKeyFactory.getInstance("DES");
		SecretKey key = keyFactory.generateSecret(desKeySpec);

		IvParameterSpec iv = new IvParameterSpec(keyBytes);
		cipher.init(Cipher.ENCRYPT_MODE, key, iv);

		return byte2HexString(cipher.doFinal(message.getBytes("UTF-8")))
				.toUpperCase();
	}

	public static String decrypt(String message, String secretKey)
			throws Exception {
		byte[] byteSrc = decodeBase64Hex(message.getBytes(DEFAULT_CHARSET));
		byte[] keyBytes = decodeBase64Hex(secretKey.getBytes(DEFAULT_CHARSET));

		Cipher cipher = Cipher.getInstance("DES/CBC/PKCS5Padding");
		DESKeySpec desKeySpec = new DESKeySpec(keyBytes);

		SecretKeyFactory keyFactory = SecretKeyFactory.getInstance("DES");
		SecretKey key = keyFactory.generateSecret(desKeySpec);

		IvParameterSpec iv = new IvParameterSpec(keyBytes);
		cipher.init(Cipher.DECRYPT_MODE, key, iv);

		return new String(cipher.doFinal(byteSrc), DEFAULT_CHARSET);
	}

	public static String decryptHex(String hexMessage, String secretKey)
			throws Exception {
		byte[] byteSrc = hex2ByteArray(hexMessage.toLowerCase());
		byte[] keyBytes = decodeBase64Hex(secretKey.getBytes(DEFAULT_CHARSET));

		Cipher cipher = Cipher.getInstance("DES/CBC/PKCS5Padding");
		DESKeySpec desKeySpec = new DESKeySpec(keyBytes);

		SecretKeyFactory keyFactory = SecretKeyFactory.getInstance("DES");
		SecretKey key = keyFactory.generateSecret(desKeySpec);

		IvParameterSpec iv = new IvParameterSpec(keyBytes);
		cipher.init(Cipher.DECRYPT_MODE, key, iv);

		return new String(cipher.doFinal(byteSrc), DEFAULT_CHARSET);
	}

	public static String decryptMix(String message) throws Exception {
		String encryptStr = StringUtils.substring(message, 6,
				message.length() - 6);
		String encryptKey = StringUtils.substring(message, 0, 6);
		encryptKey += StringUtils.substring(message, message.length() - 6);

		return decrypt(encryptStr, encryptKey);
	}

	public static String byte2HexString(byte[] bytes) {
		StringBuilder stringBuilder = new StringBuilder("");
		if (bytes == null || bytes.length <= 0)
			return StringUtils.EMPTY;

		for (int i = 0; i < bytes.length; i++) {
			int v = bytes[i] & 0xFF;
			String hv = Integer.toHexString(v);
			if (hv.length() < 2) {
				stringBuilder.append(0);
			}
			stringBuilder.append(hv);
		}

		return stringBuilder.toString();
	}

	public static byte[] hex2ByteArray(String inHex) {
		int hexlen = inHex.length();
		byte[] result;
		if (hexlen % 2 == 1) {
			// 奇数
			hexlen++;
			result = new byte[(hexlen / 2)];
			inHex = "0" + inHex;
		} else {
			// 偶数
			result = new byte[(hexlen / 2)];
		}
		int j = 0;
		for (int i = 0; i < hexlen; i += 2) {
			result[j] = (byte) Integer.parseInt(inHex.substring(i, i + 2), 16);
			j++;
		}
		return result;
	}

	public static String mixEncrypt(String encryptStr, String encryptKey) {
		return StringUtils.substring(encryptKey, 0, 6) + encryptStr
				+ StringUtils.substring(encryptKey, 6);
	}

	public static String getRandomString(int length) {
		String str = "zxcvbnmlkjhgfdsaqwertyuiopQWERTYUIOPASDFGHJKLZXCVBNM1234567890,./?{}-_";
		Random random = new Random(System.currentTimeMillis());
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < length; ++i) {
			// 从62个的数字或字母中选择
			int number = random.nextInt(70);
			sb.append(str.charAt(number));
		}
		return sb.toString();
	}
}
