package com.bettersoft.api.utils;

import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.Writer;
import java.sql.Timestamp;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Map;

import org.apache.commons.collections.MapUtils;
import org.apache.commons.io.IOUtils;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;

public class VelocityUtils {
	private static final String DEFAULT_CHARSET = "UTF-8";

	private static VelocityContext initContext(Map<String, Object> model) {
		VelocityContext context = new VelocityContext();
		NumberFormat df = new DecimalFormat(getNumberPattern());
		df.setGroupingUsed(true);
		df.setMaximumFractionDigits(Constants.DOT_LENGTH);

		model.put("dateFormat", new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"));
		model.put("decimalFormat", df);

		model.put("printDateTime", new Timestamp(System.currentTimeMillis()));
		for (String key : model.keySet()) {
			context.put(key, MapUtils.getObject(model, key));
		}

		return context;
	}

	public static void renderVelocity(OutputStream output,
			Map<String, Object> model, String templateName) throws Exception {
		VelocityContext context = initContext(model);

		Writer writer = null;
		Reader reader = null;

		try {
			writer = new OutputStreamWriter(output, DEFAULT_CHARSET);
			reader = new InputStreamReader(
					VelocityUtils.class.getResourceAsStream("/" + templateName),
					DEFAULT_CHARSET);

			Velocity.evaluate(context, writer, "", reader);
			writer.flush();
		} catch (Exception ex) {
			ex.printStackTrace();
		} finally {
			if (null != writer)
				IOUtils.closeQuietly(writer);
			if (null != reader)
				IOUtils.closeQuietly(reader);
		}
	}

	private static String getNumberPattern() {
		String patten = "#";
		if (0 < Constants.DOT_LENGTH)
			patten += ".";
		for (int i = 0; i < Constants.DOT_LENGTH; i++) {
			patten += "0";
		}

		return patten;
	}
}
