package com.bettersoft.api.validmodel;

import com.alibaba.fastjson.JSONObject;
import com.bettersoft.api.annotation.FieldCommon;
import com.bettersoft.api.model.OnlineItemBean;
import com.bettersoft.api.utils.StringDigestUtil;
import com.fasterxml.jackson.annotation.JsonFormat;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

public class OnlineItemPackageValidate extends BaseValidate {
    private static final long serialVersionUID = -2115548412258030196L;

    private String uuid;
    @FieldCommon(common = "商品ID")
    private int itemId;
    @FieldCommon(common = "套餐组编号")
    private String packageCode;
    @FieldCommon(common = "套餐组或商品描述")
    private String packageDesc;
    @FieldCommon(common = "上级套餐组编号")
    private String parentPackage;
    @FieldCommon(common = "商品单价")
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    private BigDecimal price = new BigDecimal(0l);
    @FieldCommon(common = "商品数量")
    private int quantity = 1;
    @FieldCommon(common = "套餐包类型")
    private String packageType; // P：套餐，I：商品，M：扩展组，A：附加属性
    @FieldCommon(common = "套餐或商品图片")
    private String imagePath;
    @FieldCommon(common = "订单号")
    private String orderNo;
    @FieldCommon(common = "是否已选")
    private boolean checked;
    @FieldCommon(common = "行号")
    transient private int lineNo;
    transient private String pmodify;
    transient boolean isOffline;
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    private BigDecimal totalPrice;
    private String custom;
    private JSONObject commodityInfo;
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    private BigDecimal countTotalPrice;

    private String defaultflag;

    public String getDefaultflag() {
        return defaultflag;
    }

    public void setDefaultflag(String defaultflag) {
        this.defaultflag = defaultflag;
    }

    private String strpagecode;

    public String getStrpagecode() {
        return strpagecode;
    }

    public void setStrpagecode(String strpagecode) {
        this.strpagecode = strpagecode;
    }

    public BigDecimal getCountTotalPrice() {
       return countTotalPrice==null ?new BigDecimal(0.00) :countTotalPrice.setScale(2, BigDecimal.ROUND_DOWN);
    }

    public void setCountTotalPrice(BigDecimal countTotalPrice ) {
        this.countTotalPrice = countTotalPrice;
    }

    public BigDecimal setCTotalPrice(int quantity,BigDecimal totalPrice){
        this.countTotalPrice=BigDecimal.valueOf(quantity).multiply(totalPrice);
        return countTotalPrice;
    }
    @FieldCommon(common = "套餐或商品子项")
    private List<OnlineItemPackageValidate> childPackages = new ArrayList<OnlineItemPackageValidate>();

    public OnlineItemPackageValidate() {
        this.uuid = StringDigestUtil.generalUUID();
    }

    public String getUuid() {
        return uuid;
    }

    public void setUuid(String uuid) {
        this.uuid = uuid;
    }

    public String getPackageCode() {
        return packageCode;
    }

    public void setPackageCode(String packageCode) {
        this.packageCode = packageCode;
    }

    public String getPackageDesc() {
        return packageDesc;
    }

    public void setPackageDesc(String packageDesc) {
        this.packageDesc = packageDesc;
    }

    public String getParentPackage() {
        return parentPackage;
    }

    public void setParentPackage(String parentPackage) {
        this.parentPackage = parentPackage;
    }

    public int getItemId() {
        return itemId;
    }

    public void setItemId(int itemId) {
        this.itemId = itemId;
    }

    public BigDecimal getPrice() {
        return price==null ?new BigDecimal(0.00) :price.setScale(2, BigDecimal.ROUND_DOWN);
    }

    public void setPrice(BigDecimal price) {
        this.price = price;
    }

    public int getQuantity() {
        return quantity;
    }

    public void setQuantity(int quantity) {
        this.quantity = quantity;
        this.setChildItemQuantity(null, quantity);
    }

    public String getPackageType() {
        return packageType;
    }

    public void setPackageType(String packageType) {
        this.packageType = packageType;
    }

    public String getImagePath() {
        return imagePath;
    }

    public void setImagePath(String imagePath) {
        this.imagePath = imagePath;
    }

    public String getOrderNo() {
        return orderNo;
    }

    public void setOrderNo(String orderNo) {
        this.orderNo = orderNo;
    }

    public int getLineNo() {
        return lineNo;
    }

    public void setLineNo(int lineNo) {
        this.lineNo = lineNo;
    }

    public boolean isChecked() {
        return checked;
    }

    public void setChecked(boolean checked) {
        this.checked = checked;
    }

    public String getPmodify() {
        return pmodify;
    }

    public void setPmodify(String pmodify) {
        this.pmodify = pmodify;
    }

    public boolean isOffline() {
        return isOffline;
    }

    public void setOffline(boolean offline) {
        isOffline = offline;
    }

    public List<OnlineItemPackageValidate> getChildPackages() {
        return childPackages;
    }

    public void setChildPackages(List<OnlineItemPackageValidate> childPackages) {
        this.childPackages = childPackages;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        OnlineItemPackageValidate that = (OnlineItemPackageValidate) o;
        return itemId == that.itemId &&
                Objects.equals(packageCode, that.packageCode) &&
                Objects.equals(orderNo, that.orderNo) &&
                //先添加有客制化的商品，再添加没有的，会导致有客制化的商品数量加一，所以我修改了equal条件，之后别的地方有问题再说
//                (childPackages == that.childPackages || childPackages.containsAll(that.childPackages));
//                (childPackages == that.childPackages || (childPackages.containsAll(that.childPackages))&&(that.childPackages.containsAll(childPackages)));
                Objects.equals(that.packageDesc,packageDesc) && childPackages .equals(that.childPackages);
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + itemId;
        return result;
    }

    private void setChildItemQuantity(List<OnlineItemPackageValidate> childPackages, int quantity) {
        if (null != childPackages && !childPackages.isEmpty()) {
            for (OnlineItemPackageValidate childItem : childPackages) {
                if (null != childItem.getChildPackages() && !childItem.getChildPackages().isEmpty()) {
                    setChildItemQuantity(childItem.getChildPackages(), quantity);
                }

                childItem.setQuantity(quantity);
            }
        } else {
            if (null != this.childPackages && !this.childPackages.isEmpty()) {
                for (OnlineItemPackageValidate childItem : this.childPackages) {
                    if (null != childItem.getChildPackages() && !childItem.getChildPackages().isEmpty()) {
                        setChildItemQuantity(childItem.getChildPackages(), quantity);
                    }
                }
            }
        }
    }

    public BigDecimal getTotalPrice() {
        return totalPrice;
    }

    public void setTotalPrice(BigDecimal totalPrice){
        this.totalPrice = totalPrice;
    }

    public BigDecimal setChildItemTotalPrice(List<OnlineItemPackageValidate> childPackages) {
        this.totalPrice = new BigDecimal(0);
        this.countTotalPrice=new BigDecimal(0);
        if (null != childPackages && !childPackages.isEmpty()) {
            for (OnlineItemPackageValidate childItem : childPackages) {
                if (null != childItem.getChildPackages() && !childItem.getChildPackages().isEmpty()) {
                    this.totalPrice = this.totalPrice.add(setChildItemTotalPrice(childItem.getChildPackages()));
                }
                this.totalPrice =this.totalPrice.add(childItem.getPrice());
            }
        } else {
            if (null != this.childPackages && !this.childPackages.isEmpty()) {
                for (OnlineItemPackageValidate childItem : this.childPackages) {
                    if (null != childItem.getChildPackages() && !childItem.getChildPackages().isEmpty()) {
                        this.totalPrice = this.totalPrice.add(setChildItemTotalPrice(childItem.getChildPackages()));
                    }
                }
            }
            this.totalPrice =this.totalPrice.add(this.price);

        }
        return this.totalPrice;
    }

    public String getCustom() {
        return custom;
    }

    public void setCustom(String custom) {
        this.custom = custom;
    }

    public JSONObject getCommodityInfo() {
        return commodityInfo;
    }

    public void setCommodityInfo(JSONObject commodityInfo) {
        this.commodityInfo = commodityInfo;
    }
}
