package com.bettersoft.api.validmodel;

import com.bettersoft.api.model.BuyItem;
import com.bettersoft.api.model.OnlineItemBean;
import com.fasterxml.jackson.annotation.JsonFormat;
import org.apache.commons.lang.StringUtils;

import java.math.BigDecimal;
import java.util.*;

public class ShoppingCartValidate extends BaseValidate {
    private static final long serialVersionUID = 5631393421982662437L;

    private String storeCode;
    private String tableCode;
    private int totalQuantity;
    private int categoryQuantity;
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    private BigDecimal totalAmount;
    private int totalSettleQuantity;
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    private BigDecimal totalSettleAmount;
    private int totalOfflineQuantity;
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    private BigDecimal totalOfflineAmount;
    private List<OnlineItemPackageValidate> cartItems = new ArrayList<OnlineItemPackageValidate>();
    private String remark;


    public ShoppingCartValidate(String storeCode, String tableCode) {
        this.storeCode = storeCode;
        this.tableCode = tableCode;
    }

    public String getStoreCode() {
        return storeCode;
    }

    public void setStoreCode(String storeCode) {
        this.storeCode = storeCode;
    }

    public String getTableCode() {
        return tableCode;
    }

    public void setTableCode(String tableCode) {
        this.tableCode = tableCode;
    }

    public int getTotalQuantity() {
        return totalQuantity;
    }

    public BigDecimal getTotalAmount() {
        return totalAmount==null ?new BigDecimal(0.00) :totalAmount.setScale(2, BigDecimal.ROUND_DOWN);
    }

    public int getCategoryQuantity() {
        return categoryQuantity;
    }

    public int getTotalSettleQuantity() {
        return totalSettleQuantity;
    }

    public BigDecimal getTotalSettleAmount() {
        return totalSettleAmount ==null ?new BigDecimal(0.00) :totalSettleAmount.setScale(2, BigDecimal.ROUND_DOWN);
    }

    public String getRemark() {
        return remark;
    }

    public void setRemark(String remark) {
        this.remark = remark;
    }

    public List<OnlineItemPackageValidate> getCartItems() {
        List<OnlineItemPackageValidate> items = new ArrayList<OnlineItemPackageValidate>();
        for (OnlineItemPackageValidate item : this.cartItems) {
            if (StringUtils.isEmpty(item.getOrderNo())) {
                items.add(item);
            }
        }
        return items;
    }

    public List<OnlineItemPackageValidate> getOrderItems() {
        List<OnlineItemPackageValidate> items = new ArrayList<OnlineItemPackageValidate>();
        for (OnlineItemPackageValidate item : this.cartItems) {
            if (!StringUtils.isEmpty(item.getOrderNo()) && !item.isOffline()) {
                items.add(item);
            }
        }
        return items;
    }

    public void setOrderItem(OnlineItemPackageValidate orderItem) {
        this.cartItems.add(orderItem);
        summaryCart(null);
    }

    public List<OnlineItemPackageValidate> getOfflineItems() {
        List<OnlineItemPackageValidate> items = new ArrayList<OnlineItemPackageValidate>();
        for (OnlineItemPackageValidate item : this.cartItems) {
            if (!StringUtils.isEmpty(item.getOrderNo()) && item.isOffline()) {
                items.add(item);
            }
        }
        return items;
    }

    public int addCartItemQrOrder(OnlineItemPackageValidate cartItem) {
        int flag = 0;
        int inputQty = 0;
        for (OnlineItemPackageValidate it : this.cartItems) {
            if (it.equals(cartItem)) {
                it.setQuantity(it.getQuantity() + cartItem.getQuantity());
                it.setChecked(true);
                it.setCTotalPrice(it.getQuantity(),it.getTotalPrice());

                inputQty = it.getQuantity();
                flag = 1;
                break;
            }
        }
        if (0 > inputQty) {
            removeCartItem(cartItem);
            return -1;
        }

        if (flag==0) {
            cartItem.setChecked(true);
            cartItem.setQuantity(cartItem.getQuantity());
            cartItem.setChildItemTotalPrice(null);
            cartItem.setCTotalPrice(cartItem.getQuantity(),cartItem.getTotalPrice());

            inputQty = cartItem.getQuantity();
            this.cartItems.add(cartItem);
        }

        this.categoryQuantity = this.cartItems.size();
        this.summaryCart(null);
        return inputQty;
    }

    public void addCartItem(OnlineItemPackageValidate cartItem) {
        int inputQty = 0;
        if (this.cartItems.contains(cartItem)) {
            for (OnlineItemPackageValidate it : this.cartItems) {
                if (it.equals(cartItem)) {
                    it.setQuantity(it.getQuantity() + cartItem.getQuantity());
                    it.setChecked(true);
//                    it.setCountTotalPrice(BigDecimal.valueOf(it.getQuantity()).multiply(it.getTotalPrice()));
                    inputQty = it.getQuantity();
                }
            }

//            if (0 >= inputQty) {
//                removeCartItem(cartItem);
//                return;
//            }
            if (0 > inputQty) {
                removeCartItem(cartItem);
                return;
            }
        } else if (0 < cartItem.getQuantity()) {
            cartItem.setChecked(true);
//            cartItem.setCountTotalPrice(BigDecimal.valueOf(cartItem.getQuantity()).multiply(cartItem.getTotalPrice()));
            cartItem.setQuantity(cartItem.getQuantity());
            this.cartItems.add(cartItem);
        }

        this.categoryQuantity = this.cartItems.size();
        this.summaryCart(null);
    }

    public void removeCartItem(OnlineItemPackageValidate cartItem) {
        if (null != this.cartItems && !this.cartItems.isEmpty()) {
            if (this.cartItems.contains(cartItem)) {
                this.cartItems.remove(cartItem);
                this.summaryCart(null);
            }
        }

        this.categoryQuantity = this.cartItems.size();
    }

    public void resetCartItems() {
        List<OnlineItemPackageValidate> cartItems = getCartItems();
        if (null != cartItems && !cartItems.isEmpty()) {
            for (OnlineItemPackageValidate cartItem : cartItems) {
                removeCartItem(cartItem.getItemId());
            }
        }
        this.summaryCart(null);
    }

    public void removeCartItem(int itemId) {
        OnlineItemPackageValidate removeItem = null;
        if (null != this.cartItems && !this.cartItems.isEmpty()) {
            for (OnlineItemPackageValidate cartItem : this.cartItems) {
                if (cartItem.getItemId() == itemId) {
                    removeItem = cartItem;
                    break;
                }
            }
        }

        if (null != removeItem)
            removeCartItem(removeItem);
    }

    public void resetCartItemsClear() {
        List<OnlineItemPackageValidate> cartItems = getCartItems();
        if (null != cartItems && !cartItems.isEmpty()) {
            for (OnlineItemPackageValidate cartItem : cartItems) {
                removeCartItemClear(cartItem.getItemId());
            }
        }
        this.summaryCart(null);
    }

    public void removeCartItemClear(int itemId) {
        OnlineItemPackageValidate removeItem = null;
        if (null != this.cartItems && !this.cartItems.isEmpty()) {
            for (OnlineItemPackageValidate cartItem : this.cartItems) {
                if (cartItem.getItemId() == itemId&&(cartItem.getOrderNo()==null||cartItem.getOrderNo().isEmpty())) {
                    removeItem = cartItem;
                    break;
                }
            }
        }

        if (null != removeItem)
            removeCartItem(removeItem);
    }

    public List<OnlineItemBean> getOnlineItems(String orderNo) {
        return getOnlineItems(orderNo, null, null, null, false);
    }

    private List<OnlineItemBean> getOnlineItems(String orderNo, List<OnlineItemBean> items, List<OnlineItemPackageValidate> childPackages, OnlineItemBean modifyItem, boolean isPackage) {
        if (null == items)
            items = new ArrayList<OnlineItemBean>();

        if (null != childPackages && !childPackages.isEmpty()) {
            for (OnlineItemPackageValidate cartItem : childPackages) {
                cartItem.setOrderNo(orderNo);
                OnlineItemBean onlineItem = generalOnlineItem(cartItem);
                if (null != onlineItem && onlineItem.getItemType().equals("S") && !items.contains(onlineItem)) {
                    isPackage = true;
                    items.add(onlineItem);
                } else if (null != onlineItem && onlineItem.getItemType().equals("I")) {
                    if (isPackage)
                        onlineItem.setItemType("SI");
                    modifyItem = onlineItem;
                    items.add(onlineItem);
                } else if (null != onlineItem && onlineItem.getIntitemno() != 0) {
                    if (onlineItem.getItemType().equals("A")){
                        onlineItem.setItemType("A");
                        items.add(onlineItem);
                    }
                }

                if (null != onlineItem && onlineItem.getItemType().equals("A") && null != modifyItem) {
                    modifyItem.setPmodify(modifyItem.getPmodify() + "[" +onlineItem.getPmodify() + "] ");
                }

                if (null != cartItem.getChildPackages() && !cartItem.getChildPackages().isEmpty())
                    getOnlineItems(orderNo, items, cartItem.getChildPackages(), modifyItem, isPackage);
            }
        } else {
            List<OnlineItemPackageValidate> cartItems = getCartItems();
            if (null != cartItems && !cartItems.isEmpty()) {
                for (OnlineItemPackageValidate cartItem : cartItems) {
                    cartItem.setOrderNo(orderNo);
                    OnlineItemBean onlineItem = generalOnlineItem(cartItem);
                    //套餐
                    if (null != onlineItem && onlineItem.getItemType().equals("S")) {
                        isPackage = true;
                        items.add(onlineItem);
                        //商品
                    } else if (null != onlineItem && onlineItem.getItemType().equals("I")) {
                        isPackage = false;
//                        if (isPackage)
//                            onlineItem.setItemType("SI");
                        modifyItem = onlineItem;
                        items.add(onlineItem);
                    } else if (null != onlineItem && onlineItem.getIntitemno() != 0 && !items.contains(onlineItem)) {
                        items.add(onlineItem);
                    }

                    if (null != onlineItem && onlineItem.getItemType().equals("A") && null != modifyItem) {
                        modifyItem.setPmodify(modifyItem.getPmodify() + "[" +onlineItem.getPmodify() + "] ");
                    }

                    if (null != cartItem.getChildPackages() && !cartItem.getChildPackages().isEmpty())
                        getOnlineItems(orderNo, items, cartItem.getChildPackages(), modifyItem, isPackage);
                }
            }
        }
        return items;
    }

    public List<OnlineItemBean> getOnlineItemsTakeOff() {
        return getOnlineItemsTakeOff(null, null, null, false);
    }

    private List<OnlineItemBean> getOnlineItemsTakeOff(List<OnlineItemBean> items, List<OnlineItemPackageValidate> childPackages, OnlineItemBean modifyItem, boolean isPackage) {
        if (null == items)
            items = new ArrayList<OnlineItemBean>();

        if (null != childPackages && !childPackages.isEmpty()) {
            for (OnlineItemPackageValidate cartItem : childPackages) {
                OnlineItemBean onlineItem = generalOnlineItem(cartItem);
                if (null != onlineItem && onlineItem.getItemType().equals("S") && !items.contains(onlineItem)) {
                    isPackage = true;
                    items.add(onlineItem);
                } else if (null != onlineItem && onlineItem.getItemType().equals("I")) {
                    if (isPackage)
                        onlineItem.setItemType("SI");
                    modifyItem = onlineItem;
                    items.add(onlineItem);
                } else if (null != onlineItem && onlineItem.getIntitemno() != 0) {
                    if (onlineItem.getItemType().equals("A")){
                        onlineItem.setItemType("A");
                        items.add(onlineItem);
                    }
                }

                if (null != onlineItem && onlineItem.getItemType().equals("A") && null != modifyItem) {
                    modifyItem.setPmodify(modifyItem.getPmodify() + "[" + onlineItem.getPmodify() + "] ");
                }

                if (null != cartItem.getChildPackages() && !cartItem.getChildPackages().isEmpty())
                    getOnlineItemsTakeOff(items, cartItem.getChildPackages(), modifyItem, isPackage);
            }
        } else {
            List<OnlineItemPackageValidate> cartItems = getCartItems();
            if (null != cartItems && !cartItems.isEmpty()) {
                for (OnlineItemPackageValidate cartItem : cartItems) {
                    OnlineItemBean onlineItem = generalOnlineItem(cartItem);
                    //套餐
                    if (null != onlineItem && onlineItem.getItemType().equals("S")) {
                        isPackage = true;
                        items.add(onlineItem);
                        //商品
                    } else if (null != onlineItem && onlineItem.getItemType().equals("I")) {
                        isPackage = false;
//                        if (isPackage)
//                            onlineItem.setItemType("SI");
                        modifyItem = onlineItem;
                        items.add(onlineItem);
                    } else if (null != onlineItem && onlineItem.getIntitemno() != 0 && !items.contains(onlineItem)) {
                        items.add(onlineItem);
                    }

                    if (null != onlineItem && onlineItem.getItemType().equals("A") && null != modifyItem) {
                        modifyItem.setPmodify(modifyItem.getPmodify() + "[" +onlineItem.getPmodify() + "] ");
                    }

                    if (null != cartItem.getChildPackages() && !cartItem.getChildPackages().isEmpty())
                        getOnlineItemsTakeOff(items, cartItem.getChildPackages(), modifyItem, isPackage);
                }
            }
        }
        return items;
    }

    private void summaryCart(List<OnlineItemPackageValidate> cartItems) {
        if (null != cartItems && !cartItems.isEmpty()) {
            for (OnlineItemPackageValidate item : cartItems) {
                item.setChecked(true);
                if (null != item.getChildPackages() && !item.getChildPackages().isEmpty()) {
                    this.summaryCart(item.getChildPackages());
                }
                if (item.getPackageType().equals("A") || item.getPackageType().equals("I")) {
                    if (StringUtils.isEmpty(item.getOrderNo())) {
                        this.totalAmount = this.totalAmount.add(item.getPrice().multiply(new BigDecimal(item.getQuantity())));
                    } else {
                        if (item.isOffline()) {
                            this.totalOfflineAmount = this.totalOfflineAmount.add(item.getPrice().multiply(new BigDecimal(item.getQuantity())));
                        } else {
                            this.totalSettleAmount = this.totalSettleAmount.add(item.getPrice().multiply(new BigDecimal(item.getQuantity())));
                        }
                    }
                }
            }
        } else {
            this.totalQuantity = 0;
            this.totalAmount = new BigDecimal(0);
            this.totalSettleQuantity = 0;
            this.totalSettleAmount = new BigDecimal(0);
            this.totalOfflineQuantity = 0;
            this.totalOfflineAmount = new BigDecimal(0.00);
            if (null != this.cartItems && !this.cartItems.isEmpty()) {
                for (OnlineItemPackageValidate item : this.cartItems) {
                    item.setChecked(true);
                    if (null != item.getChildPackages() && !item.getChildPackages().isEmpty()) {
                        this.summaryCart(item.getChildPackages());
                    }
                    if (StringUtils.isEmpty(item.getOrderNo())) {
                        this.totalAmount = this.totalAmount.add(item.getPrice().multiply(new BigDecimal(item.getQuantity())));
                    } else {
                        if (item.isOffline()) {
                            this.totalOfflineAmount = this.totalOfflineAmount.add(item.getPrice().multiply(new BigDecimal(item.getQuantity())));
                        } else {
                            this.totalSettleAmount = this.totalSettleAmount.add(item.getPrice().multiply(new BigDecimal(item.getQuantity())));
                        }
                    }

                    if ((item.getPackageType().equals("I") || item.getPackageType().equals("P"))) {
                        if (StringUtils.isEmpty(item.getOrderNo())) {
                            this.totalQuantity += item.getQuantity();
                        } else {
                            if (item.isOffline()) {
                                this.totalOfflineQuantity += item.getQuantity();
                            } else {
                                this.totalSettleQuantity += item.getQuantity();
                            }
                        }
                    }
                }
            }
        }
    }

    private OnlineItemBean generalOnlineItem(OnlineItemPackageValidate cartItem) {
        OnlineItemBean onlineItem = new OnlineItemBean();
        if (cartItem.getPackageType().equals("P") && 0 < cartItem.getItemId() && null != cartItem.getPrice()) {
            onlineItem.setItemType("S");
            onlineItem.setQuantity(cartItem.getQuantity());
            onlineItem.setPrice(cartItem.getPrice().doubleValue());
            onlineItem.setIntitemno(cartItem.getItemId());
        } else if (cartItem.getPackageType().equals("I") && 0 < cartItem.getItemId()) {
            onlineItem.setIntitemno(cartItem.getItemId());
            onlineItem.setQuantity(cartItem.getQuantity());
            onlineItem.setPrice(cartItem.getPrice().doubleValue());
            onlineItem.setItemType("I");
        } else if (cartItem.getPackageType().equals("A") && 0 < cartItem.getItemId()) {
            onlineItem.setIntitemno(cartItem.getItemId());
            onlineItem.setItemType("A");
            onlineItem.setQuantity(cartItem.getQuantity());
            onlineItem.setPrice(cartItem.getPrice().doubleValue());
            onlineItem.setPmodify(cartItem.getPackageDesc());
        } else if (cartItem.getPackageType().equals("A") && 0 == cartItem.getItemId()) {
            if (!StringUtils.isEmpty(cartItem.getPackageCode()) && 0.0f < cartItem.getPrice().floatValue()) {
                try {
                    onlineItem.setIntitemno(Integer.parseInt(cartItem.getPackageCode()));
                } catch (Exception ex) {
                    onlineItem.setIntitemno(0);
                }
            }
            onlineItem.setItemType("A");
            onlineItem.setQuantity(cartItem.getQuantity());
            onlineItem.setPrice(cartItem.getPrice().doubleValue());
            onlineItem.setPmodify(cartItem.getPackageDesc());
        } else {
            onlineItem = null;
        }

        return onlineItem;
    }

    public List<OnlineItemPackageValidate> getRealCartItems() {
        return this.cartItems;
    }

    public void delOrderOfflineItem() {
        List<OnlineItemPackageValidate> cartItems = getRealCartItems();
        if (null != cartItems && !cartItems.isEmpty()) {
            cartItems.removeIf(cartItem -> cartItem.getOrderNo() != null && !cartItem.getOrderNo().isEmpty());
        }
    }

    public BigDecimal getTotalOfflineAmount() {
        return totalOfflineAmount;
    }

    public void setTotalOfflineAmount(BigDecimal totalOfflineAmount) {
        this.totalOfflineAmount = totalOfflineAmount;
    }
}
