package com.bettersoft.api.websocket;

import io.jsonwebtoken.Claims;

import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.lang.StringUtils;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.http.server.ServletServerHttpRequest;
import org.springframework.stereotype.Component;
import org.springframework.web.socket.WebSocketHandler;
import org.springframework.web.socket.server.support.HttpSessionHandshakeInterceptor;

import com.bettersoft.api.utils.EntityUtil;
import com.bettersoft.api.utils.JwtUtils;
import com.bettersoft.api.utils.LoginUserInfo;
import com.bettersoft.api.utils.RedisCacheUtil;

@Component
public class BettersoftHandShakeInterceptor extends
		HttpSessionHandshakeInterceptor {
	@Resource(name = "redisCacheUtil")
	private RedisCacheUtil redisCacheUtil;

	public boolean beforeHandshake(ServerHttpRequest request,
			ServerHttpResponse response, WebSocketHandler wsHandler,
			Map<String, Object> attributes) throws Exception {
		boolean handled = true;
		if (request instanceof ServletServerHttpRequest) {
			ServletServerHttpRequest servletRequest = (ServletServerHttpRequest) request;

			String ticket = (String) servletRequest.getServletRequest()
					.getParameter("ticket");

			try {
				// 标记用户
				if (!StringUtils.isEmpty(ticket)) {
					Claims claims = JwtUtils
							.verifyToken(ticket, redisCacheUtil);

					LoginUserInfo loginUser = EntityUtil.convertMapToEntity(
							LoginUserInfo.class, claims);

					attributes.put("connectUser", loginUser);
				} else {
					handled = false;
				}
			} catch (Exception e) {
				handled = false;
			}
		}
		return handled;
	}

	public void afterHandshake(ServerHttpRequest request,
			ServerHttpResponse response, WebSocketHandler wsHandler,
			Exception exception) {
		super.afterHandshake(request, response, wsHandler, exception);
	}
}
