package com.bettersoft.api.websocket;

import java.io.IOException;
import java.text.MessageFormat;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import net.sf.ehcache.store.chm.ConcurrentHashMap;

import org.apache.commons.collections.MapUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.stereotype.Component;
import org.springframework.web.socket.CloseStatus;
import org.springframework.web.socket.TextMessage;
import org.springframework.web.socket.WebSocketHandler;
import org.springframework.web.socket.WebSocketMessage;
import org.springframework.web.socket.WebSocketSession;

import com.bettersoft.api.utils.Constants;
import com.bettersoft.api.utils.LoginUserInfo;

@Component
public class BettersoftWebSocketHandler implements WebSocketHandler {
	private Log logger = LogFactory.getLog(getClass());

	// 用于保存HttpSession与WebSocketSession的映射关系
	private static final Map<String, WebSocketSession> SOCKET_SESSION_MAP;

	static {
		SOCKET_SESSION_MAP = new ConcurrentHashMap<String, WebSocketSession>();
	}

	public void afterConnectionEstablished(WebSocketSession session)
			throws Exception {
		LoginUserInfo loginUser = (LoginUserInfo) session.getAttributes().get(
				"connectUser");

		logger.info(MessageFormat.format(
				"WebSocket connected userCode/session id:{0}/{1}",
				loginUser.getUserCode(), loginUser.getSessionId()));

		String sessionKey = Constants.WEBSOCKET_SESSION_KEY
				+ loginUser.getSessionId();

		if (null == MapUtils.getObject(SOCKET_SESSION_MAP, sessionKey, null)) {
			SOCKET_SESSION_MAP.put(sessionKey, session);
		}
	}

	public void handleMessage(WebSocketSession session,
			WebSocketMessage<?> message) throws Exception {
		if (0 == message.getPayloadLength())
			return;
	}

	public void handleTransportError(WebSocketSession session,
			Throwable exception) throws Exception {
		logger.error("websocket HandleError.", exception);
		if (session.isOpen()) {
			session.close();
		}

		Iterator<Entry<String, WebSocketSession>> it = SOCKET_SESSION_MAP
				.entrySet().iterator();
		// 移除当前抛出异常用户的Socket会话
		while (it.hasNext()) {
			Entry<String, WebSocketSession> entry = it.next();
			if (entry.getValue().getId().equals(session.getId())) {
				SOCKET_SESSION_MAP.remove(entry.getKey());
				break;
			}
		}
	}

	public void afterConnectionClosed(WebSocketSession session,
			CloseStatus closeStatus) throws Exception {
		LoginUserInfo loginUser = (LoginUserInfo) session.getAttributes().get(
				"connectUser");

		Iterator<Entry<String, WebSocketSession>> it = SOCKET_SESSION_MAP
				.entrySet().iterator();

		String sessionKey = Constants.WEBSOCKET_SESSION_KEY
				+ loginUser.getSessionId();
		// 移除当前用户的Socket会话
		while (it.hasNext()) {
			Entry<String, WebSocketSession> entry = it.next();
			if (entry.getKey().equals(sessionKey)) {
				logger.info(MessageFormat.format(
						"WebSocket disconnected id:{0}", entry.getKey()));

				SOCKET_SESSION_MAP.remove(entry.getKey());
				break;
			}
		}
	}

	public boolean supportsPartialMessages() {
		return false;
	}

	public void broadcast(final TextMessage message) throws IOException {
		Iterator<Entry<String, WebSocketSession>> iter = SOCKET_SESSION_MAP
				.entrySet().iterator();

		// 多线程群发
		while (iter.hasNext()) {
			final Entry<String, WebSocketSession> entry = iter.next();

			if (entry.getValue().isOpen()) {
				// entry.getValue().sendMessage(message);
				new Thread(new Runnable() {
					public void run() {
						try {
							if (entry.getValue().isOpen()) {
								entry.getValue().sendMessage(message);
							}
						} catch (IOException e) {
							e.printStackTrace();
						}
					}

				}).start();
			}
		}
	}

	public void pushMessageToAccount(String uuid, TextMessage message)
			throws IOException {
		WebSocketSession session = getWebSocketSession(uuid);

		if (null != session && session.isOpen()) {
			session.sendMessage(message);
		}
	}

	public static WebSocketSession getWebSocketSession(String uuid) {
		WebSocketSession session = null;

		Iterator<Entry<String, WebSocketSession>> iter = SOCKET_SESSION_MAP
				.entrySet().iterator();

		while (iter.hasNext()) {
			final Entry<String, WebSocketSession> entry = iter.next();
			if (0 <= entry.getKey().indexOf(uuid)) {
				session = entry.getValue();
				break;
			}
		}

		return session;
	}
}
