package comm.hibernate;

import java.io.Serializable;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import com.bettersoft.api.model.OnlineorderhBean;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.LockMode;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.CriteriaSpecification;

import com.bettersoft.api.utils.Constants;
import org.springframework.stereotype.Repository;

@Repository
public class BaseHibernateDaoImpl implements IBaseHibernateDao {
    private static String LEFT_BRACKET = "(";
    private static String RIGHT_BRACKET = ")";

    protected Log logger = LogFactory.getLog(getClass());

    @Resource
    private SessionFactory sessionFactory;

    @SuppressWarnings({"rawtypes", "unchecked"})
    public <T> List<T> generalSearch(String fromHql,
                                     Map<String, Object> queryParam, String orderByHql) {
        Map<String, Object> cloneParam = null;
        String groupBy = StringUtils.EMPTY;
        if (null != queryParam && !queryParam.isEmpty())
            cloneParam = new HashMap<String, Object>(queryParam);
        else
            cloneParam = new HashMap<String, Object>();

        if (StringUtils.isEmpty(fromHql))
            return null;

        if (0 < fromHql.toUpperCase().lastIndexOf("GROUP BY")) {
            int groupByIndex = fromHql.toUpperCase().lastIndexOf("GROUP BY");
            groupBy = fromHql.substring(groupByIndex);
            fromHql = fromHql.substring(0, groupByIndex);
        }

        if (0 < fromHql.toUpperCase().lastIndexOf("ORDER BY")) {
            int orderByIndex = fromHql.toUpperCase().lastIndexOf("ORDER BY");
            orderByHql = fromHql.substring(orderByIndex);
            fromHql = fromHql.substring(0, orderByIndex);
        }

        StringBuffer hqlBuffer = new StringBuffer(fromHql);
        if (fromHql.toUpperCase().indexOf("WHERE") < 0) {
            hqlBuffer.append(" WHERE 1=1 ");
        }

        boolean boolOrderBy = !StringUtils.isEmpty(orderByHql);
        String filterKey = StringUtils.EMPTY;
        int firstResults = 0;
        int maxResults = 0;
        if (queryParam != null && !queryParam.isEmpty()) {
            for (Iterator<String> it = queryParam.keySet().iterator(); it
                    .hasNext(); ) {
                String key = (String) it.next();
                hqlBuffer.append(getCondition(key, queryParam.get(key),
                        cloneParam));
                if (Constants.QUERY_FIRST_RESULTS.equals(key)) {
                    firstResults = Integer.valueOf(
                            queryParam.get(key).toString()).intValue();
                }
                if (Constants.QUERY_MAX_RESULTS.equals(key)) {
                    maxResults = Integer
                            .valueOf(queryParam.get(key).toString()).intValue();
                }

                if (Constants.QUERY_GROUP_BY.equals(key))
                    groupBy = " group by "
                            + MapUtils.getString(queryParam, key);

                if (StringUtils.startsWith(key, Constants.QUERY_FILTER))
                    filterKey = key.replace(Constants.QUERY_FILTER, "");

                // Order By
                if (boolOrderBy) {
                    if (Constants.QUERY_ORDER_BY.equals(key)) {
                        String orderBy = MapUtils.getString(queryParam, key,
                                StringUtils.EMPTY).replace("order by", "");
                        if (!StringUtils.isEmpty(orderBy)) {
                            orderByHql = " order by " + orderBy;
                        }
                    }
                }
            }

            if (queryParam.containsKey(Constants.QUERY_SPEC_TAIL_CONDITION))
                hqlBuffer.append(" AND "
                        + MapUtils.getString(queryParam,
                        Constants.QUERY_SPEC_TAIL_CONDITION));
        }

        if (!StringUtils.isEmpty(groupBy))
            hqlBuffer.append(" " + groupBy);

        if (!StringUtils.isEmpty(orderByHql)) {
            hqlBuffer.append(" " + orderByHql.trim());
        }

        Session session = null;
        try {
            session = getSession();
            if (!StringUtils.isEmpty(filterKey)) {
                String[] filterParam = StringUtils.split(filterKey, ".");

                session.enableFilter(filterParam[0]).setParameter(
                        filterParam[1],
                        MapUtils.getObject(queryParam, Constants.QUERY_FILTER
                                + filterKey, null));
            }

            Query query = session.createQuery(hqlBuffer.toString());
            for (Iterator it = cloneParam.keySet().iterator(); it.hasNext(); ) {
                String key = (String) it.next();

                if (!isMaskByKey(key)) {
                    query.setParameter(key.replace(".", "_"),
                            MapUtils.getObject(cloneParam, key));
                }
            }

            if (firstResults > 0) {
                query.setFirstResult(firstResults);
            }
            if (maxResults > 0) {
                query.setMaxResults(maxResults);
            }
            return query.list();
        } catch (Exception ex) {
            logger.error("generalSearch error.", ex);
        }

        return null;
    }

    @SuppressWarnings({"rawtypes", "unchecked"})
    public <T> List<T> generalSearch(String fromHql,
                                     Map<String, Object> queryParam, String orderByHQL, boolean lockFlag) {
        Map<String, Object> cloneParam = null;
        String groupBy = StringUtils.EMPTY;
        if (null != queryParam && !queryParam.isEmpty())
            cloneParam = new HashMap<String, Object>(queryParam);
        else
            cloneParam = new HashMap<String, Object>();

        if (StringUtils.isEmpty(fromHql))
            return null;

        if (0 < fromHql.toUpperCase().lastIndexOf("GROUP BY")) {
            int groupByIndex = fromHql.toUpperCase().lastIndexOf("GROUP BY");
            groupBy = fromHql.substring(groupByIndex);
            fromHql = fromHql.substring(0, groupByIndex);
        }

        if (0 < fromHql.toUpperCase().lastIndexOf("ORDER BY")) {
            int orderByIndex = fromHql.toUpperCase().lastIndexOf("ORDER BY");
            orderByHQL = fromHql.substring(orderByIndex);
            fromHql = fromHql.substring(0, orderByIndex);
        }

        StringBuffer hqlBuffer = new StringBuffer(fromHql);
        if (fromHql.toUpperCase().indexOf("WHERE") < 0) {
            hqlBuffer.append(" WHERE 1=1 ");
        }
        boolean boolOrderBy = !StringUtils.isEmpty(orderByHQL);
        String filterKey = StringUtils.EMPTY;
        int firstResults = 0;
        int maxResults = 0;
        if (queryParam != null && !queryParam.isEmpty()) {
            for (Iterator it = queryParam.keySet().iterator(); it.hasNext(); ) {
                String key = (String) it.next();
                hqlBuffer.append(getCondition(key, queryParam.get(key),
                        cloneParam));
                if (Constants.QUERY_FIRST_RESULTS.equals(key)) {
                    firstResults = Integer.valueOf(
                            queryParam.get(key).toString()).intValue();
                }
                if (Constants.QUERY_MAX_RESULTS.equals(key)) {
                    maxResults = Integer
                            .valueOf(queryParam.get(key).toString()).intValue();
                }

                if (Constants.QUERY_GROUP_BY.equals(key))
                    groupBy = " group by "
                            + MapUtils.getString(queryParam, key);

                if (StringUtils.startsWith(key, Constants.QUERY_FILTER))
                    filterKey = key.replace(Constants.QUERY_FILTER, "");

                // Order By
                if (boolOrderBy) {
                    if (Constants.QUERY_ORDER_BY.equals(key)) {
                        String orderBy = MapUtils.getString(queryParam, key,
                                StringUtils.EMPTY).replace("order by", "");
                        if (!StringUtils.isEmpty(orderBy)) {
                            orderByHQL = " order by " + orderBy;
                        }
                    }
                }
            }

            if (queryParam.containsKey(Constants.QUERY_SPEC_TAIL_CONDITION))
                hqlBuffer.append(" AND "
                        + MapUtils.getString(queryParam,
                        Constants.QUERY_SPEC_TAIL_CONDITION));
        }

        if (!StringUtils.isEmpty(groupBy))
            hqlBuffer.append(" " + groupBy);

        if (!StringUtils.isEmpty(orderByHQL)) {
            hqlBuffer.append(" " + orderByHQL.trim());
        }

        Session session = null;
        try {
            session = getSession();
            if (!StringUtils.isEmpty(filterKey)) {
                String[] filterParam = StringUtils.split(filterKey, ".");

                session.enableFilter(filterParam[0]).setParameter(
                        filterParam[1],
                        MapUtils.getObject(queryParam, Constants.QUERY_FILTER
                                + filterKey, null));
            }

            Query query = session.createQuery(hqlBuffer.toString());
            for (Iterator it = cloneParam.keySet().iterator(); it.hasNext(); ) {
                String key = (String) it.next();

                if (!isMaskByKey(key)) {
                    query.setParameter(key.replace(".", "_"),
                            MapUtils.getObject(cloneParam, key));
                }
            }

            if (firstResults > 0) {
                query.setFirstResult(firstResults);
            }
            if (maxResults > 0) {
                query.setMaxResults(maxResults);
            }
            return lockFlag ? query.setLockMode(fromHql.split(" ")[2],
                    LockMode.UPGRADE_NOWAIT).list() : query.list();
        } catch (Exception ex) {
            logger.error("generalSearch error.", ex);
        }

        return null;
    }

    @SuppressWarnings({"rawtypes", "unchecked"})
    public Number generalCount(String fromHQL, Map queryParam,
                               boolean... holdOn) {
        Map<String, Object> cloneParam = null;
        String groupBy = StringUtils.EMPTY;
        String filterKey = StringUtils.EMPTY;

        if (null != queryParam && !queryParam.isEmpty())
            cloneParam = new HashMap<String, Object>(queryParam);
        else
            cloneParam = new HashMap<String, Object>();

        if (StringUtils.isEmpty(fromHQL))
            return 0;

        if (0 < fromHQL.toUpperCase().lastIndexOf("GROUP BY")) {
            int groupByIndex = fromHQL.toUpperCase().lastIndexOf("GROUP BY");
            groupBy = fromHQL.substring(groupByIndex);
            fromHQL = fromHQL.substring(0, groupByIndex);
        }

        if (0 < fromHQL.toUpperCase().lastIndexOf("ORDER BY"))
            fromHQL = fromHQL.substring(0,
                    fromHQL.toUpperCase().lastIndexOf("ORDER BY"));

        StringBuffer hqlBuffer = new StringBuffer("select count (*) ");

        if (0 < holdOn.length && holdOn[0])
            hqlBuffer = new StringBuffer(fromHQL);
        else
            hqlBuffer.append(removeSelect(fromHQL));

        if (fromHQL.toUpperCase().indexOf("WHERE") < 0) {
            hqlBuffer.append(" WHERE 1=1 ");
        }

        if (queryParam != null && !queryParam.isEmpty()) {
            for (Iterator it = queryParam.keySet().iterator(); it.hasNext(); ) {
                String key = (String) it.next();
                hqlBuffer.append(getCondition(key, queryParam.get(key),
                        cloneParam));

                if (StringUtils.startsWith(key, Constants.QUERY_FILTER))
                    filterKey = key.replace(Constants.QUERY_FILTER, "");
            }

            if (queryParam.containsKey(Constants.QUERY_SPEC_TAIL_CONDITION))
                hqlBuffer.append(" AND "
                        + MapUtils.getString(queryParam,
                        Constants.QUERY_SPEC_TAIL_CONDITION));
        }

        if (!StringUtils.isEmpty(groupBy))
            hqlBuffer.append(" " + groupBy);

        Session session = null;

        try {
            session = getSession();
            if (!StringUtils.isEmpty(filterKey)) {
                String[] filterParam = StringUtils.split(filterKey, ".");

                session.enableFilter(filterParam[0]).setParameter(
                        filterParam[1],
                        MapUtils.getObject(queryParam, Constants.QUERY_FILTER
                                + filterKey, null));
            }

            Query query = session.createQuery(hqlBuffer.toString());
            for (Iterator<String> itKey = cloneParam.keySet().iterator(); itKey
                    .hasNext(); ) {
                String key = itKey.next();
                Object value = MapUtils.getObject(cloneParam, key);
                if (isMaskByKey(key) | isMaskByValue(value))
                    continue;

                query.setParameter(key.replace('.', '_'), value);
            }

            if (!StringUtils.isEmpty(groupBy)
                    || (0 < holdOn.length && holdOn[0]))
                return (Number) query.list().size();

            return (Number) query.uniqueResult();
        } catch (Exception ex) {
            logger.error("generalSearchCount error.", ex);
        }

        return null;
    }

    @SuppressWarnings({"rawtypes", "unchecked"})
    public <T> List<T> generalSearchBySql(String fromSql,
                                          Map<String, Object> queryParam, String orderBySql) {
        Map<String, Object> cloneParam = null;
        String groupBy = StringUtils.EMPTY;
        int whereIndex = findWhereIndex(fromSql, -1);
        String whereStr = StringUtils.EMPTY;

        if (null != queryParam && !queryParam.isEmpty())
            cloneParam = new HashMap<String, Object>(queryParam);
        else
            cloneParam = new HashMap<String, Object>();

        if (StringUtils.isEmpty(fromSql))
            return null;

        if (0 < whereIndex) {
            whereStr = fromSql.substring(whereIndex);
            fromSql = fromSql.substring(0, whereIndex);

            if (0 < findLastSqlKeyWords(whereStr, "GROUP BY")) {
                int groupByIndex = whereStr.toUpperCase().lastIndexOf(
                        "GROUP BY");
                groupBy = whereStr.substring(groupByIndex);
                fromSql = whereStr.substring(0, groupByIndex);
            }

            if (0 < findLastSqlKeyWords(whereStr, "ORDER BY")) {
                int orderByIndex = whereStr.toUpperCase().lastIndexOf(
                        "ORDER BY");
                orderBySql = whereStr.substring(orderByIndex);
                fromSql = whereStr.substring(0, orderByIndex);
            }
        } else {
            if (0 < findLastSqlKeyWords(fromSql, "GROUP BY")) {
                int groupByIndex = findLastSqlKeyWords(fromSql, "GROUP BY");
                groupBy = fromSql.substring(groupByIndex);
                fromSql = fromSql.substring(0, groupByIndex);
            }

            if (0 < findLastSqlKeyWords(fromSql, "ORDER BY")) {
                int orderByIndex = findLastSqlKeyWords(fromSql, "ORDER BY");
                orderBySql = fromSql.substring(orderByIndex);
                fromSql = fromSql.substring(0, orderByIndex);
            }

            if (0 < findLastSqlKeyWords(groupBy, "ORDER BY")) {
                int orderByIndex = findLastSqlKeyWords(groupBy, "ORDER BY");
                orderBySql = groupBy.substring(orderByIndex);
                groupBy = groupBy.substring(0, orderByIndex);
            }
        }

        StringBuffer sqlBuffer = new StringBuffer(fromSql);
        if (0 > whereIndex)
            sqlBuffer.append(" WHERE 1=1 ");
        else
            sqlBuffer.append(" " + whereStr);

        boolean boolOrderBy = !StringUtils.isEmpty(orderBySql);
        int firstResults = 0;
        int maxResults = 0;
        if (queryParam != null && !queryParam.isEmpty()) {
            for (Iterator it = queryParam.keySet().iterator(); it.hasNext(); ) {
                String key = (String) it.next();
                sqlBuffer.append(getCondition(key, queryParam.get(key),
                        cloneParam));
                if (Constants.QUERY_FIRST_RESULTS.equals(key)) {
                    firstResults = Integer.valueOf(
                            queryParam.get(key).toString()).intValue();
                }
                if (Constants.QUERY_MAX_RESULTS.equals(key)) {
                    maxResults = Integer
                            .valueOf(queryParam.get(key).toString()).intValue();
                }

                if (Constants.QUERY_GROUP_BY.equals(key))
                    groupBy = " group by "
                            + MapUtils.getString(queryParam, key);
                // Order By
                if (boolOrderBy) {
                    if (Constants.QUERY_ORDER_BY.equals(key)) {
                        String orderBy = MapUtils.getString(queryParam, key,
                                StringUtils.EMPTY).replace("order by", "");
                        if (!StringUtils.isEmpty(orderBy)) {
                            orderBySql = " order by " + orderBy;
                        }
                    }
                }
            }

            if (queryParam.containsKey(Constants.QUERY_SPEC_TAIL_CONDITION))
                sqlBuffer.append(" AND "
                        + MapUtils.getString(queryParam,
                        Constants.QUERY_SPEC_TAIL_CONDITION));
        }

        if (!StringUtils.isEmpty(groupBy))
            sqlBuffer.append(" " + groupBy);

        if (!StringUtils.isEmpty(orderBySql)) {
            sqlBuffer.append(" " + orderBySql.trim());
        }

        Session session = null;
        try {
            session = getSession();
            Query query = session.createSQLQuery(sqlBuffer.toString());
            query.setResultTransformer(CriteriaSpecification.ALIAS_TO_ENTITY_MAP);
            for (Iterator it = cloneParam.keySet().iterator(); it.hasNext(); ) {
                String key = (String) it.next();

                if (!isMaskByKey(key)) {
                    query.setParameter(key.replace(".", "_"),
                            MapUtils.getObject(cloneParam, key));
                }
            }

            if (firstResults > 0) {
                query.setFirstResult(firstResults);
            }
            if (maxResults > 0) {
                query.setMaxResults(maxResults);
            }

            return query.list();
        } catch (Exception ex) {
            logger.error("generalSearchBySql error.", ex);
        }

        return null;
    }

    public <T> List<OnlineorderhBean> generalSearchByFullSql(String fromSql,
                                                             Map<String, Object> queryParam, String orderBySql) {
        Map<String, Object> cloneParam = null;
        String groupBy = StringUtils.EMPTY;
        int whereIndex = fromSql.indexOf("where");
        String whereStr = StringUtils.EMPTY;

        if (null != queryParam && !queryParam.isEmpty())
            cloneParam = new HashMap<String, Object>(queryParam);
        else
            cloneParam = new HashMap<String, Object>();

        if (StringUtils.isEmpty(fromSql))
            return null;

        if (0 < whereIndex) {
            whereStr = fromSql.substring(whereIndex);
            fromSql = fromSql.substring(0, whereIndex);

            if (0 < findLastSqlKeyWords(whereStr, "GROUP BY")) {
                int groupByIndex = whereStr.toUpperCase().lastIndexOf(
                        "GROUP BY");
                groupBy = whereStr.substring(groupByIndex);
                fromSql = whereStr.substring(0, groupByIndex);
            }

            if (0 < findLastSqlKeyWords(whereStr, "ORDER BY")) {
                int orderByIndex = whereStr.toUpperCase().lastIndexOf(
                        "ORDER BY");
                orderBySql = whereStr.substring(orderByIndex);
                fromSql = whereStr.substring(0, orderByIndex);
            }
        } else {
            if (0 < findLastSqlKeyWords(fromSql, "GROUP BY")) {
                int groupByIndex = findLastSqlKeyWords(fromSql, "GROUP BY");
                groupBy = fromSql.substring(groupByIndex);
                fromSql = fromSql.substring(0, groupByIndex);
            }

            if (0 < findLastSqlKeyWords(fromSql, "ORDER BY")) {
                int orderByIndex = findLastSqlKeyWords(fromSql, "ORDER BY");
                orderBySql = fromSql.substring(orderByIndex);
                fromSql = fromSql.substring(0, orderByIndex);
            }

            if (0 < findLastSqlKeyWords(groupBy, "ORDER BY")) {
                int orderByIndex = findLastSqlKeyWords(groupBy, "ORDER BY");
                orderBySql = groupBy.substring(orderByIndex);
                groupBy = groupBy.substring(0, orderByIndex);
            }
        }

        StringBuffer sqlBuffer = new StringBuffer(fromSql);
        if (0 > whereIndex)
            sqlBuffer.append(" WHERE 1=1 ");
        else
            sqlBuffer.append(" " + whereStr);

        boolean boolOrderBy = !StringUtils.isEmpty(orderBySql);
        int firstResults = 0;
        int maxResults = 0;
        if (queryParam != null && !queryParam.isEmpty()) {
            for (Iterator it = queryParam.keySet().iterator(); it.hasNext(); ) {
                String key = (String) it.next();
                sqlBuffer.append(getCondition(key, queryParam.get(key),
                        cloneParam));
                if (Constants.QUERY_FIRST_RESULTS.equals(key)) {
                    firstResults = Integer.valueOf(
                            queryParam.get(key).toString()).intValue();
                }
                if (Constants.QUERY_MAX_RESULTS.equals(key)) {
                    maxResults = Integer
                            .valueOf(queryParam.get(key).toString()).intValue();
                }

                if (Constants.QUERY_GROUP_BY.equals(key))
                    groupBy = " group by "
                            + MapUtils.getString(queryParam, key);
                // Order By
                if (boolOrderBy) {
                    if (Constants.QUERY_ORDER_BY.equals(key)) {
                        String orderBy = MapUtils.getString(queryParam, key,
                                StringUtils.EMPTY).replace("order by", "");
                        if (!StringUtils.isEmpty(orderBy)) {
                            orderBySql = " order by " + orderBy;
                        }
                    }
                }
            }

            if (queryParam.containsKey(Constants.QUERY_SPEC_TAIL_CONDITION))
                sqlBuffer.append(" AND "
                        + MapUtils.getString(queryParam,
                        Constants.QUERY_SPEC_TAIL_CONDITION));
        }

        if (!StringUtils.isEmpty(groupBy))
            sqlBuffer.append(" " + groupBy);

        if (!StringUtils.isEmpty(orderBySql)) {
            sqlBuffer.append(" " + orderBySql.trim());
        }

        Session session = null;
        try {
            session = getSession();
            Query query = session.createSQLQuery(sqlBuffer.toString());
            query.setResultTransformer(CriteriaSpecification.ALIAS_TO_ENTITY_MAP);
            for (Iterator it = cloneParam.keySet().iterator(); it.hasNext(); ) {
                String key = (String) it.next();

                if (!isMaskByKey(key)) {
                    query.setParameter(key.replace(".", "_"),
                            MapUtils.getObject(cloneParam, key));
                }
            }

            if (firstResults > 0) {
                query.setFirstResult(firstResults);
            }
            if (maxResults > 0) {
                query.setMaxResults(maxResults);
            }

            return query.list();
        } catch (Exception ex) {
            logger.error("generalSearchBySql error.", ex);
        }

        return null;
    }

    @SuppressWarnings({"rawtypes"})
    public Number generalCountBySql(String fromSql,
                                    Map<String, Object> queryParam, boolean... holdOn) {
        Map<String, Object> cloneParam = null;
        String groupBy = StringUtils.EMPTY;
        int whereIndex = findWhereIndex(fromSql, -1);
        String whereStr = StringUtils.EMPTY;

        if (null != queryParam && !queryParam.isEmpty())
            cloneParam = new HashMap<String, Object>(queryParam);
        else
            cloneParam = new HashMap<String, Object>();

        if (StringUtils.isEmpty(fromSql))
            return null;

        if (0 < whereIndex) {
            whereStr = fromSql.substring(whereIndex);
            fromSql = fromSql.substring(0, whereIndex);

            if (0 < findLastSqlKeyWords(whereStr, "GROUP BY")) {
                int groupByIndex = findLastSqlKeyWords(whereStr, "GROUP BY");
                groupBy = whereStr.substring(groupByIndex);
                fromSql = whereStr.substring(0, groupByIndex);
            }

            if (0 < findLastSqlKeyWords(whereStr, "ORDER BY"))
                fromSql = whereStr.substring(0, whereStr.toUpperCase()
                        .lastIndexOf("ORDER BY"));
        } else {
            if (0 < findLastSqlKeyWords(fromSql, "GROUP BY")) {
                int groupByIndex = findLastSqlKeyWords(fromSql, "GROUP BY");
                groupBy = fromSql.substring(groupByIndex);
                fromSql = fromSql.substring(0, groupByIndex);
            }

            if (0 < findLastSqlKeyWords(fromSql, "ORDER BY"))
                fromSql = fromSql.substring(0,
                        findLastSqlKeyWords(fromSql, "ORDER BY"));

            if (0 < findLastSqlKeyWords(groupBy, "ORDER BY")) {
                int orderByIndex = findLastSqlKeyWords(groupBy, "ORDER BY");
                groupBy = groupBy.substring(0, orderByIndex);
            }
        }

        StringBuffer sqlBuffer = new StringBuffer("select count (*) ")
                .append(removeSelect(fromSql));
        if (0 > whereIndex)
            sqlBuffer.append(" WHERE 1=1 ");
        else
            sqlBuffer.append(" " + whereStr);

        if (queryParam != null && !queryParam.isEmpty()) {
            for (Iterator it = queryParam.keySet().iterator(); it.hasNext(); ) {
                String key = (String) it.next();
                sqlBuffer.append(getCondition(key, queryParam.get(key),
                        cloneParam));
            }

            if (queryParam.containsKey(Constants.QUERY_SPEC_TAIL_CONDITION))
                sqlBuffer.append(" AND "
                        + MapUtils.getString(queryParam,
                        Constants.QUERY_SPEC_TAIL_CONDITION));
        }

        if (!StringUtils.isEmpty(groupBy))
            sqlBuffer.append(" " + groupBy);

        Session session = null;

        try {
            session = getSession();
            Query query = session.createSQLQuery(sqlBuffer.toString());
            for (Iterator<String> itKey = cloneParam.keySet().iterator(); itKey
                    .hasNext(); ) {
                String key = itKey.next();
                Object value = MapUtils.getObject(cloneParam, key);
                if (isMaskByKey(key) | isMaskByValue(value))
                    continue;

                query.setParameter(key.replace('.', '_'), value);
            }

            if (!StringUtils.isEmpty(groupBy))
                return (Number) query.list().size();

            return (Number) query.uniqueResult();
        } catch (Exception ex) {
            logger.error("generalSearchCount error.", ex);
        }

        return null;
    }

    public Number generalCountByFullSql(String fromSql) {
        Session session = null;

        try {
            session = getSession();
            Query query = session.createSQLQuery(fromSql);

            return (Number) query.uniqueResult();
        } catch (Exception ex) {
            logger.error("generalSearchCount error.", ex);
        }

        return null;
    }

    public <T> void delete(Class<T> entity, Serializable id) {
        T delEntity = findEntity(entity, id);
        getSession().delete(delEntity);
    }

    public void delete(Object object) {
        getSession().delete(object);
    }

    public <T> void deleteAll(Collection<T> collection) {
        for (T t : collection) {
            getSession().delete(t);
        }
    }

    @SuppressWarnings("unchecked")
    public <T> T findEntity(Class<T> entity) {
        List<T> entities = getSession().createCriteria(entity).list();

        return null != entities && !entities.isEmpty() ? entities.get(0) : null;
    }

    @SuppressWarnings("unchecked")
    public <T> T findEntity(Class<T> entity, Serializable id) {
        return id == null ? null : (T) getSession().get(entity, id);
    }

    @SuppressWarnings("unchecked")
    public <T> T loadEntity(Class<T> entity, Serializable id) {
        return id == null ? null : (T) getSession().load(entity, id);
    }

    public Serializable save(Object entity) {
        return getSession().save(entity);
    }

    public <T> void saveOrUpdate(Object entity) {
        getSession().saveOrUpdate(entity);
    }

    public <T> void saveOrUpdateAll(Collection<T> collection) {
        for (T t : collection) {
            getSession().saveOrUpdate(t);
        }
    }

    @SuppressWarnings("unchecked")
    public Number executeByHql(String fromHql, Map<String, Object> queryParam,
                               Map<String, Object>... valueParams) throws Exception {
        Map<String, Object> cloneParam = null;
        if (null != queryParam && !queryParam.isEmpty())
            cloneParam = new HashMap<String, Object>(queryParam);
        else
            cloneParam = new HashMap<String, Object>();

        StringBuffer hqlBuffer = new StringBuffer(fromHql);
        if (fromHql.toUpperCase().indexOf("WHERE") < 0) {
            hqlBuffer.append(" WHERE 1=1 ");
        }

        if (null != queryParam && !queryParam.isEmpty()) {
            for (Iterator<String> it = queryParam.keySet().iterator(); it
                    .hasNext(); ) {
                String key = (String) it.next();
                hqlBuffer.append(getCondition(key, queryParam.get(key),
                        cloneParam));
            }
        }

        Session session = null;

        try {
            session = getSession();
            Query query = session.createQuery(hqlBuffer.toString());
            for (Iterator<String> itKey = cloneParam.keySet().iterator(); itKey
                    .hasNext(); ) {
                String key = itKey.next();
                Object value = MapUtils.getObject(cloneParam, key);
                if (isMaskByKey(key) | isMaskByValue(value))
                    continue;

                query.setParameter(key.replace('.', '_'), value);
            }

            if (null != valueParams && 0 < valueParams.length) {
                for (Map<String, Object> valueParam : valueParams) {
                    if (null != valueParam && !valueParam.isEmpty()) {
                        for (Iterator<String> itKey = valueParam.keySet()
                                .iterator(); itKey.hasNext(); ) {
                            String vKey = itKey.next();
                            Object value = MapUtils.getObject(valueParam, vKey);

                            if (isMaskByValue(value))
                                continue;

                            query.setParameter(vKey.replace(".", "_"), value);
                        }
                    }
                }
            }

            return query.executeUpdate();
        } catch (Exception ex) {
            logger.error("generalSearchCount error.", ex);
            throw ex;
        }
    }

    public <T> void update(Object entity) {
        getSession().update(entity);
    }

    public int updataByOriginSql(final String updateOriginSql) {
        Query query = getSession().createSQLQuery(updateOriginSql);
        int size = query.executeUpdate();
        return size;
    }

    // private Session openSession() {
    // return this.sessionFactory.openSession();
    // }

    public Session getSession() {
        return this.sessionFactory.getCurrentSession();
    }

    private String getCondition(String key, Object value,
                                Map<String, Object> queryParam) {
        StringBuffer buffer = new StringBuffer("");
        String dateFormat = "yyyy-MM-dd HH:mm:ss";
        if (isMaskByKey(key)) {
            queryParam.remove(key);
            return buffer.toString();
        }
        if (isMaskByValue(value)) {
            queryParam.remove(key);
            return buffer.toString();
        }

        if (value instanceof String) {
            value = ((String) value).trim();
        }

        if (key.startsWith(Constants.QUERY_START_DATE)) {
            if (!(value instanceof Date)) {
                try {
                    if (value.toString().length() < dateFormat.length())
                        value = value.toString() + " 00:00:00";
                    value = new SimpleDateFormat(dateFormat).parse(value
                            .toString());
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
            queryParam.remove(key);
            queryParam.put(key.replace(".", "_"), value);

            return buffer.append(" AND ")
                    .append(key.substring(Constants.QUERY_START_DATE.length()))
                    .append(" >=:" + key.replace(".", "_")).toString();
        } else if (key.startsWith(Constants.QUERY_END_DATE)) {
            if (!(value instanceof Date)) {
                try {
                    if (value.toString().length() < dateFormat.length())
                        value = value.toString() + " 23:59:59";

                    value = new SimpleDateFormat(dateFormat).parse(value
                            .toString());
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
            queryParam.remove(key);
            queryParam.put(key.replace(".", "_"), value);

            return buffer.append(" AND ")
                    .append(key.substring(Constants.QUERY_END_DATE.length()))
                    .append(" <=:" + key.replace(".", "_")).toString();
        } else if (key.startsWith(Constants.QUERY_GREATE_DATE)) {
            if (!(value instanceof Date)) {
                try {
                    if (value.toString().length() < dateFormat.length())
                        value = value.toString() + " 23:59:59";

                    value = new SimpleDateFormat(dateFormat).parse(value
                            .toString());
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
            queryParam.remove(key);
            queryParam.put(key.replace(".", "_"), value);

            return buffer
                    .append(" AND ")
                    .append(key.substring(Constants.QUERY_GREATE_DATE.length()))
                    .append(" >:" + key.replace(".", "_")).toString();
        } else if (key.startsWith(Constants.QUERY_GREATE_EQUAL_DATE)) {
            if (!(value instanceof Date)) {
                try {
                    if (value.toString().length() < dateFormat.length())
                        value = value.toString() + " 00:00:00";

                    value = new SimpleDateFormat(dateFormat).parse(value
                            .toString());
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
            queryParam.remove(key);
            queryParam.put(key.replace(".", "_"), value);

            return buffer
                    .append(" AND ")
                    .append(key.substring(Constants.QUERY_GREATE_EQUAL_DATE
                            .length())).append(" >=:" + key.replace(".", "_"))
                    .toString();
        } else if (key.startsWith(Constants.QUERY_LESS_DATE)) {
            if (!(value instanceof Date)) {
                try {
                    if (value.toString().length() < dateFormat.length())
                        value = value.toString() + " 23:59:59";

                    value = new SimpleDateFormat(dateFormat).parse(value
                            .toString());
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
            queryParam.remove(key);
            queryParam.put(key.replace(".", "_"), value);

            return buffer.append(" AND ")
                    .append(key.substring(Constants.QUERY_LESS_DATE.length()))
                    .append(" <:" + key.replace(".", "_")).toString();
        } else if (key.startsWith(Constants.QUERY_LESS_EQUAL_DATE)) {
            if (!(value instanceof Date)) {
                try {
                    if (value.toString().length() < dateFormat.length())
                        value = value.toString() + " 23:59:59";

                    value = new SimpleDateFormat(dateFormat).parse(value
                            .toString());
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
            queryParam.remove(key);
            queryParam.put(key.replace(".", "_"), value);

            return buffer
                    .append(" AND ")
                    .append(key.substring(Constants.QUERY_LESS_EQUAL_DATE
                            .length())).append(" <=:" + key.replace(".", "_"))
                    .toString();
        } else if (key.startsWith(Constants.EQUAL_DATE)) {
            if (!(value instanceof Date)) {
                if (value.toString().length() > "yyyy-MM-dd".length())
                    value = value.toString()
                            .substring(0, "yyyy-MM-dd".length());

                try {
                    value = new SimpleDateFormat("yyyy-MM-dd").parse(value
                            .toString());
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
            queryParam.remove(key);
            queryParam.put(key.replace(".", "_"), value);

            return buffer.append(" AND ")
                    .append(key.substring(Constants.EQUAL_DATE.length()))
                    .append(" =:" + key.replace(".", "_")).toString();
        } else if (key.startsWith(Constants.QUERY_NOTEQUAL_DATE)) {
            if (!(value instanceof Date)) {
                try {
                    if (value.toString().length() > "yyyy-MM-dd".length())
                        value = value.toString().substring(0,
                                "yyyy-MM-dd".length());

                    value = new SimpleDateFormat("yyyy-MM-dd").parse(value
                            .toString());
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
            queryParam.remove(key);
            queryParam.put(key.replace(".", "_"), value);

            return buffer
                    .append(" AND ")
                    .append(key.substring(Constants.QUERY_NOTEQUAL_DATE
                            .length())).append("!=:" + key.replace(".", "_"))
                    .toString();
        } else if (key.startsWith(Constants.QUERY_NOTEQUAL)) {
            queryParam.remove(key);
            queryParam.put(key.replace(".", "_"), value);

            return buffer.append(" AND ")
                    .append(key.substring(Constants.QUERY_NOTEQUAL.length()))
                    .append("!=:" + key.replace(".", "_")).toString();
        } else if (key.startsWith(Constants.QUERY_BEGIN_WITH)) {
            value = value + "%";
            queryParam.remove(key);
            queryParam.put(key.replace(".", "_"), value);

            return buffer.append(" AND ")
                    .append(key.substring(Constants.QUERY_BEGIN_WITH.length()))
                    .append(" LIKE :").append(key.replace(".", "_")).toString();
        } else if (key.startsWith(Constants.QUERY_END_WITH)) {
            value = "%" + value;
            queryParam.remove(key);
            queryParam.put(key.replace(".", "_"), value);

            return buffer.append(" AND ")
                    .append(key.substring(Constants.QUERY_END_WITH.length()))
                    .append(" LIKE :").append(key.replace(".", "_")).toString();
        } else if (key.startsWith(Constants.QUERY_LIKE)) {
            value = "%" + value + "%";
            queryParam.remove(key);
            queryParam.put(key.replace(".", "_"), value);

            return buffer.append(" AND ")
                    .append(key.substring(Constants.QUERY_LIKE.length()))
                    .append(" LIKE :").append(key.replace(".", "_")).toString();
        } else if (key.startsWith(Constants.QUERY_GREATE)) {
            return buffer.append(" AND ")
                    .append(key.substring(Constants.QUERY_GREATE.length()))
                    .append(" >:").append(key.replace(".", "_")).toString();
        } else if (key.startsWith(Constants.QUERY_GREATE_EQUAL)) {
            queryParam.remove(key);
            queryParam.put(key.replace(".", "_"), value);
            return buffer
                    .append(" AND ")
                    .append(key.substring(Constants.QUERY_GREATE_EQUAL.length()))
                    .append(" >=:").append(key.replace(".", "_")).toString();
        } else if (key.startsWith(Constants.QUERY_LESS)) {
            queryParam.remove(key);
            queryParam.put(key.replace(".", "_"), value);

            return buffer.append(" AND ")
                    .append(key.substring(Constants.QUERY_LESS.length()))
                    .append(" <:").append(key.replace(".", "_")).toString();
        } else if (key.startsWith(Constants.QUERY_LESS_EQUAL)) {
            queryParam.remove(key);
            queryParam.put(key.replace(".", "_"), value);

            return buffer.append(" AND ")
                    .append(key.substring(Constants.QUERY_LESS_EQUAL.length()))
                    .append(" <=:").append(key.replace(".", "_")).toString();
        } else if (key.startsWith(Constants.QUERY_SPEC_CONDITION)) {
            queryParam.remove(key);
            return buffer.append(" AND ").append(value).toString();
        } else if (key.startsWith(Constants.QUERY_EITHER)) {
            queryParam.remove(key);
            queryParam.put(key.replace(".", "_"), value);

            return buffer.append(" OR ")
                    .append(key.substring(Constants.QUERY_EITHER.length()))
                    .append("=:").append(key.replace(".", "_")).toString();
        } else {
            queryParam.remove(key);
            queryParam.put(key.replace(".", "_"), value);
            return buffer.append(" AND ").append(key).append("=:")
                    .append(key.replace(".", "_")).toString();
        }
    }

    private boolean isMaskByKey(String key) {
        return Constants.QUERY_FETCH_SIZE.equals(key)
                || Constants.QUERY_MAX_RESULTS.equals(key)
                || Constants.QUERY_FIRST_RESULTS.equals(key)
                || Constants.QUERY_ORDER_BY.equals(key)
                || Constants.QUERY_GROUP_BY.equals(key)
                || Constants.QUERY_SPEC_TAIL_CONDITION.equals(key)
                || StringUtils.startsWith(key, Constants.QUERY_FILTER);
    }

    private boolean isMaskByValue(Object value) {
        if (value == null) {
            return true;
        }

        if (value instanceof String) {
            return StringUtils.isEmpty((String) value);
        }

        return false;
    }

    private String removeSelect(String hql) {
        int beginPos = hql.toUpperCase().indexOf("FROM");
        return hql.substring(beginPos);
    }

    private int findWhereIndex(String sql, int lastWhereIndex) {
        int lastRightBracket = sql.toUpperCase().lastIndexOf(RIGHT_BRACKET);

        int whereIndex = sql.toUpperCase().lastIndexOf("WHERE");

        if (0 > whereIndex || whereIndex > lastRightBracket) {
            return whereIndex;
        }

        if (0 < whereIndex || lastRightBracket < whereIndex)
            sql = sql.substring(whereIndex + "WHERE".length());

        return findWhereIndex(sql, whereIndex);
    }

    private int findLastSqlKeyWords(String sql, String keyWords) {
        int lastRightBracketIndex = sql.toUpperCase()
                .lastIndexOf(RIGHT_BRACKET);
        int lastLeftBracketIndex = sql.toUpperCase().lastIndexOf(LEFT_BRACKET);
        int keyWordsIndex = sql.toUpperCase().lastIndexOf(keyWords);
        int lastFromIndex = sql.toUpperCase().lastIndexOf("FROM");
        if (keyWordsIndex > lastRightBracketIndex)
            return keyWordsIndex;
        else if (lastLeftBracketIndex > keyWordsIndex
                && lastRightBracketIndex > keyWordsIndex
                && keyWordsIndex > lastFromIndex)
            return keyWordsIndex;
        else if ((0 > keyWordsIndex && keyWordsIndex == lastRightBracketIndex)
                || 0 == lastRightBracketIndex)
            return -1;

        sql = StringUtils.substring(sql, lastRightBracketIndex);

        return findLastSqlKeyWords(sql, keyWords);
    }


}
