package com.bettersoft.api.unittest.testapi;

import java.lang.reflect.Type;
import java.util.List;

import javax.annotation.Resource;

import com.bettersoft.api.utils.StringDigestUtil;
import org.springframework.test.annotation.Commit;
import org.testng.annotations.BeforeTest;
import org.testng.annotations.Test;

import com.bettersoft.api.model.MenuTreeBean;
import com.bettersoft.api.model.SystemPermissionBean;
import com.bettersoft.api.service.interfaces.ISystemMenuTreeService;
import com.bettersoft.api.service.interfaces.ISystemPermissionService;
import com.bettersoft.api.unittest.utils.Bettersoft4TestNG;
import com.bettersoft.api.utils.LoginUserInfo;
import com.bettersoft.api.validmodel.SystemMenuTreeValidate;
import com.bettersoft.api.validmodel.SystemPermissionValidate;
import com.google.gson.reflect.TypeToken;

@Commit
public class BettersoftApiConfigurationTester extends Bettersoft4TestNG {

	@Resource(name = "systemMenuTreeService")
	private ISystemMenuTreeService systemMenuTreeService;

	@Resource(name = "systemPermissionService")
	private ISystemPermissionService systemPermissionService;

	@BeforeTest
	public void setupBefore() {
		System.out.println("setup success...");
		bettersoftTestDao.executeSqlCommand("truncate table bs_menutree");
		bettersoftTestDao.executeSqlCommand("truncate table bs_permission");
		bettersoftTestDao.executeSqlCommand("truncate table bs_menupermission");
	}

	@Test(enabled = true, priority = 1)
	public void savePermissions() throws Exception {
		Type listType = new TypeToken<List<SystemPermissionBean>>() {
		}.getType();
		StringBuffer jsonBuilder = new StringBuffer();
		jsonBuilder.append("[{");
		jsonBuilder.append("	permissionCode: \"ACCESS\",");
		jsonBuilder.append("	permissionDesc: \"访问权限\",");
		jsonBuilder.append("	permissionType: \"A\",");
		jsonBuilder.append("	createDateTime: \"2019-01-08 12:18:54\"");
		jsonBuilder.append("},");
		jsonBuilder.append("{");
		jsonBuilder.append("	permissionCode: \"INSERT\",");
		jsonBuilder.append("	permissionDesc: \"新增权限\",");
		jsonBuilder.append("	permissionType: \"B\",");
		jsonBuilder.append("	createDateTime: \"2019-01-08 12:18:54\"");
		jsonBuilder.append("},{");
		jsonBuilder.append("	permissionCode: \"UPDATE\",");
		jsonBuilder.append("	permissionDesc: \"修改权限\",");
		jsonBuilder.append("	permissionType: \"B\",");
		jsonBuilder.append("	createDateTime: \"2019-01-08 12:18:54\"");
		jsonBuilder.append("},{");
		jsonBuilder.append("	permissionCode: \"DELETE\",");
		jsonBuilder.append("	permissionDesc: \"删除权限\",");
		jsonBuilder.append("	permissionType: \"B\",");
		jsonBuilder.append("	createDateTime: \"2019-01-08 12:18:54\"");
		jsonBuilder.append("},{");
		jsonBuilder.append("	permissionCode: \"SEARCH\",");
		jsonBuilder.append("	permissionDesc: \"查询权限\",");
		jsonBuilder.append("	permissionType: \"B\",");
		jsonBuilder.append("	createDateTime: \"2019-01-08 12:18:54\"");
		jsonBuilder.append("},{");
		jsonBuilder.append("	permissionCode: \"APPROVE\",");
		jsonBuilder.append("	permissionDesc: \"审核权限\",");
		jsonBuilder.append("	permissionType: \"B\",");
		jsonBuilder.append("	createDateTime: \"2019-01-08 12:18:54\"");
		jsonBuilder.append("},{");
		jsonBuilder.append("	permissionCode: \"REJECT\",");
		jsonBuilder.append("	permissionDesc: \"驳回权限\",");
		jsonBuilder.append("	permissionType: \"B\",");
		jsonBuilder.append("	createDateTime: \"2019-01-08 12:18:54\"");
		jsonBuilder.append("},{");
		jsonBuilder.append("	permissionCode: \"PRINT\",");
		jsonBuilder.append("	permissionDesc: \"打印权限\",");
		jsonBuilder.append("	permissionType: \"B\",");
		jsonBuilder.append("	createDateTime: \"2019-01-08 12:18:54\"");
		jsonBuilder.append("}]");

		List<SystemPermissionValidate> permissions = gson.fromJson(
				jsonBuilder.toString(), listType);
		if (null != permissions && !permissions.isEmpty()) {
			for (SystemPermissionValidate permission : permissions)
				systemPermissionService.savePermission(permission,
						new LoginUserInfo());
		}
	}

	@Test(enabled = true, priority = 2)
	public void saveMenuTree() throws Exception {
		Type listType = new TypeToken<List<MenuTreeBean>>() {
		}.getType();
		StringBuffer jsonBuilder = new StringBuffer();
		jsonBuilder
				.append("[{                                                                  ");
		jsonBuilder
				.append("	menuCode: \"SYS\",                                              ");
		jsonBuilder
				.append("	menuName: \"系统管理\",                                         ");
		jsonBuilder
				.append("	parentCode: null,                                               ");
		jsonBuilder
				.append("	priority: 1,                                                    ");
		jsonBuilder
				.append("	enable: true,                                                   ");
		jsonBuilder
				.append("	menuIcon: \"fa fa-cog\",                                        ");
		jsonBuilder
				.append("	menuUrl: null,                                                  ");
		jsonBuilder
				.append("	menuType: \"D\",                                                ");
		jsonBuilder
				.append("	createDateTime: \"2019-01-08 10:51:22\",                        ");
		jsonBuilder
				.append("	childMenus: [{                                                  ");
		jsonBuilder
				.append("		menuCode: \"SYS01\",                                        ");
		jsonBuilder
				.append("		menuName: \"组织架构维护\",                                 ");
		jsonBuilder
				.append("		parentCode: \"SYS\",                                        ");
		jsonBuilder
				.append("		priority: 1,                                                ");
		jsonBuilder
				.append("		enable: true,                                               ");
		jsonBuilder
				.append("		menuIcon: \"fa fa-sitemap\",                                ");
		jsonBuilder
				.append("		menuUrl: null,                                              ");
		jsonBuilder
				.append("		menuType: \"M\",                                            ");
		jsonBuilder
				.append("		permissions: [{                                             ");
		jsonBuilder
				.append("			permissionCode: \"INSERT\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"UPDATE\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"SEARCH\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"DELETE\"                              ");
		jsonBuilder
				.append("		}],                                                         ");
		jsonBuilder
				.append("		createDateTime: \"2019-01-08 10:51:22\"                     ");
		jsonBuilder
				.append("	},                                                              ");
		jsonBuilder
				.append("	{                                                               ");
		jsonBuilder
				.append("		menuCode: \"SYS02\",                                        ");
		jsonBuilder
				.append("		menuName: \"系统用户维护\",                                 ");
		jsonBuilder
				.append("		parentCode: \"SYS\",                                        ");
		jsonBuilder
				.append("		priority: 2,                                                ");
		jsonBuilder
				.append("		enable: true,                                               ");
		jsonBuilder
				.append("		menuIcon: \"fa fa-cog\",                                    ");
		jsonBuilder
				.append("		menuUrl: null,                                              ");
		jsonBuilder
				.append("		menuType: \"M\",                                            ");
		jsonBuilder
				.append("		permissions: [{                                             ");
		jsonBuilder
				.append("			permissionCode: \"INSERT\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"UPDATE\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"SEARCH\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"DELETE\"                              ");
		jsonBuilder
				.append("		}],                                                         ");
		jsonBuilder
				.append("		createDateTime: \"2019-01-08 10:51:22\"                     ");
		jsonBuilder
				.append("	},                                                              ");
		jsonBuilder
				.append("	{                                                               ");
		jsonBuilder
				.append("		menuCode: \"SYS03\",                                        ");
		jsonBuilder
				.append("		menuName: \"系统权限维护\",                                 ");
		jsonBuilder
				.append("		parentCode: \"SYS\",                                        ");
		jsonBuilder
				.append("		priority: 3,                                                ");
		jsonBuilder
				.append("		enable: true,                                               ");
		jsonBuilder
				.append("		menuIcon: \"fa fa-book\",                                   ");
		jsonBuilder
				.append("		menuUrl: null,                                              ");
		jsonBuilder
				.append("		menuType: \"M\",                                            ");
		jsonBuilder
				.append("		permissions: [{                                             ");
		jsonBuilder
				.append("			permissionCode: \"INSERT\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"UPDATE\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"SEARCH\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"DELETE\"                              ");
		jsonBuilder
				.append("		}],                                                         ");
		jsonBuilder
				.append("		createDateTime: \"2019-01-08 10:51:22\"                     ");
		jsonBuilder
				.append("	},                                                              ");
		jsonBuilder
				.append("	{                                                               ");
		jsonBuilder
				.append("		menuCode: \"SYS04\",                                        ");
		jsonBuilder
				.append("		menuName: \"角色权限维护\",                                 ");
		jsonBuilder
				.append("		parentCode: \"SYS\",                                        ");
		jsonBuilder
				.append("		priority: 4,                                                ");
		jsonBuilder
				.append("		enable: true,                                               ");
		jsonBuilder
				.append("		menuIcon: \"fa fa-user-secret\",                            ");
		jsonBuilder
				.append("		menuUrl: null,                                              ");
		jsonBuilder
				.append("		menuType: \"M\",                                            ");
		jsonBuilder
				.append("		permissions: [{                                             ");
		jsonBuilder
				.append("			permissionCode: \"INSERT\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"UPDATE\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"SEARCH\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"DELETE\"                              ");
		jsonBuilder
				.append("		}],                                                         ");
		jsonBuilder
				.append("		createDateTime: \"2019-01-08 10:51:22\"                     ");
		jsonBuilder
				.append("	}]                                                              ");
		jsonBuilder
				.append("},                                                                  ");
		jsonBuilder
				.append("{                                                                   ");
		jsonBuilder
				.append("	menuCode: \"MA\",                                               ");
		jsonBuilder
				.append("	menuName: \"主档资料管理\",                                     ");
		jsonBuilder
				.append("	parentCode: null,                                               ");
		jsonBuilder
				.append("	priority: 2,                                                    ");
		jsonBuilder
				.append("	enable: true,                                                   ");
		jsonBuilder
				.append("	menuIcon: \"fa fa-cog\",                                        ");
		jsonBuilder
				.append("	menuUrl: null,                                                  ");
		jsonBuilder
				.append("	menuType: \"D\",                                                ");
		jsonBuilder
				.append("	createDateTime: \"2019-01-08 10:51:22\",                        ");
		jsonBuilder
				.append("	childMenus: [{                                                  ");
		jsonBuilder
				.append("		menuCode: \"MA01\",                                         ");
		jsonBuilder
				.append("		menuName: \"食用菌国标维护\",                               ");
		jsonBuilder
				.append("		parentCode: \"MA\",                                         ");
		jsonBuilder
				.append("		priority: 1,                                                ");
		jsonBuilder
				.append("		enable: true,                                               ");
		jsonBuilder
				.append("		menuIcon: \"fa fa-book\",                                   ");
		jsonBuilder
				.append("		menuUrl: null,                                              ");
		jsonBuilder
				.append("		menuType: \"M\",                                            ");
		jsonBuilder
				.append("		permissions: [{                                             ");
		jsonBuilder
				.append("			permissionCode: \"INSERT\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"UPDATE\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"SEARCH\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"DELETE\"                              ");
		jsonBuilder
				.append("		}],                                                         ");
		jsonBuilder
				.append("		createDateTime: \"2019-01-08 10:51:22\"                     ");
		jsonBuilder
				.append("	},                                                              ");
		jsonBuilder
				.append("	{                                                               ");
		jsonBuilder
				.append("		menuCode: \"MA02\",                                         ");
		jsonBuilder
				.append("		menuName: \"实验参数设定\",                                 ");
		jsonBuilder
				.append("		parentCode: \"MA\",                                         ");
		jsonBuilder
				.append("		priority: 2,                                                ");
		jsonBuilder
				.append("		enable: true,                                               ");
		jsonBuilder
				.append("		menuIcon: \"fa fa-book\",                                   ");
		jsonBuilder
				.append("		menuUrl: null,                                              ");
		jsonBuilder
				.append("		menuType: \"M\",                                            ");
		jsonBuilder
				.append("		permissions: [{                                             ");
		jsonBuilder
				.append("			permissionCode: \"INSERT\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"UPDATE\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"SEARCH\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"DELETE\"                              ");
		jsonBuilder
				.append("		}],                                                         ");
		jsonBuilder
				.append("		createDateTime: \"2019-01-08 10:51:22\"                     ");
		jsonBuilder
				.append("	},                                                              ");
		jsonBuilder
				.append("	{                                                               ");
		jsonBuilder
				.append("		menuCode: \"MA03\",                                         ");
		jsonBuilder
				.append("		menuName: \"菌种二维码生成\",                               ");
		jsonBuilder
				.append("		parentCode: \"MA\",                                         ");
		jsonBuilder
				.append("		priority: 3,                                                ");
		jsonBuilder
				.append("		enable: true,                                               ");
		jsonBuilder
				.append("		menuIcon: \"fa fa-book\",                                   ");
		jsonBuilder
				.append("		menuUrl: null,                                              ");
		jsonBuilder
				.append("		menuType: \"M\",                                            ");
		jsonBuilder
				.append("		permissions: [{                                             ");
		jsonBuilder
				.append("			permissionCode: \"INSERT\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"UPDATE\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"SEARCH\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"DELETE\"                              ");
		jsonBuilder
				.append("		}],                                                         ");
		jsonBuilder
				.append("		createDateTime: \"2019-01-08 10:51:22\"                     ");
		jsonBuilder
				.append("	},                                                              ");
		jsonBuilder
				.append("	{                                                               ");
		jsonBuilder
				.append("		menuCode: \"MA04\",                                         ");
		jsonBuilder
				.append("		menuName: \"菌种分类标签\",                                 ");
		jsonBuilder
				.append("		parentCode: \"MA\",                                         ");
		jsonBuilder
				.append("		priority: 4,                                                ");
		jsonBuilder
				.append("		enable: true,                                               ");
		jsonBuilder
				.append("		menuIcon: \"fa fa-book\",                                   ");
		jsonBuilder
				.append("		menuUrl: null,                                              ");
		jsonBuilder
				.append("		menuType: \"M\",                                            ");
		jsonBuilder
				.append("		permissions: [{                                             ");
		jsonBuilder
				.append("			permissionCode: \"INSERT\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"UPDATE\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"SEARCH\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"DELETE\"                              ");
		jsonBuilder
				.append("		}],                                                         ");
		jsonBuilder
				.append("		createDateTime: \"2019-01-08 10:51:22\"                     ");
		jsonBuilder
				.append("	},                                                              ");
		jsonBuilder
				.append("	{                                                               ");
		jsonBuilder
				.append("		menuCode: \"MA05\",                                         ");
		jsonBuilder
				.append("		menuName: \"菌种主档维护\",                                 ");
		jsonBuilder
				.append("		parentCode: \"MA\",                                         ");
		jsonBuilder
				.append("		priority: 4,                                                ");
		jsonBuilder
				.append("		enable: true,                                               ");
		jsonBuilder
				.append("		menuIcon: \"fa fa-book\",                                   ");
		jsonBuilder
				.append("		menuUrl: null,                                              ");
		jsonBuilder
				.append("		menuType: \"M\",                                            ");
		jsonBuilder
				.append("		permissions: [{                                             ");
		jsonBuilder
				.append("			permissionCode: \"INSERT\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"UPDATE\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"SEARCH\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"DELETE\"                              ");
		jsonBuilder
				.append("		}],                                                         ");
		jsonBuilder
				.append("		createDateTime: \"2019-01-08 10:51:22\"                     ");
		jsonBuilder
				.append("	},                                                              ");
		jsonBuilder
				.append("	{                                                               ");
		jsonBuilder
				.append("		menuCode: \"MA06\",                                         ");
		jsonBuilder
				.append("		menuName: \"培养流程定义\",                                 ");
		jsonBuilder
				.append("		parentCode: \"MA\",                                         ");
		jsonBuilder
				.append("		priority: 5,                                                ");
		jsonBuilder
				.append("		enable: true,                                               ");
		jsonBuilder
				.append("		menuIcon: \"fa fa-book\",                                   ");
		jsonBuilder
				.append("		menuUrl: null,                                              ");
		jsonBuilder
				.append("		menuType: \"M\",                                            ");
		jsonBuilder
				.append("		permissions: [{                                             ");
		jsonBuilder
				.append("			permissionCode: \"INSERT\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"UPDATE\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"SEARCH\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"DELETE\"                              ");
		jsonBuilder
				.append("		}],                                                         ");
		jsonBuilder
				.append("		createDateTime: \"2019-01-08 10:51:22\"                     ");
		jsonBuilder
				.append("	}]                                                              ");
		jsonBuilder
				.append("},                                                                  ");
		jsonBuilder
				.append("{                                                                   ");
		jsonBuilder
				.append("	menuCode: \"CR\",                                               ");
		jsonBuilder
				.append("	menuName: \"培养流程管理\",                                      ");
		jsonBuilder
				.append("	parentCode: \"MA\",                                             ");
		jsonBuilder
				.append("	priority: 3,                                                    ");
		jsonBuilder
				.append("	enable: true,                                                   ");
		jsonBuilder
				.append("	menuIcon: \"fa fa-book\",                                       ");
		jsonBuilder
				.append("	menuUrl: null,                                                  ");
		jsonBuilder
				.append("	menuType: \"D\",                                                ");
		jsonBuilder
				.append("	createDateTime: \"2019-01-08 10:51:22\",                        ");
		jsonBuilder
				.append("	childMenus: [{                                                  ");
		jsonBuilder
				.append("		menuCode: \"CR01\",                                         ");
		jsonBuilder
				.append("		menuName: \"菌种培养立项\",                                  ");
		jsonBuilder
				.append("		parentCode: \"CR\",                                         ");
		jsonBuilder
				.append("		priority: 1,                                                ");
		jsonBuilder
				.append("		enable: true,                                               ");
		jsonBuilder
				.append("		menuIcon: \"fa fa-book\",                                   ");
		jsonBuilder
				.append("		menuUrl: null,                                              ");
		jsonBuilder
				.append("		menuType: \"M\",                                            ");
		jsonBuilder
				.append("		permissions: [{                                             ");
		jsonBuilder
				.append("			permissionCode: \"INSERT\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"UPDATE\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"SEARCH\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"DELETE\"                              ");
		jsonBuilder
				.append("		}],                                                         ");
		jsonBuilder
				.append("		createDateTime: \"2019-01-08 10:51:22\"                     ");
		jsonBuilder
				.append("	},                                                              ");
		jsonBuilder
				.append("	{                                                               ");
		jsonBuilder
				.append("		menuCode: \"CR02\",                                         ");
		jsonBuilder
				.append("		menuName: \"菌种培养项目审批\",                             ");
		jsonBuilder
				.append("		parentCode: \"CR\",                                         ");
		jsonBuilder
				.append("		priority: 2,                                                ");
		jsonBuilder
				.append("		enable: true,                                               ");
		jsonBuilder
				.append("		menuIcon: \"fa fa-book\",                                   ");
		jsonBuilder
				.append("		menuUrl: null,                                              ");
		jsonBuilder
				.append("		menuType: \"M\",                                            ");
		jsonBuilder
				.append("		permissions: [{                                             ");
		jsonBuilder
				.append("			permissionCode: \"INSERT\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"UPDATE\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"SEARCH\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"DELETE\"                              ");
		jsonBuilder
				.append("		}],                                                         ");
		jsonBuilder
				.append("		createDateTime: \"2019-01-08 10:51:22\"                     ");
		jsonBuilder
				.append("	},                                                              ");
		jsonBuilder
				.append("	{                                                               ");
		jsonBuilder
				.append("		menuCode: \"CR03\",                                         ");
		jsonBuilder
				.append("		menuName: \"菌种培养项目维护\",                             ");
		jsonBuilder
				.append("		parentCode: \"CR\",                                         ");
		jsonBuilder
				.append("		priority: 3,                                                ");
		jsonBuilder
				.append("		enable: true,                                               ");
		jsonBuilder
				.append("		menuIcon: \"fa fa-book\",                                   ");
		jsonBuilder
				.append("		menuUrl: null,                                              ");
		jsonBuilder
				.append("		menuType: \"M\",                                            ");
		jsonBuilder
				.append("		permissions: [{                                             ");
		jsonBuilder
				.append("			permissionCode: \"INSERT\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"UPDATE\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"SEARCH\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"DELETE\"                              ");
		jsonBuilder
				.append("		}],                                                         ");
		jsonBuilder
				.append("		createDateTime: \"2019-01-08 10:51:22\"                     ");
		jsonBuilder
				.append("	},                                                              ");
		jsonBuilder
				.append("	{                                                               ");
		jsonBuilder
				.append("		menuCode: \"CR04\",                                         ");
		jsonBuilder
				.append("		menuName: \"培养数据对比\",                                 ");
		jsonBuilder
				.append("		parentCode: \"CR\",                                         ");
		jsonBuilder
				.append("		priority: 4,                                                ");
		jsonBuilder
				.append("		enable: true,                                               ");
		jsonBuilder
				.append("		menuIcon: \"fa fa-book\",                                   ");
		jsonBuilder
				.append("		menuUrl: null,                                              ");
		jsonBuilder
				.append("		menuType: \"M\",                                            ");
		jsonBuilder
				.append("		permissions: [{                                             ");
		jsonBuilder
				.append("			permissionCode: \"INSERT\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"UPDATE\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"SEARCH\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"DELETE\"                              ");
		jsonBuilder
				.append("		}],                                                         ");
		jsonBuilder
				.append("		createDateTime: \"2019-01-08 10:51:22\"                     ");
		jsonBuilder
				.append("	},                                                              ");
		jsonBuilder
				.append("	{                                                               ");
		jsonBuilder
				.append("		menuCode: \"CR05\",                                         ");
		jsonBuilder
				.append("		menuName: \"历年科研数据分析\",                             ");
		jsonBuilder
				.append("		parentCode: \"CR\",                                         ");
		jsonBuilder
				.append("		priority: 5,                                                ");
		jsonBuilder
				.append("		enable: true,                                               ");
		jsonBuilder
				.append("		menuIcon: \"fa fa-book\",                                   ");
		jsonBuilder
				.append("		menuUrl: null,                                              ");
		jsonBuilder
				.append("		menuType: \"M\",                                            ");
		jsonBuilder
				.append("		permissions: [{                                             ");
		jsonBuilder
				.append("			permissionCode: \"INSERT\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"UPDATE\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"SEARCH\"                              ");
		jsonBuilder
				.append("		},                                                          ");
		jsonBuilder
				.append("		{                                                           ");
		jsonBuilder
				.append("			permissionCode: \"DELETE\"                              ");
		jsonBuilder
				.append("		}],                                                         ");
		jsonBuilder
				.append("		createDateTime: \"2019-01-08 10:51:22\"                     ");
		jsonBuilder
				.append("	}]                                                              ");
		jsonBuilder
				.append("}]                                                                  ");
	}
}
